module mod_interpolate

	implicit none
	
	!This module contains functions for interpolating between grid points
	
contains

function ly(yval) result(low)

	use mod_globvar

	implicit none

	!******* input/output variables *******
	real(8), intent(in) :: yval
	real(8) :: yval1, y_mod, y1
	integer :: low

	!******* compute low *******
	if (yval >= 0.0) then
		yval1 = min(max_y,yval)
		y_mod = yval1**curv_ypos
		low   = ny_neg + 1 + floor(y_mod/dy_pos)
	else
		yval1 = max(min_y,yval)
		y_mod = -(-yval1)**curv_yneg
		y1    = -(-min_y)**curv_yneg
		low   = 1 + floor((y_mod - y1)/dy_neg)
	endif

end function ly

function uy(yval) result(high)

	use mod_globvar

	implicit none

	!******* input/output variables *******
	real(8), intent(in) :: yval
	real(8) :: yval1, y_mod, y1
	integer :: high

	!******* compute high *******
	if (yval >= 0.0) then
		yval1 = min(max_y,yval)
		y_mod = yval1**curv_ypos
		high  = ny_neg + 1 + ceiling(y_mod/dy_pos)
	else
		yval1 = max(min_y,yval)
		y_mod = -(-yval1)**curv_yneg
		y1    = -(-min_y)**curv_yneg
		high  = 1 + ceiling((y_mod - y1)/dy_neg)
	endif

end function uy

function wy(yval) result(weight)

	use mod_globvar

	implicit none

	!******* input/output variables *******
	real(8), intent(in) :: yval
	real(8) :: yval1, y_mod, y1, weight
	integer :: low, high

	!******* compute weight *******
	if (yval >= 0.0) then
		yval1 = min(max_y,yval)
		y_mod = yval1**curv_ypos
		low   = ny_neg + 1 + floor(y_mod/dy_pos)
		high  = ny_neg + 1 + ceiling(y_mod/dy_pos)
	else
		yval1 = max(min_y,yval)
		y_mod = -(-yval1)**curv_yneg
		y1    = -(-min_y)**curv_yneg
		low   = 1 + floor((y_mod - y1)/dy_neg)
		high  = 1 + ceiling((y_mod - y1)/dy_neg)
	endif
	if (low == high) then
		weight = 0.0
	else
		weight = (yval1 - yy(low))/(yy(high) - yy(low))
	endif

end function wy

function lyfine(yval) result(low)

	use mod_globvar

	implicit none

	!******* input/output variables *******
	real(8), intent(in) :: yval
	real(8) :: yval1, y_mod, y1
	integer :: low

	!******* compute low *******
	if (yval >= 0.0) then
		yval1 = min(max_yfine,yval)
		y_mod = yval1**curv_yfinepos
		low   = nyfine_neg + 1 + floor(y_mod/dyfine_pos)
	else
		yval1 = max(min_yfine,yval)
		y_mod = -(-yval1)**curv_yfineneg
		y1    = -(-min_yfine)**curv_yfineneg
		low   = 1 + floor((y_mod - y1)/dyfine_neg)
	endif

end function lyfine

function uyfine(yval) result(high)

	use mod_globvar

	implicit none

	!******* input/output variables *******
	real(8), intent(in) :: yval
	real(8) :: yval1, y_mod, y1
	integer :: high

	!******* compute high *******
	if (yval >= 0.0) then
		yval1 = min(max_yfine,yval)
		y_mod = yval1**curv_yfinepos
		high  = nyfine_neg + 1 + ceiling(y_mod/dyfine_pos)
	else
		yval1 = max(min_yfine,yval)
		y_mod = -(-yval1)**curv_yfineneg
		y1    = -(-min_yfine)**curv_yfineneg
		high  = 1 + ceiling((y_mod - y1)/dyfine_neg)
	endif

end function uyfine

function wyfine(yval) result(weight)

	use mod_globvar

	implicit none

	!******* input/output variables *******
	real(8), intent(in) :: yval
	real(8) :: yval1, y_mod, y1, weight
	integer :: low, high

	!******* compute weight *******
	if (yval >= 0.0) then
		yval1 = min(max_yfine,yval)
		y_mod = yval1**curv_yfinepos
		low   = nyfine_neg + 1 + floor(y_mod/dyfine_pos)
		high  = nyfine_neg + 1 + ceiling(y_mod/dyfine_pos)
	else
		yval1 = max(min_yfine,yval)
		y_mod = -(-yval1)**curv_yfineneg
		y1    = -(-min_yfine)**curv_yfineneg
		low   = 1 + floor((y_mod - y1)/dyfine_neg)
		high  = 1 + ceiling((y_mod - y1)/dyfine_neg)
	endif
	if (low == high) then
		weight = 0.0
	else
		weight = (yval1 - yyfine(low))/(yyfine(high) - yyfine(low))
	endif

end function wyfine

end module mod_interpolate