subroutine sub_aggresults

	use mod_globvar
	use mod_interpolate

	implicit none

	!This program computes aggregate results, including welfare, consumption inequality, default premia, etc.

	!Indices
	integer :: is, ig, ih, im, iy, ibprime, imprime
	
	!Mortgage prices without search frictions
	real(8), dimension(:,:,:,:), allocatable :: qm0_nosearch
	
	!Value functions
	real(8), dimension(:,:,:,:), allocatable :: Vown_f0_sp3, Vown_f0_sp3_nosearch, welfare_own_f0
	real(8), dimension(:,:,:), allocatable :: Vown_f1_nosearch, welfare_own_f1
	real(8), dimension(:,:), allocatable :: Vrent_f0_nosearch, Vrent_f1_nosearch, welfare_rent_f0, welfare_rent_f1
	
	!Lower bound of budget set in Walrasian model
	real(8), dimension(ng,ns) :: ylb_nosearch
	
	!Other internal variables
	integer :: compare_nosearch, loadvals, iylow, iyupp
	real(8) :: qm0_orig, num_orig, mean_qm0_orig, mean_qm0ltv80_orig, mean_qm0ltv90_orig, mean_qm0ltv95_orig, min_q, min_q_nosearch, mean_cons, sd_cons, mean_owncons, sd_owncons, &
	mean_rentcons, sd_rentcons, meas_own, meas_rent, mean_ltv80cons, sd_ltv80cons, meas_ltv80, mean_ltv90cons, sd_ltv90cons, meas_ltv90, mean_lowltvcons, sd_lowltvcons, meas_lowltv, &
	ypay, yweight, welfare_loss, welfare_loss_rent, welfare_loss_own, welfare_loss_ltv80, welfare_loss_ltv90, mean_bonds, sd_bonds, mean_ownbonds, sd_ownbonds, mean_rentbonds, &
	sd_rentbonds, mean_ltv80bonds, sd_ltv80bonds, mean_ltv90bonds, sd_ltv90bonds, mean_lowltvbonds, sd_lowltvbonds, ltv80bonds, ltv90bonds
	
	!!!Mortgage Default Premia!!!	
	print *, ' '
	write(*,*), 'Compare baseline to model without search?'
	read(*,*) compare_nosearch
	if (compare_nosearch == 1) then
		!Allocate and load no-search mortgage prices
		allocate(qm0_nosearch(nm,nb,ng,ns))
		open(unit=11,	file='Input\qm0_nosearch.txt')
		do is = 1, ns
			do ig = 1, ng
				do ibprime = 1, nb
					read (11, *) (qm0_nosearch(imprime,ibprime,ig,is), imprime = 1,nm)
				enddo
			enddo
		enddo
		close(11)
	endif
	
	!Compute average mortgage prices
	!Overall, at origination
	qm0_orig = 0.d0
	num_orig = 0.d0
	do is = 1, ns
		do ig = 1, ng
			do iy = 1, ny
				qm0_orig = qm0_orig + qm0(locm_f0(iy,1,ig,is),locbown_f0(iy,1,ig,is),ig,is)*mu3own_f0(iy,1,ig,is)
				num_orig = num_orig + mu3own_f0(iy,1,ig,is)
			enddo
		enddo
	enddo
	mean_qm0_orig = qm0_orig/num_orig
	!For mortgages with ltv > 80%, at origination
	qm0_orig = 0.d0
	num_orig = 0.d0
	do is = 1, ns
		do ig = 1, ng
			do iy = 1, ny
				if (qm0(locm_f0(iy,1,ig,is),locbown_f0(iy,1,ig,is),ig,is)*m_f0(iy,1,ig,is) >= 0.8d0) then
					qm0_orig = qm0_orig + qm0(locm_f0(iy,1,ig,is),locbown_f0(iy,1,ig,is),ig,is)*mu3own_f0(iy,1,ig,is)
					num_orig = num_orig + mu3own_f0(iy,1,ig,is)
				endif
			enddo
		enddo
	enddo
	mean_qm0ltv80_orig = qm0_orig/num_orig
	!For mortgages with ltv > 90%, at origination
	qm0_orig = 0.d0
	num_orig = 0.d0
	do is = 1, ns
		do ig = 1, ng
			do iy = 1, ny
				if (qm0(locm_f0(iy,1,ig,is),locbown_f0(iy,1,ig,is),ig,is)*m_f0(iy,1,ig,is) >= 0.90d0) then
					qm0_orig = qm0_orig + qm0(locm_f0(iy,1,ig,is),locbown_f0(iy,1,ig,is),ig,is)*mu3own_f0(iy,1,ig,is)
					num_orig = num_orig + mu3own_f0(iy,1,ig,is)
				endif
			enddo
		enddo
	enddo
	mean_qm0ltv90_orig = qm0_orig/num_orig
	!For mortgages with ltv > 95%, at origination
	qm0_orig = 0.d0
	num_orig = 0.d0
	do is = 1, ns
		do ig = 1, ng
			do iy = 1, ny
				if (qm0(locm_f0(iy,1,ig,is),locbown_f0(iy,1,ig,is),ig,is)*m_f0(iy,1,ig,is) >= 0.95d0) then
					qm0_orig = qm0_orig + qm0(locm_f0(iy,1,ig,is),locbown_f0(iy,1,ig,is),ig,is)*mu3own_f0(iy,1,ig,is)
					num_orig = num_orig + mu3own_f0(iy,1,ig,is)
				endif
			enddo
		enddo
	enddo
	mean_qm0ltv95_orig = qm0_orig/num_orig
	
	print *, ' '
	print *, 'Average qm0 for all borrowers'
	print *, mean_qm0_orig
	print *, qm/(mean_qm0_orig*(1 + rocost)) - 1
	print *, ' '
	print *, 'Average qm0 for borrowers choosing ltv > 80%'
	print *, mean_qm0ltv80_orig
	print *, qm/(mean_qm0ltv80_orig*(1 + rocost)) - 1
	print *, ' '
	print *, 'Average qm0 for borrowers choosing ltv > 90%'
	print *, mean_qm0ltv90_orig
	print *, qm/(mean_qm0ltv90_orig*(1 + rocost)) - 1
	print *, ' '
	print *, 'Average qm0 for borrowers choosing ltv > 95%'
	print *, mean_qm0ltv95_orig
	print *, qm/(mean_qm0ltv95_orig*(1 + rocost)) - 1
	
	if (compare_nosearch == 1) then
		!Compute average mortgage prices WITHOUT SEARCH FRICTIONS
		!Overall, at origination
		qm0_orig = 0.d0
		num_orig = 0.d0
		do is = 1, ns
			do ig = 1, ng
				do iy = 1, ny
					qm0_orig = qm0_orig + qm0_nosearch(locm_f0(iy,1,ig,is),locbown_f0(iy,1,ig,is),ig,is)*mu3own_f0(iy,1,ig,is)
					num_orig = num_orig + mu3own_f0(iy,1,ig,is)
				enddo
			enddo
		enddo
		mean_qm0_orig = qm0_orig/num_orig
		!For mortgages with ltv > 80%, at origination
		qm0_orig = 0.d0
		num_orig = 0.d0
		do is = 1, ns
			do ig = 1, ng
				do iy = 1, ny
					if (qm0_nosearch(locm_f0(iy,1,ig,is),locbown_f0(iy,1,ig,is),ig,is)*m_f0(iy,1,ig,is) >= 0.8d0) then
						qm0_orig = qm0_orig + qm0_nosearch(locm_f0(iy,1,ig,is),locbown_f0(iy,1,ig,is),ig,is)*mu3own_f0(iy,1,ig,is)
						num_orig = num_orig + mu3own_f0(iy,1,ig,is)
					endif
				enddo
			enddo
		enddo
		mean_qm0ltv80_orig = qm0_orig/num_orig
		!For mortgages with ltv > 90%, at origination
		qm0_orig = 0.d0
		num_orig = 0.d0
		do is = 1, ns
			do ig = 1, ng
				do iy = 1, ny
					if (qm0_nosearch(locm_f0(iy,1,ig,is),locbown_f0(iy,1,ig,is),ig,is)*m_f0(iy,1,ig,is) >= 0.90d0) then
						qm0_orig = qm0_orig + qm0_nosearch(locm_f0(iy,1,ig,is),locbown_f0(iy,1,ig,is),ig,is)*mu3own_f0(iy,1,ig,is)
						num_orig = num_orig + mu3own_f0(iy,1,ig,is)
					endif
				enddo
			enddo
		enddo
		mean_qm0ltv90_orig = qm0_orig/num_orig
		!For mortgages with ltv > 95%, at origination
		qm0_orig = 0.d0
		num_orig = 0.d0
		do is = 1, ns
			do ig = 1, ng
				do iy = 1, ny
					if (qm0_nosearch(locm_f0(iy,1,ig,is),locbown_f0(iy,1,ig,is),ig,is)*m_f0(iy,1,ig,is) >= 0.95d0) then
						qm0_orig = qm0_orig + qm0_nosearch(locm_f0(iy,1,ig,is),locbown_f0(iy,1,ig,is),ig,is)*mu3own_f0(iy,1,ig,is)
						num_orig = num_orig + mu3own_f0(iy,1,ig,is)
					endif
				enddo
			enddo
		enddo
		mean_qm0ltv95_orig = qm0_orig/num_orig
		
		print *, ' '
		print *, 'WITHOUT SEARCH FRICTIONS'
		print *, ' '
		print *, 'Average qm0 for all borrowers'
		print *, mean_qm0_orig
		print *, qm/(mean_qm0_orig*(1 + rocost)) - 1
		print *, ' '
		print *, 'Average qm0 for borrowers choosing ltv > 80%'
		print *, mean_qm0ltv80_orig
		print *, qm/(mean_qm0ltv80_orig*(1 + rocost)) - 1
		print *, ' '
		print *, 'Average qm0 for borrowers choosing ltv > 90%'
		print *, mean_qm0ltv90_orig
		print *, qm/(mean_qm0ltv90_orig*(1 + rocost)) - 1
		print *, ' '
		print *, 'Average qm0 for borrowers choosing ltv > 95%'
		print *, mean_qm0ltv95_orig
		print *, qm/(mean_qm0ltv95_orig*(1 + rocost)) - 1
	endif
	
	!Lowest chosen qm0 and what it would be without search frictions
	min_q = 1.d0
	do is = 1, ns
		do ig = 1, ng
			do iy = 1, ny
				if (mu3own_f0(iy,1,ig,is) > 0.d0) then
					if (qm0(locm_f0(iy,1,ig,is),locbown_f0(iy,1,ig,is),ig,is) < min_q) then
						min_q = qm0(locm_f0(iy,1,ig,is),locbown_f0(iy,1,ig,is),ig,is)
						if (compare_nosearch == 1) then
							min_q_nosearch = qm0_nosearch(locm_f0(iy,1,ig,is),locbown_f0(iy,1,ig,is),ig,is)
						endif
					endif
				endif
			enddo
		enddo
	enddo
	
	print *, ' '
	print *, 'Highest actively chosen default premium'
	print *, qm/(min_q*(1 + rocost)) - 1
	if (compare_nosearch == 1) then
		print *, qm/(min_q_nosearch*(1 + rocost)) - 1
	endif
	
	!!!Consumption Inequality!!!
	
	!Mean of log consumption
	mean_cons       = 0.d0
	mean_owncons    = 0.d0
	mean_rentcons   = 0.d0
	mean_ltv80cons  = 0.d0
	mean_ltv90cons  = 0.d0
	mean_lowltvcons = 0.d0
	meas_own        = 0.d0
	meas_rent       = 0.d0
	meas_ltv80      = 0.d0
	meas_ltv90      = 0.d0
	meas_lowltv     = 0.d0
	do is = 1, ns
		!Renters
		do iy = ny_neg + 1, ny
			!Renters with good credit
			mean_cons     = mean_cons + log(crent_f0(iy,is))*mu3rent_f0(iy,is)
			mean_rentcons = mean_rentcons + log(crent_f0(iy,is))*mu3rent_f0(iy,is)
			meas_rent     = meas_rent + mu3rent_f0(iy,is)
			!Renters with bad credit
			mean_cons     = mean_cons + log(crent_f1(iy,is))*mu3rent_f1(iy,is)
			mean_rentcons = mean_rentcons + log(crent_f1(iy,is))*mu3rent_f1(iy,is)
			meas_rent     = meas_rent + mu3rent_f1(iy,is)
		enddo
		!Homeowners
		do ig = 1, ng
			!Homeowners with bad credit
			do iy = ny_neg + 1, ny
				mean_cons    = mean_cons + log(cown_f1(iy,ig,is))*mu3own_f1(iy,ig,is)
				mean_owncons = mean_owncons + log(cown_f1(iy,ig,is))*mu3own_f1(iy,ig,is)
				meas_own     = meas_own + mu3own_f1(iy,ig,is)
			enddo
			!Homeowners with good credit but no mortgage
			do iy = 1, ny
				mean_cons    = mean_cons + log(cown_f0(iy,1,ig,is))*mu3own_f0(iy,1,ig,is)
				mean_owncons = mean_owncons + log(cown_f0(iy,1,ig,is))*mu3own_f0(iy,1,ig,is)
				meas_own     = meas_own + mu3own_f0(iy,1,ig,is)
				if (qm0(locm_f0(iy,1,ig,is),locbown_f0(iy,1,ig,is),ig,is)*m_f0(iy,1,ig,is) >= 0.8d0) then
					mean_ltv80cons = mean_ltv80cons + log(cown_f0(iy,1,ig,is))*mu3own_f0(iy,1,ig,is)
					meas_ltv80     = meas_ltv80 + mu3own_f0(iy,1,ig,is)
				endif
				if (qm0(locm_f0(iy,1,ig,is),locbown_f0(iy,1,ig,is),ig,is)*m_f0(iy,1,ig,is) >= 0.9d0) then
					mean_ltv90cons = mean_ltv90cons + log(cown_f0(iy,1,ig,is))*mu3own_f0(iy,1,ig,is)
					meas_ltv90     = meas_ltv90 + mu3own_f0(iy,1,ig,is)
				endif
				if (qm0(locm_f0(iy,1,ig,is),locbown_f0(iy,1,ig,is),ig,is)*m_f0(iy,1,ig,is) < 0.8d0) then
					mean_lowltvcons = mean_lowltvcons + log(cown_f0(iy,1,ig,is))*mu3own_f0(iy,1,ig,is)
					meas_lowltv     = meas_lowltv + mu3own_f0(iy,1,ig,is)
				endif
			enddo
			do im = 2, nm
				!Homeowners with mortgages
				do iy = ny_neg + 1, ny
					if (mm(im) >= 0.8d0) then
						mean_ltv80cons = mean_ltv80cons + log(cown_f0(iy,im,ig,is))*mu3own_f0(iy,im,ig,is)
						meas_ltv80     = meas_ltv80 + mu3own_f0(iy,im,ig,is)
					endif
					if (mm(im) >= 0.9d0) then
						mean_ltv90cons = mean_ltv90cons + log(cown_f0(iy,im,ig,is))*mu3own_f0(iy,im,ig,is)
						meas_ltv90     = meas_ltv90 + mu3own_f0(iy,im,ig,is)
					endif
					if (mm(im) < 0.8d0) then
						mean_lowltvcons = mean_lowltvcons + log(cown_f0(iy,im,ig,is))*mu3own_f0(iy,im,ig,is)
						meas_lowltv     = meas_lowltv + mu3own_f0(iy,im,ig,is)
					endif
					mean_cons    = mean_cons + log(cown_f0(iy,im,ig,is))*mu3own_f0(iy,im,ig,is)
					mean_owncons = mean_owncons + log(cown_f0(iy,im,ig,is))*mu3own_f0(iy,im,ig,is)
					meas_own     = meas_own + mu3own_f0(iy,im,ig,is)
				enddo
			enddo
		enddo
	enddo
	mean_rentcons   = mean_rentcons/meas_rent
	mean_owncons    = mean_owncons/meas_own
	mean_ltv80cons  = mean_ltv80cons/meas_ltv80
	mean_ltv90cons  = mean_ltv90cons/meas_ltv90
	mean_lowltvcons = mean_lowltvcons/meas_lowltv
	
	!Standard deviation of consumption
	sd_cons       = 0.d0
	sd_owncons    = 0.d0
	sd_rentcons   = 0.d0
	sd_ltv80cons  = 0.d0
	sd_ltv90cons  = 0.d0
	sd_lowltvcons = 0.d0
	do is = 1, ns
		!Renters
		do iy = ny_neg + 1, ny
			!Renters with good credit
			sd_cons     = sd_cons + ((log(crent_f0(iy,is)) - mean_cons)**2)*mu3rent_f0(iy,is)
			sd_rentcons = sd_rentcons + ((log(crent_f0(iy,is)) - mean_rentcons)**2)*mu3rent_f0(iy,is)
			!Renters with bad credit
			sd_cons     = sd_cons + ((log(crent_f1(iy,is)) - mean_cons)**2)*mu3rent_f0(iy,is)
			sd_rentcons = sd_rentcons + ((log(crent_f1(iy,is)) - mean_rentcons)**2)*mu3rent_f0(iy,is)
		enddo
		!Homeowners
		do ig = 1, ng
			!Homeowners with bad credit
			do iy = ny_neg + 1, ny
				sd_cons	   = sd_cons + ((log(cown_f1(iy,ig,is)) - mean_cons)**2)*mu3own_f1(iy,ig,is)
				sd_owncons = sd_owncons + ((log(cown_f1(iy,ig,is)) - mean_owncons)**2)*mu3own_f1(iy,ig,is)
			enddo
			!Homeowners with good credit but no mortgage
			do iy = 1, ny
				sd_cons    = sd_cons + ((log(cown_f0(iy,1,ig,is)) - mean_cons)**2)*mu3own_f0(iy,1,ig,is)
				sd_owncons = sd_owncons + ((log(cown_f0(iy,1,ig,is)) - mean_owncons)**2)*mu3own_f0(iy,1,ig,is)
				if (qm0(locm_f0(iy,1,ig,is),locbown_f0(iy,1,ig,is),ig,is)*m_f0(iy,1,ig,is) >= 0.8d0) then
					sd_ltv80cons = sd_ltv80cons + ((log(cown_f0(iy,1,ig,is)) - mean_ltv80cons)**2)*mu3own_f0(iy,1,ig,is)
				endif
				if (qm0(locm_f0(iy,1,ig,is),locbown_f0(iy,1,ig,is),ig,is)*m_f0(iy,1,ig,is) >= 0.9d0) then
					sd_ltv90cons = sd_ltv90cons + ((log(cown_f0(iy,1,ig,is)) - mean_ltv90cons)**2)*mu3own_f0(iy,1,ig,is)
				endif
				if (qm0(locm_f0(iy,1,ig,is),locbown_f0(iy,1,ig,is),ig,is)*m_f0(iy,1,ig,is) < 0.8d0) then
					sd_lowltvcons = sd_lowltvcons + ((log(cown_f0(iy,1,ig,is)) - mean_lowltvcons)**2)*mu3own_f0(iy,1,ig,is)
				endif
			enddo
			do im = 2, nm
				!Homeowners with mortgages
				do iy = ny_neg + 1, ny
					if (mm(im) >= 0.8d0) then
						sd_ltv80cons = sd_ltv80cons + ((log(cown_f0(iy,im,ig,is)) - mean_ltv80cons)**2)*mu3own_f0(iy,im,ig,is)
					endif
					if (mm(im) >= 0.9d0) then
						sd_ltv90cons = sd_ltv90cons + ((log(cown_f0(iy,im,ig,is)) - mean_ltv90cons)**2)*mu3own_f0(iy,im,ig,is)
					endif
					if (mm(im) < 0.8d0) then
						sd_lowltvcons = sd_lowltvcons + ((log(cown_f0(iy,im,ig,is)) - mean_lowltvcons)**2)*mu3own_f0(iy,im,ig,is)
					endif
					sd_cons    = sd_cons + ((log(cown_f0(iy,im,ig,is)) - mean_cons)**2)*mu3own_f0(iy,im,ig,is)
					sd_owncons = sd_owncons + ((log(cown_f0(iy,im,ig,is)) - mean_owncons)**2)*mu3own_f0(iy,im,ig,is)
				enddo
			enddo
		enddo
	enddo
	sd_owncons    = sd_owncons/meas_own
	sd_rentcons   = sd_rentcons/meas_rent
	sd_ltv80cons  = sd_ltv80cons/meas_ltv80
	sd_ltv90cons  = sd_ltv90cons/meas_ltv90
	sd_lowltvcons = sd_lowltvcons/meas_lowltv
	sd_cons       = sd_cons**(0.5)
	sd_owncons    = sd_owncons**(0.5)
	sd_rentcons   = sd_rentcons**(0.5)
	sd_ltv80cons  = sd_ltv80cons**(0.5)
	sd_ltv90cons  = sd_ltv90cons**(0.5)
	sd_lowltvcons = sd_lowltvcons**(0.5)
	
	print *, ' '
	print *, 'SD(log(c)) for all households'
	print *, sd_cons
	print *, ' '
	print *, 'SD(log(c)) for renters'
	print *, sd_rentcons
	print *, ' '
	print *, 'SD(log(c)) for all homeowners'
	print *, sd_owncons
	print *, ' '
	print *, 'SD(log(c)) for homeowners with ltv > 80%'
	print *, sd_ltv80cons
	print *, ' '
	print *, 'SD(log(c)) for homeowners with ltv > 90%'
	print *, sd_ltv90cons
	print *, ' '
	print *, 'SD(log(c)) for homeowners with ltv < 80%'
	print *, sd_lowltvcons
	
	!!!Asset Inequality!!!
	
	!Mean of log bond holdings
	mean_bonds       = 0.d0
	mean_ownbonds    = 0.d0
	mean_rentbonds   = 0.d0
	ltv80bonds       = 0.d0
	ltv90bonds       = 0.d0
	mean_ltv80bonds  = 0.d0
	mean_ltv90bonds  = 0.d0
	mean_lowltvbonds = 0.d0
	do is = 1, ns
		!Renters
		do iy = ny_neg + 1, ny
			if (brent_f0(iy,is) > 0.d0) then
				!Renters with good credit
				mean_bonds     = mean_bonds + log(brent_f0(iy,is))*mu3rent_f0(iy,is)
				mean_rentbonds = mean_rentbonds + log(brent_f0(iy,is))*mu3rent_f0(iy,is)
			endif
			if (brent_f1(iy,is) > 0.d0) then
				!Renters with bad credit
				mean_bonds     = mean_bonds + log(brent_f1(iy,is))*mu3rent_f1(iy,is)
				mean_rentbonds = mean_rentbonds + log(brent_f1(iy,is))*mu3rent_f1(iy,is)
			endif
		enddo
		!Homeowners
		do ig = 1, ng
			!Homeowners with bad credit
			do iy = ny_neg + 1, ny
				if (bown_f1(iy,ig,is) > 0.d0) then
					mean_bonds    = mean_bonds + log(bown_f1(iy,ig,is))*mu3own_f1(iy,ig,is)
					mean_ownbonds = mean_ownbonds + log(bown_f1(iy,ig,is))*mu3own_f1(iy,ig,is)
				endif
			enddo
			!Homeowners with good credit but no mortgage
			do iy = 1, ny
				if (bown_f0(iy,1,ig,is) > 0.d0) then
					mean_bonds    = mean_bonds + log(bown_f0(iy,1,ig,is))*mu3own_f0(iy,1,ig,is)
					mean_ownbonds = mean_ownbonds + log(bown_f0(iy,1,ig,is))*mu3own_f0(iy,1,ig,is)
					if (qm0(locm_f0(iy,1,ig,is),locbown_f0(iy,1,ig,is),ig,is)*m_f0(iy,1,ig,is) >= 0.8d0) then
						mean_ltv80bonds = mean_ltv80bonds + log(bown_f0(iy,1,ig,is))*mu3own_f0(iy,1,ig,is)
						ltv80bonds      = ltv80bonds + bown_f0(iy,1,ig,is)*mu3own_f0(iy,1,ig,is)
					endif
					if (qm0(locm_f0(iy,1,ig,is),locbown_f0(iy,1,ig,is),ig,is)*m_f0(iy,1,ig,is) >= 0.9d0) then
						mean_ltv90bonds = mean_ltv90bonds + log(bown_f0(iy,1,ig,is))*mu3own_f0(iy,1,ig,is)
						ltv90bonds      = ltv90bonds + bown_f0(iy,1,ig,is)*mu3own_f0(iy,1,ig,is)
					endif
					if (qm0(locm_f0(iy,1,ig,is),locbown_f0(iy,1,ig,is),ig,is)*m_f0(iy,1,ig,is) < 0.8d0) then
						mean_lowltvbonds = mean_lowltvbonds + log(bown_f0(iy,1,ig,is))*mu3own_f0(iy,1,ig,is)
					endif
				endif
			enddo
			do im = 2, nm
				!Homeowners with mortgages
				do iy = ny_neg + 1, ny
					if (bown_f0(iy,im,ig,is) > 0.d0) then
						if (mm(im) >= 0.8d0) then
							mean_ltv80bonds = mean_ltv80bonds + log(bown_f0(iy,im,ig,is))*mu3own_f0(iy,im,ig,is)
							ltv80bonds      = ltv80bonds + bown_f0(iy,im,ig,is)*mu3own_f0(iy,im,ig,is)
						endif
						if (mm(im) >= 0.9d0) then
							mean_ltv90bonds = mean_ltv90bonds + log(bown_f0(iy,im,ig,is))*mu3own_f0(iy,im,ig,is)
							ltv90bonds      = ltv90bonds + bown_f0(iy,im,ig,is)*mu3own_f0(iy,im,ig,is)
						endif
						if (mm(im) < 0.8d0) then
							mean_lowltvbonds = mean_lowltvbonds + log(bown_f0(iy,im,ig,is))*mu3own_f0(iy,im,ig,is)
						endif
						mean_bonds    = mean_bonds + log(bown_f0(iy,im,ig,is))*mu3own_f0(iy,im,ig,is)
						mean_ownbonds = mean_ownbonds + log(bown_f0(iy,im,ig,is))*mu3own_f0(iy,im,ig,is)
					endif
				enddo
			enddo
		enddo
	enddo
	mean_rentbonds   = mean_rentbonds/meas_rent
	mean_ownbonds    = mean_ownbonds/meas_own
	mean_ltv80bonds  = mean_ltv80bonds/meas_ltv80
	mean_ltv90bonds  = mean_ltv90bonds/meas_ltv90
	mean_lowltvbonds = mean_lowltvbonds/meas_lowltv
	ltv80bonds       = ltv80bonds/meas_ltv80
	ltv90bonds       = ltv90bonds/meas_ltv90
	
	!Standard deviation of bond holdings
	sd_bonds       = 0.d0
	sd_ownbonds    = 0.d0
	sd_rentbonds   = 0.d0
	sd_ltv80bonds  = 0.d0
	sd_ltv90bonds  = 0.d0
	sd_lowltvbonds = 0.d0
	do is = 1, ns
		!Renters
		do iy = ny_neg + 1, ny
			if (brent_f0(iy,is) > 0.d0) then
				!Renters with good credit
				sd_bonds     = sd_bonds + ((log(brent_f0(iy,is)) - mean_bonds)**2)*mu3rent_f0(iy,is)
				sd_rentbonds = sd_rentbonds + ((log(brent_f0(iy,is)) - mean_rentbonds)**2)*mu3rent_f0(iy,is)
			endif
			if (brent_f1(iy,is) > 0.d0) then
				!Renters with bad credit
				sd_bonds     = sd_bonds + ((log(brent_f1(iy,is)) - mean_bonds)**2)*mu3rent_f0(iy,is)
				sd_rentbonds = sd_rentbonds + ((log(brent_f1(iy,is)) - mean_rentbonds)**2)*mu3rent_f0(iy,is)
			endif
		enddo
		!Homeowners
		do ig = 1, ng
			!Homeowners with bad credit
			do iy = ny_neg + 1, ny
				if (bown_f1(iy,ig,is) > 0.d0) then
					sd_bonds	   = sd_bonds + ((log(bown_f1(iy,ig,is)) - mean_bonds)**2)*mu3own_f1(iy,ig,is)
					sd_ownbonds = sd_ownbonds + ((log(bown_f1(iy,ig,is)) - mean_ownbonds)**2)*mu3own_f1(iy,ig,is)
				endif
			enddo
			!Homeowners with good credit but no mortgage
			do iy = 1, ny
				if (bown_f0(iy,1,ig,is) > 0.d0) then
					sd_bonds    = sd_bonds + ((log(bown_f0(iy,1,ig,is)) - mean_bonds)**2)*mu3own_f0(iy,1,ig,is)
					sd_ownbonds = sd_ownbonds + ((log(bown_f0(iy,1,ig,is)) - mean_ownbonds)**2)*mu3own_f0(iy,1,ig,is)
					if (qm0(locm_f0(iy,1,ig,is),locbown_f0(iy,1,ig,is),ig,is)*m_f0(iy,1,ig,is) >= 0.8d0) then
						sd_ltv80bonds = sd_ltv80bonds + ((log(bown_f0(iy,1,ig,is)) - mean_ltv80bonds)**2)*mu3own_f0(iy,1,ig,is)
					endif
					if (qm0(locm_f0(iy,1,ig,is),locbown_f0(iy,1,ig,is),ig,is)*m_f0(iy,1,ig,is) >= 0.9d0) then
						sd_ltv90bonds = sd_ltv90bonds + ((log(bown_f0(iy,1,ig,is)) - mean_ltv90bonds)**2)*mu3own_f0(iy,1,ig,is)
					endif
					if (qm0(locm_f0(iy,1,ig,is),locbown_f0(iy,1,ig,is),ig,is)*m_f0(iy,1,ig,is) < 0.8d0) then
						sd_lowltvbonds = sd_lowltvbonds + ((log(bown_f0(iy,1,ig,is)) - mean_lowltvbonds)**2)*mu3own_f0(iy,1,ig,is)
					endif
				endif
			enddo
			do im = 2, nm
				!Homeowners with mortgages
				do iy = ny_neg + 1, ny
					if (bown_f0(iy,im,ig,is) > 0.d0) then
						if (mm(im) >= 0.8d0) then
							sd_ltv80bonds = sd_ltv80bonds + ((log(bown_f0(iy,im,ig,is)) - mean_ltv80bonds)**2)*mu3own_f0(iy,im,ig,is)
						endif
						if (mm(im) >= 0.9d0) then
							sd_ltv90bonds = sd_ltv90bonds + ((log(bown_f0(iy,im,ig,is)) - mean_ltv90bonds)**2)*mu3own_f0(iy,im,ig,is)
						endif
						if (mm(im) < 0.8d0) then
							sd_lowltvbonds = sd_lowltvbonds + ((log(bown_f0(iy,im,ig,is)) - mean_lowltvbonds)**2)*mu3own_f0(iy,im,ig,is)
						endif
						sd_bonds    = sd_bonds + ((log(bown_f0(iy,im,ig,is)) - mean_bonds)**2)*mu3own_f0(iy,im,ig,is)
						sd_ownbonds = sd_ownbonds + ((log(bown_f0(iy,im,ig,is)) - mean_ownbonds)**2)*mu3own_f0(iy,im,ig,is)
					endif
				enddo
			enddo
		enddo
	enddo
	sd_ownbonds    = sd_ownbonds/meas_own
	sd_rentbonds   = sd_rentbonds/meas_rent
	sd_ltv80bonds  = sd_ltv80bonds/meas_ltv80
	sd_ltv90bonds  = sd_ltv90bonds/meas_ltv90
	sd_lowltvbonds = sd_lowltvbonds/meas_lowltv
	sd_bonds       = sd_bonds**(0.5)
	sd_ownbonds    = sd_ownbonds**(0.5)
	sd_rentbonds   = sd_rentbonds**(0.5)
	sd_ltv80bonds  = sd_ltv80bonds**(0.5)
	sd_ltv90bonds  = sd_ltv90bonds**(0.5)
	sd_lowltvbonds = sd_lowltvbonds**(0.5)
	
	print *, ' '
	print *, 'SD(log(b)) for all households'
	print *, sd_bonds
	print *, sd_cons/sd_bonds
	print *, ' '
	print *, 'SD(log(b)) for renters'
	print *, sd_rentbonds
	print *, sd_rentcons/sd_rentbonds
	print *, ' '
	print *, 'SD(log(b)) for all homeowners'
	print *, sd_ownbonds
	print *, sd_owncons/sd_ownbonds
	print *, ' '
	print *, 'SD(log(b)) for homeowners with ltv > 80%'
	print *, sd_ltv80bonds
	print *, sd_ltv80cons/sd_ltv80bonds
	print *, ltv80bonds
	print *, ' '
	print *, 'SD(log(b)) for homeowners with ltv > 90%'
	print *, sd_ltv90bonds
	print *, sd_ltv90cons/sd_ltv90bonds
	print *, ltv90bonds
	print *, ' '
	print *, 'SD(log(b)) for homeowners with ltv < 80%'
	print *, sd_lowltvbonds
	print *, sd_lowltvcons/sd_lowltvbonds
	
	!!!Welfare!!!
	
	!Calculate actual subperiod 3 homeowner value function with (y,m) prior to the mortgage payment choice in the computation
	allocate(Vown_f0_sp3(ny,nm,ng,ns))
	do is = 1, ns
		do ig = 1, ng
			ih = hfun(ig)
			!Homeowners without a mortgage
			do iy = 1, ny
				Vown_f0_sp3(iy,1,ig,is) = Vown_f0(iy,1,ig,is)
			enddo
			!Households with a mortgage
			do im = 2, nm
				do iy = ny_neg + 1, ny
					imprime = locm_f0(iy,im,ig,is)
					ypay    = yy(iy) - (mm(im) - qm*mm(imprime))*ph*hh(ih)
					if (ypay < 0.d0) then
						ypay = ypay/abs(ylb(ig,is))
					endif
					iylow   = ly(ypay)
					iyupp   = uy(ypay)
					yweight = wy(ypay)
					Vown_f0_sp3(iy,im,ig,is) = (1 - yweight)*Vown_f0(iylow,imprime,ig,is) + yweight*Vown_f0(iyupp,imprime,ig,is)
				enddo
				do iy = 1, ny_neg
					Vown_f0_sp3(iy,im,ig,is) = Vown_f0_sp3(ny_neg + 1,im,ig,is)
				enddo
			enddo
		enddo
	enddo
	!Print
	open(unit=11,	file='Input\Vown_f0_sp3.txt', status = 'replace')
	do is = 1, ns
		do ig = 1, ng
			write (11, '(45f20.10)') (Vown_f0_sp3(iy,:,ig,is), iy = 1,ny)
		enddo
	enddo
	close(11)
	
	print *, ' '
	write(*,*), 'Load value functions and calculate welfare?'
	read(*,*) loadvals
	if (loadvals == 1) then
		allocate(Vown_f0_sp3_nosearch(ny,nm,ng,ns))
		allocate(Vown_f1_nosearch(ny,ng,ns))
		allocate(Vrent_f0_nosearch(ny,ns))
		allocate(Vrent_f1_nosearch(ny,ns))
		allocate(welfare_own_f0(ny,nm,ng,ns))
		allocate(welfare_own_f1(ny,ng,ns))
		allocate(welfare_rent_f0(ny,ns))
		allocate(welfare_rent_f1(ny,ns))
		open(unit=11,	file='Input\Vown_f0_sp3_nosearch.txt')
		do is = 1, ns
			do ig = 1, ng
				do iy = 1, ny
					read (11, *) (Vown_f0_sp3_nosearch(iy,im,ig,is), im = 1,nm)
				enddo
			enddo
		enddo
		close(11)
		open(unit=11,	file='Input\Vown_f1_nosearch.txt')
		do is = 1, ns
			do iy = 1, ny
				read (11, *) (Vown_f1_nosearch(iy,ig,is), ig = 1,ng)
			enddo
		enddo
		close(11)
		open(unit=11,	file='Input\Vrent_f0_nosearch.txt')
			do iy = 1, ny
				read (11, *) (Vrent_f0_nosearch(iy,is), is = 1,ns)
			enddo
		close(11)
		open(unit=11,	file='Input\Vrent_f1_nosearch.txt')
			do iy = 1, ny
				read (11, *) (Vrent_f1_nosearch(iy,is), is = 1,ns)
			enddo
		close(11)
		!Calculate welfare cost of search frictions: "How much consumption would you be willing to give up to enter the Walrasian world?"
		welfare_loss      = 0.d0
		welfare_loss_rent = 0.d0
		welfare_loss_own  = 0.d0
		do is = 1, ns
			!Renters
			do iy = ny_neg + 1, ny
				welfare_rent_f0(iy,is) = 100.d0*(Vrent_f0_nosearch(iy,is)/Vrent_f0(iy,is) - 1)
				welfare_rent_f1(iy,is) = 100.d0*(Vrent_f1_nosearch(iy,is)/Vrent_f1(iy,is) - 1)
				welfare_loss           = welfare_loss + welfare_rent_f0(iy,is)*mu3rent_f0(iy,is) + welfare_rent_f1(iy,is)*mu3rent_f1(iy,is)
				welfare_loss_rent      = welfare_loss_rent + welfare_rent_f0(iy,is)*mu3rent_f0(iy,is) + welfare_rent_f1(iy,is)*mu3rent_f1(iy,is)
			enddo
			do ig = 1, ng
				!Homeowners with bad credit
				do iy = ny_neg + 1, ny
					welfare_own_f1(iy,ig,is) = 100.d0*(Vown_f1_nosearch(iy,ig,is)/Vown_f1(iy,ig,is) - 1)
					welfare_loss             = welfare_loss + welfare_own_f1(iy,ig,is)*mu3own_f1(iy,ig,is)
					welfare_loss_own         = welfare_loss_own + welfare_own_f1(iy,ig,is)*mu3own_f1(iy,ig,is)
				enddo
				!Homeowners with good credit but no mortgage
				do iy = 1, ny
					welfare_own_f0(iy,1,ig,is) = 100.d0*(Vown_f0_sp3_nosearch(iy,1,ig,is)/Vown_f0_sp3(iy,1,ig,is) - 1)
					welfare_loss               = welfare_loss + welfare_own_f0(iy,1,ig,is)*mu3own_f0(iy,1,ig,is)
					welfare_loss_own           = welfare_loss_own + welfare_own_f0(iy,1,ig,is)*mu3own_f0(iy,1,ig,is)
					if (qm0(locm_f0(iy,1,ig,is),locbown_f0(iy,1,ig,is),ig,is)*m_f0(iy,1,ig,is) >= 0.8d0) then
						welfare_loss_ltv80 = welfare_loss_ltv80 + welfare_own_f0(iy,1,ig,is)*mu3own_f0(iy,1,ig,is)
					endif
					if (qm0(locm_f0(iy,1,ig,is),locbown_f0(iy,1,ig,is),ig,is)*m_f0(iy,1,ig,is) >= 0.9d0) then
						welfare_loss_ltv90 = welfare_loss_ltv90 + welfare_own_f0(iy,1,ig,is)*mu3own_f0(iy,1,ig,is)
					endif
				enddo
				!Homeowners with mortgages
				do im = 2, nm
					do iy = ny_neg + 1, ny
						welfare_own_f0(iy,im,ig,is) = 100.d0*(Vown_f0_sp3_nosearch(iy,im,ig,is)/Vown_f0_sp3(iy,im,ig,is) - 1)
						welfare_loss                = welfare_loss + welfare_own_f0(iy,im,ig,is)*mu3own_f0(iy,im,ig,is)
						welfare_loss_own            = welfare_loss_own + welfare_own_f0(iy,im,ig,is)*mu3own_f0(iy,im,ig,is)
						if (mm(im) >= 0.8d0) then
							welfare_loss_ltv80 = welfare_loss_ltv80 + welfare_own_f0(iy,im,ig,is)*mu3own_f0(iy,im,ig,is)
						endif
						if (mm(im) >= 0.9d0) then
							welfare_loss_ltv90 = welfare_loss_ltv90 + welfare_own_f0(iy,im,ig,is)*mu3own_f0(iy,im,ig,is)
						endif
					enddo
				enddo
			enddo
		enddo
		welfare_loss_rent  = welfare_loss_rent/meas_rent
		welfare_loss_own   = welfare_loss_own/meas_own
		welfare_loss_ltv80 = welfare_loss_ltv80/meas_ltv80
		welfare_loss_ltv90 = welfare_loss_ltv90/meas_ltv90
		print *, ' '
		print *, 'Welfare gain of removing search frictions for all households'
		print *, welfare_loss
		print *, ' '
		print *, 'Welfare gain of removing search frictions for renters'
		print *, welfare_loss_rent
		print *, ' '
		print *, 'Welfare gain of removing search frictions for homeowners'
		print *, welfare_loss_own
		print *, ' '
		print *, 'Welfare gain of removing search frictions for homeowners with LTV > 80%'
		print *, welfare_loss_ltv80
		print *, ' '
		print *, 'Welfare gain of removing search frictions for homeowners with LTV > 90%'
		print *, welfare_loss_ltv90
	endif
	
end subroutine sub_aggresults