subroutine sub_aggresults_supplement

	use mod_globvar
	use mod_interpolate

	implicit none

	!This program computes the welfare change between two equilibria (baseline and new)

	!Indices
	integer :: iy, im, ig, is, ie
	
	!Value functions
	real(8), dimension(:,:,:,:), allocatable :: Wown_f0_baseline, Wown_f0_new
	real(8), dimension(:,:,:), allocatable :: Wown_f1_baseline, Wown_f1_new
	real(8), dimension(:,:), allocatable :: Wrent_f0_baseline, Wrent_f1_baseline, Wrent_f0_new, Wrent_f1_new
	
	!Other internal variables
	real(8) :: welfare_change_all, welfare_change_renters, welfare_change_ltv80, welfare_change_ltv90, welfare_change_ltv95, meas, meas_buyers, mean_earn, buyerwealth_q1, buyerwealth_q2, buyerwealth_q3, buyerwealth_q4, &
	buyerwealth_q5
	
	!Allocate
	allocate(Wown_f0_baseline(ny,nm,ng,ns))
	allocate(Wown_f1_baseline(ny,ng,ns))
	allocate(Wrent_f0_baseline(ny,ns))
	allocate(Wrent_f1_baseline(ny,ns))
	allocate(Wown_f0_new(ny,nm,ng,ns))
	allocate(Wown_f1_new(ny,ng,ns))
	allocate(Wrent_f0_new(ny,ns))
	allocate(Wrent_f1_new(ny,ns))
	
	!Load files
	open(unit=11,	file='Results\Baseline\mu1own_f0.txt')
	do is = 1, ns
		do ig = 1, ng
			do im = 1, nm
				do iy = 1, ny
					read (11, *) mu1own_f0(iy,im,ig,is)
				enddo
			enddo
		enddo
	enddo
	close(11)
	open(unit=11,	file='Results\Baseline\mu1own_f1.txt')
	do is = 1, ns
		do ig = 1, ng
			do iy = 1, ny
				read (11, *) mu1own_f1(iy,ig,is)
			enddo
		enddo
	enddo
	close(11)
	open(unit=11,	file='Results\Baseline\mu1rent_f0.txt')
	do is = 1, ns
		do iy = 1, ny
			read (11, *) mu1rent_f0(iy,is)
		enddo
	enddo
	close(11)
	open(unit=11,	file='Results\Baseline\mu1rent_f1.txt')
	do is = 1, ns
		do iy = 1, ny
			read (11, *) mu1rent_f1(iy,is)
		enddo
	enddo
	close(11)
	open(unit=11,	file='Results\Baseline\Wown_f0.txt')
	do is = 1, ns
		do ig = 1, ng
			do im = 1, nm
				do iy = 1, ny
					read (11, *) Wown_f0_baseline(iy,im,ig,is)
				enddo
			enddo
		enddo
	enddo
	close(11)
	open(unit=11,	file='Results\Baseline\Wown_f1.txt')
	do is = 1, ns
		do ig = 1, ng
			do iy = 1, ny
				read (11, *) Wown_f1_baseline(iy,ig,is)
			enddo
		enddo
	enddo
	close(11)
	open(unit=11,	file='Results\Baseline\Wrent_f0.txt')
	do is = 1, ns
		do iy = 1, ny
			read (11, *) Wrent_f0_baseline(iy,is)
		enddo
	enddo
	close(11)
	open(unit=11,	file='Results\Baseline\Wrent_f1.txt')
	do is = 1, ns
		do iy = 1, ny
			read (11, *) Wrent_f1_baseline(iy,is)
		enddo
	enddo
	close(11)
	open(unit=11,	file='Results\New\Wown_f0.txt')
	do is = 1, ns
		do ig = 1, ng
			do im = 1, nm
				do iy = 1, ny
					read (11, *) Wown_f0_new(iy,im,ig,is)
				enddo
			enddo
		enddo
	enddo
	close(11)
	open(unit=11,	file='Results\New\Wown_f1.txt')
	do is = 1, ns
		do ig = 1, ng
			do iy = 1, ny
				read (11, *) Wown_f1_new(iy,ig,is)
			enddo
		enddo
	enddo
	close(11)
	open(unit=11,	file='Results\New\Wrent_f0.txt')
	do is = 1, ns
		do iy = 1, ny
			read (11, *) Wrent_f0_new(iy,is)
		enddo
	enddo
	close(11)
	open(unit=11,	file='Results\New\Wrent_f1.txt')
	do is = 1, ns
		do iy = 1, ny
			read (11, *) Wrent_f1_new(iy,is)
		enddo
	enddo
	close(11)
	
	!Average welfare gain -- all agents
	welfare_change_all = 0.d0
	do is = 1, ns
		do ig = 1, ng
			!Owners with bad credit
			do iy = ny_neg + 1, ny
				welfare_change_all = welfare_change_all + 100.d0*(Wown_f1_new(iy,ig,is)/Wown_f1_baseline(iy,ig,is) - 1.d0)*mu1own_f1(iy,ig,is)
			enddo
			do im = 1, nm
				!Owners with good credit
				do iy = ny_neg + 1, ny
					welfare_change_all = welfare_change_all + 100.d0*(Wown_f0_new(iy,im,ig,is)/Wown_f0_baseline(iy,im,ig,is) - 1.d0)*mu1own_f0(iy,im,ig,is)
				enddo
			enddo
		enddo
		!Renters
		do iy = ny_neg + 1, ny
			welfare_change_all = welfare_change_all + 100.d0*(Wrent_f0_new(iy,is)/Wrent_f0_baseline(iy,is) - 1.d0)*mu1rent_f0(iy,is)
			welfare_change_all = welfare_change_all + 100.d0*(Wrent_f1_new(iy,is)/Wrent_f1_baseline(iy,is) - 1.d0)*mu1rent_f1(iy,is)
		enddo
	enddo
	
	!Average welfare gain -- renters with good credit
	welfare_change_renters = 0.d0
	meas				   = 0.d0
	do is = 1, ns
		!Renters
		do iy = ny_neg + 1, ny
			welfare_change_renters = welfare_change_renters + 100.d0*(Wrent_f0_new(iy,is)/Wrent_f0_baseline(iy,is) - 1.d0)*mu1rent_f0(iy,is)
			meas                   = meas + mu1rent_f0(iy,is)
		enddo
	enddo
	welfare_change_renters = welfare_change_renters/meas
	
	!Average welfare gain -- homeowners with LTV > 80%
	welfare_change_ltv80 = 0.d0
	meas                 = 0.d0
	do is = 1, ns
		do ig = 1, ng
			do im = 1, nm
				if (mm(im) >= 0.8d0) then
					!Owners with good credit
					do iy = ny_neg + 1, ny
						welfare_change_ltv80 = welfare_change_ltv80 + 100.d0*(Wown_f0_new(iy,im,ig,is)/Wown_f0_baseline(iy,im,ig,is) - 1.d0)*mu1own_f0(iy,im,ig,is)
						meas                 = meas + mu1own_f0(iy,im,ig,is)
					enddo
				endif
			enddo
		enddo
	enddo
	welfare_change_ltv80 = welfare_change_ltv80/meas
	
	!Average welfare gain -- homeowners with LTV > 90%
	welfare_change_ltv90 = 0.d0
	meas                 = 0.d0
	do is = 1, ns
		do ig = 1, ng
			do im = 1, nm
				if (mm(im) >= 0.9d0) then
					!Owners with good credit
					do iy = ny_neg + 1, ny
						welfare_change_ltv90 = welfare_change_ltv90 + 100.d0*(Wown_f0_new(iy,im,ig,is)/Wown_f0_baseline(iy,im,ig,is) - 1.d0)*mu1own_f0(iy,im,ig,is)
						meas                 = meas + mu1own_f0(iy,im,ig,is)
					enddo
				endif
			enddo
		enddo
	enddo
	welfare_change_ltv90 = welfare_change_ltv90/meas
	
	!Average welfare gain -- homeowners with LTV > 95%
	welfare_change_ltv95 = 0.d0
	meas                 = 0.d0
	do is = 1, ns
		do ig = 1, ng
			do im = 1, nm
				if (mm(im) >= 0.95d0) then
					!Owners with good credit
					do iy = ny_neg + 1, ny
						welfare_change_ltv95 = welfare_change_ltv95 + 100.d0*(Wown_f0_new(iy,im,ig,is)/Wown_f0_baseline(iy,im,ig,is) - 1.d0)*mu1own_f0(iy,im,ig,is)
						meas                 = meas + mu1own_f0(iy,im,ig,is)
					enddo
				endif
			enddo
		enddo
	enddo
	welfare_change_ltv95 = welfare_change_ltv95/meas
	
	print *, ' '
	print *, 'Welfare change: all'
	print *, welfare_change_all
	print *, 'Welfare change: renters'
	print *, welfare_change_renters
	print *, 'Welfare change: homeowners with ltv > 80'
	print *, welfare_change_ltv80
	print *, 'Welfare change: homeowners with ltv > 90'
	print *, welfare_change_ltv90
	print *, 'Welfare change: homeowners with ltv > 95'
	print *, welfare_change_ltv95
	
	!!!Buyer wealth quintiles!!!
	
	!Mean earnings
	mean_earn = 0.d0
	do is = 1, ns
		do ie = 1, ne
			mean_earn = mean_earn + labor(is,ie)*w*emeas(ie)*(sum(mu3own_f0(:,:,:,is)) + sum(mu3own_f1(:,:,is)) + sum(mu3rent_f0(:,is)) + sum(mu3rent_f1(:,is)))
		enddo
	enddo
	
	!Total number of buyers
	meas_buyers = 0.d0
	do is = 1, ns
		do iy = ny_neg + 1, ny
			meas_buyers = meas_buyers + pbuy_f0(iy,is)*mu1rent_f0(iy,is)
			meas_buyers = meas_buyers + pbuy_f1(iy,is)*mu1rent_f1(iy,is)
		enddo
	enddo
	
	!Buyer wealth quintiles
	buyerwealth_q1 = 0.d0
	meas           = 0.d0
	do iy = ny_neg + 1, ny
		meas = meas + dot_product(pbuy_f0(iy,:),mu1rent_f0(iy,:)) + dot_product(pbuy_f1(iy,:),mu1rent_f1(iy,:))
		if (meas > (0.1d0 - 1e-6)*meas_buyers) then
			buyerwealth_q1 = yy(iy)/(4.d0*mean_earn)
			exit
		endif
	enddo
	buyerwealth_q2 = 0.d0
	meas           = 0.d0
	do iy = ny_neg + 1, ny
		meas = meas + dot_product(pbuy_f0(iy,:),mu1rent_f0(iy,:)) + dot_product(pbuy_f1(iy,:),mu1rent_f1(iy,:))
		if (meas > (0.3d0 - 1e-6)*meas_buyers) then
			buyerwealth_q2 = yy(iy)/(4.d0*mean_earn)
			exit
		endif
	enddo
	buyerwealth_q3 = 0.d0
	meas           = 0.d0
	do iy = ny_neg + 1, ny
		meas = meas + dot_product(pbuy_f0(iy,:),mu1rent_f0(iy,:)) + dot_product(pbuy_f1(iy,:),mu1rent_f1(iy,:))
		if (meas > (0.5d0 - 1e-6)*meas_buyers) then
			buyerwealth_q3 = yy(iy)/(4.d0*mean_earn)
			exit
		endif
	enddo
	buyerwealth_q4 = 0.d0
	meas           = 0.d0
	do iy = ny_neg + 1, ny
		meas = meas + dot_product(pbuy_f0(iy,:),mu1rent_f0(iy,:)) + dot_product(pbuy_f1(iy,:),mu1rent_f1(iy,:))
		if (meas > (0.7d0 - 1e-6)*meas_buyers) then
			buyerwealth_q4 = yy(iy)/(4.d0*mean_earn)
			exit
		endif
	enddo
	buyerwealth_q5 = 0.d0
	meas           = 0.d0
	do iy = ny_neg + 1, ny
		meas = meas + dot_product(pbuy_f0(iy,:),mu1rent_f0(iy,:)) + dot_product(pbuy_f1(iy,:),mu1rent_f1(iy,:))
		if (meas > (0.9d0 - 1e-6)*meas_buyers) then
			buyerwealth_q5 = yy(iy)/(4.d0*mean_earn)
			exit
		endif
	enddo
	
	print *, ' '
	print *, 'Buyer wealth quintiles'
	print *, buyerwealth_q1
	print *, buyerwealth_q2
	print *, buyerwealth_q3
	print *, buyerwealth_q4
	print *, buyerwealth_q5
	
end subroutine sub_aggresults_supplement