subroutine sub_equilibrium

	use mod_globvar
	use mod_interpolate

	implicit none

	!This program computes the stationary equilibrium for the housing model with borrowing and default

	!Indices -- individual states and choice variables
	integer :: ix, im, imprime, ibprime, iy, ig, ih, iu, igshock, is, isprime, ieprime
	
	!Indices -- miscellaneous
	integer :: iter_kc, iter_q, iter_qtemp
	
	!Updated mortgage prices
	real(8), dimension(ne) :: qm0_e
	real(8), dimension(ns) :: qm0_s
	real(8), dimension(nu) :: qm0_u
	real(8), dimension(:,:,:,:), allocatable :: qm0new, qm0temp
	real(8) :: yprime_f0, y0weight, mrecov, d_low, d_upp, pxs_low, pxs_upp, qm0_low, qm0_upp
	integer :: iy0low, iy0upp, im_low, im_upp, ib_low, ib_upp
	
	!Vector of configuration parameters for equilibrium subroutine
	integer, dimension(8) :: config_equil
	
	!Normed difference between successive iterations
	real(8) :: err_qtemp                                                    !Inner qm0 loop
	real(8) :: err_kc                                                       !Outer risk-free rate loop
	
	!Error tolerances
	real(8) :: toler_excessdemand                                           !Error tolerance for excess demand
	real(8) :: toler_kc														!Error tolerance for risk-free rate
	
	!Equilibrium ph algorithm
	real(8) :: min_dph, supply, demand, excess_demand1, excess_demand0, dph, pctdiff
	integer :: sign_excessdemand1, sign_excessdemand0
	
	!Equilibrium kc algorithm
	real(8) :: kstock, assets, mortcost, dkc, kc_min, kc_max
	
	!Maximum iterations
	integer :: max_iterkc													!Maximum number of kc loop iterations
	integer :: max_iterph													!Maximum number of ph loop iterations
	integer :: max_iterqtemp                                                !Maximum number of inner qm0 loop iterations
	integer :: max_iterq                                                    !Maximum number of outer qm0 loop iterations
	integer :: max_iterv_1                                                  !Maximum number of value function iterations when iter_coeff = 1 and iter_q = 1
	integer :: max_itervaccel_1                                             !Maximum number of accelerated value function iterations when iter_coeff = 1 and iter_q = 1
	integer :: max_iterv_2                                                  !Maximum number of value function iterations when iter_coeff >= 2 or iter_q >= 2
	integer :: max_itervaccel_2                                             !Maximum number of accelerated value function iterations when iter_coeff >=2 and iter_q >= 2
	
	!Other internal variables
	real(8) :: ylower, spending_f0, c_f0, hc_f0, spending_f1, c_f1, hc_f1, hpredict, hactual
	
	!Load configuration
	open(unit=11,	file='Input\config_equil.txt')
	read (11, *) config_equil
	close(11)
	max_iterkc       = config_equil(1)
	max_iterph       = config_equil(2)
	max_iterqtemp    = config_equil(3)
	max_iterq        = config_equil(4)
	max_iterv_1      = config_equil(5)
	max_itervaccel_1 = config_equil(6)
	max_iterv_2      = config_equil(7)
	max_itervaccel_2 = config_equil(8)
	
	!Allocate updated mortgage prices
	allocate(qm0new(nm,nb,ng,ns))
	allocate(qm0temp(nm,nb,ng,ns))
	
	!Initial guess for price of apartment space
	pa = rentratio*ph
	
	!Calculate factor prices
	call sub_outputprices_sh(kc,eavg,ph,zh,w,rk,yc,yh)
	
	!Bond and mortgage prices
	rf = rk - deprec_k
	qb = 1/(1 + rf)
	if (input_zc == 0) then
		qm     = 1/((1 + rf)**4 + rmspread)**0.25
		rscost = (1 - deprec_h)/((1 + rf)*qm) - 1
		if (rscost < 0.d0) then
			print *, 'Negative mortgage servicing cost!'
			stop
		endif
	else
		qm = qb*(1 - deprec_h)/(1 + rscost)
	endif
	
	!Initialize value functions and mortgage prices
	if (load_data == 1) then
		!Load value functions and mortgage prices
		call sub_loadfiles
		!Compute optimal REO behavior
		Jreo = -100.0
		call sub_valfn_reo
	else
		!Initialize value and policy functions, mortgage prices, and steady-state distributions
		d_f0           = 0.1d0
		psell_f0       = 0.d0
		locm_f0        = 1
		locbown_f0_sp1 = 1
		!Value functions
		do is = 1, ns
			do iy = 1, ny
				spending_f0 = max(min_spend,yy(iy))
				c_f0        = cshare*spending_f0
				hc_f0       = (1 - cshare)*spending_f0/pa
				spending_f1 = max(min_spend,(1 - fincloss)*yy(iy))
				c_f1        = cshare*spending_f1
				hc_f1       = (1 - cshare)*spending_f1/pa
				Vrent_f0(iy,is) = c_f0**cshare*hc_f0**(1 - cshare)
				Vrent_f1(iy,is) = c_f1**cshare*hc_f1**(1 - cshare)
			enddo
			do ig = 1, ng
				ih = hfun(ig)
				do iy = 1, ny
					spending_f1 = max(min_spend,(1 - fincloss)*yy(iy) + ph*hh(ih))
					c_f1        = cshare*spending_f1
					hc_f1       = (1 - cshare)*spending_f1/pa
					Vown_f1(iy,ig,is) = c_f1**cshare*hc_f1**(1 - cshare)
				enddo
				do im = 1, nm
					do iy = 1, ny
						spending_f0 = max(min_spend,yy(iy) + (1 - mm(im))*ph*hh(ih))
						c_f0        = cshare*spending_f0
						hc_f0       = (1 - cshare)*spending_f0/pa
						Vown_f0(iy,im,ig,is) = c_f0**cshare*hc_f0**(1 - cshare)
					enddo
				enddo
			enddo
		enddo
		!Compute optimal REO behavior
		Jreo = -100.0
		call sub_valfn_reo		
		qm0temp(:,:,:,:) = qm/(1 + rocost)
		do iter_qtemp = 1, max_iterqtemp
			qm0(1,:,:,:) = qm/(1 + rocost)
			do is = 1, ns
				do ig = 1, ng
					ih      = hfun(ig)
					iu      = ufun(ig)
					igshock = gfun(ih,2)
					call omp_set_num_threads(num_threads)
					!$omp parallel private(imprime,isprime,ieprime,yprime_f0,iy0low,iy0upp,y0weight,im_low,ib_low,d_low,pxs_low,im_upp,ib_upp,d_upp,pxs_upp,mrecov,qm0_low,qm0_upp,qm0_e,qm0_s,qm0_u)
					!$omp do
					do ibprime = 1, nb
						do imprime = 2, nm
							if (iu == 1 .AND. nu == 2) then
								do isprime = 1, ns
									!No housing shock
									do ieprime = 1, ne
										yprime_f0 = labor(isprime,ieprime)*w + bb(ibprime)
										iy0low    = ly(yprime_f0)
										iy0upp    = uy(yprime_f0)
										y0weight  = wy(yprime_f0)
										im_low    = locm_f0(iy0low,imprime,ig,isprime)
										ib_low    = locbown_f0_sp1(iy0low,imprime,ig,isprime)
										d_low     = d_f0(iy0low,imprime,ig,isprime)
										pxs_low   = psell_f0(iy0low,imprime,ig,isprime)
										im_upp    = locm_f0(iy0upp,imprime,ig,isprime)
										ib_upp    = locbown_f0_sp1(iy0upp,imprime,ig,isprime)
										d_upp     = d_f0(iy0upp,imprime,ig,isprime)
										pxs_upp   = psell_f0(iy0upp,imprime,ig,isprime)
										mrecov    = min(1.d0,(Jreo(ih) + fdeficiency*yprime_f0)/(mm(imprime)*ph*hh(ih)))
										qm0_low   = qm0temp(im_low,ib_low,ig,isprime)
										qm0_upp   = qm0temp(im_upp,ib_upp,ig,isprime)
										!Update mortgage prices
										qm0_e(ieprime) = (1 - y0weight)*(pxs_low + (1 - pxs_low)*(d_low*mrecov + (1 - d_low)*(1 + (1 + rscost)*(qm0_low*(1 + rocost) - qm)&
										*mm(im_low)/mm(imprime)))) + y0weight*(pxs_upp + (1 - pxs_upp)*(d_upp*mrecov + (1 - d_upp)*(1 + (1 + rscost)*(qm0_upp*(1 + rocost) - qm)&
										*mm(im_upp)/mm(imprime))))
									enddo
									qm0_u(1) = dot_product(qm0_e,emeas)
									!Housing shock
									do ieprime = 1, ne
										yprime_f0 = labor(isprime,ieprime)*w + bb(ibprime)
										iy0low    = ly(yprime_f0)
										iy0upp    = uy(yprime_f0)
										y0weight  = wy(yprime_f0)
										im_low    = locm_f0(iy0low,imprime,igshock,isprime)
										ib_low    = locbown_f0_sp1(iy0low,imprime,igshock,isprime)
										d_low     = d_f0(iy0low,imprime,igshock,isprime)
										pxs_low   = psell_f0(iy0low,imprime,igshock,isprime)
										im_upp    = locm_f0(iy0upp,imprime,igshock,isprime)
										ib_upp    = locbown_f0_sp1(iy0upp,imprime,igshock,isprime)
										d_upp     = d_f0(iy0upp,imprime,igshock,isprime)
										pxs_upp   = psell_f0(iy0upp,imprime,igshock,isprime)
										mrecov    = min(1.d0,(Jreo(ih) + fdeficiency*yprime_f0)/(mm(imprime)*ph*hh(ih)))
										qm0_low   = qm0temp(im_low,ib_low,igshock,isprime)
										qm0_upp   = qm0temp(im_upp,ib_upp,igshock,isprime)
										!Update mortgage prices
										qm0_e(ieprime) = (1 - y0weight)*(pxs_low + (1 - pxs_low)*(d_low*mrecov + (1 - d_low)*(1 + (1 + rscost)*(qm0_low*(1 + rocost) - qm)&
										*mm(im_low)/mm(imprime)))) + y0weight*(pxs_upp + (1 - pxs_upp)*(d_upp*mrecov + (1 - d_upp)*(1 + (1 + rscost)*(qm0_upp*(1 + rocost) - qm)&
										*mm(im_upp)/mm(imprime))))
									enddo
									qm0_u(2) = dot_product(qm0_e,emeas)
									qm0_s(isprime) = (1 - pseparate)*qm0_u(1) + pseparate*qm0_u(2)
								enddo
							else
								do isprime = 1, ns
									do ieprime = 1, ne
										yprime_f0 = labor(isprime,ieprime)*w + bb(ibprime)
										iy0low    = ly(yprime_f0)
										iy0upp    = uy(yprime_f0)
										y0weight  = wy(yprime_f0)
										im_low    = locm_f0(iy0low,imprime,ig,isprime)
										ib_low    = locbown_f0_sp1(iy0low,imprime,ig,isprime)
										d_low     = d_f0(iy0low,imprime,ig,isprime)
										pxs_low   = psell_f0(iy0low,imprime,ig,isprime)
										im_upp    = locm_f0(iy0upp,imprime,ig,isprime)
										ib_upp    = locbown_f0_sp1(iy0upp,imprime,ig,isprime)
										d_upp     = d_f0(iy0upp,imprime,ig,isprime)
										pxs_upp   = psell_f0(iy0upp,imprime,ig,isprime)
										mrecov    = min(1.d0,(Jreo(ih) + fdeficiency*yprime_f0)/(mm(imprime)*ph*hh(ih)))
										qm0_low   = qm0temp(im_low,ib_low,ig,isprime)
										qm0_upp   = qm0temp(im_upp,ib_upp,ig,isprime)
										!Update mortgage prices
										qm0_e(ieprime) = (1 - y0weight)*(pxs_low + (1 - pxs_low)*(d_low*mrecov + (1 - d_low)*(1 + (1 + rscost)*(qm0_low*(1 + rocost) - qm)&
										*mm(im_low)/mm(imprime)))) + y0weight*(pxs_upp + (1 - pxs_upp)*(d_upp*mrecov + (1 - d_upp)*(1 + (1 + rscost)*(qm0_upp*(1 + rocost) - qm)&
										*mm(im_upp)/mm(imprime))))
									enddo
									qm0_s(isprime) = dot_product(qm0_e,emeas)
								enddo
							endif
							qm0(imprime,ibprime,ig,is) = qm/(1 + rocost)*dot_product(qm0_s,prob_ss(is,:))
						enddo
					enddo
					!$omp end do
					!$omp end parallel
				enddo
			enddo
			err_qtemp = maxval(abs(qm0 - qm0temp))
			if (iter_qtemp - 1 == 10*floor(real(iter_qtemp - 1,8)/10.0)) then
				write(*, '(a31 i4, a13, f8.5)'), '          Inner mortgage loop =',iter_qtemp,', err_qtemp =',err_qtemp
			endif
			if (err_qtemp < 1e-5 .OR. iter_qtemp == max_iterqtemp) then
				exit
			endif
			qm0temp = qm0
		enddo
		!Steady-state distributions
		mu1rent_f0 = 0.d0
		mu1rent_f1 = 0.d0
		mu1own_f1  = 0.d0
		mu1own_f0  = 0.d0
		do is = 1, ns
			do ig = 1, ng
				do im = 1, nm
					do iy = ny_neg + 1, ny
						if (iy == ny_neg + 1 .AND. im == 1 .AND. ig == 1) then
							mu1own_f0(iy,im,ig,is) = 1.d0/real(ns,8)
						endif
					enddo
				enddo
			enddo
		enddo
	endif
	
	!!!Compute stationary equilibrium!!!
	!Outer loop -- risk-free rate
	toler_kc = 1e-4
	err_kc   = 1.d0
	dkc      = 0.1d0
	kc_min   = 1e-2
	kc_max   = 10.d0
	do iter_kc = 1, max_iterkc
		!Middle loop -- shadow price of housing
		min_dph            = 1e-5
		toler_excessdemand = 1e-4
		dph                = 999.d0
		do iter_ph = 1, max_iterph
			print *, ' '
			!Calculate factor prices, output, and bond/mortgage prices
			call sub_outputprices_sh(kc,eavg,ph,zh,w,rk,yc,yh)
			rf = rk - deprec_k
			qb = 1/(1 + rf)
			if (input_zc == 0) then
				qm     = 1/((1 + rf)**4 + rmspread)**0.25
				rscost = (1 - deprec_h)/((1 + rf)*qm) - 1
				if (rscost < 0.d0) then
					print *, 'Negative mortgage servicing cost!'
					stop
				endif
			else
				qm = qb*(1 - deprec_h)/(1 + rscost)
			endif
			!Inner loop -- mortgage prices
			do iter_q = 1, max_iterq
				!Calculate lower bound of y at mortgage origination
				do is = 1, ns
					do ig = 1, ng
						ih = hfun(ig)
						ylb(ig,is) = max_y
						do imprime = 1, nm
							do ibprime = 1, nb
								if (qm0(imprime,ibprime,ig,is)*mm(imprime) <= max_ltv) then
									ylower = qb*bb(ibprime) - qm0(imprime,ibprime,ig,is)*mm(imprime)*ph*hh(ih)
									if (ylower < ylb(ig,is)) then
										ylb(ig,is) = ylower
									endif
								endif
							enddo
						enddo
					enddo
				enddo
				!Compute value function iteration
				if (iter_ph == 1 .AND. iter_q == 1) then
					itermax       = max_iterv_1
					itermax_accel = max_itervaccel_1
				else
					itermax       = max_iterv_2
					itermax_accel = max_itervaccel_2
				endif
				toler = 1e-6
				if (valfn_type == 1) then
					call sub_valfn_ces
				elseif (valfn_type == 2) then
					call sub_valfn_ez_ces
                endif
				if (print_input == 2) then
					call sub_print_input
				endif
				if (print_output == 2) then
					call sub_print_readable
					call sub_print_final
				endif
				!Calculate updated mortgage prices
				qm0temp = qm0
				do iter_qtemp = 1, max_iterqtemp
					qm0new(1,:,:,:) = qm/(1 + rocost)
					do is = 1, ns
						do ig = 1, ng
							ih      = hfun(ig)
							iu      = ufun(ig)
							igshock = gfun(ih,2)
							call omp_set_num_threads(num_threads)
							!$omp parallel private(imprime,isprime,ieprime,yprime_f0,iy0low,iy0upp,y0weight,im_low,ib_low,d_low,pxs_low,im_upp,ib_upp,d_upp,pxs_upp,mrecov,qm0_low,qm0_upp,qm0_e,qm0_s,qm0_u)
							!$omp do
							do ibprime = 1, nb
								do imprime = 2, nm
									if (iu == 1 .AND. nu == 2) then
										do isprime = 1, ns
											!No housing shock
											do ieprime = 1, ne
												yprime_f0 = labor(isprime,ieprime)*w + bb(ibprime)
												iy0low    = ly(yprime_f0)
												iy0upp    = uy(yprime_f0)
												y0weight  = wy(yprime_f0)
												im_low    = locm_f0(iy0low,imprime,ig,isprime)
												ib_low    = locbown_f0_sp1(iy0low,imprime,ig,isprime)
												d_low     = d_f0(iy0low,imprime,ig,isprime)
												pxs_low   = psell_f0(iy0low,imprime,ig,isprime)
												im_upp    = locm_f0(iy0upp,imprime,ig,isprime)
												ib_upp    = locbown_f0_sp1(iy0upp,imprime,ig,isprime)
												d_upp     = d_f0(iy0upp,imprime,ig,isprime)
												pxs_upp   = psell_f0(iy0upp,imprime,ig,isprime)
												mrecov    = min(1.d0,(Jreo(ih) + fdeficiency*yprime_f0)/(mm(imprime)*ph*hh(ih)))
												qm0_low   = qm0temp(im_low,ib_low,ig,isprime)
												qm0_upp   = qm0temp(im_upp,ib_upp,ig,isprime)
												!Update mortgage prices
												qm0_e(ieprime) = (1 - y0weight)*(pxs_low + (1 - pxs_low)*(d_low*mrecov + (1 - d_low)*(1 + (1 + rscost)*(qm0_low*(1 + rocost) - qm)&
												*mm(im_low)/mm(imprime)))) + y0weight*(pxs_upp + (1 - pxs_upp)*(d_upp*mrecov + (1 - d_upp)*(1 + (1 + rscost)*(qm0_upp*(1 + rocost) - qm)&
												*mm(im_upp)/mm(imprime))))
											enddo
											qm0_u(1) = dot_product(qm0_e,emeas)
											!Housing shock
											do ieprime = 1, ne
												yprime_f0 = labor(isprime,ieprime)*w + bb(ibprime)
												iy0low    = ly(yprime_f0)
												iy0upp    = uy(yprime_f0)
												y0weight  = wy(yprime_f0)
												im_low    = locm_f0(iy0low,imprime,igshock,isprime)
												ib_low    = locbown_f0_sp1(iy0low,imprime,igshock,isprime)
												d_low     = d_f0(iy0low,imprime,igshock,isprime)
												pxs_low   = psell_f0(iy0low,imprime,igshock,isprime)
												im_upp    = locm_f0(iy0upp,imprime,igshock,isprime)
												ib_upp    = locbown_f0_sp1(iy0upp,imprime,igshock,isprime)
												d_upp     = d_f0(iy0upp,imprime,igshock,isprime)
												pxs_upp   = psell_f0(iy0upp,imprime,igshock,isprime)
												mrecov    = min(1.d0,(Jreo(ih) + fdeficiency*yprime_f0)/(mm(imprime)*ph*hh(ih)))
												qm0_low   = qm0temp(im_low,ib_low,igshock,isprime)
												qm0_upp   = qm0temp(im_upp,ib_upp,igshock,isprime)
												!Update mortgage prices
												qm0_e(ieprime) = (1 - y0weight)*(pxs_low + (1 - pxs_low)*(d_low*mrecov + (1 - d_low)*(1 + (1 + rscost)*(qm0_low*(1 + rocost) - qm)&
												*mm(im_low)/mm(imprime)))) + y0weight*(pxs_upp + (1 - pxs_upp)*(d_upp*mrecov + (1 - d_upp)*(1 + (1 + rscost)*(qm0_upp*(1 + rocost) - qm)&
												*mm(im_upp)/mm(imprime))))
											enddo
											qm0_u(2) = dot_product(qm0_e,emeas)
											qm0_s(isprime) = (1 - pseparate)*qm0_u(1) + pseparate*qm0_u(2)
										enddo
									else
										do isprime = 1, ns
											do ieprime = 1, ne
												yprime_f0 = labor(isprime,ieprime)*w + bb(ibprime)
												iy0low    = ly(yprime_f0)
												iy0upp    = uy(yprime_f0)
												y0weight  = wy(yprime_f0)
												im_low    = locm_f0(iy0low,imprime,ig,isprime)
												ib_low    = locbown_f0_sp1(iy0low,imprime,ig,isprime)
												d_low     = d_f0(iy0low,imprime,ig,isprime)
												pxs_low   = psell_f0(iy0low,imprime,ig,isprime)
												im_upp    = locm_f0(iy0upp,imprime,ig,isprime)
												ib_upp    = locbown_f0_sp1(iy0upp,imprime,ig,isprime)
												d_upp     = d_f0(iy0upp,imprime,ig,isprime)
												pxs_upp   = psell_f0(iy0upp,imprime,ig,isprime)
												mrecov    = min(1.d0,(Jreo(ih) + fdeficiency*yprime_f0)/(mm(imprime)*ph*hh(ih)))
												qm0_low   = qm0temp(im_low,ib_low,ig,isprime)
												qm0_upp   = qm0temp(im_upp,ib_upp,ig,isprime)
												!Update mortgage prices
												qm0_e(ieprime) = (1 - y0weight)*(pxs_low + (1 - pxs_low)*(d_low*mrecov + (1 - d_low)*(1 + (1 + rscost)*(qm0_low*(1 + rocost) - qm)&
												*mm(im_low)/mm(imprime)))) + y0weight*(pxs_upp + (1 - pxs_upp)*(d_upp*mrecov + (1 - d_upp)*(1 + (1 + rscost)*(qm0_upp*(1 + rocost) - qm)&
												*mm(im_upp)/mm(imprime))))
											enddo
											qm0_s(isprime) = dot_product(qm0_e,emeas)
										enddo
									endif
									qm0new(imprime,ibprime,ig,is) = qm/(1 + rocost)*dot_product(qm0_s,prob_ss(is,:))
								enddo
							enddo
							!$omp end do
							!$omp end parallel
						enddo
					enddo
					err_qtemp = maxval(abs(qm0new - qm0temp))
					if (err_qtemp < 1e-5 .OR. iter_qtemp == max_iterqtemp) then
						exit
					endif
					qm0temp = qm0new
				enddo
				err_q = maxval(abs(qm0new - qm0))
				write(*, '(a11, i4, a9, f8.5, a6, f9.5, a6, f9.6)'), '   Iter_q =' ,iter_q, ', err_q =' ,err_q, ', ph =' ,ph, ', kc =' ,kc
				if ((err_q < 1e-5 .AND. err_valfn < toler) .OR. iter_q == max_iterq) then
					exit
				endif
				if (err_q >= 0.01) then
					qm0 = qm0 + 0.5*(qm0new - qm0)
				else
					qm0 = qm0 + 0.9*(qm0new - qm0)
				endif
			enddo
			call sub_stationary
			if (print_input >= 1) then
				call sub_print_input
			endif
			if (print_output >= 1) then
				call sub_print_readable
				call sub_print_final
			endif
			!Compute supply and demand of housing
			!Supply of new housing
			supply = yh
			!Total demand and supply of housing
			demand = 0.d0
			do is = 1, ns
				do iy = ny_neg + 1, ny
					!Renters with good credit
					demand = demand + pbuy_f0(iy,is)*h_f0(iy,is)*mu2rent_f0(iy,is)
					!Renters with bad credit
					demand = demand + pbuy_f1(iy,is)*h_f1(iy,is)*mu2rent_f1(iy,is)
				enddo
				!Total supply of housing
				do ig = 1, ng
					ih = hfun(ig)
					!Homeowners with bad credit
					do iy = ny_neg + 1, ny
						supply = supply + psell_f1(iy,ig,is)*hh(ih)*mu1own_f1(iy,ig,is)
					enddo
					!Homeowners with good credit
					do im = 1, nm
						do iy = ny_neg + 1, ny
							!Supply of housing from owners who voluntarily sell
							supply = supply + psell_f0(iy,im,ig,is)*hh(ih)*mu1own_f0(iy,im,ig,is)
							!Supply of housing from new REOs
							supply = supply + (1 - psell_f0(iy,im,ig,is))*d_f0(iy,im,ig,is)*mu1own_f0(iy,im,ig,is)&
							*hh(ih)*psell_reo(ih)
						enddo
					enddo
				enddo
			enddo
			!Supply of housing from existing REOs
			do ih = 1, nh
				supply = supply + hh(ih)*htot_reo(ih)*psell_reo(ih)
			enddo
			!Compute end of period housing stock (recall that htot_reo is the beginning of period REO housing stock)
			hpredict = yh/deprec_h
			hactual  = 0.d0
			do ig = 1, ng
				ih      = hfun(ig)
				hactual = hactual + hh(ih)*(sum(mu3own_f0(:,:,ig,:)) + sum(mu3own_f1(:,ig,:)))
			enddo
			do ih = 1, nh
				hactual = hactual + hh(ih)*htot_reo(ih)/(1 - deprec_h)
			enddo
			!Compute excess demand and update ph
			excess_demand1 = demand - supply
			if (excess_demand1 >= 0.d0) then
				sign_excessdemand1 = 1
			else
				sign_excessdemand1 = -1
			endif
			if (iter_ph >=2 ) then
				if (excess_demand0 >= 0.d0) then
					sign_excessdemand0 = 1
				else
					sign_excessdemand0 = -1
				endif
			endif
			print *, ' '
			write(*, '(a11, f9.6, a10, f9.6, a12, f8.5, a11, f8.5)'), '   demand =' ,demand, ', supply =' ,supply, ', Hpredict =' ,hpredict, ', Hactual =' , hactual
			write(*, '(a8, f9.6, a8, f9.6, a13, f9.6)'), '   own =', sum(mu3own_f0) + sum(mu3own_f1), ', rent =', sum(mu3rent_f0) + sum(mu3rent_f1), ', badcredit =', sum(mu3own_f1) + sum(mu3rent_f1)
			if (iter_ph == 1) then
				if (demand > 0.d0 .AND. supply > 0.d0) then
					pctdiff = abs(excess_demand1)/(0.5*demand + 0.5*supply)
					dph     = max(-0.5d0*ph,min(1.d0,pctdiff)*ph*sign_excessdemand1)
				else
					dph = 0.1d0*ph*sign_excessdemand1
				endif
			elseif (sign_excessdemand1*sign_excessdemand0 == -1) then
				dph     = max(-0.5d0*ph,-0.5d0*dph)
				pctdiff = abs(excess_demand1)/(0.5*demand + 0.5*supply)
				if (abs(dph) > min(1.d0,pctdiff)*ph) then
					dph = dph*min(1.d0,pctdiff)*ph/abs(dph)
				endif
			else
				dph     = max(-0.5d0*ph,dph)
				pctdiff = abs(excess_demand1)/(0.5*demand + 0.5*supply)
				if (abs(dph) > min(1.d0,pctdiff)*ph) then
					dph = dph*min(1.d0,pctdiff)*ph/abs(dph)
				endif
			endif
			excess_demand0 = excess_demand1
			print *, ' '
			write(*, '(a12, i4, a6, f9.5, a7, f9.5, a6, f8.5, a10, f8.5)'), '   Iter_ph =' ,iter_ph, ', ph =' ,ph, ', dph =' ,dph, ', kc =' ,kc, ', err_kc =' ,err_kc
			if (abs(dph) < min_dph .OR. (demand > 0.d0 .AND. supply > 0.d0 .AND. abs(excess_demand1)/(0.5*demand + 0.5*supply) < toler_excessdemand) .OR. iter_ph == max_iterph) then
				exit
			endif
			!Exit in the event of nonexistence
			if (ph < 0.1d0 .AND. excess_demand1 < 0.d0) then
				exit
			endif
			ph = ph + dph
			!Update apartment rental price
			pa = rentratio*ph
			!Compute optimal REO behavior
			call sub_valfn_reo
		enddo
		!Compute capital stock and implied rental rate
		!Liquid assets
		assets = 0.d0
		do is = 1, ns
			!Renters
			do iy = ny_neg + 1, ny
				!Renters with good credit
				assets = assets + brent_f0(iy,is)*mu3rent_f0(iy,is)
				!Renters with bad credit
				assets = assets + brent_f1(iy,is)*mu3rent_f1(iy,is)
			enddo
			!Homeowners
			do ig = 1, ng
				!Homeowners with bad credit
				do iy = ny_neg + 1, ny
					assets = assets + bown_f1(iy,ig,is)*mu3own_f1(iy,ig,is)
				enddo
				!Homeowners with good credit but no mortgage
				do iy = 1, ny
					assets = assets + bown_f0(iy,1,ig,is)*mu3own_f0(iy,1,ig,is)
				enddo
				do im = 2, nm
					!Homeowners with mortgages
					do iy = ny_neg + 1, ny
						assets = assets + bown_f0(iy,im,ig,is)*mu3own_f0(iy,im,ig,is)
					enddo
				enddo
			enddo
		enddo
		!Intermediary mortgage costs (= new mortgages + existing mortgages + purchases of REOs)
		mortcost = 0.d0
		do is = 1, ns
			do ig = 1, ng
				ih = hfun(ig)
				!Subperiod 1
				do im = 2, nm
					do iy = ny_neg + 1, ny
						!!!Ignores maintenance costs and compensation to borrowers in the event of ex-ante REO profits!!!
						mortcost = mortcost + (Jreo(ih) - psell_reo(ih)*xs_reo(ih))*(1 - psell_f0(iy,im,ig,is))*d_f0(iy,im,ig,is)*mu1own_f0(iy,im,ig,is)
					enddo
				enddo
				!Subperiod 3
				!New mortgages
				do iy = 1, ny
					mortcost = mortcost + qm0(locm_f0(iy,1,ig,is),locbown_f0(iy,1,ig,is),ig,is)*(1 + rocost)*(1 + rscost)*m_f0(iy,1,ig,is)*ph*hh(ih)*mu3own_f0(iy,1,ig,is)
				enddo
				!Existing mortgages
				do im = 2, nm
					do iy = ny_neg + 1, ny
						mortcost = mortcost + qm0(im,locbown_f0(iy,im,ig,is),ig,is)*(1 + rocost)*(1 + rscost)*mm(im)*ph*hh(ih)*mu3own_f0(iy,im,ig,is)
					enddo
				enddo
			enddo
		enddo
		!Existing REO stock
		do ih = 1, nh
			mortcost = mortcost + (Jreo(ih) - psell_reo(ih)*xs_reo(ih))*htot_reo(ih)
		enddo
		!Steady state capital stock
		kstock = qb*assets - mortcost
		err_kc = abs(kstock - kc)
		kc_min = max(kc_min,min(kc,kstock))
		kc_max = min(kc_max,max(kc,kstock))
		print *, ' '
		write(*, '(a12, i4, a10, f8.5, a6, f8.5, a5, f8.5, a9, f8.5)'), '   Iter_kc =' ,iter_kc, ', err_kc =' ,err_kc, ', Kc =' ,kc, ', K =' ,kstock, ', Kcnew =' ,min(kc + 0.5*(kc_max - kc),max(kc + 0.5*(kc_min - kc),kc + dkc*(kstock - kc)))
		if (err_kc < toler_kc .OR. abs(kc_max - kc_min) < toler_kc .OR. iter_kc == max_iterkc) then
			exit
		endif
		kc = min(kc + 0.5*(kc_max - kc),max(kc + 0.5*(kc_min - kc),kc + dkc*(kstock - kc)))
	enddo
	
end subroutine sub_equilibrium