subroutine sub_initialize

	use mod_globvar
	use mod_toolbox

	implicit none

	!This program generates grids, transition probabilities, etc.
	
	!Indices
	integer :: im, iy, ib, ih, ix, is, isprime, ie, iu, ig, iyfine

	!Other internal variables
	integer :: printcorr, printgrids, rouwenhorst_s, ns_temp
	real(8) :: mmod, ymod, bmod, hmod, xmod, mu, upp, low
	real(8), dimension(:), allocatable :: ss_temp
	real(8), dimension(:,:), allocatable :: prob_ss_temp

	!******* External functions to be called *******
	real(8), external :: pmatchbuy, pmatchsell, normcdf, evalue

	!------------------------------------------------
	!State variable correspondences
	!------------------------------------------------
	
	!Generate link between g and (h,u) values
	do iu = 1, nu
		do ih = 1, nh
			ig = nh*(iu - 1) + ih
			gfun(ih,iu) = ig
			hfun(ig)    = ih
			ufun(ig)    = iu
		enddo
	enddo
	
	printcorr = 0
	if (printcorr == 1) then
		print *, 'g --> (h,u)'
		do ig = 1, ng
			write(*, '(a3, i4, a5, i2, a5, i2)'), 'g =',ig, ', h =',hfun(ig), ', u =' ,ufun(ig)
		enddo
		print *, ' '
		print *, '(h,u) --> g'
		do iu = 1, nu
			do ih = 1, nh
				write(*, '(a3, i2, a5, i2, a5, i2)'), 'h =',ih, ', u =',iu, ', g =' ,gfun(ih,iu)
			enddo
		enddo
	endif
	
	!------------------------------------------------
	!Grids and transition probabilities for individual states
	!------------------------------------------------
	
	!Generate grid for leverage values
	do im = 1, nm
		mmod   = dm*real(im - 1,8)
		mm(im) = mmod**(1/curv_m)
	enddo
	
	!Generate grid for cash at hand
	do iy = 1, ny
		if (iy <= ny_neg) then
			ymod   = -(-min_y)**curv_yneg + dy_neg*real(iy - 1,8)
			ymod   = -ymod
			yy(iy) = -ymod**(1/curv_yneg)
		else
			ymod   = dy_pos*real(iy - ny_neg - 1,8)
			yy(iy) = ymod**(1/curv_ypos)
		endif
	enddo
	
	!Generate extra fine grid for cash at hand
	do iyfine = 1, nyfine
		if (iyfine <= nyfine_neg) then
			ymod   = -(-min_yfine)**curv_yfineneg + dyfine_neg*real(iyfine - 1,8)
			ymod   = -ymod
			yyfine(iyfine) = -ymod**(1/curv_yfineneg)
		else
			ymod   = dyfine_pos*real(iyfine - nyfine_neg - 1,8)
			yyfine(iyfine) = ymod**(1/curv_yfinepos)
		endif
	enddo

	!Generate grid for bonds
	do ib = 1, nb
		bmod   = db*real(ib - 1,8)
		bb(ib) = bmod**(1/curv_b)
	enddo
	
	!Generate grid for housing
	do ih = 1, nh
!		hmod   = dh*real(ih - 1,8)
!		hh(ih) = hmod**(1/curv_h) + min_h
		hh(ih) = min_h*(1 + hincrement)**real(ih - 1,8)
	enddo

	!Generate grid for housing preference shocks
	uu(1) = 1.0
	if (nu == 2) then
		uu(2) = useparate
	endif

	!Generate grid for housing submarkets
	do ix = 1, nx
		xmod   = dx*real(ix - 1,8)
		xx(ix) = xmod**(1/curv_x)
	enddo
	
	!Generate submarket match probabilities
	do ix = 1, nx
		prob_xb(ix) = pmatchbuy(xx(ix))
		prob_xs(ix) = pmatchsell(xx(ix))
		if (monthly_quarterly == 1) then
			prob_xb(ix) = 1 - (1 - prob_xb(ix))**3.0
			prob_xs(ix) = 1 - (1 - prob_xs(ix))**3.0
		endif
	enddo
	
	!Generate grid for transitory labor productivity shocks
	ee(ne) = cover_e*sig_e
	ee(1)  = -ee(ne)	
	do ie = 2, ne - 1
		ee(ie) = ee(1) + (ee(ne) - ee(1))*real(ie - 1,8)/real(ne - 1,8)
	enddo
	
	!Pdf for transitory labor productivity shocks
	upp = (ee(2) + ee(1))/2.0
	upp = upp/sig_e
	emeas(1) = normcdf(upp)
	low = (ee(ne) + ee(ne - 1))/2.0
	low = low/sig_e
	emeas(ne) = 1.0 - normcdf(low)
	do ie = 2, ne - 1
		low = (ee(ie - 1) + ee(ie))/2.0
		upp = (ee(ie + 1) + ee(ie))/2.0
		low = low/sig_e
		upp = upp/sig_e
		emeas(ie) = normcdf(upp) - normcdf(low)
	enddo
	
	!Discretize AR(1) process for permanent labor productivity shock
	rouwenhorst_s = 1
	if (rich_shock == 1) then
		ns_temp = ns - 1
	else
		ns_temp = ns
	endif
	allocate(ss_temp(ns_temp))
	allocate(prob_ss_temp(ns_temp,ns_temp))
	if (rouwenhorst_s == 1) then
		!Rouwenhorst method
		call rouwenhorst(rho_s,sig_s/(1 - rho_s**2.0)**0.5,prob_ss_temp,ss_temp)
	else
		!Tauchen method
		ss_temp(ns_temp) = cover_s*sig_s/sqrt(1.0 - rho_s**2.0)
		ss_temp(1)       = -ss_temp(ns_temp)	
		do is = 2, ns_temp - 1
			ss_temp(is) = ss_temp(1) + (ss_temp(ns_temp) - ss_temp(1))*real(is - 1,8)/real(ns_temp - 1,8)
		enddo
		do is = 1, ns_temp
			mu  = rho_s*ss_temp(is)
			upp = (ss_temp(2) + ss_temp(1))/2.0
			upp = (upp - mu)/sig_s
			prob_ss_temp(is,1) = normcdf(upp)
			low = (ss_temp(ns_temp) + ss_temp(ns_temp - 1))/2.0
			low = (low - mu)/sig_s	
			prob_ss_temp(is,ns_temp) = 1.0 - normcdf(low)
			do isprime = 2, ns_temp - 1		
				low = (ss_temp(isprime - 1) + ss_temp(isprime))/2.0
				upp = (ss_temp(isprime + 1) + ss_temp(isprime))/2.0
				low = (low - mu)/sig_s
				upp = (upp - mu)/sig_s
				prob_ss_temp(is,isprime) = normcdf(upp) - normcdf(low)
			enddo
		enddo
	endif
	
	!Adjust transition probabilities to quarterly frequency
	do is = 1, ns_temp
		do isprime = 1, ns_temp
			if (is == isprime) then
				prob_ss_temp(is,isprime) = 0.25d0*prob_ss_temp(is,isprime) + 0.75d0
			else
				prob_ss_temp(is,isprime) = 0.25d0*prob_ss_temp(is,isprime)
			endif
		enddo
	enddo
	
	if (rich_shock == 1) then
		!Add lucky high shock
		do is = 1, ns - 1
			ss(is) = ss_temp(is)
		enddo
		ss(ns) = ss(ns - 1) + log(s_rich)
		do is = 1, ns - 2
			do isprime = 1, ns - 1
				prob_ss(is,isprime) = prob_ss_temp(is,isprime)
			enddo
			prob_ss(is,ns) = 0.d0
		enddo
		do isprime = 1, ns - 1
			prob_ss(ns - 1,isprime) = prob_ss_temp(ns - 1,isprime)*(1 - prob_getrich)
		enddo
		prob_ss(ns - 1,ns) = prob_getrich
		do isprime = 1, ns - 2
			prob_ss(ns,isprime) = 0.d0
		enddo
		prob_ss(ns,ns)     = prob_stayrich
		prob_ss(ns,ns - 1) = 1 - prob_stayrich
	else
		prob_ss = prob_ss_temp
		ss      = ss_temp
	endif
	
	!Generate stationary distribution of permanent labor productivity shocks
	call ergodist(ns,prob_ss,stationary_s)
	
	!Normalize average labor productivity to 1
	enorm = 1.d0
	eavg  = 0.d0
	do is = 1, ns
		do ie = 1,ne
			labor(is,ie) = evalue(ss(is),ee(ie))
			eavg         = eavg + labor(is,ie)*emeas(ie)*stationary_s(is)
		enddo
	enddo
!	enorm = eavg
!	eavg  = 0.d0
!	do is = 1, ns
!		do ie = 1,ne
!			eavg = eavg + evalue(ss(is),ee(ie))*emeas(ie)*stationary_s(is)
!		enddo
!	enddo

	!Display grids (to make sure nothing is obviously wrong)
	printgrids = 0
	if (printgrids == 1) then
		print *, 'Leverage'
		print *, mm
		print *, ' '
		print *, 'Cash at hand'
		print *, yy
		print *, ' '
		print *, 'Bonds'
		print *, bb
		print *, ' '
		print *, 'Submarkets'
		print *, xx
	endif
	
!	do is = 1, ns
!		print *, ' '
!		print *, prob_ss(is,:)
!		print *, sum(prob_ss(is,:))
!	enddo
!	print *, ' '
!	print *, stationary_s	
!	stop

end subroutine sub_initialize