subroutine sub_outputprices_sh(kstock,ltot,p_h,z_h,wage,rent,y_c,y_h)

	use mod_globvar

	implicit none

	!This program computes factor prices and output when structures are an intermediate good
	
	real(8), intent(in) :: kstock, ltot, p_h, z_h
	real(8), intent(out) :: wage, rent, y_c, y_h
	
	!Other internal variables
	real(8) :: n_c, kn_c, s_h, n_h, A, B, feval, fderiv, n_c1	
	integer :: iter
	
	if (input_zc == 1) then
		!*** Use Newton's method to compute labor in the consumption good sector***!
		A   = ((1 - lshare)*kshare_h*p_h*z_h*(kshare_h/(1 - kshare_h)*(1 - kshare_c)*zc)**(kshare_h*(1 - lshare) - 1))**(1/lshare)
		B   = kshare_c*(kshare_h*(1 - lshare) - 1)/lshare
		n_c = 0.05*ltot
		do iter = 1, 1000
			feval = ltot - A*kstock**B*n_c**(-B) - n_c
			if (abs(feval) < 1e-8) then
				exit
			endif
			fderiv = A*B*kstock**B*n_c**(-B - 1) - 1
			n_c1   = max(1e-6,n_c - feval/fderiv)
			n_c    = n_c1
		enddo
	
		!Finish computing sectoral allocation of capital and labor
		n_h  = ltot - n_c
		kn_c = kstock/n_c
		s_h  = kshare_h/(1 - kshare_h)*(1 - kshare_c)*zc*A*kn_c**(B + kshare_c)
	else		
		!Calculate sectoral labor!
		n_h = ((1 - lshare)*kshare_h*p_h*z_h*(kshare_h/(1 - kshare_h)*0.25d0/ltot)**(kshare_h*(1 - lshare) - 1))**(1.d0/lshare)
		n_c = ltot - n_h
	
		!Calculate capital-to-labor ratio in the consumption good sector
		kn_c = kstock/n_c
	
		!Calculate TFP in the consumption good sector
		zc = 0.25/((1 - kshare_c)*ltot*kn_c**kshare_c)
	
		!Calculate structures in the housing sector
		A   = ((1 - lshare)*kshare_h*p_h*z_h*(kshare_h/(1 - kshare_h)*(1 - kshare_c)*zc)**(kshare_h*(1 - lshare) - 1))**(1/lshare)
		B   = kshare_c*(kshare_h*(1 - lshare) - 1)/lshare
		s_h = kshare_h/(1 - kshare_h)*(1 - kshare_c)*zc*A*kn_c**(B + kshare_c)
	endif	
	
	!Compute factor prices
	wage = (1 - kshare_c)*zc*kn_c**kshare_c
	rent = kshare_c*zc*kn_c**(kshare_c - 1)
	
	!Compute output
	y_c = zc*kn_c**kshare_c*n_c
	y_h = z_h*(s_h**kshare_h*n_h**(1 - kshare_h))**(1 - lshare)
	
	!!!TEMP Modify for Change in Land Share!!!
	!y_h = 0.29850764d0*y_h		!San Francisco
	!y_h = 1.26865641d0*y_h		!Houston

end subroutine sub_outputprices_sh