subroutine sub_simulate

	use mod_globvar
	use mod_interpolate

	implicit none

	!This program computes simulates dynamic household behavior
	
	!Indices
	integer :: it, isprime, ieprime, iy, im, ig, ih, is
	
	!Simulation time series
	real(8), dimension(:), allocatable :: erand_t, srand_t, buyrand_t, sellrand_t, flagrand_t, deprecrand_t, separaterand_t, earn_t, sell_t, default_t, buy_t, assets_t, mort_t, cons_t, &
	hown_t, hcons_t, xsratio_t, xbratio_t, psell_t, pbuy_t
	integer, dimension(:), allocatable :: ieprime_t, isprime_t, is_t, ig_t, im_t, iy_t, flag_t
	
	!Other internal variables
	real(8) :: ysell, ypay, ydefault, ybuy, yprime
	integer :: T, flag
	
	!Length of simulation
	T = 1000
	
	!Allocate
	allocate(erand_t(T))
	allocate(srand_t(T))
	allocate(buyrand_t(T))
	allocate(sellrand_t(T))
	allocate(flagrand_t(T))
	allocate(deprecrand_t(T))
	allocate(separaterand_t(T))
	allocate(ieprime_t(T))
	allocate(isprime_T(T))
	allocate(is_t(T + 1))
	allocate(ig_t(T + 1))
	allocate(im_t(T + 1))
	allocate(iy_t(T + 1))
	allocate(flag_t(T + 1))
	allocate(earn_t(T + 1))
	allocate(sell_t(T))
	allocate(default_t(T))
	allocate(buy_t(T))
	allocate(assets_t(T))
	allocate(mort_t(T))
	allocate(cons_t(T))
	allocate(hcons_t(T))
	allocate(hown_t(T))
	allocate(xsratio_t(T))
	allocate(xbratio_t(T))
	allocate(psell_t(T))
	allocate(pbuy_t(T))
	
	!Initialize
	print *, ' '
	write(*,*), 'Initial foreclosure flag?'
	read(*,*) flag_t(1)
	write(*,*), 'Initial persistent earnings shock?'
	read(*,*) is_t(1)
	write(*,*), 'Initial house?'
	read(*,*) ig_t(1)
	write(*,*), 'Initial mortgage?'
	read(*,*) im_t(1)
	write(*,*), 'Initial cash at hand?'
	read(*,*) iy_t(1)
	iy_t(1)      = ny_neg + iy_t(1)
	sell_t(:)    = 0.d0
	default_t(:) = 0.d0
	buy_t(:)     = 0.d0
	assets_t(:)  = 0.d0
	mort_t(:)    = 0.d0
	cons_t(:)    = 0.d0
	hcons_t(:)   = 0.d0
	hown_t(:)    = 0.d0
	
	!Generate sequence of idiosyncratic earnings shocks
	do it = 1, T
		call random_number(erand_t(it))
		call random_number(srand_t(it))
		call random_number(buyrand_t(it))
		call random_number(sellrand_t(it))
		call random_number(flagrand_t(it))
		call random_number(deprecrand_t(it))
		call random_number(separaterand_t(it))
		do isprime = 1, ns
			if (srand_t(it) <= sum(prob_ss(is_t(it),1:isprime))) then
				isprime_t(it) = isprime
				exit
			endif
		enddo
		is_t(it + 1) = isprime_t(it)
		do ieprime = 1, ne
			if (erand_t(it) <= sum(emeas(1:ieprime))) then
				ieprime_t(it) = ieprime
				exit
			endif
		enddo
		earn_t(it + 1) = labor(isprime_t(it),ieprime_t(it))*w
	enddo
	
	!Simulate
	do it = 1, T
		flag = flag_t(it)
		is   = is_t(it)
		ig   = ig_t(it)
		im   = im_t(it)
		iy   = iy_t(it)
		if (flag == 1 .OR. ig == 0) then
			im = 1
		endif
		!Subperiod 1
		if (ig > 0) then
			!Homeowners
			ih         = hfun(ig)
			hown_t(it) = ph*hh(ih)
			if (flag == 1) then
				!Homeowners with bad credit
				xsratio_t(it) = xsratio_f1(iy,ig,is)
				psell_t(it)   = psell_f1(iy,ig,is)
				if (sellrand_t(it) <= psell_f1(iy,ig,is)) then
					!Successful sale
					!!!
					sell_t(it) = 1.d0
					!!!
					ysell = yy(iy) + xs_f1(iy,ig,is)*(1 - sellcost)
					if (wy(ysell) >= 0.5d0) then
						iy = uy(ysell)
					else
						iy = ly(ysell)
					endif
					ig = 0
				endif
			else
				!Homeowners with good credit
				xsratio_t(it) = xsratio_f0(iy,im,ig,is)
				psell_t(it)   = psell_f0(iy,im,ig,is)
				if (sellrand_t(it) <= psell_f0(iy,im,ig,is)) then
					!Successful sale
					!!!
					sell_t(it) = 1.d0
					!!!
					ysell = yy(iy) - mm(im)*ph*hh(ih) + xs_f0(iy,im,ig,is)*(1 - sellcost)
					if (wy(ysell) >= 0.5d0) then
						iy = uy(ysell)
					else
						iy = ly(ysell)
					endif
					ig = 0
					im = 1
				else
					if (d_f0(iy,im,ig,is) == 1.d0) then
						!!!
						default_t(it) = 1.d0
						!!!
						ydefault = yy(iy) - max(0.d0,min(fdeficiency*yy(iy),mm(im)*ph*hh(ih) - Jreo(ih)))
						if (wy(ydefault) >= 0.5d0) then
							iy = uy(ydefault)
						else
							iy = ly(ydefault)
						endif
						ig   = 0
						im   = 1
						flag = 1
					else
						ypay = yy(iy) - (mm(im) - qm*m_f0(iy,im,ig,is))*ph*hh(ih)
						if (ypay < 0.d0) then
							ypay = ypay/abs(ylb(ig,is))
						endif
						if (wy(ypay) >= 0.5d0) then
							iy = uy(ypay)
						else
							iy = ly(ypay)
						endif
						im = locm_f0(iy,im,ig,is)
					endif
				endif
			endif
		endif
		!Subperiod 2
		if (ig == 0) then
			if (flag == 0) then
				xbratio_t(it) = xbratio_f0(iy,is)
				pbuy_t(it)    = pbuy_f0(iy,is)
				if (buyrand_t(it) <= pbuy_f0(iy,is)) then
					!!!
					buy_t(it) = 1.d0
					!!!
					ig   = gfun(loch_f0(iy,is),1)
					ybuy = yy(iy) - xb_f0(iy,is)
					if (ybuy < 0.d0) then
						ybuy = ybuy/abs(ylb(ig,is))
					endif
					if (wy(ybuy) >= 0.5d0) then
						iy = uy(ybuy)
					else
						iy = ly(ybuy)
					endif
				endif
			else
				xbratio_t(it) = xbratio_f1(iy,is)
				pbuy_t(it)    = pbuy_f1(iy,is)
				if (buyrand_t(it) <= pbuy_f1(iy,is)) then
					!!!
					buy_t(it) = 1.d0
					!!!
					ig   = gfun(loch_f1(iy,is),1)
					ybuy = yy(iy) - xb_f1(iy,is)
					if (ybuy < 0.d0) then
						ybuy = ybuy/abs(ylb(ig,is))
					endif
					if (wy(ybuy) >= 0.5d0) then
						iy = uy(ybuy)
					else
						iy = ly(ybuy)
					endif
				endif
			endif
		endif
		!Subperiod 3
		if (ig == 0) then
			!Renters
			if (flag == 0) then
				!Renters with good credit
				!!!
				cons_t(it)   = crent_f0(iy,is)
				hcons_t(it)  = pa*hcrent_f0(iy,is)
				assets_t(it) = brent_f0(iy,is)
				!!!
				flag_t(it + 1) = 0
				ig_t(it + 1)   = 0
				im_t(it + 1)   = 1
				yprime         = earn_t(it + 1) + brent_f0(iy,is)
				if (wy(yprime) >= 0.5d0) then
					iy_t(it + 1) = uy(yprime)
				else
					iy_t(it + 1) = ly(yprime)
				endif
			else
				!Renters with bad credit
				!!!
				cons_t(it)   = crent_f1(iy,is)
				hcons_t(it)  = pa*hcrent_f1(iy,is)
				assets_t(it) = brent_f1(iy,is)
				!!!
				ig_t(it + 1) = 0
				im_t(it + 1) = 1
				if (flagrand_t(it) <= pflag) then
					flag_t(it + 1) = 1
					yprime         = (1 - fincloss)*earn_t(it + 1) + brent_f1(iy,is)
				else
					flag_t(it + 1) = 0
					yprime         = earn_t(it + 1) + brent_f1(iy,is)
				endif
				if (wy(yprime) >= 0.5d0) then
					iy_t(it + 1) = uy(yprime)
				else
					iy_t(it + 1) = ly(yprime)
				endif
			endif
		else
			!Homeowners
			ih = hfun(ig)
			if (flag == 1) then
				!Homeowners with bad credit
				!!!
				cons_t(it)   = cown_f1(iy,ig,is)
				hcons_t(it)  = pa*(hcown_f1(iy,ig,is) + hh(ih))
				assets_t(it) = bown_f1(iy,ig,is)
				!!!
				im_t(it + 1) = 1
				if (deprecrand_t(it) <= deprec_h) then
					!House fully depreciates
					ig_t(it + 1) = 0
				else
					!House does not depreciate
					if (separaterand_t(it) <= pseparate) then
						!Housing preference shock
						ig_t(it + 1) = gfun(ih,2)
					else
						!No housing preference shock
						ig_t(it + 1) = ig
					endif
				endif
				if (flagrand_t(it) <= pflag) then
					flag_t(it + 1) = 1
					yprime         = (1 - fincloss)*earn_t(it + 1) + bown_f1(iy,ig,is)
				else
					flag_t(it + 1) = 0
					yprime         = earn_t(it + 1) + bown_f1(iy,ig,is)
				endif
				if (wy(yprime) >= 0.5d0) then
					iy_t(it + 1) = uy(yprime)
				else
					iy_t(it + 1) = ly(yprime)
				endif
			else
				!Homeowners with good credit
				!!!
				cons_t(it)   = cown_f0(iy,im,ig,is)
				hcons_t(it)  = pa*(hcown_f0(iy,im,ig,is) + hh(ih))
				assets_t(it) = bown_f0(iy,im,ig,is)
				!!!
				flag_t(it + 1) = 0
				if (deprecrand_t(it) <= deprec_h) then
					!House fully depreciates
					ig_t(it + 1) = 0
					im_t(it + 1) = 1
				else
					!House does not depreciate
					if (im == 1) then
						!!!
						mort_t(it) = m_f0(iy,1,ig,is)
						!!!
						im_t(it + 1) = locm_f0(iy,1,ig,is)
					else
						!!!
						mort_t(it) = mm(im)
						!!!
						im_t(it + 1) = im
					endif
					if (separaterand_t(it) <= pseparate) then
						!Housing preference shock
						ig_t(it + 1) = gfun(ih,2)
					else
						!No housing preference shock
						ig_t(it + 1) = ig
					endif
				endif
				yprime = earn_t(it + 1) + bown_f0(iy,im,ig,is)
				if (wy(yprime) >= 0.5d0) then
					iy_t(it + 1) = uy(yprime)
				else
					iy_t(it + 1) = ly(yprime)
				endif
			endif
		endif
	enddo
	
	do it = 1, T
		if (it == 1) then
			open(unit=11,	file='simul_individual.txt', status = 'replace')
			write(11, '(i4, i4, f10.5, f10.5, f10.5, f10.5, f10.5, f10.5, f10.5, f10.5, f10.5, f10.5, f10.5, f10.5, f10.5, f10.5, f10.5, f10.5)'), it, flag_t(it), earn_t(it), yy(iy_t(it)), hown_t(it), mm(im_t(it)), xsratio_t(it), psell_t(it), sell_t(it), default_t(it), xbratio_t(it), pbuy_t(it), buy_t(it), assets_t(it), mort_t(it), cons_t(it), hcons_t(it)
			close(11)
		else
			open(unit=11,	file='simul_individual.txt', access = 'append')
			write(11, '(i4, i4, f10.5, f10.5, f10.5, f10.5, f10.5, f10.5, f10.5, f10.5, f10.5, f10.5, f10.5, f10.5, f10.5, f10.5, f10.5, f10.5)'), it, flag_t(it), earn_t(it), yy(iy_t(it)), hown_t(it), mm(im_t(it)), xsratio_t(it), psell_t(it), sell_t(it), default_t(it), xbratio_t(it), pbuy_t(it), buy_t(it), assets_t(it), mort_t(it), cons_t(it), hcons_t(it)
			close(11)
		endif
	enddo
	
end subroutine sub_simulate