subroutine sub_valfn_ez_ces

	use mod_globvar
	use mod_interpolate

	implicit none

	!This program computes value function iteration
	!Features: 1) Owners choose between living in their house or renting an apartment; 2) There is a utility cost to failing to sell.
	!Note 1: Since all value functions are > 0, owners do not attempt to sell if searchcost > Vown.
	!Note 2: It is easy to modify the setup to have a utility cost to entering the selling market, as was previously the case.
	
	!Indices for today's state
	integer :: is, ig, ih, iu, im, iy
	!Indices for tomorrow's state
	integer :: isprime, ieprime, igshock
	!Indices for grid search during optimization
	integer :: imprime, ibprime, ix

	!Continuation utilities
	real(8), dimension(ne) :: Vcont_e_f0, Vcont_e_f1
	real(8), dimension(ns) :: Vcont_s_f0, Vcont_s_f1
	real(8), dimension(nb,nm) :: Vcont_own_f0
	real(8), dimension(nb) :: Vcont_rent_f0, Vcont_own_f1, Vcont_rent_f1
		
	!Updated value functions
	real(8), dimension(:,:), allocatable :: Vnew_rent_f0, Vnew_rent_f1
	real(8), dimension(:,:,:), allocatable :: Vnew_own_f1
	real(8), dimension(:,:,:,:), allocatable :: Vnew_own_f0, Wnosell_f0
		
	!Other internal variables
	real(8) :: yprime_f0, yprime_f1, V_maxsofar, c_maxsofar, hc_maxsofar, spend_maxsofar, spending, cstay, hcstay, ustay, cmove, hcmove, umove, c, hc, &
	util, val, cash_at_hand, ybuy, ydefault, wdefault, W_maxsofar, ypay, wval, ysell, y0weight, y1weight, yweight, minerr_noaccel, minerr_accel
	integer :: iter_valfn, b_maxsofar, m_maxsofar, ibprime_start, h_maxsofar, x_maxsofar, iy0low, iy0upp, iy1low, iy1upp, iylow, iyupp
	
	!******* External functions to be called *******
	real(8), external :: xb, xs
	
	!Allocate updated value functions
	allocate(Vnew_rent_f0(ny,ns))
	allocate(Vnew_rent_f1(ny,ns))
	allocate(Vnew_own_f1(ny,ng,ns))
	allocate(Vnew_own_f0(ny,nm,ng,ns))
	allocate(Wnosell_f0(ny,nm,ng,ns))
		
!	print *, ' '
!	print *, '     Compute value function iteration'
!	print *, ' '
	minerr_noaccel = 1e8
	minerr_accel   = 1e8
	do iter_valfn = 1, itermax
		if (iter_valfn - 1 == 10*floor((real(iter_valfn)-1)/10.0) .OR. iter_valfn >= itermax_accel .OR. (minerr_noaccel < 10.0*toler .AND. minerr_accel < toler) .OR. minerr_accel < 0.5*toler) then
			!House buying subperiod
			do is = 1, ns
				call omp_set_num_threads(num_threads)
				!$omp parallel private(W_maxsofar,h_maxsofar,x_maxsofar,ih,ig,ix,ybuy,iylow,iyupp,yweight,wval)
				!$omp do
				do iy = ny_neg + 1, ny
					!Households with good credit
					W_maxsofar = -1e9
					h_maxsofar = 1
					x_maxsofar = 1
					do ih = 1, nh
						ig = gfun(ih,1)
						do ix = 2, nx
							ybuy = yy(iy) - xb(xx(ix),hh(ih),ph)
							if (ybuy < 0.d0) then
								ybuy = ybuy/abs(ylb(ig,is))
							endif
							iylow   = ly(ybuy)
							iyupp   = uy(ybuy)
							yweight = wy(ybuy)
							if (ybuy >= -1.d0) then
								wval = ((1 - prob_xb(ix))*Vrent_f0(iy,is)**(1 - gamma) + prob_xb(ix)*((1 - yweight)*Vown_f0(iylow,1,ig,is)**(1 - gamma)&
								+ yweight*Vown_f0(iyupp,1,ig,is)**(1 - gamma)))**(1/(1 - gamma))
								if (wval > W_maxsofar) then
									W_maxsofar = wval
									h_maxsofar = ih
									x_maxsofar = ix
								endif
							else
								exit
							endif
						enddo
					enddo
					if (W_maxsofar > Vrent_f0(iy,is)) then
						Wrent_f0(iy,is)   = W_maxsofar
						loch_f0(iy,is)    = h_maxsofar
						h_f0(iy,is)       = hh(h_maxsofar)
						locxb_f0(iy,is)   = x_maxsofar
						xb_f0(iy,is)      = xb(xx(x_maxsofar),hh(h_maxsofar),ph)
						xbratio_f0(iy,is) = xb(xx(x_maxsofar),hh(h_maxsofar),ph)/(ph*hh(h_maxsofar))
						pbuy_f0(iy,is)    = prob_xb(x_maxsofar)
					else
						Wrent_f0(iy,is)   = Vrent_f0(iy,is)
						loch_f0(iy,is)    = 1			!Renters who don't buy any house choose ih = 1 and pbuy = 0
						h_f0(iy,is)       = 0.d0
						locxb_f0(iy,is)   = 1
						xb_f0(iy,is)      = 0.d0
						xbratio_f0(iy,is) = 0.d0
						pbuy_f0(iy,is)    = 0.d0
					endif
					!Households with bad credit
					W_maxsofar = -1e9
					h_maxsofar = 1
					x_maxsofar = 1
					do ih = 1, nh
						ig = gfun(ih,1)
						do ix = 2, nx
							ybuy    = yy(iy) - xb(xx(ix),hh(ih),ph)
							iylow   = ly(ybuy)
							iyupp   = uy(ybuy)
							yweight = wy(ybuy)
							if (ybuy >= 0.d0) then
								wval = ((1 - prob_xb(ix))*Vrent_f1(iy,is)**(1 - gamma) + prob_xb(ix)*((1 - yweight)*Vown_f1(iylow,ig,is)**(1 - gamma)&
								+ yweight*Vown_f1(iyupp,ig,is)**(1 - gamma)))**(1/(1 - gamma))
								if (wval > W_maxsofar) then
									W_maxsofar = wval
									h_maxsofar = ih
									x_maxsofar = ix
								endif
							else
								exit
							endif
						enddo
					enddo
					if (W_maxsofar > Vrent_f1(iy,is)) then
						Wrent_f1(iy,is)   = W_maxsofar
						loch_f1(iy,is)    = h_maxsofar
						h_f1(iy,is)       = hh(h_maxsofar)
						locxb_f1(iy,is)   = x_maxsofar
						xb_f1(iy,is)      = xb(xx(x_maxsofar),hh(h_maxsofar),ph)
						xbratio_f1(iy,is) = xb(xx(x_maxsofar),hh(h_maxsofar),ph)/(ph*hh(h_maxsofar))
						pbuy_f1(iy,is)    = prob_xb(x_maxsofar)
					else
						Wrent_f1(iy,is)   = Vrent_f1(iy,is)
						loch_f1(iy,is)    = 1			!Renters who don't buy any house choose ih = 1 and pbuy = 0
						h_f1(iy,is)       = 0.d0
						locxb_f1(iy,is)   = 1
						xb_f1(iy,is)      = 0.d0
						xbratio_f1(iy,is) = 0.d0
						pbuy_f1(iy,is)    = 0.d0
					endif
				enddo
				!$omp end do
				!$omp end parallel
				do iy = 1, ny_neg
					Wrent_f0(iy,is)   = Wrent_f0(ny_neg + 1,is)
					loch_f0(iy,is)    = loch_f0(ny_neg + 1,is)
					h_f0(iy,is)       = h_f0(ny_neg + 1,is)
					locxb_f0(iy,is)   = locxb_f0(ny_neg + 1,is)
					xb_f0(iy,is)      = xb_f0(ny_neg + 1,is)
					xbratio_f0(iy,is) = xbratio_f0(ny_neg + 1,is)
					pbuy_f0(iy,is)    = pbuy_f0(ny_neg + 1,is)
					Wrent_f1(iy,is)   = Wrent_f1(ny_neg + 1,is)
					loch_f1(iy,is)    = loch_f1(ny_neg + 1,is)
					h_f1(iy,is)       = h_f1(ny_neg + 1,is)
					locxb_f1(iy,is)   = locxb_f1(ny_neg + 1,is)
					xb_f1(iy,is)      = xb_f1(ny_neg + 1,is)
					xbratio_f1(iy,is) = xbratio_f1(ny_neg + 1,is)
					pbuy_f1(iy,is)    = pbuy_f1(ny_neg + 1,is)
				enddo
			enddo
			!Mortgage foreclosure/payment subperiod
			do is = 1, ns
				do ig = 1, ng
					ih = hfun(ig)
					!Households without a mortgage
					do iy = 1, ny
						Wnosell_f0(iy,1,ig,is) = Vown_f0(iy,1,ig,is)
						d_f0(iy,1,ig,is)       = 0.d0
					enddo
					!Households with a mortgage
					call omp_set_num_threads(num_threads)
					!$omp parallel private(iy,ydefault,iylow,iyupp,yweight,wdefault,W_maxsofar,m_maxsofar,imprime,ypay,wval)
					!$omp do
					do im = 2, nm
						do iy = ny_neg + 1, ny
							!Utility of defaulting
							ydefault = yy(iy) - max(0.d0,min(fdeficiency*yy(iy),mm(im)*ph*hh(ih) - Jreo(ih)))
							iylow    = ly(ydefault)
							iyupp    = uy(ydefault)
							yweight  = wy(ydefault)
							wdefault = (1 - yweight)*Wrent_f1(iylow,is) + yweight*Wrent_f1(iyupp,is)
							!Utility of making an optimal payment
							W_maxsofar = -1e9
							m_maxsofar = im
							do imprime = 1, im
								ypay = yy(iy) - (mm(im) - qm*mm(imprime))*ph*hh(ih)
								if (ypay < 0.d0) then
									ypay = ypay/abs(ylb(ig,is))
								endif
								iylow   = ly(ypay)
								iyupp   = uy(ypay)
								yweight = wy(ypay)
								if ((imprime == 1 .AND. ypay >= -1.d0) .OR. (imprime > 1 .AND. ypay >= 0.d0)) then
									wval = (1 - yweight)*Vown_f0(iylow,imprime,ig,is) + yweight*Vown_f0(iyupp,imprime,ig,is)
									if (wval > W_maxsofar) then
										W_maxsofar = wval
										m_maxsofar = imprime
									endif
								endif					        
							enddo
							!Optimal default/payment decision
							if (W_maxsofar >= wdefault) then
								Wnosell_f0(iy,im,ig,is) = W_maxsofar
								d_f0(iy,im,ig,is)       = 0.d0
								locm_f0(iy,im,ig,is)    = m_maxsofar
								m_f0(iy,im,ig,is)       = mm(m_maxsofar)
							else
								Wnosell_f0(iy,im,ig,is) = wdefault
								d_f0(iy,im,ig,is)       = 1.d0
								locm_f0(iy,im,ig,is)    = 1
								m_f0(iy,im,ig,is)       = 0.d0
							endif
						enddo
						do iy = 1, ny_neg
							Wnosell_f0(iy,im,ig,is) = Wnosell_f0(ny_neg + 1,im,ig,is)
							d_f0(iy,im,ig,is)       = d_f0(ny_neg + 1,im,ig,is)
							locm_f0(iy,im,ig,is)    = locm_f0(ny_neg + 1,im,ig,is)
							m_f0(iy,im,ig,is)       = m_f0(ny_neg + 1,im,ig,is)
						enddo
					enddo
					!$omp end do
					!$omp end parallel
				enddo
			enddo
			!House selling subperiod
			do is = 1, ns
				do ig = 1, ng
					ih = hfun(ig)
					!Households with bad credit
					call omp_set_num_threads(num_threads)
					!$omp parallel private(W_maxsofar,x_maxsofar,ix,ysell,iylow,iyupp,yweight,wval)
					!$omp do
					do iy = ny_neg + 1, ny
						W_maxsofar = -1e9
						x_maxsofar = nx
						do ix = 1, nx - 1
							ysell   = yy(iy) + xs(xx(ix),hh(ih),ph)*(1 - sellcost)
							iylow   = ly(ysell)
							iyupp   = uy(ysell)
							yweight = wy(ysell)
							if (Vown_f1(iy,ig,is) - searchcost < 0.d0) then
								wval = -1e9
							else
								wval    = ((1 - prob_xs(ix))*(Vown_f1(iy,ig,is) - searchcost)**(1 - gamma) + prob_xs(ix)&
								*((1 - yweight)*Wrent_f1(iylow,is)**(1 - gamma) + yweight*Wrent_f1(iyupp,is)**(1 - gamma)))**(1/(1 - gamma))
							endif
							if (wval > W_maxsofar) then
								W_maxsofar = wval
								x_maxsofar = ix
							endif
						enddo
						if (W_maxsofar > Vown_f1(iy,ig,is)) then
							Wown_f1(iy,ig,is)    = W_maxsofar
							locxs_f1(iy,ig,is)   = x_maxsofar
							xs_f1(iy,ig,is)      = xs(xx(x_maxsofar),hh(ih),ph)
							xsratio_f1(iy,ig,is) = xs(xx(x_maxsofar),hh(ih),ph)/(ph*hh(ih))
							psell_f1(iy,ig,is)   = prob_xs(x_maxsofar)
						else
							Wown_f1(iy,ig,is)    = Vown_f1(iy,ig,is)
							locxs_f1(iy,ig,is)   = nx
							xs_f1(iy,ig,is)      = xs(xx(nx),hh(ih),ph)
							xsratio_f1(iy,ig,is) = xs(xx(nx),hh(ih),ph)/(ph*hh(ih))
							psell_f1(iy,ig,is)   = 0.d0
						endif
					enddo
					!$omp end do
					!$omp end parallel
					do iy = 1, ny_neg
						Wown_f1(iy,ig,is)    = Wown_f1(ny_neg + 1,ig,is)
						locxs_f1(iy,ig,is)   = locxs_f1(ny_neg + 1,ig,is)
						xs_f1(iy,ig,is)      = xs_f1(ny_neg + 1,ig,is)
						xsratio_f1(iy,ig,is) = xsratio_f1(ny_neg + 1,ig,is)
						psell_f1(iy,ig,is)   = psell_f1(ny_neg + 1,ig,is)
					enddo
					!Households with good credit
					call omp_set_num_threads(num_threads)
					!$omp parallel private(iy,W_maxsofar,x_maxsofar,ix,ysell,iylow,iyupp,yweight,wval)
					!$omp do
					do im = 1, nm
						do iy = ny_neg + 1, ny
							W_maxsofar = -1e9
							x_maxsofar = nx
							do ix = 2, nx
								ysell   = yy(iy) - mm(im)*ph*hh(ih) + xs(xx(nx - ix + 1),hh(ih),ph)*(1 - sellcost)
								iylow   = ly(ysell)
								iyupp   = uy(ysell)
								yweight = wy(ysell)
								if (ysell >= 0.d0) then
									if (Wnosell_f0(iy,im,ig,is) - searchcost < 0.d0) then
										wval = -1e9
									else
										wval = ((1 - prob_xs(nx - ix + 1))*(Wnosell_f0(iy,im,ig,is) - searchcost)**(1 - gamma) + prob_xs(nx - ix + 1)&
										*((1 - yweight)*Wrent_f0(iylow,is)**(1 - gamma) + yweight*Wrent_f0(iyupp,is)**(1 - gamma)))**(1/(1 - gamma))
									endif
									if (wval > W_maxsofar) then
										W_maxsofar = wval
										x_maxsofar = nx - ix + 1
									endif
								else
									exit
								endif
							enddo
							if (W_maxsofar > Wnosell_f0(iy,im,ig,is)) then
								Wown_f0(iy,im,ig,is)    = W_maxsofar
								locxs_f0(iy,im,ig,is)   = x_maxsofar
								xs_f0(iy,im,ig,is)      = xs(xx(x_maxsofar),hh(ih),ph)
								xsratio_f0(iy,im,ig,is) = xs(xx(x_maxsofar),hh(ih),ph)/(ph*hh(ih))
								psell_f0(iy,im,ig,is)   = prob_xs(x_maxsofar)
							else
								Wown_f0(iy,im,ig,is)    = Wnosell_f0(iy,im,ig,is)
								locxs_f0(iy,im,ig,is)   = nx
								xs_f0(iy,im,ig,is)      = xs(xx(nx),hh(ih),ph)
								xsratio_f0(iy,im,ig,is) = xs(xx(nx),hh(ih),ph)/(ph*hh(ih))
								psell_f0(iy,im,ig,is)   = 0.d0
							endif
						enddo
						do iy = 1, ny_neg
							Wown_f0(iy,im,ig,is)    = Wown_f0(ny_neg + 1,im,ig,is)
							locxs_f0(iy,im,ig,is)   = locxs_f0(ny_neg + 1,im,ig,is)
							xs_f0(iy,im,ig,is)      = xs_f0(ny_neg + 1,im,ig,is)
							xsratio_f0(iy,im,ig,is) = xsratio_f0(ny_neg + 1,im,ig,is)
							psell_f0(iy,im,ig,is)   = psell_f0(ny_neg + 1,im,ig,is)
						enddo
					enddo
					!$omp end do
					!$omp end parallel
				enddo
			enddo
			!Consumption/savings subperiod		    
			do is = 1, ns
				!Renters
				!Continuation utilities
				call omp_set_num_threads(num_threads)
				!$omp parallel private(isprime,ieprime,yprime_f0,yprime_f1,iy0low,iy0upp,y0weight,iy1low,iy1upp,y1weight,Vcont_e_f0,Vcont_e_f1,Vcont_s_f0,Vcont_s_f1)
				!$omp do
				do ibprime = 1, nb
					do isprime = 1, ns
						do ieprime = 1, ne
							yprime_f0 = labor(isprime,ieprime)*w + bb(ibprime)
							iy0low    = ly(yprime_f0)
							iy0upp    = uy(yprime_f0)
							y0weight  = wy(yprime_f0)
							yprime_f1 = (1 - fincloss)*labor(isprime,ieprime)*w + bb(ibprime)
							iy1low    = ly(yprime_f1)
							iy1upp    = uy(yprime_f1)
							y1weight  = wy(yprime_f1)
							!Households with good credit
							Vcont_e_f0(ieprime) = (1 - y0weight)*Wrent_f0(iy0low,isprime)**(1 - gamma) + y0weight*Wrent_f0(iy0upp,isprime)**(1 - gamma)
							!Households with bad credit
							Vcont_e_f1(ieprime) = pflag*((1 - y1weight)*Wrent_f1(iy1low,isprime)**(1 - gamma) + y1weight*Wrent_f1(iy1upp,isprime)**(1 - gamma))&
							+ (1 - pflag)*((1 - y0weight)*Wrent_f0(iy0low,isprime)**(1 - gamma)	+ y0weight*Wrent_f0(iy0upp,isprime)**(1 - gamma))
						enddo
						Vcont_s_f0(isprime) = dot_product(Vcont_e_f0,emeas)
						Vcont_s_f1(isprime) = dot_product(Vcont_e_f1,emeas)
					enddo
					Vcont_rent_f0(ibprime) = dot_product(Vcont_s_f0,prob_ss(is,:))
					Vcont_rent_f1(ibprime) = dot_product(Vcont_s_f1,prob_ss(is,:))
				enddo
				!$omp end do
				!$omp end parallel
				!Optimization
				call omp_set_num_threads(num_threads)
				!$omp parallel private(V_maxsofar,b_maxsofar,c_maxsofar,hc_maxsofar,spend_maxsofar,ibprime,spending,hc,c,util,val)
				!$omp do
				do iy = ny_neg + 1, ny
					!Households with good credit
					V_maxsofar     = -1e9
					b_maxsofar     = 1
					c_maxsofar     = cshare*min_spend
					hc_maxsofar    = (1 - cshare)*min_spend/pa
					spend_maxsofar = min_spend
					do ibprime = 1, nb
						spending = min_spend + yy(iy) - qb*bb(ibprime)
						if (spending > 0.d0) then
							if (spending >= max_a*(pa + (cshare/(1 - cshare)*pa)**ies)) then
								hc = max_a
								c  = spending - pa*max_a
							else
								hc = spending/(pa + (cshare/(1 - cshare)*pa)**ies)
								c  = spending*(cshare/(1 - cshare)*pa)**ies/(pa + (cshare/(1 - cshare)*pa)**ies)
							endif
							util = (cshare*c**((ies - 1)/ies) + (1 - cshare)*hc**((ies - 1)/ies))**(ies/(ies - 1))
							!Lifetime utility
							val = ((1 - beta)*util**((1 - gamma)/theta) + beta*Vcont_rent_f0(ibprime)**(1/theta))**(theta/(1 - gamma))
							if (val > V_maxsofar) then
								V_maxsofar     = val
								b_maxsofar     = ibprime
								c_maxsofar     = c
								hc_maxsofar    = hc
								spend_maxsofar = spending
							endif
						else
							exit
						endif
					enddo
					Vnew_rent_f0(iy,is) = V_maxsofar
					locbrent_f0(iy,is)  = b_maxsofar
					brent_f0(iy,is)     = bb(b_maxsofar)
					crent_f0(iy,is)     = c_maxsofar
					hcrent_f0(iy,is)    = hc_maxsofar
					spendrent_f0(iy,is) = spend_maxsofar
					!Households with bad credit
					V_maxsofar     = -1e9
					b_maxsofar     = 1
					c_maxsofar     = cshare*min_spend
					hc_maxsofar    = (1 - cshare)*min_spend/pa
					spend_maxsofar = min_spend
					do ibprime = 1, nb
						spending = min_spend + yy(iy) - qb*bb(ibprime)
						if (spending > 0.d0) then
							if (spending >= max_a*(pa + (cshare/(1 - cshare)*pa)**ies)) then
								hc = max_a
								c  = spending - pa*max_a
							else
								hc = spending/(pa + (cshare/(1 - cshare)*pa)**ies)
								c  = spending*(cshare/(1 - cshare)*pa)**ies/(pa + (cshare/(1 - cshare)*pa)**ies)
							endif
							util = (cshare*c**((ies - 1)/ies) + (1 - cshare)*hc**((ies - 1)/ies))**(ies/(ies - 1))
							!Lifetime utility
							val = ((1 - beta)*util**((1 - gamma)/theta)	+ beta*Vcont_rent_f1(ibprime)**(1/theta))**(theta/(1 - gamma))
							if (val > V_maxsofar) then
								V_maxsofar     = val
								b_maxsofar     = ibprime
								c_maxsofar     = c
								hc_maxsofar    = hc
								spend_maxsofar = spending
							endif
						else
							exit
						endif
					enddo
					Vnew_rent_f1(iy,is) = V_maxsofar
					locbrent_f1(iy,is)  = b_maxsofar
					brent_f1(iy,is)     = bb(b_maxsofar)
					crent_f1(iy,is)     = c_maxsofar
					hcrent_f1(iy,is)    = hc_maxsofar
					spendrent_f1(iy,is) = spend_maxsofar
				enddo
				!$omp end do
				!$omp end parallel
				do iy = 1, ny_neg
					Vnew_rent_f0(iy,is) = Vnew_rent_f0(ny_neg + 1,is)
					locbrent_f0(iy,is)  = locbrent_f0(ny_neg + 1,is)
					brent_f0(iy,is)     = brent_f0(ny_neg + 1,is)
					crent_f0(iy,is)     = crent_f0(ny_neg + 1,is)
					hcrent_f0(iy,is)    = hcrent_f0(ny_neg + 1,is)
					spendrent_f0(iy,is) = spendrent_f0(ny_neg + 1,is)
					Vnew_rent_f1(iy,is) = Vnew_rent_f1(ny_neg + 1,is)
					locbrent_f1(iy,is)  = locbrent_f1(ny_neg + 1,is)
					brent_f1(iy,is)     = brent_f1(ny_neg + 1,is)
					crent_f1(iy,is)     = crent_f1(ny_neg + 1,is)
					hcrent_f1(iy,is)    = hcrent_f1(ny_neg + 1,is)
					spendrent_f1(iy,is) = spendrent_f1(ny_neg + 1,is)
				enddo
				do ig = 1, ng
					ih      = hfun(ig)
					iu      = ufun(ig)
					igshock = gfun(ih,2)
					!Homeowners with bad credit
					!Continuation utilities
					call omp_set_num_threads(num_threads)
					!$omp parallel private(isprime,ieprime,yprime_f0,yprime_f1,iy0low,iy0upp,y0weight,iy1low,iy1upp,y1weight,Vcont_e_f1,Vcont_s_f1)
					!$omp do
					do ibprime = 1, nb
						if (iu == 1 .AND. nu == 2) then
							do isprime = 1, ns
								do ieprime = 1, ne
									yprime_f0 = labor(isprime,ieprime)*w + bb(ibprime)
									iy0low    = ly(yprime_f0)
									iy0upp    = uy(yprime_f0)
									y0weight  = wy(yprime_f0)
									yprime_f1 = (1 - fincloss)*labor(isprime,ieprime)*w + bb(ibprime)
									iy1low    = ly(yprime_f1)
									iy1upp    = uy(yprime_f1)
									y1weight  = wy(yprime_f1)
									Vcont_e_f1(ieprime) = pflag*((1 - y1weight)*((1 - deprec_h)*((1 - pseparate)*Wown_f1(iy1low,ig,isprime)**(1 - gamma)&
									+ pseparate*Wown_f1(iy1low,igshock,isprime)**(1 - gamma)) + deprec_h*Wrent_f1(iy1low,isprime)**(1 - gamma)) + y1weight*((1 - deprec_h)&
									*((1 - pseparate)*Wown_f1(iy1upp,ig,isprime)**(1 - gamma) + pseparate*Wown_f1(iy1upp,igshock,isprime)**(1 - gamma))&
									+ deprec_h*Wrent_f1(iy1upp,isprime)**(1 - gamma))) + (1 - pflag)*((1 - y0weight)*((1 - deprec_h)&
									*((1 - pseparate)*Wown_f0(iy0low,1,ig,isprime)**(1 - gamma)	+ pseparate*Wown_f0(iy0low,1,igshock,isprime)**(1 - gamma))&
									+ deprec_h*Wrent_f0(iy0low,isprime)**(1 - gamma)) + y0weight*((1 - deprec_h)*((1 - pseparate)*Wown_f0(iy0upp,1,ig,isprime)**(1 - gamma)&
									+ pseparate*Wown_f0(iy0upp,1,igshock,isprime)**(1 - gamma))	+ deprec_h*Wrent_f0(iy0upp,isprime)**(1 - gamma)))
								enddo
								Vcont_s_f1(isprime) = dot_product(Vcont_e_f1,emeas)
							enddo
						else
							do isprime = 1, ns
								do ieprime = 1, ne
									yprime_f0 = labor(isprime,ieprime)*w + bb(ibprime)
									iy0low    = ly(yprime_f0)
									iy0upp    = uy(yprime_f0)
									y0weight  = wy(yprime_f0)
									yprime_f1 = (1 - fincloss)*labor(isprime,ieprime)*w + bb(ibprime)
									iy1low    = ly(yprime_f1)
									iy1upp    = uy(yprime_f1)
									y1weight  = wy(yprime_f1)
									Vcont_e_f1(ieprime) = pflag*((1 - y1weight)*((1 - deprec_h)*Wown_f1(iy1low,ig,isprime)**(1 - gamma)&
									+ deprec_h*Wrent_f1(iy1low,isprime)**(1 - gamma)) + y1weight*((1 - deprec_h)*Wown_f1(iy1upp,ig,isprime)**(1 - gamma)&
									+ deprec_h*Wrent_f1(iy1upp,isprime)**(1 - gamma))) + (1 - pflag)*((1 - y0weight)*((1 - deprec_h)*Wown_f0(iy0low,1,ig,isprime)**(1 - gamma)&
									+ deprec_h*Wrent_f0(iy0low,isprime)**(1 - gamma)) + y0weight*((1 - deprec_h)*Wown_f0(iy0upp,1,ig,isprime)**(1 - gamma)&
									+ deprec_h*Wrent_f0(iy0upp,isprime)**(1 - gamma)))
								enddo
								Vcont_s_f1(isprime) = dot_product(Vcont_e_f1,emeas)
							enddo
						endif
						Vcont_own_f1(ibprime) = dot_product(Vcont_s_f1,prob_ss(is,:))
					enddo
					!$omp end do
					!$omp end parallel
					!Optimization
					call omp_set_num_threads(num_threads)
					!$omp parallel private(V_maxsofar,b_maxsofar,c_maxsofar,hc_maxsofar,spend_maxsofar,ibprime,spending,hcstay,cstay,ustay,hcmove,cmove,umove,hc,c,util,val)
					!$omp do
					do iy = ny_neg + 1, ny
						V_maxsofar     = -1e9
						b_maxsofar     = 1
						c_maxsofar     = cshare*min_spend
						hc_maxsofar    = (1 - cshare)*min_spend/pa
						spend_maxsofar = min_spend
						do ibprime = 1, nb
							spending = min_spend + yy(iy) - qb*bb(ibprime)
							if (spending > 0.d0) then
								!Utility of occupying
								hcstay = utilown*uu(iu)*hh(ih)
								cstay  = spending
								ustay  = (cshare*cstay**((ies - 1)/ies) + (1 - cshare)*hcstay**((ies - 1)/ies))**(ies/(ies - 1))
								!Utility of renting elsewhere
								if (spending >= max_a*(pa + (cshare/(1 - cshare)*pa)**ies)) then
									hcmove = max_a
									cmove  = spending - pa*max_a
								else
									hcmove = spending/(pa + (cshare/(1 - cshare)*pa)**ies)
									cmove  = spending*(cshare/(1 - cshare)*pa)**ies/(pa + (cshare/(1 - cshare)*pa)**ies)
								endif
								umove = (cshare*cmove**((ies - 1)/ies) + (1 - cshare)*hcmove**((ies - 1)/ies))**(ies/(ies - 1))
								!Optimal occupation decision
								if (ustay >= umove) then
									util = ustay
									c    = cstay
									hc   = 0.d0
								else
									util = umove
									c    = cmove
									hc   = hcmove
								endif
								!Lifetime utility
								val = ((1 - beta)*util**((1 - gamma)/theta) + beta*Vcont_own_f1(ibprime)**(1/theta))**(theta/(1 - gamma))
								if (val > V_maxsofar) then
									V_maxsofar     = val
									b_maxsofar     = ibprime
									c_maxsofar     = c
									hc_maxsofar    = hc
									spend_maxsofar = spending
								endif
							else
								exit
							endif
						enddo
						Vnew_own_f1(iy,ig,is) = V_maxsofar
						locbown_f1(iy,ig,is)  = b_maxsofar
						bown_f1(iy,ig,is)     = bb(b_maxsofar)
						cown_f1(iy,ig,is)     = c_maxsofar
						hcown_f1(iy,ig,is)    = hc_maxsofar
						spendown_f1(iy,ig,is) = spend_maxsofar
					enddo
					!$omp end do
					!$omp end parallel
					do iy = 1, ny_neg
						Vnew_own_f1(iy,ig,is) = Vnew_own_f1(ny_neg + 1,ig,is)
						locbown_f1(iy,ig,is)  = locbown_f1(ny_neg + 1,ig,is)
						bown_f1(iy,ig,is)     = bown_f1(ny_neg + 1,ig,is)
						cown_f1(iy,ig,is)     = cown_f1(ny_neg + 1,ig,is)
						hcown_f1(iy,ig,is)    = hcown_f1(ny_neg + 1,ig,is)
						spendown_f1(iy,ig,is) = spendown_f1(ny_neg + 1,ig,is)
					enddo
					!Homeowners with good credit
					!Continuation utilities
					call omp_set_num_threads(num_threads)
					!$omp parallel private(ibprime,isprime,ieprime,yprime_f0,iy0low,iy0upp,y0weight,Vcont_e_f0,Vcont_s_f0)
					!$omp do
					do imprime = 1, nm
						do ibprime = 1, nb
							if (iu == 1 .AND. nu == 2) then
								do isprime = 1, ns
									do ieprime = 1, ne
										yprime_f0 = labor(isprime,ieprime)*w + bb(ibprime)
										iy0low    = ly(yprime_f0)
										iy0upp    = uy(yprime_f0)
										y0weight  = wy(yprime_f0)
										Vcont_e_f0(ieprime) = (1 - y0weight)*((1 - deprec_h)*((1 - pseparate)*Wown_f0(iy0low,imprime,ig,isprime)**(1 - gamma) + pseparate&
										*Wown_f0(iy0low,imprime,igshock,isprime)**(1 - gamma)) + deprec_h*Wrent_f0(iy0low,isprime)**(1 - gamma))&
										+ y0weight*((1 - deprec_h)*((1 - pseparate)*Wown_f0(iy0upp,imprime,ig,isprime)**(1 - gamma) + pseparate&
										*Wown_f0(iy0upp,imprime,igshock,isprime)**(1 - gamma)) + deprec_h*Wrent_f0(iy0upp,isprime)**(1 - gamma))
									enddo											
									Vcont_s_f0(isprime) = dot_product(Vcont_e_f0,emeas)
								enddo
							else
								do isprime = 1, ns
									do ieprime = 1, ne
										yprime_f0 = labor(isprime,ieprime)*w + bb(ibprime)
										iy0low    = ly(yprime_f0)
										iy0upp    = uy(yprime_f0)
										y0weight  = wy(yprime_f0)
										Vcont_e_f0(ieprime) = (1 - y0weight)*((1 - deprec_h)*Wown_f0(iy0low,imprime,ig,isprime)**(1 - gamma) + deprec_h&
										*Wrent_f0(iy0low,isprime)**(1 - gamma)) + y0weight*((1 - deprec_h)*Wown_f0(iy0upp,imprime,ig,isprime)**(1 - gamma) + deprec_h&
										*Wrent_f0(iy0upp,isprime)**(1 - gamma))
									enddo											
									Vcont_s_f0(isprime) = dot_product(Vcont_e_f0,emeas)
								enddo
							endif
							Vcont_own_f0(ibprime,imprime) = dot_product(Vcont_s_f0,prob_ss(is,:))
						enddo
					enddo
					!$omp end do
					!$omp end parallel
					!Optimization
					!Households without a mortgage
					call omp_set_num_threads(num_threads)
					!$omp parallel private(cash_at_hand,V_maxsofar,m_maxsofar,b_maxsofar,c_maxsofar,hc_maxsofar,spend_maxsofar,imprime,ibprime,spending,hcstay,cstay,ustay,hcmove,cmove,umove,hc,c,util,val)
					!$omp do
					do iy = 1, ny
						if (iy <= ny_neg) then
							cash_at_hand = yy(iy)*abs(ylb(ig,is))
						else
							cash_at_hand = yy(iy)
						endif
						V_maxsofar     = -1e9
						m_maxsofar     = 1
						b_maxsofar     = 1
						c_maxsofar     = cshare*min_spend
						hc_maxsofar    = (1 - cshare)*min_spend/pa
						spend_maxsofar = min_spend
						do imprime = 1, nm
							do ibprime = 1, nb
								spending = min_spend + cash_at_hand + qm0(imprime,ibprime,ig,is)*mm(imprime)*ph*hh(ih) - qb*bb(ibprime)
								if (spending > 0.d0 .AND. qm0(imprime,ibprime,ig,is)*mm(imprime) <= max_ltv) then
									!Utility of occupying
									hcstay = utilown*uu(iu)*hh(ih)
									cstay  = spending
									ustay  = (cshare*cstay**((ies - 1)/ies) + (1 - cshare)*hcstay**((ies - 1)/ies))**(ies/(ies - 1))
									!Utility of renting elsewhere
									if (spending >= max_a*(pa + (cshare/(1 - cshare)*pa)**ies)) then
										hcmove = max_a
										cmove  = spending - pa*max_a
									else
										hcmove = spending/(pa + (cshare/(1 - cshare)*pa)**ies)
										cmove  = spending*(cshare/(1 - cshare)*pa)**ies/(pa + (cshare/(1 - cshare)*pa)**ies)
									endif
									umove = (cshare*cmove**((ies - 1)/ies) + (1 - cshare)*hcmove**((ies - 1)/ies))**(ies/(ies - 1))
									!Optimal occupation decision
									if (ustay >= umove) then
										util = ustay
										c    = cstay
										hc   = 0.d0
									else
										util = umove
										c    = cmove
										hc   = hcmove
									endif
									!Lifetime utility
									val = ((1 - beta)*util**((1 - gamma)/theta)	+ beta*Vcont_own_f0(ibprime,imprime)**(1/theta))**(theta/(1 - gamma))
									if (val > V_maxsofar) then
										V_maxsofar     = val
										m_maxsofar     = imprime
										b_maxsofar     = ibprime
										c_maxsofar     = c
										hc_maxsofar    = hc
										spend_maxsofar = spending
									endif
								endif
							enddo
						enddo
						Vnew_own_f0(iy,1,ig,is) = V_maxsofar
						locm_f0(iy,1,ig,is)     = m_maxsofar
						m_f0(iy,1,ig,is)        = mm(m_maxsofar)
						locbown_f0(iy,1,ig,is)  = b_maxsofar
						bown_f0(iy,1,ig,is)     = bb(b_maxsofar)
						cown_f0(iy,1,ig,is)     = c_maxsofar
						hcown_f0(iy,1,ig,is)    = hc_maxsofar
						spendown_f0(iy,1,ig,is) = spend_maxsofar
					enddo
					!$omp end do
					!$omp end parallel
					call omp_set_num_threads(num_threads)
					!$omp parallel private(iy,V_maxsofar,b_maxsofar,c_maxsofar,hc_maxsofar,spend_maxsofar,ibprime_start,ibprime,spending,hcstay,cstay,ustay,hcmove,cmove,umove,hc,c,util,val)
					!$omp do
					do im = 2, nm
						!Households with a mortgage
						do iy = ny_neg + 1, ny
							V_maxsofar     = -1e9
							b_maxsofar     = 1
							c_maxsofar     = cshare*min_spend
							hc_maxsofar    = (1 - cshare)*min_spend/pa
							spend_maxsofar = min_spend
							if (iy == ny_neg + 1) then
								ibprime_start = 1
							else
								ibprime_start = locbown_f0(iy - 1,im,ig,is)
							endif
							do ibprime = ibprime_start, nb
								spending = min_spend + yy(iy) - qb*bb(ibprime)
								if (spending > 0.d0) then
									!Utility of occupying
									hcstay = utilown*uu(iu)*hh(ih)
									cstay  = spending
									ustay  = (cshare*cstay**((ies - 1)/ies) + (1 - cshare)*hcstay**((ies - 1)/ies))**(ies/(ies - 1))
									!Utility of renting elsewhere
									if (spending >= max_a*(pa + (cshare/(1 - cshare)*pa)**ies)) then
										hcmove = max_a
										cmove  = spending - pa*max_a
									else
										hcmove = spending/(pa + (cshare/(1 - cshare)*pa)**ies)
										cmove  = spending*(cshare/(1 - cshare)*pa)**ies/(pa + (cshare/(1 - cshare)*pa)**ies)
									endif
									umove = (cshare*cmove**((ies - 1)/ies) + (1 - cshare)*hcmove**((ies - 1)/ies))**(ies/(ies - 1))
									!Optimal occupation decision
									if (ustay >= umove) then
										util = ustay
										c    = cstay
										hc   = 0.d0
									else
										util = umove
										c    = cmove
										hc   = hcmove
									endif
									!Lifetime utility
									val = ((1 - beta)*util**((1 - gamma)/theta)	+ beta*Vcont_own_f0(ibprime,im)**(1/theta))**(theta/(1 - gamma))
									if (val > V_maxsofar) then
										V_maxsofar     = val
										b_maxsofar     = ibprime
										c_maxsofar     = c
										hc_maxsofar    = hc
										spend_maxsofar = spending
									endif
								else
									exit
								endif
							enddo
							Vnew_own_f0(iy,im,ig,is) = V_maxsofar
							locbown_f0(iy,im,ig,is)  = b_maxsofar
							bown_f0(iy,im,ig,is)     = bb(b_maxsofar)
							cown_f0(iy,im,ig,is)     = c_maxsofar
							hcown_f0(iy,im,ig,is)    = hc_maxsofar
							spendown_f0(iy,im,ig,is) = spend_maxsofar
						enddo
						do iy = 1, ny_neg
							Vnew_own_f0(iy,im,ig,is) = Vnew_own_f0(ny_neg + 1,im,ig,is)
							locbown_f0(iy,im,ig,is)  = locbown_f0(ny_neg + 1,im,ig,is)
							bown_f0(iy,im,ig,is)     = bown_f0(ny_neg + 1,im,ig,is)
							cown_f0(iy,im,ig,is)     = cown_f0(ny_neg + 1,im,ig,is)
							hcown_f0(iy,im,ig,is)    = hcown_f0(ny_neg + 1,im,ig,is)
							spendown_f0(iy,im,ig,is) = spendown_f0(ny_neg + 1,im,ig,is)
						enddo
					enddo
					!$omp end do
					!$omp end parallel
				enddo
			enddo
			err_valfn = max(maxval(abs(Vnew_own_f0 - Vown_f0)),maxval(abs(Vnew_rent_f0 - Vrent_f0)),maxval(abs(Vnew_own_f1 - Vown_f1)),maxval(abs(Vnew_rent_f1 - Vrent_f1)))
			if (err_valfn < minerr_noaccel) then
				minerr_noaccel = err_valfn
			endif
			Vown_f0  = Vnew_own_f0
			Vrent_f0 = Vnew_rent_f0
			Vown_f1  = Vnew_own_f1
			Vrent_f1 = Vnew_rent_f1
			write(*, '(a22, i4, a12, f20.6)'), '          Iteration = ' ,iter_valfn, ', err_val = ' ,err_valfn
			if (err_valfn < toler) then
				exit
			endif
		else
			!Accelerator
			!House buying subperiod
			do is = 1, ns
				call omp_set_num_threads(num_threads)
				!$omp parallel private(ih,ig,ix,ybuy,iylow,iyupp,yweight)
				!$omp do
				do iy = ny_neg + 1, ny
					!Households with good credit
					ih   = loch_f0(iy,is)
					ig   = gfun(ih,1)
					ix   = locxb_f0(iy,is)
					ybuy = yy(iy) - xb(xx(ix),hh(ih),ph)
					if (ybuy < 0.d0) then
						ybuy = ybuy/abs(ylb(ig,is))
					endif
					iylow           = ly(ybuy)
					iyupp           = uy(ybuy)
					yweight         = wy(ybuy)
					Wrent_f0(iy,is) = max(Vrent_f0(iy,is),((1 - prob_xb(ix))*Vrent_f0(iy,is)**(1 - gamma) + prob_xb(ix)*((1 - yweight)*Vown_f0(iylow,1,ig,is)**(1 - gamma)&
					+ yweight*Vown_f0(iyupp,1,ig,is)**(1 - gamma)))**(1/(1 - gamma)))
					!Households with bad credit
					ih              = loch_f1(iy,is)
					ig              = gfun(ih,1)
					ix              = locxb_f1(iy,is)
					ybuy			= yy(iy) - xb(xx(ix),hh(ih),ph)
					iylow           = ly(ybuy)
					iyupp           = uy(ybuy)
					yweight         = wy(ybuy)
					Wrent_f1(iy,is) = max(Vrent_f1(iy,is),((1 - prob_xb(ix))*Vrent_f1(iy,is)**(1 - gamma) + prob_xb(ix)*((1 - yweight)*Vown_f1(iylow,ig,is)**(1 - gamma)&
					+ yweight*Vown_f1(iyupp,ig,is)**(1 - gamma)))**(1/(1 - gamma)))
				enddo
				!$omp end do
				!$omp end parallel
				do iy = 1, ny_neg
					Wrent_f0(iy,is) = Wrent_f0(ny_neg + 1,is)
					Wrent_f1(iy,is) = Wrent_f1(ny_neg + 1,is)
				enddo
			enddo
			!Mortgage foreclosure/payment subperiod
			do is = 1, ns
				do ig = 1, ng
					ih = hfun(ig)
					!Households without a mortgage
					do iy = 1, ny
						Wnosell_f0(iy,1,ig,is) = Vown_f0(iy,1,ig,is)
					enddo
					!Households with a mortgage
					call omp_set_num_threads(num_threads)
					!$omp parallel private(iy,ydefault,iylow,iyupp,yweight,wdefault,imprime,ypay,wval)
					!$omp do
					do im = 2, nm
						do iy = ny_neg + 1, ny
							!Utility of defaulting
							ydefault = yy(iy) - max(0.d0,min(fdeficiency*yy(iy),mm(im)*ph*hh(ih) - Jreo(ih)))
							iylow    = ly(ydefault)
							iyupp    = uy(ydefault)
							yweight  = wy(ydefault)
							wdefault = (1 - yweight)*Wrent_f1(iylow,is) + yweight*Wrent_f1(iyupp,is)
							!Utility of making an optimal payment
							imprime = locm_f0(iy,im,ig,is)
							ypay    = yy(iy) - (mm(im) - qm*mm(imprime))*ph*hh(ih)
							if (ypay < 0.d0) then
								ypay = ypay/abs(ylb(ig,is))
							endif
							iylow   = ly(ypay)
							iyupp   = uy(ypay)
							yweight = wy(ypay)
							if ((imprime == 1 .AND. ypay >= -1.d0) .OR. (imprime > 1 .AND. ypay >= 0.d0)) then
								wval = (1 - yweight)*Vown_f0(iylow,imprime,ig,is) + yweight*Vown_f0(iyupp,imprime,ig,is)
								!Optimal default/payment decision
								if (wval >= wdefault) then
									Wnosell_f0(iy,im,ig,is) = wval
								else
									Wnosell_f0(iy,im,ig,is) = wdefault
								endif
							else
								Wnosell_f0(iy,im,ig,is) = wdefault
							endif
						enddo
						do iy = 1, ny_neg
							Wnosell_f0(iy,im,ig,is) = Wnosell_f0(ny_neg + 1,im,ig,is)
						enddo
					enddo
					!$omp end do
					!$omp end parallel
				enddo
			enddo
			!House selling subperiod
			do is = 1, ns
				do ig = 1, ng
					ih = hfun(ig)
					!Households with bad credit
					call omp_set_num_threads(num_threads)
					!$omp parallel private(ix,ysell,iylow,iyupp,yweight)
					!$omp do
					do iy = ny_neg + 1, ny
						ix                = locxs_f1(iy,ig,is)
						ysell             = yy(iy) + xs(xx(ix),hh(ih),ph)*(1 - sellcost)
						iylow             = ly(ysell)
						iyupp			  = uy(ysell)
						yweight           = wy(ysell)
						Wown_f1(iy,ig,is) = max(Vown_f1(iy,ig,is),((1 - prob_xs(ix))*(max(0.d0,Vown_f1(iy,ig,is) - searchcost))**(1 - gamma) + prob_xs(ix)&
						*((1 - yweight)*Wrent_f1(iylow,is)**(1 - gamma) + yweight*Wrent_f1(iyupp,is)**(1 - gamma)))**(1/(1 - gamma)))
					enddo
					!$omp end do
					!$omp end parallel
					do iy = 1, ny_neg
						Wown_f1(iy,ig,is) = Wown_f1(ny_neg + 1,ig,is)
					enddo
					!Households with good credit
					call omp_set_num_threads(num_threads)
					!$omp parallel private(iy,ix,ysell,iylow,iyupp,yweight)
					!$omp do
					do im = 1, nm
						do iy = ny_neg + 1, ny
							ix      = locxs_f0(iy,im,ig,is)
							ysell   = yy(iy) - mm(im)*ph*hh(ih) + xs(xx(ix),hh(ih),ph)*(1 - sellcost)
							iylow   = ly(ysell)
							iyupp   = uy(ysell)
							yweight = wy(ysell)
							if (ysell >= 0.d0) then
								Wown_f0(iy,im,ig,is) = max(Wnosell_f0(iy,im,ig,is),((1 - prob_xs(ix))*(max(0.d0,Wnosell_f0(iy,im,ig,is) - searchcost))**(1 - gamma)&
								+ prob_xs(ix)*((1 - yweight)*Wrent_f0(iylow,is)**(1 - gamma) + yweight*Wrent_f0(iyupp,is)**(1 - gamma)))**(1/(1 - gamma)))
							endif
						enddo
						do iy = 1, ny_neg
							Wown_f0(iy,im,ig,is) = Wown_f0(ny_neg + 1,im,ig,is)
						enddo
					enddo
					!$omp end do
					!$omp end parallel
				enddo
			enddo
			!Consumption/savings subperiod
			 do is = 1, ns
				!Renters
				call omp_set_num_threads(num_threads)
				!$omp parallel private(V_maxsofar,ibprime,spending,hc,c,util,isprime,ieprime,yprime_f0,iy0low,iy0upp,y0weight,Vcont_e_f0,Vcont_s_f0,yprime_f1,iy1low,iy1upp,y1weight,Vcont_e_f1,Vcont_s_f1,val)
				!$omp do
				do iy = ny_neg + 1, ny
					!Households with good credit
					V_maxsofar = -1e9
					ibprime    = locbrent_f0(iy,is)
					spending   = min_spend + yy(iy) - qb*bb(ibprime)
					if (spending > 0.d0) then
						if (spending >= max_a*(pa + (cshare/(1 - cshare)*pa)**ies)) then
							hc = max_a
							c  = spending - pa*max_a
						else
							hc = spending/(pa + (cshare/(1 - cshare)*pa)**ies)
							c  = spending*(cshare/(1 - cshare)*pa)**ies/(pa + (cshare/(1 - cshare)*pa)**ies)
						endif
						util = (cshare*c**((ies - 1)/ies) + (1 - cshare)*hc**((ies - 1)/ies))**(ies/(ies - 1))
						!Continuation utilities
						do isprime = 1, ns
							do ieprime = 1, ne
								yprime_f0 = labor(isprime,ieprime)*w + bb(ibprime)
								iy0low    = ly(yprime_f0)
								iy0upp    = uy(yprime_f0)
								y0weight  = wy(yprime_f0)
								!Households with good credit
								Vcont_e_f0(ieprime) = (1 - y0weight)*Wrent_f0(iy0low,isprime)**(1 - gamma) + y0weight*Wrent_f0(iy0upp,isprime)**(1 - gamma)
							enddo
							Vcont_s_f0(isprime) = dot_product(Vcont_e_f0,emeas)
						enddo
						!Lifetime utility
						val = ((1 - beta)*util**((1 - gamma)/theta) + beta*dot_product(Vcont_s_f0,prob_ss(is,:))**(1/theta))**(theta/(1 - gamma))
						if (val > V_maxsofar) then
							Vnew_rent_f0(iy,is) = val
						endif
					endif
					!Households with bad credit
					V_maxsofar = -1e9
					ibprime    = locbrent_f1(iy,is)
					spending   = min_spend + yy(iy) - qb*bb(ibprime)
					if (spending > 0.d0) then
						if (spending >= max_a*(pa + (cshare/(1 - cshare)*pa)**ies)) then
							hc = max_a
							c  = spending - pa*max_a
						else
							hc = spending/(pa + (cshare/(1 - cshare)*pa)**ies)
							c  = spending*(cshare/(1 - cshare)*pa)**ies/(pa + (cshare/(1 - cshare)*pa)**ies)
						endif
						util = (cshare*c**((ies - 1)/ies) + (1 - cshare)*hc**((ies - 1)/ies))**(ies/(ies - 1))
						!Continuation utilities
						do isprime = 1, ns
							do ieprime = 1, ne
								yprime_f0 = labor(isprime,ieprime)*w + bb(ibprime)
								iy0low    = ly(yprime_f0)
								iy0upp    = uy(yprime_f0)
								y0weight  = wy(yprime_f0)
								yprime_f1 = (1 - fincloss)*labor(isprime,ieprime)*w + bb(ibprime)
								iy1low    = ly(yprime_f1)
								iy1upp    = uy(yprime_f1)
								y1weight  = wy(yprime_f1)
								!Households with bad credit
								Vcont_e_f1(ieprime) = pflag*((1 - y1weight)*Wrent_f1(iy1low,isprime)**(1 - gamma) + y1weight*Wrent_f1(iy1upp,isprime)**(1 - gamma))&
								+ (1 - pflag)*((1 - y0weight)*Wrent_f0(iy0low,isprime)**(1 - gamma) + y0weight*Wrent_f0(iy0upp,isprime)**(1 - gamma))
							enddo
							Vcont_s_f1(isprime) = dot_product(Vcont_e_f1,emeas)
						enddo
						!Lifetime utility
						val = ((1 - beta)*util**((1 - gamma)/theta) + beta*dot_product(Vcont_s_f1,prob_ss(is,:))**(1/theta))**(theta/(1 - gamma))
						if (val > V_maxsofar) then
							Vnew_rent_f1(iy,is) = val
						endif
					endif
				enddo
				!$omp end do
				!$omp end parallel
				do iy = 1, ny_neg
					Vnew_rent_f0(iy,is) = Vnew_rent_f0(ny_neg + 1,is)
					Vnew_rent_f1(iy,is) = Vnew_rent_f1(ny_neg + 1,is)
				enddo
				do ig = 1, ng
					ih      = hfun(ig)
					iu      = ufun(ig)
					igshock = gfun(ih,2)
					!Homeowners with bad credit
					call omp_set_num_threads(num_threads)
					!$omp parallel private(V_maxsofar,ibprime,spending,hcstay,cstay,ustay,hcmove,cmove,umove,hc,c,util,isprime,ieprime,yprime_f0,yprime_f1,iy0low,iy0upp,y0weight,iy1low,iy1upp,y1weight,Vcont_e_f1,Vcont_s_f1,val)
					!$omp do
					do iy = ny_neg + 1, ny
						V_maxsofar = -1e9
						ibprime    = locbown_f1(iy,ig,is)
						spending   = min_spend + yy(iy) - qb*bb(ibprime)
						if (spending > 0.d0) then
							!Utility of occupying
							hcstay = utilown*uu(iu)*hh(ih)
							cstay  = spending
							ustay  = (cshare*cstay**((ies - 1)/ies) + (1 - cshare)*hcstay**((ies - 1)/ies))**(ies/(ies - 1))
							!Utility of renting elsewhere
							if (spending >= max_a*(pa + (cshare/(1 - cshare)*pa)**ies)) then
								hcmove = max_a
								cmove  = spending - pa*max_a
							else
								hcmove = spending/(pa + (cshare/(1 - cshare)*pa)**ies)
								cmove  = spending*(cshare/(1 - cshare)*pa)**ies/(pa + (cshare/(1 - cshare)*pa)**ies)
							endif
							umove = (cshare*cmove**((ies - 1)/ies) + (1 - cshare)*hcmove**((ies - 1)/ies))**(ies/(ies - 1))
							!Optimal occupation decision
							if (ustay >= umove) then
								util = ustay
								c    = cstay
								hc   = 0.d0
							else
								util = umove
								c    = cmove
								hc   = hcmove
							endif
							!Continuation utilities
							if (iu == 1 .AND. nu == 2) then
								do isprime = 1, ns
									do ieprime = 1, ne
										yprime_f0 = labor(isprime,ieprime)*w + bb(ibprime)
										iy0low    = ly(yprime_f0)
										iy0upp    = uy(yprime_f0)
										y0weight  = wy(yprime_f0)
										yprime_f1 = (1 - fincloss)*labor(isprime,ieprime)*w + bb(ibprime)
										iy1low    = ly(yprime_f1)
										iy1upp    = uy(yprime_f1)
										y1weight  = wy(yprime_f1)
										Vcont_e_f1(ieprime) = pflag*((1 - y1weight)*((1 - deprec_h)*((1 - pseparate)*Wown_f1(iy1low,ig,isprime)**(1 - gamma)&
										+ pseparate*Wown_f1(iy1low,igshock,isprime)**(1 - gamma)) + deprec_h*Wrent_f1(iy1low,isprime)**(1 - gamma)) + y1weight*((1 - deprec_h)&
										*((1 - pseparate)*Wown_f1(iy1upp,ig,isprime)**(1 - gamma) + pseparate*Wown_f1(iy1upp,igshock,isprime)**(1 - gamma))&
										+ deprec_h*Wrent_f1(iy1upp,isprime)**(1 - gamma))) + (1 - pflag)*((1 - y0weight)*((1 - deprec_h)&
										*((1 - pseparate)*Wown_f0(iy0low,1,ig,isprime)**(1 - gamma)	+ pseparate*Wown_f0(iy0low,1,igshock,isprime)**(1 - gamma))&
										+ deprec_h*Wrent_f0(iy0low,isprime)**(1 - gamma)) + y0weight*((1 - deprec_h)*((1 - pseparate)*Wown_f0(iy0upp,1,ig,isprime)**(1 - gamma)&
										+ pseparate*Wown_f0(iy0upp,1,igshock,isprime)**(1 - gamma)) + deprec_h*Wrent_f0(iy0upp,isprime)**(1 - gamma)))
									enddo
									Vcont_s_f1(isprime) = dot_product(Vcont_e_f1,emeas)
								enddo
							else
								do isprime = 1, ns
									do ieprime = 1, ne
										yprime_f0 = labor(isprime,ieprime)*w + bb(ibprime)
										iy0low    = ly(yprime_f0)
										iy0upp    = uy(yprime_f0)
										y0weight  = wy(yprime_f0)
										yprime_f1 = (1 - fincloss)*labor(isprime,ieprime)*w + bb(ibprime)
										iy1low    = ly(yprime_f1)
										iy1upp    = uy(yprime_f1)
										y1weight  = wy(yprime_f1)
										Vcont_e_f1(ieprime) = pflag*((1 - y1weight)*((1 - deprec_h)*Wown_f1(iy1low,ig,isprime)**(1 - gamma)&
										+ deprec_h*Wrent_f1(iy1low,isprime)**(1 - gamma)) + y1weight*((1 - deprec_h)*Wown_f1(iy1upp,ig,isprime)**(1 - gamma) + deprec_h&
										*Wrent_f1(iy1upp,isprime)**(1 - gamma))) + (1 - pflag)*((1 - y0weight)*((1 - deprec_h)*Wown_f0(iy0low,1,ig,isprime)**(1 - gamma)&
										+ deprec_h*Wrent_f0(iy0low,isprime)**(1 - gamma)) + y0weight*((1 - deprec_h)*Wown_f0(iy0upp,1,ig,isprime)**(1 - gamma) + deprec_h&
										*Wrent_f0(iy0upp,isprime)**(1 - gamma)))
									enddo
									Vcont_s_f1(isprime) = dot_product(Vcont_e_f1,emeas)
								enddo
							endif
							!Lifetime utility
							val = ((1 - beta)*util**((1 - gamma)/theta) + beta*dot_product(Vcont_s_f1,prob_ss(is,:))**(1/theta))**(theta/(1 - gamma))
							if (val > V_maxsofar) then
								Vnew_own_f1(iy,ig,is) = val
							endif
						endif
					enddo
					!$omp end do
					!$omp end parallel
					do iy = 1, ny_neg
						Vnew_own_f1(iy,ig,is) = Vnew_own_f1(ny_neg + 1,ig,is)
					enddo
					!Homeowners with good credit
					!Households without a mortgage
					call omp_set_num_threads(num_threads)
					!$omp parallel private(cash_at_hand,V_maxsofar,imprime,ibprime,spending,hcstay,cstay,ustay,hcmove,cmove,umove,hc,c,util,isprime,ieprime,yprime_f0,iy0low,iy0upp,y0weight,Vcont_e_f0,Vcont_s_f0,val)
					!$omp do
					do iy = 1, ny
						if (iy <= ny_neg) then
							cash_at_hand = yy(iy)*abs(ylb(ig,is))
						else
							cash_at_hand = yy(iy)
						endif
						V_maxsofar = -1e9
						imprime    = locm_f0(iy,1,ig,is)
						ibprime    = locbown_f0(iy,1,ig,is)
						spending   = min_spend + cash_at_hand + qm0(imprime,ibprime,ig,is)*mm(imprime)*ph*hh(ih)&
						- qb*bb(ibprime)
						if (spending > 0.d0) then
							!Utility of occupying
							hcstay = utilown*uu(iu)*hh(ih)
							cstay  = spending
							ustay  = (cshare*cstay**((ies - 1)/ies) + (1 - cshare)*hcstay**((ies - 1)/ies))**(ies/(ies - 1))
							!Utility of renting elsewhere
							if (spending >= max_a*(pa + (cshare/(1 - cshare)*pa)**ies)) then
								hcmove = max_a
								cmove  = spending - pa*max_a
							else
								hcmove = spending/(pa + (cshare/(1 - cshare)*pa)**ies)
								cmove  = spending*(cshare/(1 - cshare)*pa)**ies/(pa + (cshare/(1 - cshare)*pa)**ies)
							endif
							umove = (cshare*cmove**((ies - 1)/ies) + (1 - cshare)*hcmove**((ies - 1)/ies))**(ies/(ies - 1))							    
							!Optimal occupation decision
							if (ustay >= umove) then
								util = ustay
								c    = cstay
								hc   = 0.d0
							else
								util = umove
								c    = cmove
								hc   = hcmove
							endif
							!Continuation utilities
							if (iu == 1 .AND. nu == 2) then
								do isprime = 1, ns
									do ieprime = 1, ne
										yprime_f0 = labor(isprime,ieprime)*w + bb(ibprime)
										iy0low    = ly(yprime_f0)
										iy0upp    = uy(yprime_f0)
										y0weight  = wy(yprime_f0)
										Vcont_e_f0(ieprime) = (1 - y0weight)*((1 - deprec_h)*((1 - pseparate)*Wown_f0(iy0low,imprime,ig,isprime)**(1 - gamma) + pseparate&
										*Wown_f0(iy0low,imprime,igshock,isprime)**(1 - gamma)) + deprec_h*Wrent_f0(iy0low,isprime)**(1 - gamma))&
										+ y0weight*((1 - deprec_h)*((1 - pseparate)*Wown_f0(iy0upp,imprime,ig,isprime)**(1 - gamma) + pseparate&
										*Wown_f0(iy0upp,imprime,igshock,isprime)**(1 - gamma)) + deprec_h*Wrent_f0(iy0upp,isprime)**(1 - gamma))
									enddo											
									Vcont_s_f0(isprime) = dot_product(Vcont_e_f0,emeas)
								enddo
							else
								do isprime = 1, ns
									do ieprime = 1, ne
										yprime_f0 = labor(isprime,ieprime)*w + bb(ibprime)
										iy0low    = ly(yprime_f0)
										iy0upp    = uy(yprime_f0)
										y0weight  = wy(yprime_f0)
										Vcont_e_f0(ieprime) = (1 - y0weight)*((1 - deprec_h)*Wown_f0(iy0low,imprime,ig,isprime)**(1 - gamma) + deprec_h&
										*Wrent_f0(iy0low,isprime)**(1 - gamma))	+ y0weight*((1 - deprec_h)*Wown_f0(iy0upp,imprime,ig,isprime)**(1 - gamma) + deprec_h&
										*Wrent_f0(iy0upp,isprime)**(1 - gamma))
									enddo											
									Vcont_s_f0(isprime) = dot_product(Vcont_e_f0,emeas)
								enddo
							endif
							!Lifetime utility
							val = ((1 - beta)*util**((1 - gamma)/theta) + beta*dot_product(Vcont_s_f0,prob_ss(is,:))**(1/theta))**(theta/(1 - gamma))
							if (val > V_maxsofar) then
								Vnew_own_f0(iy,1,ig,is) = val
							endif
						endif
					enddo
					!$omp end do
					!$omp end parallel
					!Households with a mortgage
					call omp_set_num_threads(num_threads)
					!$omp parallel private(iy,V_maxsofar,ibprime,spending,hcstay,cstay,ustay,hcmove,cmove,umove,hc,c,util,isprime,ieprime,yprime_f0,iy0low,iy0upp,y0weight,Vcont_e_f0,Vcont_s_f0,val)
					!$omp do
					do im = 2, nm
						do iy = ny_neg + 1, ny
							V_maxsofar = -1e9
							ibprime    = locbown_f0(iy,im,ig,is)
							spending   = min_spend + yy(iy) - qb*bb(ibprime)
							if (spending > 0.d0) then
								!Utility of occupying
								hcstay = utilown*uu(iu)*hh(ih)
								cstay  = spending
								ustay  = (cshare*cstay**((ies - 1)/ies) + (1 - cshare)*hcstay**((ies - 1)/ies))**(ies/(ies - 1))
								!Utility of renting elsewhere
								if (spending >= max_a*(pa + (cshare/(1 - cshare)*pa)**ies)) then
									hcmove = max_a
									cmove  = spending - pa*max_a
								else
									hcmove = spending/(pa + (cshare/(1 - cshare)*pa)**ies)
									cmove  = spending*(cshare/(1 - cshare)*pa)**ies/(pa + (cshare/(1 - cshare)*pa)**ies)
								endif
								umove = (cshare*cmove**((ies - 1)/ies) + (1 - cshare)*hcmove**((ies - 1)/ies))**(ies/(ies - 1))								    
								!Optimal occupation decision
								if (ustay >= umove) then
									util = ustay
									c    = cstay
									hc   = 0.d0
								else
									util = umove
									c    = cmove
									hc   = hcmove
								endif
								!Continuation utilities
								if (iu == 1 .AND. nu == 2) then
									do isprime = 1, ns
										do ieprime = 1, ne
											yprime_f0 = labor(isprime,ieprime)*w + bb(ibprime)
											iy0low    = ly(yprime_f0)
											iy0upp    = uy(yprime_f0)
											y0weight  = wy(yprime_f0)
											Vcont_e_f0(ieprime) = (1 - y0weight)*((1 - deprec_h)*((1 - pseparate)*Wown_f0(iy0low,im,ig,isprime)**(1 - gamma) + pseparate&
											*Wown_f0(iy0low,im,igshock,isprime)**(1 - gamma)) + deprec_h*Wrent_f0(iy0low,isprime)**(1 - gamma))&
											+ y0weight*((1 - deprec_h)*((1 - pseparate)*Wown_f0(iy0upp,im,ig,isprime)**(1 - gamma) + pseparate&
											*Wown_f0(iy0upp,im,igshock,isprime)**(1 - gamma)) + deprec_h*Wrent_f0(iy0upp,isprime)**(1 - gamma))
										enddo											
										Vcont_s_f0(isprime) = dot_product(Vcont_e_f0,emeas)
									enddo
								else
									do isprime = 1, ns
										do ieprime = 1, ne
											yprime_f0 = labor(isprime,ieprime)*w + bb(ibprime)
											iy0low    = ly(yprime_f0)
											iy0upp    = uy(yprime_f0)
											y0weight  = wy(yprime_f0)
											Vcont_e_f0(ieprime) = (1 - y0weight)*((1 - deprec_h)*Wown_f0(iy0low,im,ig,isprime)**(1 - gamma)&
											+ deprec_h*Wrent_f0(iy0low,isprime)**(1 - gamma)) + y0weight*((1 - deprec_h)&
											*Wown_f0(iy0upp,im,ig,isprime)**(1 - gamma)	+ deprec_h*Wrent_f0(iy0upp,isprime)**(1 - gamma))
										enddo											
										Vcont_s_f0(isprime) = dot_product(Vcont_e_f0,emeas)
									enddo
								endif
								!Lifetime utility
								val = ((1 - beta)*util**((1 - gamma)/theta) + beta*dot_product(Vcont_s_f0,prob_ss(is,:))**(1/theta))**(theta/(1 - gamma))
								if (val > V_maxsofar) then
									Vnew_own_f0(iy,im,ig,is) = val
								endif
							endif
						enddo
						do iy = 1, ny_neg
							Vnew_own_f0(iy,im,ig,is) = Vnew_own_f0(ny_neg + 1,im,ig,is)
						enddo
					enddo
					!$omp end do
					!$omp end parallel
				enddo
			enddo
			err_valfn = max(maxval(abs(Vnew_own_f0 - Vown_f0)),maxval(abs(Vnew_rent_f0 - Vrent_f0)),maxval(abs(Vnew_own_f1 - Vown_f1)),maxval(abs(Vnew_rent_f1 - Vrent_f1)))
			if (err_valfn < minerr_accel) then
				minerr_accel = err_valfn
			endif
			Vown_f0  = Vnew_own_f0
			Vrent_f0 = Vnew_rent_f0
			Vown_f1  = Vnew_own_f1
			Vrent_f1 = Vnew_rent_f1
!			write(*, '(a22, i4, a12, f20.6)'), '          Iteration = ' ,iter_valfn, ', err_val = ' ,err_valfn
		endif
	enddo
	
	!Compute optimal bond holdings as of the subperiod 1 state for households with good credit
	do is = 1, ns
		do ig = 1, ng
			ih = hfun(ig)
			!Households without a mortgage
			do iy = ny_neg + 1, ny
				bown_f0_sp1(iy,1,ig,is)    = bown_f0(iy,1,ig,is)
				locbown_f0_sp1(iy,1,ig,is) = locbown_f0(iy,1,ig,is)
			enddo
			do iy = 1, ny_neg
				bown_f0_sp1(iy,1,ig,is)	   = bown_f0_sp1(ny_neg + 1,1,ig,is)
				locbown_f0_sp1(iy,1,ig,is) = locbown_f0_sp1(ny_neg + 1,1,ig,is)
			enddo
			!Households with a mortgage
			do im = 2, nm
				do iy = ny_neg + 1, ny
					imprime = locm_f0(iy,im,ig,is)
					ypay    = yy(iy) - (mm(im) - qm*mm(imprime))*ph*hh(ih)
					if (ypay < 0.d0) then
						ypay = ypay/abs(ylb(ig,is))
					endif
					iylow   = ly(ypay)
					iyupp   = uy(ypay)
					yweight = wy(ypay)
					!Bond holdings as of the subperiod 1 state
					bown_f0_sp1(iy,im,ig,is) = (1 - yweight)*bown_f0(iylow,imprime,ig,is) + yweight*bown_f0(iyupp,imprime,ig,is)
					if (yweight >= 0.5d0) then
						locbown_f0_sp1(iy,im,ig,is) = locbown_f0(iyupp,imprime,ig,is)
					else
						locbown_f0_sp1(iy,im,ig,is) = locbown_f0(iylow,imprime,ig,is)
					endif
				enddo
				do iy = 1, ny_neg
					bown_f0_sp1(iy,im,ig,is)	= bown_f0_sp1(ny_neg + 1,im,ig,is)
					locbown_f0_sp1(iy,im,ig,is) = locbown_f0_sp1(ny_neg + 1,im,ig,is)
				enddo
			enddo
		enddo
	enddo

end subroutine sub_valfn_ez_ces