module mod_krusellsmith_regress

	use mod_globvar
	use mkl95_LAPACK
    use mod_matlab

	implicit none
	
	!This module contains a function for computing restricted or unrestricted OLS
	
contains

subroutine sub_krusellsmith_regress(y_t,x1_t,x2_t,locz_t,locz_tprime,version,coeff,Rsquared)
    
    implicit none
    
    !Input/output variables
    real(8), dimension(1:), intent(in) :: y_t, x1_t, x2_t
    integer, dimension(1:), intent(in) :: locz_t, locz_tprime
    integer, intent(in) :: version
    real(8), dimension(1,3,nz,nz) :: coeff
    real(8) :: Rsquared
    
    !Internal variables
    integer :: tmax, it, iz, izprime, numvar
    real(8), dimension(:), allocatable :: Ydata, Ypredict, Ymeans, coeffmod
    real(8), dimension(:,:), allocatable :: Xdata
    
    !Length of data series
    tmax = size(y_t)
	
	!Number of regressors (excluding constant)
	numvar = 2
    
    if ((tmax .NE. size(x1_t)) .OR. (tmax .NE. size(x2_t)) .OR. (tmax .NE. size(locz_t)) .OR. (tmax .NE. size(locz_tprime))) then
		print *, 'Unbalanced time series!'
		stop
	endif
    
    !Define matrix of independent variables
	if (version == 1) then
		!Version 1: y = a0(z,z'<>=z) + a1(z,z'<>=z)x1 + a2(z,z'<>=z)x2
		allocate(Xdata(tmax,(numvar + 1)*(4 + 3*(nz - 2))))
		Xdata = 0.d0
		do it = 1, tmax
			if (locz_t(it) == 1) then
				if (locz_tprime(it) == 1) then
					Xdata(it,1)                      = 1
					Xdata(it,4 + 3*(nz - 2) + 1)     = x1_t(it)
					Xdata(it,2*(4 + 3*(nz - 2)) + 1) = x2_t(it)
				else
					Xdata(it,2)                      = 1
					Xdata(it,4 + 3*(nz - 2) + 2)     = x1_t(it)
					Xdata(it,2*(4 + 3*(nz - 2)) + 2) = x2_t(it)
				endif
			elseif (locz_t(it) > 1 .AND. locz_t(it) < nz) then
				if (locz_tprime(it) < locz_t(it)) then
					Xdata(it,2 + 3*(locz_t(it) - 2) + 1)					  = 1
					Xdata(it,4 + 3*(nz - 2) + 2 + 3*(locz_t(it) - 2) + 1)     = x1_t(it)
					Xdata(it,2*(4 + 3*(nz - 2)) + 2 + 3*(locz_t(it) - 2) + 1) = x2_t(it)
				elseif (locz_tprime(it) == locz_t(it)) then
	                Xdata(it,2 + 3*(locz_t(it) - 2) + 2)                      = 1
					Xdata(it,4 + 3*(nz - 2) + 2 + 3*(locz_t(it) - 2) + 2)     = x1_t(it)
					Xdata(it,2*(4 + 3*(nz - 2)) + 2 + 3*(locz_t(it) - 2) + 2) = x2_t(it)
				else
	                Xdata(it,2 + 3*(locz_t(it) - 2) + 3)					  = 1
					Xdata(it,4 + 3*(nz - 2) + 2 + 3*(locz_t(it) - 2) + 3)     = x1_t(it)
					Xdata(it,2*(4 + 3*(nz - 2)) + 2 + 3*(locz_t(it) - 2) + 3) = x2_t(it)
				endif
			else
	            if (locz_tprime(it) < nz) then
					Xdata(it,2 + 3*(nz - 2) + 1)                      = 1
					Xdata(it,4 + 3*(nz - 2) + 2 + 3*(nz - 2) + 1)     = x1_t(it)
					Xdata(it,2*(4 + 3*(nz - 2)) + 2 + 3*(nz - 2) + 1) = x2_t(it)
				else
	                Xdata(it,2 + 3*(nz - 2) + 2)                      = 1
					Xdata(it,4 + 3*(nz - 2) + 2 + 3*(nz - 2) + 2)     = x1_t(it)
					Xdata(it,2*(4 + 3*(nz - 2)) + 2 + 3*(nz - 2) + 2) = x2_t(it)
				endif
			endif
		enddo
	elseif (version == 2) then
		!Version 2: y = a0(z,z'<>=z) + a1(z)x1 + a2(z)x2
		allocate(Xdata(tmax,4 + 3*(nz - 2) + numvar*nz))
		Xdata = 0.d0
		do it = 1, tmax
			Xdata(it,4 + 3*(nz - 2) + locz_t(it))        = x1_t(it)
			Xdata(it,4 + 3*(nz - 2) + nz + locz_t(it))   = x2_t(it)
			if (locz_t(it) == 1) then
				if (locz_tprime(it) == 1) then
					Xdata(it,1)	= 1
				else
					Xdata(it,2) = 1
				endif
			elseif (locz_t(it) > 1 .AND. locz_t(it) < nz) then
				if (locz_tprime(it) < locz_t(it)) then
					Xdata(it,2 + 3*(locz_t(it) - 2) + 1) = 1
				elseif (locz_tprime(it) == locz_t(it)) then
	                Xdata(it,2 + 3*(locz_t(it) - 2) + 2) = 1
				else
	                Xdata(it,2 + 3*(locz_t(it) - 2) + 3) = 1
				endif
			else
	            if (locz_tprime(it) < nz) then
					Xdata(it,2 + 3*(nz - 2) + 1) = 1
				else
	                Xdata(it,2 + 3*(nz - 2) + 2) = 1
				endif
			endif
		enddo
	elseif (version == 3) then
		!Version 3: y = a0(z) + a1(z)x1 + a2(z)x2
		allocate(Xdata(tmax,(numvar + 1)*nz))
		Xdata = 0.d0
		do it = 1, tmax
			Xdata(it,locz_t(it))	    = 1
			Xdata(it,nz + locz_t(it))   = x1_t(it)
			Xdata(it,2*nz + locz_t(it)) = x2_t(it)
		enddo
	elseif (version == 4) then
		!Version 4: y = a0(z,z'<>=z) + a1(z,z'<>=z)x_1
		allocate(Xdata(tmax,2*(4 + 3*(nz - 2))))
		Xdata = 0.d0
		do it = 1, tmax
			if (locz_t(it) == 1) then
				if (locz_tprime(it) == 1) then
					Xdata(it,1)                  = 1
					Xdata(it,4 + 3*(nz - 2) + 1) = x1_t(it)
				else
					Xdata(it,2)                  = 1
					Xdata(it,4 + 3*(nz - 2) + 2) = x1_t(it)
				endif
			elseif (locz_t(it) > 1 .AND. locz_t(it) < nz) then
				if (locz_tprime(it) < locz_t(it)) then
					Xdata(it,2 + 3*(locz_t(it) - 2) + 1)				  = 1
					Xdata(it,4 + 3*(nz - 2) + 2 + 3*(locz_t(it) - 2) + 1) = x1_t(it)
				elseif (locz_tprime(it) == locz_t(it)) then
	                Xdata(it,2 + 3*(locz_t(it) - 2) + 2)                  = 1
					Xdata(it,4 + 3*(nz - 2) + 2 + 3*(locz_t(it) - 2) + 2) = x1_t(it)
				else
	                Xdata(it,2 + 3*(locz_t(it) - 2) + 3)				  = 1
					Xdata(it,4 + 3*(nz - 2) + 2 + 3*(locz_t(it) - 2) + 3) = x1_t(it)
				endif
			else
	            if (locz_tprime(it) < nz) then
					Xdata(it,2 + 3*(nz - 2) + 1)                  = 1
					Xdata(it,4 + 3*(nz - 2) + 2 + 3*(nz - 2) + 1) = x1_t(it)
				else
	                Xdata(it,2 + 3*(nz - 2) + 2)                  = 1
					Xdata(it,4 + 3*(nz - 2) + 2 + 3*(nz - 2) + 2) = x1_t(it)
				endif
			endif
		enddo
	elseif (version == 5) then
		!Version 5: y = a0(z,z'<>=z) + a1(z)x_1
		allocate(Xdata(tmax,4 + 3*(nz - 2) + nz))
		Xdata = 0.d0
		do it = 1, tmax
			Xdata(it,4 + 3*(nz - 2) + locz_t(it)) = x1_t(it)
			if (locz_t(it) == 1) then
				if (locz_tprime(it) == 1) then
					Xdata(it,1)	= 1
				else
					Xdata(it,2) = 1
				endif
			elseif (locz_t(it) > 1 .AND. locz_t(it) < nz) then
				if (locz_tprime(it) < locz_t(it)) then
					Xdata(it,2 + 3*(locz_t(it) - 2) + 1) = 1
				elseif (locz_tprime(it) == locz_t(it)) then
	                Xdata(it,2 + 3*(locz_t(it) - 2) + 2) = 1
				else
	                Xdata(it,2 + 3*(locz_t(it) - 2) + 3) = 1
				endif
			else
	            if (locz_tprime(it) < nz) then
					Xdata(it,2 + 3*(nz - 2) + 1) = 1
				else
	                Xdata(it,2 + 3*(nz - 2) + 2) = 1
				endif
			endif
		enddo
	elseif (version == 6) then
		!Version 6: y = a0(z) + a1(z)x_1
		allocate(Xdata(tmax,2*nz))
		Xdata = 0.d0
		do it = 1, tmax
			Xdata(it,locz_t(it))	  = 1
			Xdata(it,nz + locz_t(it)) = x1_t(it)
		enddo
	endif
	
	!Compute OLS coefficients
	allocate(Ydata(tmax))
	allocate(coeffmod(size(Xdata,2)))
    Ydata    = y_t
	coeffmod = regress(Ydata,Xdata)
	
	!Compute R^2 statistic
	allocate(Ypredict(tmax))
	allocate(Ymeans(tmax))
	Ypredict = matmul(Xdata,coeffmod)
	Ymeans   = sum(Ydata)/real(tmax,8)
	Rsquared = 1 - dot_product(Ydata - Ypredict,Ydata - Ypredict)/dot_product(Ydata - Ymeans,Ydata - Ymeans)
	
	!Fill in missing coefficients
	if (version == 1) then
		!Version 1: y = a0(z,z'<>=z) + a1(z,z'<>=z)x1 + a2(z,z'<>=z)x2
		do iz = 1, nz
			do izprime = 1, nz
				if (iz == 1) then
					if (izprime == 1) then
						coeff(1,1,iz,izprime) = coeffmod(1)
						coeff(1,2,iz,izprime) = coeffmod(4 + 3*(nz - 2) + 1)
						coeff(1,3,iz,izprime) = coeffmod(2*(4 + 3*(nz - 2)) + 1)
					else
						coeff(1,1,iz,izprime) = coeffmod(2)
						coeff(1,2,iz,izprime) = coeffmod(4 + 3*(nz - 2) + 2)
						coeff(1,3,iz,izprime) = coeffmod(2*(4 + 3*(nz - 2)) + 2)
					endif
				elseif (iz > 1 .AND. iz < nz) then
					if (izprime < iz) then
						coeff(1,1,iz,izprime) = coeffmod(2 + 3*(iz - 2) + 1)
						coeff(1,2,iz,izprime) = coeffmod(4 + 3*(nz - 2) + 2 + 3*(iz - 2) + 1)
						coeff(1,3,iz,izprime) = coeffmod(2*(4 + 3*(nz - 2)) + 2 + 3*(iz - 2) + 1)
					elseif (izprime == iz) then
						coeff(1,1,iz,izprime) = coeffmod(2 + 3*(iz - 2) + 2)
						coeff(1,2,iz,izprime) = coeffmod(4 + 3*(nz - 2) + 2 + 3*(iz - 2) + 2)
						coeff(1,3,iz,izprime) = coeffmod(2*(4 + 3*(nz - 2)) + 2 + 3*(iz - 2) + 2)
					else
		                coeff(1,1,iz,izprime) = coeffmod(2 + 3*(iz - 2) + 3)
						coeff(1,2,iz,izprime) = coeffmod(4 + 3*(nz - 2) + 2 + 3*(iz - 2) + 3)
						coeff(1,3,iz,izprime) = coeffmod(2*(4 + 3*(nz - 2)) + 2 + 3*(iz - 2) + 3)
					endif
				else
		            if (izprime < nz) then
						coeff(1,1,iz,izprime) = coeffmod(2 + 3*(nz - 2) + 1)
						coeff(1,2,iz,izprime) = coeffmod(4 + 3*(nz - 2) + 2 + 3*(nz - 2) + 1)
						coeff(1,3,iz,izprime) = coeffmod(2*(4 + 3*(nz - 2)) + 2 + 3*(nz - 2) + 1)
					else
		                coeff(1,1,iz,izprime) = coeffmod(2 + 3*(nz - 2) + 2)
						coeff(1,2,iz,izprime) = coeffmod(4 + 3*(nz - 2) + 2 + 3*(nz - 2) + 2)
						coeff(1,3,iz,izprime) = coeffmod(2*(4 + 3*(nz - 2)) + 2 + 3*(nz - 2) + 2)						
					endif
				endif
			enddo
		enddo
	elseif (version == 2) then
		!Version 2: y = a0(z,z'<>=z) + a1(z)x1 + a2(z)x2
		do iz = 1, nz
			do izprime = 1, nz
				coeff(1,2,iz,izprime) = coeffmod(4 + 3*(nz - 2) + iz)
				coeff(1,3,iz,izprime) = coeffmod(4 + 3*(nz - 2) + nz + iz)
				if (iz == 1) then
					if (izprime == 1) then
						coeff(1,1,iz,izprime) = coeffmod(1)
					else
						coeff(1,1,iz,izprime) = coeffmod(2)
					endif
				elseif (iz > 1 .AND. iz < nz) then
					if (izprime < iz) then
						coeff(1,1,iz,izprime) = coeffmod(2 + 3*(iz - 2) + 1)
					elseif (izprime == iz) then
		                coeff(1,1,iz,izprime) = coeffmod(2 + 3*(iz - 2) + 2)
					else
						coeff(1,1,iz,izprime) = coeffmod(2 + 3*(iz - 2) + 3)
					endif
				else
		            if (izprime < nz) then
						coeff(1,1,iz,izprime) = coeffmod(2 + 3*(nz - 2) + 1)
					else
		                coeff(1,1,iz,izprime) = coeffmod(2 + 3*(nz - 2) + 2)
					endif
				endif
			enddo
		enddo
	elseif (version == 3) then
		!Version 3: y = a0(z) + a1(z)x1 + a2(z)x2
		do iz = 1, nz
			do izprime = 1, nz
				coeff(1,1,iz,izprime) = coeffmod(iz)
				coeff(1,2,iz,izprime) = coeffmod(nz + iz)
				coeff(1,3,iz,izprime) = coeffmod(2*nz + iz)
			enddo
		enddo
	elseif (version == 4) then
		!Version 4: y = a0(z,z'<>=z) + a1(z,z'<>=z)x_1
		do iz = 1, nz
			do izprime = 1, nz
				coeff(1,3,iz,izprime) = 0.d0
				if (iz == 1) then
					if (izprime == 1) then
						coeff(1,1,iz,izprime) = coeffmod(1)
						coeff(1,2,iz,izprime) = coeffmod(4 + 3*(nz - 2) + 1)
					else
						coeff(1,1,iz,izprime) = coeffmod(2)
						coeff(1,2,iz,izprime) = coeffmod(4 + 3*(nz - 2) + 2)
					endif
				elseif (iz > 1 .AND. iz < nz) then
					if (izprime < iz) then
						coeff(1,1,iz,izprime) = coeffmod(2 + 3*(iz - 2) + 1)
						coeff(1,2,iz,izprime) = coeffmod(4 + 3*(nz - 2) + 2 + 3*(iz - 2) + 1)
					elseif (izprime == iz) then
						coeff(1,1,iz,izprime) = coeffmod(2 + 3*(iz - 2) + 2)
						coeff(1,2,iz,izprime) = coeffmod(4 + 3*(nz - 2) + 2 + 3*(iz - 2) + 2)
					else
						coeff(1,1,iz,izprime) = coeffmod(2 + 3*(iz - 2) + 3)
						coeff(1,2,iz,izprime) = coeffmod(4 + 3*(nz - 2) + 2 + 3*(iz - 2) + 3)
					endif
				else
		            if (izprime < nz) then
						coeff(1,1,iz,izprime) = coeffmod(2 + 3*(nz - 2) + 1)
						coeff(1,2,iz,izprime) = coeffmod(4 + 3*(nz - 2) + 2 + 3*(nz - 2) + 1)
					else
		                coeff(1,1,iz,izprime) = coeffmod(2 + 3*(nz - 2) + 2)
						coeff(1,2,iz,izprime) = coeffmod(4 + 3*(nz - 2) + 2 + 3*(nz - 2) + 2)
					endif
				endif
			enddo
		enddo
	elseif (version == 5) then
		!Version 5: y = a0(z,z'<>=z) + a1(z)x_1
		do iz = 1, nz
			do izprime = 1, nz
				coeff(1,2,iz,izprime) = coeffmod(4 + 3*(nz - 2) + iz)
				coeff(1,3,iz,izprime) = 0.d0				
				if (iz == 1) then
					if (izprime == 1) then
						coeff(1,1,iz,izprime) = coeffmod(1)
					else
						coeff(1,1,iz,izprime) = coeffmod(2)
					endif
				elseif (iz > 1 .AND. iz < nz) then
					if (izprime < iz) then
						coeff(1,1,iz,izprime) = coeffmod(2 + 3*(iz - 2) + 1)
					elseif (izprime == iz) then
		                coeff(1,1,iz,izprime) = coeffmod(2 + 3*(iz - 2) + 2)
					else
		                coeff(1,1,iz,izprime) = coeffmod(2 + 3*(iz - 2) + 3)
					endif
				else
		            if (izprime < nz) then
						coeff(1,1,iz,izprime) = coeffmod(2 + 3*(nz - 2) + 1)
					else
		                coeff(1,1,iz,izprime) = coeffmod(2 + 3*(nz - 2) + 2)
					endif
				endif
			enddo
		enddo
	else
		!Version 6: y = a0(z) + a1(z)x_1
		do iz = 1, nz
			do izprime = 1, nz
				coeff(1,1,iz,izprime) = coeffmod(iz)
				coeff(1,2,iz,izprime) = coeffmod(nz + iz)
				coeff(1,3,iz,izprime) = 0.d0
			enddo
		enddo
	endif		

end subroutine sub_krusellsmith_regress

end module mod_krusellsmith_regress