! mod_matlab: 
! A collection of Matlab style routines in Fortran.  Uses Lapack libraries. 
! These routines have not been extensively tested, and some have not really been
! tested at all.  Use with caution!
!
! Author: Grey Gordon 2010
! Soli Deo Gloria
!
module mod_matlab
    implicit none

    interface corr
        module procedure corr_mat, corr_vecs, corr_mats, corr_matvec, corr_vecmat
    end interface corr
    interface cumsum
        module procedure cumsum_vec, cumsum_mat
    end interface cumsum
    interface cumprod
        module procedure cumprod_dp, cumprod_i
    end interface cumprod
    interface diag
        module procedure diag_mat, diag_vec
    end interface diag
    interface diff
        module procedure diff_vec, diff_mat
    end interface diff
    interface disp
        module procedure disp_vec, disp_mat, disp_vec_i, disp_mat_i
    end interface disp
    interface eye
        module procedure eye_sqmat, eye_mat
    end interface eye
    interface find
        module procedure find_x, find_xk, find_xks
    end interface find
    interface ind2sub
        module procedure ind2sub_2dim,ind2sub_3dim
    end interface ind2sub
    interface interp1
        module procedure interp1_mat, interp1_vec, interp1_scal
    end interface interp1
    interface interp1q
        module procedure interp1q_scal,interp1q_vec
    end interface interp1q
    interface mean
        module procedure mean_vec, mean_mat
    end interface mean   
    interface mergesort
        module procedure mergesort_i, mergesort_r
    end interface mergesort
    interface mtimes
        module procedure mtimes_matmat, mtimes_matvec
    end interface 
    interface num2str
        module procedure num2str_double, num2str_int
    end interface
    interface ones
        module procedure ones_sqmat, ones_mat, ones_vec2
    end interface ones
    interface plot
        module procedure plot_xy,plot_y
    end interface plot
    interface rand
        module procedure scal_rand,vec_rand,mat_rand
    end interface rand
    interface randN
        module procedure vec_randN
    end interface randN
    interface sort
        module procedure fn_sort_vec
    end interface sort   
    interface sort2
        module procedure sub_sort_vec_i, sub_sort_vec, sub_sort_mat
    end interface sort2
    interface squeeze
        module procedure squeeze_vec, squeeze_mat, squeeze_vec_i
    end interface squeeze
    interface std
        module procedure std_vec, std_mat
    end interface std
    interface sub2ind
        module procedure sub2ind_2dim,sub2ind_3dim
    end interface sub2ind
    interface toc
        module procedure toc_print, toc_pr_return
    end interface toc
    interface true
        module procedure true_sqmat, true_mat
    end interface true
    interface vec
        module procedure vec_mat, vec_3d, vec_4d, vec_mat_i, vec_3d_i, vec_4d_i
    end interface vec

    interface operator (.colon.)
        module procedure colon
    end interface
    interface operator (.mtimes.)
        module procedure mtimes_matmat, mtimes_matvec
    end interface 

    type mat
        real(8), dimension(:,:), allocatable :: r
        integer, dimension(:,:), allocatable :: i
        logical, dimension(:,:), allocatable :: l
        integer, dimension(1:2) :: dims 
        character :: type        
    end type mat

    real(4), private :: time_start

    real(8), parameter :: pi = 3.14159265358979323846264338327950288419716939937510582097494459d0

    logical, private :: randomSeedInitialized = .FALSE.
    type stream
        integer :: m
        integer, allocatable :: seeds(:)
    end type

contains

function num2str_double(A,format) result(str)
    implicit none
    real(8), intent(in) :: A
    character(len=*), intent(in) :: format
    character(len=50) :: str
    write(str,format) A
end function
function num2str_int(A,format) result(str)
    implicit none
    integer, intent(in) :: A
    character(len=*), intent(in) :: format
    character(len=50) :: str
    write(str,format) A
end function

!Very basic plotting to std out
!Only have a limited number of pixels to work with    
subroutine plot_y(y)
    implicit none
    real(8), dimension(1:), intent(in) :: y
    !local
    real(8), dimension(1:size(y)) :: x
    x = real(1.colon.size(y),8)
    call plot(x,y)
end subroutine plot_y
subroutine plot_xy(x,y)
    implicit none
    real(8), dimension(1:), intent(in) :: x,y
    !local
    integer, parameter :: xpixels = 40,ypixels=20
    real(8) :: x_axis(1:xpixels),y_axis(1:ypixels), xpt,ypt
    logical :: xy_filled(xpixels,ypixels)
    integer :: i, y_ind, x_ind
    character(len=xpixels) :: horizontal_line

    !Check input
    if (size(x)/=size(y)) STOP 'plot: wrong size'       

    !Generate axes
    x_axis = linspace(minval(x),maxval(x),xpixels)
    y_axis = linspace(minval(y),maxval(y),ypixels)

    !Fill plot
    xy_filled = .FALSE.
    do i = 1,size(x)
        xpt = x(i)
        ypt = y(i)
                    
        xy_filled(find(xpt>=x_axis(1:xpixels-1) .AND. &
                    xpt<=x_axis(2:xpixels)),&
                find(ypt>=y_axis(1:ypixels-1) .AND. &
                    ypt<=y_axis(2:ypixels))) = .TRUE.           
    end do

    !Draw plot to screen
    do y_ind = ypixels,1,-1
        do x_ind = 1,xpixels
            if (xy_filled(x_ind,y_ind)) then
                horizontal_line(x_ind:x_ind) = '.'
            else 
                horizontal_line(x_ind:x_ind) = ' '
            end if
        end do
        write(*,'(f10.5,A,A)') y_axis(y_ind),'|',horizontal_line
    end do
    do x_ind = 1,xpixels
        horizontal_line(x_ind:x_ind) = '-'
    end do
    write(*,*) '          ',horizontal_line
    write(*,*) 'x_range: ',x_axis(1),',',x_axis(xpixels)
    write(*,*) 'y_range: ',y_axis(1),',',y_axis(ypixels)
    
end subroutine plot_xy

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
! CUMSUM Cumulative sum of elements.
!    For vectors, CUMSUM(X) is a vector containing the cumulative sum of
!    the elements of X. 
!  
!  Not yet implemented:
!  For matrices, CUMSUM(X) is a matrix the same size
!    as X containing the cumulative sums over each column.  For N-D
!    arrays, CUMSUM(X) operates along the first non-singleton dimension.
! 
!    CUMSUM(X,DIM) works along the dimension DIM.
! 
!    Example: If X = [0 1 2
!                     3 4 5]
! 
!    then cumsum(X,1) is [0 1 2  and cumsum(X,2) is [0 1  3
!                         3 5 7]                     3 7 12]
function cumsum_vec(x_in) result(x_out)
    
    implicit none
    real(8), dimension(1:), intent(in) :: x_in
    real(8), dimension(1:size(x_in)) :: x_out
    integer :: i

    x_out(1)=x_in(1)
    do i=2,size(x_in)
        x_out(i) = x_in(i) + x_out(i-1)
    end do

end function cumsum_vec
function cumsum_mat(x_in) result(x_out)
    
    implicit none
    real(8), dimension(1:,1:), intent(in) :: x_in
    real(8), dimension(1:size(x_in,1),1:size(x_in,2)) :: x_out
    integer :: i

    x_out(1,:)=x_in(1,:)
    do i=2,size(x_in,1)
        x_out(i,:) = x_in(i,:) + x_out(i-1,:)
    end do

end function cumsum_mat

subroutine sub_cumsum(x_out,x_in)


implicit none

real(8), dimension(:), intent(in) :: x_in
real(8), dimension(1:size(x_in)), intent(inout) :: x_out
integer :: i

x_out(1)=x_in(1)
do i=2,size(x_in)
    x_out(i) = x_in(i) + x_out(i-1)
end do

end subroutine sub_cumsum

subroutine sub_cumsum_mat(x_out,x_in,DIM)


implicit none

real(8), dimension(:,:), intent(in) :: x_in
real(8), dimension(1:size(x_in,1),1:size(x_in,2)), intent(inout) :: x_out
integer, intent(in) :: DIM 
integer :: col_ind, row_ind

if (DIM==1) then
    do col_ind = 1,size(x_in,2)
        call sub_cumsum(x_out(:,col_ind),x_in(:,col_ind))
    end do
elseif (DIM==2) then
    do row_ind = 1,size(x_in,1)
        call sub_cumsum(x_out(row_ind,:),x_in(row_ind,:))
    end do
else
    STOP 'error in cumsum'
end if

end subroutine sub_cumsum_mat

subroutine mergesort_r(A,I)
    implicit none
    real(8), dimension(1:), intent(inout) :: A !list to be sorted
    integer, dimension(1:size(A)), intent(out) :: I !index of sorting s.t. to sort B according to A use B=B(I)
    !local
    integer :: na

    na = size(A)
    I = 1 .colon. na

    call sub_mergesort_recursion_r(A,I)

end subroutine mergesort_r

!Returns sorted vector A and indices I sorted according to A
recursive subroutine sub_mergesort_recursion_r(A,I)
    implicit none
    real(8), dimension(1:), intent(inout) :: A
    integer, dimension(1:), intent(inout) :: I
    !local
    real(8), dimension(1:size(A)) :: Acopy
    integer, dimension(1:size(I)) :: Icopy
    integer :: na,j,a1,a2
    na = size(a)

    !If na is 0 or 1, the list is sorted, so return
    if (na<=1) then        
        return
    end if

    !If na>1 then divide the list in half
    call sub_mergesort_recursion_r(A(1:na/2),I(1:na/2))
    call sub_mergesort_recursion_r(A(na/2+1:na),I(na/2+1:na))

    !Merge these two sorted lists
    a1 = 1
    a2 = na/2+1
    do j=1,na      
        if (a1<=na/2 .AND. a2<=na) then
            if (A(a1)<=A(a2)) then
                Acopy(j) = A(a1)
                Icopy(j) = I(a1)
                a1 = a1+1
            else
                Acopy(j) = A(a2)
                Icopy(j) = I(a2)
                a2 = a2+1
            end if
            !if (j==na) exit
        elseif (a1>na/2) then !first list has been exhausted, fill remaining elements with 2nd list
            Acopy(j:na) = A(a2:na)
            Icopy(j:na) = I(a2:na)
            exit
        elseif (a2>na) then !second list has been exhausted, fill remaining elements with 1st list
            Acopy(j:na) = A(a1:na/2)
            Icopy(j:na) = I(a1:na/2)
            exit
        end if
    end do

    A = Acopy
    I = Icopy

end subroutine sub_mergesort_recursion_r

subroutine mergesort_i(A,I)
    implicit none
    integer, dimension(1:), intent(inout) :: A !list to be sorted
    integer, dimension(1:size(A)), intent(out) :: I !index of sorting s.t. to sort B according to A use B=B(I)
    !local
    integer :: na

    na = size(A)
    I = 1 .colon. na

    call sub_mergesort_recursion_i(A,I)

end subroutine mergesort_i

!Returns sorted vector A and indices I sorted according to A
recursive subroutine sub_mergesort_recursion_i(A,I)
    implicit none
    integer, dimension(1:), intent(inout) :: A
    integer, dimension(1:), intent(inout) :: I
    !local
    integer, dimension(1:size(A)) :: Acopy
    integer, dimension(1:size(I)) :: Icopy
    integer :: na,j,a1,a2
    na = size(a)

    !If na is 0 or 1, the list is sorted, so return
    if (na<=1) then        
        return
    end if

    !If na>1 then divide the list in half
    call sub_mergesort_recursion_i(A(1:na/2),I(1:na/2))
    call sub_mergesort_recursion_i(A(na/2+1:na),I(na/2+1:na))

    !Merge these two sorted lists
    a1 = 1
    a2 = na/2+1
    do j=1,na      
        if (a1<=na/2 .AND. a2<=na) then
            if (A(a1)<=A(a2)) then
                Acopy(j) = A(a1)
                Icopy(j) = I(a1)
                a1 = a1+1
            else
                Acopy(j) = A(a2)
                Icopy(j) = I(a2)
                a2 = a2+1
            end if
            !if (j==na) exit
        elseif (a1>na/2) then !first list has been exhausted, fill remaining elements with 2nd list
            Acopy(j:na) = A(a2:na)
            Icopy(j:na) = I(a2:na)
            exit
        elseif (a2>na) then !second list has been exhausted, fill remaining elements with 1st list
            Acopy(j:na) = A(a1:na/2)
            Icopy(j:na) = I(a1:na/2)
            exit
        end if
    end do

    A = Acopy
    I = Icopy

end subroutine sub_mergesort_recursion_i

subroutine meshgrid(xx,yy,x,y)
    implicit none
    integer, intent(in) :: x(:), y(:)
    integer, intent(out) :: xx(size(y),size(x)), yy(size(y),size(x))
    xx = spread(x,1,size(y))
    yy = spread(y,2,size(x))
end subroutine meshgrid

elemental function fix(x)
    implicit none
    real(8), intent(in) :: x
    integer :: fix
    fix = floor(x)
    if (x<0.d0) fix = fix + 1
end function fix
recursive function magic(n) result(M)
    implicit none
    integer, intent(in) :: n
    integer, dimension(n,n) :: M
    !local
    integer, dimension(n,n) :: J,I,A,B
    logical, dimension(n,n) :: K
    integer, dimension(n/2,n/2) :: M_alt

    ! Odd order.
    if (mod(n,2) == 1) then
        call meshgrid(J,I, 1.colon.n,1.colon.n)
        A = mod(I+J-(n+3)/2,n);
        B = mod(I+2*J-2,n);        
        M = n*A + B + 1;

    else
        STOP 'magic can only handle odd n presently'
    end if
!     ! Doubly even order.
!     elseif (mod(n,4) == 0) then
!         call meshgrid(J,I, 1 .colon. n, 1.colon.n)
!         K = fix(real(mod(I,4),8)/2.d0) == fix(real(mod(J,4),8)/2.d0)
!         M = transpose(reshape(1 .colon. n*n,(/n,n/)))
!         where (K) M = n*n+1 - M
! 
!     ! Singly even order.
!     else
!         p = n/2
!         M_alt = magic(p)
!         M = [M_alt M_alt+2*p**2; M_alt+3*p**2 M_alt+p**2];
!         if n == 2, return, end
!         i = (1:p)';
!         k = (n-2)/4;
!         j = [1:k (n-k+2):n];
!         M([i; i+p],j) = M([i+p; i],j);
!         i = k+1;
!         j = [1 i];
!         M([i; i+p],j) = M([i+p; i],j);
!     end

end function magic

function scal_rand result(btwn01)
    implicit none
    real(8) :: btwn01
    call random_number(btwn01)
end function scal_rand
function vec_rand(n) result(btwn01)
    implicit none
    integer, intent(in) :: n
    real(8), dimension(1:n) :: btwn01
    call random_number(btwn01)
end function vec_rand
function mat_rand(n1,n2) result(btwn01)
    implicit none
    integer, intent(in) :: n1,n2
    real(8), dimension(1:n1,1:n2) :: btwn01
    call random_number(btwn01)
end function mat_rand

!Generates random normal variables using the Box-Muller method
function vec_randN(n) result(RVs)
    implicit none
    integer, intent(in) :: n
    real(8), dimension(1:n) :: RVs
    !local
    real(8), dimension(1:n/2+1) :: U1,U2,twopiU1, sqrtneg2logU2
    
    if (n==1) then
        U1 = rand(1)
        U2 = rand(1)
        twopiU1 = 2.d0*pi*U1
        sqrtneg2logU2 = sqrt(-2.d0*log(U2))
        RVs(1:n/2+1) = cos(twopiU1)*sqrtneg2logU2
    else
        U1 = rand(n/2+1)
        U2 = rand(n/2+1)
        twopiU1 = 2.d0*pi*U1
        sqrtneg2logU2 = sqrt(-2.d0*log(U2))
        RVs(1:n/2+1) = cos(twopiU1)*sqrtneg2logU2
        RVs(n/2+1:n) = sin(twopiU1(1:n-n/2))*sqrtneg2logU2(1:n-n/2)           
    end if       
end function vec_randN

!  RESET Reset a random stream to its initial internal state.
!     RESET(S) resets the generator for the random stream S to its initial
!     internal state.  This is almost equivalent to clearing S and recreating
!     it using RandStream(TYPE,...), except RESET does not set the stream's
!     RandnAlg, Antithetic, and Precision properties to their original values.
! Note to reset to a default stream, do the following:   call reset(getDefaultStream())
subroutine reset(streamIn)
    implicit none
    type(stream) :: streamIn
    integer :: m
    call random_seed(size=m)
    if (m/=streamIn%m) STOP 'reset: stream size and seed size do not agree'

    call random_seed(put=streamIn%seeds)
end subroutine
function getDefaultStream() result(streamOut)
    implicit none
    type(stream) :: streamOut
    integer :: m

    if (.not. randomSeedInitialized) then
        call random_seed()
        randomSeedInitialized = .true.
    end if

    call random_seed(size=m)
    streamOut%m = m

    if (allocated(streamOut%seeds)) deallocate(streamOut%seeds)
    allocate(streamOut%seeds(m))

    streamOut%seeds = 5

end function


!Vectorize (equivalent to matlab colon)
function vec_mat(X) result(vec)
    implicit none
    real(8), dimension(1:,1:) :: X
    real(8), dimension(1:size(X)) :: vec
    vec = reshape(X,(/size(X)/))
end function vec_mat
function vec_3d(X) result(vec)
    implicit none
    real(8), dimension(1:,1:,1:) :: X
    real(8), dimension(1:size(X)) :: vec
    vec = reshape(X,(/size(X)/))
end function vec_3d
function vec_4d(X) result(vec)
    implicit none
    real(8), dimension(1:,1:,1:,1:) :: X
    real(8), dimension(1:size(X)) :: vec
    vec = reshape(X,(/size(X)/))
end function vec_4d
function vec_mat_i(X) result(vec)
    implicit none
    integer, dimension(1:,1:) :: X
    integer, dimension(1:size(X)) :: vec
    vec = reshape(X,(/size(X)/))
end function vec_mat_i
function vec_3d_i(X) result(vec)
    implicit none
    integer, dimension(1:,1:,1:) :: X
    integer, dimension(1:size(X)) :: vec
    vec = reshape(X,(/size(X)/))
end function vec_3d_i
function vec_4d_i(X) result(vec)
    implicit none
    integer, dimension(1:,1:,1:,1:) :: X
    integer, dimension(1:size(X)) :: vec
    vec = reshape(X,(/size(X)/))
end function vec_4d_i


subroutine tic()
    implicit none
    call cpu_time(time_start)
end subroutine tic
subroutine toc_print()
    implicit none
    real(4) :: time_end
    call cpu_time(time_end)
    write(*,*) 'Elapsed time is ', (time_end-time_start)/60., 'minutes'
end subroutine toc_print
subroutine toc_pr_return(elapsed)
    
    implicit none
    real(4) :: time_end
    real(8) :: elapsed
    call cpu_time(time_end)
    write(*,*) 'Elapsed time is ', (time_end-time_start)/60., 'minutes'
    elapsed = (time_end-time_start)/60.
end subroutine toc_pr_return

pure function colon(a,b)
    integer, intent(in) :: a,b
    integer, dimension(1:b-a+1) :: colon
    integer :: i
    do i = a,b
        colon(i-a+1) = i
    end do
end function

function mtimes_matmat(A,B) result(C)
    real(8), intent(in) :: A(:,:),B(:,:)
    real(8) :: C(size(A,1),size(B,2))

    !For compatibility can use:
    !C = matmul(A,B)

    if (size(A,2)/=size(B,1)) STOP 'mtimes_matmat: size doesn''t conform'
    !C := alpha*op(A)*op(B) + beta*C,
    call dgemm('N','N',size(A,1),size(B,2),size(A,2),1.d0, A, size(A,1), B, size(B,1), 0.d0, C, size(C,1))

end function mtimes_matmat
function mtimes_matvec(A,B) result(C)
    real(8), intent(in) :: A(:,:),B(:)
    real(8) :: C(size(A,1))

    !For compatibility can use:
    !C = matmul(A,B)
    if (size(A,2)/=size(B)) STOP 'mtimes_matvec: size doesn''t conform'
    !C := alpha*op(A)*op(B) + beta*C,
    call dgemv('N',size(A,1),size(A,2),1.d0, A, size(A,1), B, 1, 0.d0, C, 1)

end function mtimes_matvec

! Construct a true matrix
function true_sqmat(a) result(true)
    integer, intent(in) :: a
    logical :: true(a,a)
    true = .TRUE.
end function true_sqmat
function true_mat(a,b) result(true)
    integer, intent(in) :: a,b
    logical :: true(a,b)
    true = .TRUE.
end function true_mat
function true_vec(a,b,option) result(true)
    integer, intent(in) :: a,b
    character,intent(in) :: option
    logical :: true(a)
    if (b/=1) STOP 'true_vec: wrong calling sequence'
    true = .TRUE.
end function true_vec


! LINSPACE Linearly spaced vector.
!    LINSPACE(X1, X2) generates a row vector of 100 linearly
!    equally spaced points between X1 and X2.
! 
!    LINSPACE(X1, X2, N) generates N points between X1 and X2.
function linspace(x1,x2,n) result(grid)
    
    implicit none
    real(8), intent(in) :: x1,x2
    integer, intent(in) :: n
    real(8), dimension(1:n) :: grid
    !local
    integer :: i 
    
    if (n>1) then
        do i=1,n
            grid(i) = x1 + (x2-x1)*(real(i,8)-1.d0)/(real(n,8)-1.d0)
        end do   
    else
        grid(1) = x1
    end if

end function linspace

! EYE Identity matrix.
!    EYE(N) is the N-by-N identity matrix.
! 
!    EYE(M,N) or EYE([M,N]) is an M-by-N matrix with 1's on
!    the diagonal and zeros elsewhere.
function eye_sqmat(dim) result(mat)
    implicit none
    integer, intent(in) :: dim
    real(8), dimension(1:dim,1:dim) :: mat    
    integer :: i
    mat = 0.d0
    forall (i=1:dim)
        mat(i,i) = 1.d0
    end forall
end function eye_sqmat
function eye_mat(dim1,dim2) result(mat)
    implicit none
    integer, intent(in) :: dim1,dim2
    real(8), dimension(1:dim1,1:dim2) :: mat    
    integer :: i
    mat = 0.d0
    forall (i=1:min(dim1,dim2))
        mat(i,i) = 1.d0
    end forall
end function eye_mat

! ONES   Ones array.
!    ONES(N) is an N-by-N matrix of ones.  
! 
!    ONES(M,N) or ONES([M,N]) is an M-by-N matrix of ones.
! 
!    ONES(M,N,P,...) or ONES([M N P ...]) is an M-by-N-by-P-by-... array of
!    ones.
function ones_sqmat(dim) result(mat)
    implicit none
    integer, intent(in) :: dim
    real(8), dimension(1:dim,1:dim) :: mat    
    mat = 1.d0
end function ones_sqmat
function ones_mat(dim1,dim2) result(mat)
    implicit none
    integer, intent(in) :: dim1,dim2
    real(8), dimension(1:dim1,1:dim2) :: mat    
    mat = 1.d0
end function ones_mat
function ones_vec2(dims) result(mat)
    implicit none
    integer, dimension(1:2),intent(in) :: dims
    real(8), dimension(1:dims(1)) :: mat
    if (dims(2)/=1) STOP 'ones_vec2: use ones_mat2 for a matrix.  This only handles vectors'
    mat = 1.d0
end function ones_vec2

function regress(Y,X) result(B)
    implicit none
    real(8), dimension(1:), intent(in) :: Y
    real(8), dimension(1:,1:), intent(in) :: X
    real(8), dimension(1:size(X,2)) :: B
    !local
    real(8), dimension(1:size(X,2),1:size(X,2)) :: XpX
    real(8), dimension(1:size(X,2)) :: XpY

    !Check inputs
    if (size(Y)/=size(X,1)) STOP 'regress: matrix dimensions dont agree: y/=size(x,1)'

    !Calc X'X
    XpX = matmul(transpose(X),X)
    XpY = matmul(transpose(X),Y)

    !Check the condition of X'X to check for possibly multicollinearity
    if (cond(XpX)>1d10) then 
        write(*,*) 'regress: warning: possible multicollinearity in regressors'
        write(*,*) 'regress: X''X has condition number', cond(XpX)
    end if

    !Solve (X'X)B=(X'Y) (i.e. B = (X'X)^-1(X'Y))
    B = mldivide(XpX,XpY)

end function regress    !regress

function cond(X) result(k)
    
    implicit none
    real(8), dimension(1:,1:), intent(in) :: X
    real(8) :: k
    !local
    real(8), dimension(1:size(X,1)) :: e 
    real(8) :: mine
    e = eig(X)
    mine = minval(abs(e))
    if (mine/=0.d0) then
        k = maxval(abs(e))/mine
    else 
        k = huge(0.d0)
    endif
end function cond

function eig(A_in) result(e)
    
    implicit none
    real(8), dimension(1:,1:), intent(in) :: A_in
    real(8), dimension(1:size(A_in,1)) :: e
    !local
    real(8), dimension(1:size(A_in,1),1:size(A_in,2)) :: A
    real(8), dimension(1:size(A_in,1),1:size(A_in,2)) :: H,Z
    real(8), dimension(1:size(A_in,1)) :: wr,wi
    integer, parameter :: blocksize = 64
    real(8), dimension(1:size(A_in,1)*blocksize) :: work
    real(8), dimension(1:size(A_in,1)-1) :: tau
    integer :: n, ilo, ihi, lda, lwork, info, ldh, ldz
    character(len=1) :: job,compz
    !Ensure the A is square
    if (size(A_in,1)/=size(A_in,2)) STOP 'A not square'

    !Compute the hessenberg form of the matrix A=Q*H*Q^{H}
    A = A_in
    n = size(A_in,1)
    ilo = 1
    ihi = n
    lda = n
    lwork = size(work)
    call dgehrd(n, ilo, ihi, a, lda, tau, work, lwork, info) !note A is overwritten with details of H and Q
    
    !Compute the eigenvalues of the hessenberg matrix
    job = 'E'
    compz = 'N'
    H = A
    ldh = size(H,1)    
    !Z is not referenced and so need not be set    
    ldz = size(Z,1)
    !Can use the same work arrays as before
    call dhseqr(job, compz, n, ilo, ihi, h, ldh, wr, wi, z, ldz, work, lwork, info)
    !wr and wi contain the real and imaginary parts of the eigenvalues unless info>0

    select case (info)
        case (0)
            !Executed successfully
            !Right only the real part of the eigenvalues
            e = wr
            if (any(abs(wi)>1d-14)) then
                write(*,*) 'eig: warning: some eigenvalues are complex!'
                write(*,*) 'eig:          only writing the real parts'
            end if                
        case (:-1)
            write(*,*) 'eig: the ', info, ' parameter has an illegal value'
            STOP
            !The info-th parameter had an illegal value
        case (1:)
            write(*,*) 'eig: not all eigenvalues found'
            STOP 
            !Elements 1,2,...,ilo-1 and i+1,i+2,...,n of wr and wi contain the real and imag parts of 
            !those eigenvalues which have been successfully found.        
    end select

end function eig


function inv(X) result(Xinv)
    
    implicit none
    real(8), dimension(1:,1:), intent(in) :: X
    real(8), dimension(1:size(X,1),1:size(X,2)) :: Xinv
    !local
    real(8) :: determ, a,b,c,d
    if (any(shape(X)/=(/2,2/))) STOP 'inv: only built for 2x2 matrices'
    a = X(1,1)
    b = X(1,2)
    c = X(2,1)
    d = X(2,2)
    determ = a*d-b*c
    Xinv = reshape((/d,-c,-b,a/),(/2,2/))/determ
end function inv


!Removes one singleton dimension from A
function squeeze_vec_i(A) result(B)
    implicit none
    integer, dimension(1:) :: A
    integer :: B

    if (size(A)>1) then
        STOP 'squeeze_vec: A is not a length 1 vector!'
    end if

    B = A(1)

end function squeeze_vec_i
function squeeze_vec(A) result(B)
    implicit none
    real(8), dimension(1:) :: A
    real(8) :: B

    if (size(A)>1) then
        STOP 'squeeze_vec: A is not a length 1 vector!'
    end if

    B = A(1)

end function squeeze_vec
function squeeze_mat(A) result(B)
    implicit none
    real(8), dimension(1:,1:) :: A
    real(8), dimension(1:size(A)) :: B

    if (.NOT.(any(shape(A)==1))) then
        write(*,*) 'squeeze_mat: No singleton dimension to remove'
        STOP
    elseif (count(shape(A)>1)) then
        write(*,*) 'squeeze_mat: More than one singleton dimension to remove'
        STOP
    end if   

    if (size(A,1)==1) then
        B = A(1,:)
    elseif (size(A,2)==1) then
        B = A(:,1)
    else
        STOP 'squeeze_mat: case not found'
    end if
end function


! MEAN   Average or mean value.
!    For vectors, MEAN(X) is the mean value of the elements in X. For
!    matrices, MEAN(X) is a row vector containing the mean value of
!    each column.  For N-D arrays, MEAN(X) is the mean value of the
!    elements along the first non-singleton dimension of X.
function mean_vec(X) result(mean)
    
    implicit none
    real(8), dimension(1:), intent(in) :: X
    real(8) :: mean
    mean = sum(X)/real(size(X),8)
end function mean_vec
function mean_mat(X) result(mean)
    
    implicit none
    real(8), dimension(1:,1:), intent(in) :: X
    real(8), dimension(1:size(X,2)) :: mean
    mean = sum(X,1)/real(size(X,1),8)
end function mean_mat

! \   Backslash or left matrix divide.
!    A\B is the matrix division of A into B, which is roughly the
!    same as INV(A)*B , except it is computed in a different way.
!    If A is an N-by-N matrix and B is a column vector with N
!    components, or a matrix with several such columns, then
!    X = A\B is the solution to the equation A*X = B. A warning 
!    message is printed if A is badly scaled or nearly singular.
!    A\EYE(SIZE(A)) produces the inverse of A.
! 
!    If A is an M-by-N matrix with M < or > N and B is a column
!    vector with M components, or a matrix with several such columns,
!    then X = A\B is the solution in the least squares sense to the
!    under- or overdetermined system of equations A*X = B. The
!    effective rank, K, of A is determined from the QR decomposition
!    with pivoting. A solution X is computed which has at most K
!    nonzero components per column. If K < N this will usually not
!    be the same solution as PINV(A)*B.  A\EYE(SIZE(A)) produces a
!    generalized inverse of A.
! 
!    C = MLDIVIDE(A,B) is called for the syntax 'A \ B' when A or B is an
!    object.
function mldivide(A,b) result(x)
    implicit none
    real(8), dimension(1:,1:), intent(in) :: A
    real(8), dimension(1:), intent(in) :: b
    real(8), dimension(1:size(A,2)) :: x
    !local
    real(8), dimension(1:size(A,1),1:size(A,2)) :: A_local 
    real(8), dimension(1:size(b),1:1) :: b_local
    integer, dimension(1:size(A,1)) :: ipiv
    integer :: info

    
    !Check inputs
    if (size(b)/=size(A,1)) then
        write(*,*) size(b), size(A,1)
        STOP 'mldivide: b wrong size'
    end if
    if (size(A,1)/=size(A,2)) then
        write(*,*) shape(A)
        STOP 'mldivide: A not square.  Cant handle this currently.'
    end if

    A_local = A
    b_local(:,1) = b
    call dgesv(size(A_local,1), 1, A_local, size(A_local,1), ipiv, b_local, size(b_local,1), info )
!    call gesv(A_local,b_local,info=info) !Solves Ax = b. Returns solution x in b and LU factorization in A    
    x = b_local(:,1)
    
    select case (info)
        case(0)
        case (:-1)
            write(*,*) 'mldivide: the ', -info, ' parameter had an illegal value'
        case(1:)
            write(*,*) 'mldivide: U(i,i) for i=', info, ' is exactly zero.  The factorization ', &
                    'has been completed, but the factor U is exactly singular, so the solution ', &
                    'could not be computed.'
    end select
    
end function mldivide

subroutine disp_mat(X)
    implicit none
    real(8), dimension(1:,1:), intent(in) :: X    
    integer :: i
    character(len=5) :: dim2
    write(dim2,'(i5)') size(X,2)
    do i = 1,size(X,1)
        write(*,'('//dim2//'e12.5)') X(i,:)
    end do
end subroutine disp_mat
subroutine disp_vec(X)
    implicit none
    real(8), dimension(1:), intent(in) :: X    
    integer :: i
    do i = 1,size(X,1)
        write(*,'(e12.5)') X(i)
    end do
end subroutine disp_vec
subroutine disp_mat_i(X)
    implicit none
    integer, dimension(1:,1:), intent(in) :: X    
    integer :: i
    character(len=5) :: dim2
    write(dim2,'(i5)') size(X,2)
    do i = 1,size(X,1)
        write(*,'('//dim2//'i8)') X(i,:)
    end do
end subroutine disp_mat_i
subroutine disp_vec_i(X)
    implicit none
    integer, dimension(1:), intent(in) :: X    
    integer :: i
    do i = 1,size(X,1)
        write(*,'(i8)') X(i)
    end do
end subroutine disp_vec_i


! SVD    Singular value decomposition.
!    [U,S,V] = SVD(X) produces a diagonal matrix S, of the same 
!    dimension as X and with nonnegative diagonal elements in
!    decreasing order, and unitary matrices U and V so that
!    X = U*S*V'.
subroutine svd(U,S,V,A_in)
    implicit none
    real(8), dimension(1:,1:), intent(in) :: A_in
    real(8), dimension(1:size(A_in,1),1:size(A_in,2)), intent(out) :: S    
    real(8), dimension(1:size(A_in,1),1:size(A_in,1)), intent(out) :: U
    real(8), dimension(1:size(A_in,2),1:size(A_in,2)), intent(out) :: V
    !local
    real(8), dimension(1:size(A_in,1),1:size(A_in,2)) :: A
    real(8), dimension(1:size(A_in,2),1:size(A_in,2)) :: Vt
    real(8), dimension(1:min(size(A_in,1),size(A_in,2))) :: s_values
    integer :: info
    character(len=1) :: jobu, jobvt
    integer :: m,n,lda,lwork,ldu,ldvt
    real(8), dimension(:), allocatable :: work

    !Fortran 95 interface (Lapack95)
!    A = A_in
!    call gesvd(A,s_values,U,Vt,info=info) !A = U*S*Vt
!    S = diag(s_values)
!    V = transpose(Vt)

    !Fortran 77 interface (Lapack)
    jobu = 'A'
    jobvt = 'A'
    m = size(A_in,1)
    n = size(A_in,2)
    lda = max(1,m)
    ldu = max(1,m)
    ldvt = max(1,n)
    lwork = 64 * max(3*min(m, n)+max(m, n), 5*min(m,n)) 
    
    allocate(work(1:max(1,lwork)))
    
    A = A_in
    call dgesvd(jobu, jobvt, m, n, A, lda, s_values, U, ldu, Vt, ldvt, work, lwork, info) !A = U*S*Vt
    S = diag(s_values)
    V = transpose(Vt)
    
    deallocate(work)

    select case (info)
        case(0)
        case (:-1)
            write(*,*) 'svd: in reduction to bidiagonal, the ', -info, ' parameter had an illegal value'
        case (1:)
            write(*,*) 'svd: WARNING: ', info, ' of the superdiagonals failed to converge.'
    end select    
   
end subroutine svd


!NULL   Null space.
!    Z = NULL(A) is an orthonormal basis for the null space of A obtained
!    from the singular value decomposition.  That is,  A*Z has negligible
!    elements, size(Z,2) is the nullity of A, and Z'*Z = I.
function null(A,option) result(x)
    implicit none
    real(8), dimension(1:,1:), intent(in) :: A
    real(8), dimension(1:size(A,2)) :: x_out
    character(len=*), intent(in), optional :: option
    !local
    real(8), dimension(1:size(A,2)) :: x
    real(8), dimension(1:size(A,1),1:size(A,2)) :: S
    real(8), dimension(1:size(A,1),1:size(A,1)) :: U
    real(8), dimension(1:size(A,2),1:size(A,2)) :: V    
    real(8) :: zero_toler    
    integer :: info, j
    logical, dimension(1:size(A,2)) :: is_singularcol
    
    call svd(U,S,V,A) !A = U*S*V'
       
    zero_toler = max(size(A,1),size(A,2))*maxval(S)*epsilon(0.d0)

    if (count(diag(S)<zero_toler)/=1) then
        write(*,*) 'null: nullity of A does not equal one.  '//&
                   'This is okay, ... but not for transition matrices which this function was designed for.  '//&
                   'Slight modifications are necessary if this is not what you want.'
        write(*,*) diag(S)
        STOP
    end if

    is_singularcol = diag(S)<zero_toler
    do j=1,size(V,2)
        if (is_singularcol(j)) x = V(:,j)
    end do

    if (present(option)) then
        if (option=='normalize') then
            x = x/sum(x)
        else 
            write(*,*) 'did you mean normalize? may experience unexpected results'
        end if
    end if

    x_out = x
    
end function null 

! FIND   Find indices of nonzero elements.
!    I = FIND(X) returns the linear indices corresponding to 
!    the nonzero entries of the array X.  X may be a logical expression. 
!    Use IND2SUB(SIZE(X),I) to calculate multiple subscripts from 
!    the linear indices I.
pure function find_x(X) result(I)
    implicit none
    logical, dimension(1:), intent(in) :: X
    integer, dimension(1:count(X)) :: I
    !local
    integer, dimension(1:size(X)) :: index_vec
    index_vec = 1 .colon. size(X)
    I = pack(index_vec,X)
end function find_x
function find_xk(X,k) result(I)
    implicit none
    logical, dimension(1:), intent(in) :: X
    integer, intent(in) :: k
    integer, dimension(1:k) :: I !min(count(X),k)
    !local
    integer, dimension(1:count(X)) :: I_all
    I_all = find_x(X)
    if (size(I_all)>=k) then
        I(1:k) = I_all(1:k)
    else 
        I(1:size(I_all)) = I_all
        I(size(I_all):size(I)) = -1
        write(*,*) 'find_xk: WARNING: k true values did not exist'
    end if
end function find_xk
function find_xks(X,k,string) result(I)
    implicit none
    logical, dimension(1:), intent(in) :: X
    integer, intent(in) :: k
    integer, dimension(1:min(count(X),k)) :: I
    character(len=*), intent(in) :: string
    !local
    integer, dimension(1:size(X)) :: index_vec
    integer, dimension(1:count(X)) :: I_all
    index_vec = 1 .colon. size(X)
    I_all = pack(index_vec,X)
    if (string=='last') then
        I = I_all(count(X)-size(I)+1:count(X)) !last 
    elseif (string=='first') then
        I = I_all(1:size(I))
    else 
        STOP 'find_xks: case not found'
    end if
end function find_xks


! STD Standard deviation.
!    For vectors, Y = STD(X) returns the standard deviation.  For matrices,
!    Y is a row vector containing the standard deviation of each column.  For
!    N-D arrays, STD operates along the first non-singleton dimension of X.
! 
!    STD normalizes Y by (N-1), where N is the sample size.  This is the
!    sqrt of an unbiased estimator of the variance of the population from
!    which X is drawn, as long as X consists of independent, identically
!    distributed samples.
function std_mat(X) result(Y)
    implicit none
    real(8), dimension(1:,1:), intent(in) :: X
    real(8), dimension(1:size(X,2)) :: Y
    !local
    integer :: N, j
    
    !Check dimensions
    if (size(X,1)<=1) STOP 'std: X must have at least N>1 rows'
    
    N = size(X,1)
    do j = 1,size(X,2)
        Y(j) = std_vec(X(:,j))
    end do
end function std_mat
function std_vec(X) result(Y)
    implicit none
    real(8), dimension(1:), intent(in) :: X
    real(8) :: Y
    !local
    integer :: N, j
    
    !Check dimensions
    if (size(X,1)<=1) STOP 'std: X must have at least N>1 rows'
    
    N = size(X,1)
    
    Y = sqrt(sum((X-mean(X))**2/real(N-1,8)))

end function std_vec

!CORR Linear or rank correlation.
!    RHO = CORR(X) returns a P-by-P matrix containing the pairwise linear
!    correlation coefficient between each pair of columns in the N-by-P
!    matrix X.
!    RHO = CORR(X,Y,...) returns a P1-by-P2 matrix containing the pairwise
!    correlation coefficient between each pair of columns in the N-by-P1 and
!    N-by-P2 matrices X and Y.
function corr_vecs(X,Y) result(rho)
    
    implicit none
    real(8), dimension(1:), intent(in) :: X, Y
    real(8) :: rho
    !local
    integer :: N, j
    
    !Check dimensions
    if (size(X,1)/=size(Y,1)) STOP 'corr_vecs: X must have at least N>1 rows'    
    N = size(X,1)
    
    rho = sum((X-mean(X))*(Y-mean(Y)))/(real(N-1,8)*std(X)*std(Y))
    
end function corr_vecs
function corr_mat(X) result(rho)
    
    implicit none
    real(8), dimension(1:,1:), intent(in) :: X
    real(8), dimension(1:size(X,2),1:size(X,2)) :: rho
    !local
    integer :: N, j1,j2
    
    !Check dimensions
    if (size(X,1)<=1) STOP 'corr: X must have at least N>1 rows'    
    N = size(X,1)
    
    do j1 = 1,size(X,2)
        do j2 = 1,size(X,2)
            rho(j1,j2) = corr_vecs(X(:,j1),X(:,j2))
        end do
    end do
    
end function corr_mat
function corr_mats(X1,X2) result(rho)
    
    implicit none
    real(8), dimension(1:,1:), intent(in) :: X1,X2
    real(8), dimension(1:size(X1,2),1:size(X2,2)) :: rho
    !local
    integer :: N, j1,j2
    
    !Check dimensions
    if (size(X1,1)<=1) STOP 'corr: X must have at least N>1 rows'    
    if (size(X1,1)/=size(X2,1)) STOP 'corr: X1 and X2 must have the same number of rows'    

    N = size(X1,1)
    
    do j1 = 1,size(X1,2)
        do j2 = 1,size(X2,2)
            rho(j1,j2) = corr_vecs(X1(:,j1),X2(:,j2))
        end do
    end do
    
end function corr_mats
function corr_matvec(X1,X2) result(rho)
    
    implicit none
    real(8), dimension(1:,1:), intent(in) :: X1
    real(8), dimension(1:), intent(in) :: X2
    real(8), dimension(1:size(X1,2)) :: rho
    rho = (/corr_mats(X1,reshape(X2,(/size(X2),1/)))/)
end function corr_matvec
function corr_vecmat(X1,X2) result(rho)
    
    implicit none
    real(8), dimension(1:), intent(in) :: X1
    real(8), dimension(1:,1:), intent(in) :: X2
    real(8), dimension(1:size(X2,2)) :: rho
    rho = (/corr_mats(reshape(X1,(/size(X1),1/)),X2)/)
end function corr_vecmat


! DIAG Diagonal matrices and diagonals of a matrix.
!    DIAG(V,K) when V is a vector with N components is a square matrix
!    of order N+ABS(K) with the elements of V on the K-th diagonal. K = 0
!    is the main diagonal, K > 0 is above the main diagonal and K < 0
!    is below the main diagonal. 
! 
!    DIAG(V) is the same as DIAG(V,0) and puts V on the main diagonal.
! 
!    DIAG(X,K) when X is a matrix is a column vector formed from
!    the elements of the K-th diagonal of X.
! 
!    DIAG(X) is the main diagonal of X. DIAG(DIAG(X)) is a diagonal matrix.
function diag_mat(X) result(V)
    
    implicit none
    real(8), dimension(1:,1:), intent(in) :: X
    real(8), dimension(1:min(size(X,1),size(X,2))) :: V
    !local
    integer :: i
    
    if (size(X,1)/=size(X,2)) write(*,*) 'diag_mat: WARNING: X is not square.'
    
    do i = 1,min(size(X,1),size(X,2))
        V(i) = X(i,i)
    end do

end function diag_mat
pure function diag_vec(V) result(X)
    
    implicit none
    real(8), dimension(1:), intent(in) :: V
    real(8), dimension(1:size(V),1:size(V)) :: X
    !local
    integer :: i      
    X = 0.d0
    forall (i = 1:size(V))
        X(i,i) = V(i)
    end forall
end function diag_vec

!DIFF Difference and approximate derivative.
!    DIFF(X), for a vector X, is [X(2)-X(1)  X(3)-X(2) ... X(n)-X(n-1)].
!    DIFF(X), for a matrix X, is the matrix of row differences,
!       [X(2:n,:) - X(1:n-1,:)].
!    DIFF(X), for an N-D array X, is the difference along the first
!       non-singleton dimension of X.
!    DIFF(X,N) is the N-th order difference along the first non-singleton 
!       dimension (denote it by DIM). If N >= size(X,DIM), DIFF takes 
!       successive differences along the next non-singleton dimension.
!    DIFF(X,N,DIM) is the Nth difference function along dimension DIM. 
!       If N >= size(X,DIM), DIFF returns an empty array.
! 
!    Examples:
!       h = .001; x = 0:h:pi;
!       diff(sin(x.^2))/h is an approximation to 2*cos(x.^2).*x
!       diff((1:10).^2) is 3:2:19
! 
!       If X = [3 7 5
!               0 9 2]
!       then diff(X,1,1) is [-3 2 -3], diff(X,1,2) is [4 -2
!                                                      9 -7],
!       diff(X,2,2) is the 2nd order difference along the dimension 2, and
!       diff(X,3,2) is the empty matrix.

function diff_vec(x) result(d)
    
    implicit none
    real(8), dimension(1:), intent(in) :: x
    real(8), dimension(1:size(x,1)-1) :: d
    !local
    integer :: n
    
    n = size(x,1)    
    d = x(2:n)-x(1:n-1)        
    
end function diff_vec

function diff_mat(x) result(d)
    
    implicit none
    real(8), dimension(1:,1:), intent(in) :: x
    real(8), dimension(1:size(x,1)-1,1:size(x,2)) :: d
    !local
    integer :: n
    
    n = size(x,1)    
    d = x(2:n,:)-x(1:n-1,:)
    
end function diff_mat


! SORT   Sort in ascending or descending order.
!    For vectors, SORT(X) sorts the elements of X in ascending order.
!    For matrices, SORT(X) sorts each column of X in ascending order.
!    For N-D arrays, SORT(X) sorts the along the first non-singleton
!    dimension of X. When X is a cell array of strings, SORT(X) sorts
!    the strings in ASCII dictionary order.
! 
!    Y = SORT(X,DIM,MODE)
!    has two optional parameters.  
!    DIM selects a dimension along which to sort.
!    MODE selects the direction of the sort
!       'ascend' results in ascending order
!       'descend' results in descending order
!    The result is in Y which has the same shape and type as X.
! 
!    [Y,I] = SORT(X,DIM,MODE) also returns an index matrix I.
!    If X is a vector, then Y = X(I).  
!    If X is an m-by-n matrix and DIM=1, then
!        for j = 1:n, Y(:,j) = X(I(:,j),j); end
! 
!    When X is complex, the elements are sorted by ABS(X).  Complex
!    matches are further sorted by ANGLE(X).
! 
!    When more than one element has the same value, the order of the
!    elements are preserved in the sorted result and the indexes of
!    equal elements will be ascending in any index matrix.
! 
!    Example: If X = [3 7 5
!                     0 4 2]
! 
!    then sort(X,1) is [0 4 2  and sort(X,2) is [3 5 7
!                       3 7 5]                   0 2 4];
! 
subroutine sub_sort_vec_i(y,i,x,dim,mode)
    
    implicit none
    integer, dimension(1:), intent(in) :: x
    integer, dimension(1:size(x)), intent(inout) :: y
    integer, dimension(1:size(x)), intent(out) :: i
    integer, intent(in), optional :: dim
    character(len=*), intent(in), optional :: mode
    !local
    integer, dimension(1:size(x)) :: ylocal
    character(len=20) :: mode_eff
        
    mode_eff = 'ascend'
    if (present(mode)) mode_eff = mode

    if (mode_eff=='ascend') then
        y = x
        call mergesort(y,i)
    elseif (mode_eff=='descend') then
        y = -x
        call mergesort(y,i)
        y = -y
    else
        write(*,*) 'sort: mode not found. stopping'
        STOP
    end if


    !if (.NOT. issorted(y,'ascend')) STOP 'sub_sort_vec failed' ! a good check to have
   
end subroutine sub_sort_vec_i
subroutine sub_sort_vec(y,i,x,dim,mode)
    
    implicit none
    real(8), dimension(1:), intent(in) :: x
    real(8), dimension(1:size(x)), intent(inout) :: y
    integer, dimension(1:size(x)), intent(out) :: i
    integer, intent(in), optional :: dim
    character(len=*), intent(in), optional :: mode
    !local
    real(8), dimension(1:size(x)) :: ylocal
    character(len=20) :: mode_eff
        
    mode_eff = 'ascend'
    if (present(mode)) mode_eff = mode

    if (mode_eff=='ascend') then
        y = x
        call mergesort(y,i)
    elseif (mode_eff=='descend') then
        y = -x
        call mergesort(y,i)
        y = -y
    else
        write(*,*) 'sort: mode not found. stopping'
        STOP
    end if

    !if (.NOT. issorted(y,'ascend')) STOP 'sub_sort_vec failed' ! a good check to have
   
end subroutine sub_sort_vec
subroutine sub_sort_mat(y,i,x,dim,mode)
    
    implicit none
    real(8), dimension(1:,1:), intent(in) :: x
    real(8), dimension(1:size(x,1),1:size(x,2)), intent(inout) :: y
    integer, dimension(1:size(x,1),1:size(x,2)), intent(out) :: i
    integer, intent(in), optional :: dim
    character(len=*), intent(in), optional :: mode
    !local
    integer :: ind    
        
    if (present(dim)) then        
        if (dim==2) then
            !sort each row
            do ind = 1,size(x,1)
                if (present(mode)) then
                    call sub_sort_vec(y(ind,:),i(ind,:),x(ind,:),1,mode)
                else
                    call sub_sort_vec(y(ind,:),i(ind,:),x(ind,:))
                end if            
            end do
            return        
        end if
    end if

    !sort each column
    do ind = 1,size(x,2)
        if (present(mode)) then
            call sub_sort_vec(y(:,ind),i(:,ind),x(:,ind),1,mode)
        else
            call sub_sort_vec(y(:,ind),i(:,ind),x(:,ind),1)
        end if
    end do
    
end subroutine sub_sort_mat
function fn_sort_vec(x,dim,mode) result(y)
    
    implicit none
    real(8), dimension(1:), intent(in) :: x
    real(8), dimension(1:size(x)) :: y
    integer, intent(in), optional :: dim
    character(len=*), intent(in), optional :: mode
    !local
    integer, dimension(1:size(x)):: i
    integer :: mycase
    
    mycase = merge(1,0,present(dim))+merge(2,0,present(mode))
       
    select case (mycase)
        case (0)
            call sub_sort_vec(y,i,x)
        case (1)
            call sub_sort_vec(y,i,x,dim=dim)
        case (2) 
            call sub_sort_vec(y,i,x,mode=mode)
        case (3)
            call sub_sort_vec(y,i,x,dim,mode)
    end select
    
end function fn_sort_vec

!ISSORTED TRUE for sorted vector and matrices.
!    ISSORTED(X), when X is a vector, returns TRUE if the elements of X
!    are in sorted order (in other words, if X and SORT(X) are identical)
!    and FALSE if not. X can be a 1xn or nx1 cell array of strings.
function issorted(x,mode) result(sorted)
    implicit none
    logical :: sorted
    real(8), dimension(1:) :: x
    character(len=*), intent(in) :: mode    !either 'ascend' or 'descend'
    !local
    integer :: i

    if (size(x)==1) then
        sorted = .TRUE. 
        return
    end if

    if (mode=='ascend') then
        do i = 2,size(x)
            if (x(i)<x(i-1)) then
                sorted = .FALSE.
                return
            end if
        end do
    elseif (mode=='descend') then
        do i = 2,size(x)
            if (x(i)>x(i-1)) then
                sorted = .FALSE.
                return
            end if
        end do
    else
        write(*,*) 'issorted: error: sorting mode not found'
        STOP 'issorted: error: sorting mode not found'
    end if

    sorted = .TRUE.

end function issorted


function interp1_scal(x_in,y_in,xi_in) result(yi_out)
    
    implicit none
    real(8), dimension(1:), intent(in) :: x_in
    real(8), dimension(1:), intent(in) :: y_in
    real(8), intent(in) :: xi_in
    real(8) :: yi_out
    !local
    real(8), dimension(1:size(y_in),1:1) :: y
    real(8), dimension(1:1,1:1) :: yi
    real(8), dimension(1:1) :: xi

    y(:,1) = y_in
    xi = xi_in
    yi = interp1_mat(x_in,y,xi)
    yi_out = yi(1,1)

end function interp1_scal

function interp1_vec(x_in,y_in,xi_in) result(yi_out)
    
    implicit none
    real(8), dimension(1:), intent(in) :: x_in
    real(8), dimension(1:), intent(in) :: y_in
    real(8), dimension(1:), intent(in) :: xi_in
    real(8), dimension(1:size(xi_in)) :: yi_out
    !local
    real(8), dimension(1:size(y_in),1:1) :: y
    real(8), dimension(1:size(xi_in),1:1) :: yi

    y(:,1) = y_in    
    yi = interp1_mat(x_in,y,xi_in)
    yi_out = yi(:,1)

end function interp1_vec

function interp1_mat(x_in,y_in,xi_in) result(yi_out)
    
    implicit none
    real(8), dimension(1:), intent(in) :: x_in
    real(8), dimension(1:,1:), intent(in) :: y_in
    real(8), dimension(1:), intent(in) :: xi_in
    real(8), dimension(1:size(xi_in),1:size(y_in,2)) :: yi_out
    !local
    real(8), dimension(1:size(x_in)) :: x
    real(8), dimension(1:size(y_in,1),1:size(y_in,2)) :: y
    real(8), dimension(1:size(xi_in)) :: xi
    real(8), dimension(1:size(xi_in),1:size(y_in,2)) :: yi
    integer, dimension(1:size(x_in)) :: Isortx
    integer, dimension(1:size(xi_in)) :: Isortxi, Irecoverxi
    integer :: nx,nxi,xlo_ind,col_ind,ncol,xi_ind,ny
    real(8) :: xlo,xhi,xhat
    logical :: issorted_ascend_x,issorted_descend_x,issorted_ascend_xi,issorted_descend_xi
    integer, dimension(1:size(xi)) :: tmp
    real(8), dimension(1:size(y,2),1:size(y,1)) :: yperm   !performance enhancements
    real(8), dimension(1:size(yi,2),1:size(yi,1)) :: yiperm !performance enhancements

    nx = size(x_in)
    ny = size(y_in,1)
    ncol = size(y_in,2)
    nxi = size(xi_in)
 
    if (nx/=ny) STOP 'interp1: x and y do not conform'

    x = x_in
    y = y_in
    xi = xi_in

    if (any(isNaN(xi))) then
        write(*,*) 'xi'
        call disp(xi)
        STOP 'interp1: xi contains NaNs'
    end if

    !Check whether data is sorted either in ascending or descending order
    issorted_ascend_x = issorted(x,'ascend')
    issorted_descend_x = .FALSE.
    if (.NOT. issorted_ascend_x) issorted_descend_x = issorted(x,'descend')

    issorted_ascend_xi = issorted(xi,'ascend')
    issorted_descend_xi = .FALSE.
    if (.NOT. issorted_ascend_xi) issorted_descend_xi = issorted(xi,'descend')

    !Sort ascending if data is not sorted
    if (issorted_ascend_x) then
        !Do nothing
    elseif (issorted_descend_x) then
        !Reverse the order or x and y
        x = reverse(x)
        y = reverse_mat(y)
    else
        call sort2(x,Isortx,x)
        do col_ind = 1,ncol
            y(:,col_ind) = y(Isortx,col_ind)
        end do
    end if
    if (issorted_ascend_xi) then
        !Do nothing
    elseif (issorted_descend_xi) then
        xi = reverse(xi)
    else
        call sort2(xi,Isortxi,xi)
        Irecoverxi(Isortxi) = 1 .colon. nxi
    end if


!     !For performance, put the data in row-major order
!     yperm = reshape(y,(/size(y,2),size(y,1)/),ORDER=(/2,1/)) !Do permute
!    !Perform the interpolation for sorted data
!     xlo_ind = 1
!     do xi_ind = 1,nxi
!         xhat = xi(xi_ind)
!         do while (.TRUE.)
!             xlo = x(xlo_ind)
!             if (xlo_ind+1>nx) STOP 'interp1: error'
!             xhi = x(xlo_ind+1)
!             !write(*,*) xhat, xlo, xhi
!             !Check whether xhat is in the range [xlo,xlo+1]
!             if (xhat>=xlo .AND. xhat<=xhi) then
!                 yiperm(:,xi_ind) = yperm(:,xlo_ind) + ((xhat-xlo)/(xhi-xlo))*(yperm(:,xlo_ind+1)-yperm(:,xlo_ind))
!                 exit
!             elseif (xhat>=xhi) then
!                 if (xlo_ind+1<nx) then
!                     xlo_ind = xlo_ind+1
!                 else
!                     !Must be extrapolation upward
!                     yiperm(:,xi_ind) = yperm(:,xlo_ind) + ((xhat-xlo)/(xhi-xlo))*(yperm(:,xlo_ind+1)-yperm(:,xlo_ind))
!                     exit
!                 end if
!             elseif (xhat<=xlo) then
!                 !Must be extrapolation downward
!                 yiperm(:,xi_ind) = yperm(:,xlo_ind) + ((xhat-xlo)/(xhi-xlo))*(yperm(:,xlo_ind+1)-yperm(:,xlo_ind))
!                 exit
!             else 
!                 write(*,*) 'xhat   xhi  xlo, xi_ind, nxi '
!                 write(*,*) xhat, xhi, xlo, xi_ind, nxi
!                 write(*,*) 'x '
!                 call disp(x)
!                 write(*,*) 'y '
!                 call disp(y)
!                 write(*,*) 'xi '
!                 call disp(xi)
!                 STOP 'interp1: case not found'
!             end if
!         end do
!     end do
!     yi = reshape(yiperm,(/size(yi,1),size(yi,2)/),ORDER=(/2,1/)) !Undo permute
    

    !Perform the interpolation for sorted data
    xlo_ind = 1
    do xi_ind = 1,nxi
        xhat = xi(xi_ind)
        do while (.TRUE.)
            xlo = x(xlo_ind)
            if (xlo_ind+1>nx) STOP 'interp1: error'
            xhi = x(xlo_ind+1)
            !write(*,*) xhat, xlo, xhi
            !Check whether xhat is in the range [xlo,xlo+1]
            if (xhat>=xlo .AND. xhat<=xhi) then
                yi(xi_ind,:) = y(xlo_ind,:) + ((xhat-xlo)/(xhi-xlo))*(y(xlo_ind+1,:)-y(xlo_ind,:))
                exit
            elseif (xhat>=xhi) then
                if (xlo_ind+1<nx) then
                    xlo_ind = xlo_ind+1
                else
                    !Must be extrapolation upward
                    yi(xi_ind,:) = y(xlo_ind,:) + ((xhat-xlo)/(xhi-xlo))*(y(xlo_ind+1,:)-y(xlo_ind,:))
                    exit
                end if
            elseif (xhat<=xlo) then
                !Must be extrapolation downward
                yi(xi_ind,:) = y(xlo_ind,:) + ((xhat-xlo)/(xhi-xlo))*(y(xlo_ind+1,:)-y(xlo_ind,:))
                exit
            else 
                write(*,*) 'xhat   xhi  xlo, xi_ind, nxi '
                write(*,*) xhat, xhi, xlo, xi_ind, nxi
                write(*,*) 'x '
                call disp(x)
                write(*,*) 'y '
                call disp(y)
                write(*,*) 'xi '
                call disp(xi)
                STOP 'interp1: case not found'
            end if
        end do
    end do

    !Unsort the data
    if (issorted_ascend_xi) then
        !Do nothing
    elseif (issorted_descend_xi) then
        yi = reverse_mat(yi)
    else
        !write(*,*) 'xi before'
        !call disp(xi)
        !write(*,*) 'xi after'
        !call disp(xi(Irecoverxi))
        !write(*,*) 'yi before'
        !call disp(yi)
        do col_ind = 1,ncol
            yi(:,col_ind) = yi(Irecoverxi,col_ind)
        end do
        !write(*,*) 'yi after'
        !call disp(yi)
    end if

    !Write to result
    yi_out = yi

end function interp1_mat

function reverse(x) result(xr)
    
    implicit none
    real(8), dimension(1:), intent(in) :: x
    real(8), dimension(1:size(x)) :: xr
    !local
    integer :: i,nx
    nx = size(x)
    do i = 1,nx
        xr(i) = x(nx-i+1)
    end do
end function reverse
function reverse_mat(y) result(yr)
    
    implicit none
    real(8), dimension(1:,1:), intent(in) :: y
    real(8), dimension(1:size(y,1),1:size(y,2)) :: yr
    !local
    integer :: i,ny
    ny = size(y,1)
    do i = 1,ny
        yr(i,:) = y(ny-i+1,:)
    end do
end function reverse_mat


! %INTERP1Q Quick 1-D linear interpolation.
! %   F=INTERP1Q(X,Y,XI) returns the value of the 1-D function Y at the points
! %   of column vector XI using linear interpolation. Length(F)=length(XI).
! %   The vector X specifies the coordinates of the underlying interval.
! %   
! %   If Y is a matrix, then the interpolation is performed for each column
! %   of Y in which case F is length(XI)-by-size(Y,2).
! %
! %   NaN's are returned for values of XI outside the coordinates in X.
! %
! %   INTERP1Q is quicker than INTERP1 on non-uniformly spaced data because
! %   it does no input checking. For INTERP1Q to work properly:
! %   X must be a monotonically increasing column vector.
! %   Y must be a column vector or matrix with length(X) rows.
! %
! %   Class support for inputs x, y, xi:
! %      float: double, single
! %
! %   See also INTERP1.
! 
! %   Copyright 1984-2008 The MathWorks, Inc.
! %   $Revision: 1.15.4.3 $  $Date: 2008/03/28 15:24:12 $
function interp1q_vec(x,y,xi) result(yi)
    
    implicit none
    real(8), dimension(1:), intent(in) :: x,xi
    real(8), dimension(1:), intent(in) :: y
    real(8), dimension(1:size(xi)) :: yi
    !local
    real(8), dimension(1:size(xi)) :: xxi
    integer, dimension(1:size(xi)) :: k
    real(8), dimension(1:size(x)+size(xxi)) :: dum
    integer, dimension(1:size(x)+size(xxi)) :: j,ra
    integer, dimension(1:size(xxi)) :: rb
    integer, dimension(:), allocatable :: ind,rind
    real(8), dimension(:), allocatable :: u,yrind,xrind
    integer, dimension(:), allocatable :: rc
    real(8), dimension(1:1) :: yr
    integer, dimension(1:1) :: mintmp,maxtmp
!     siz = size(xi);
    if (size(xi)>1) then 
        call sub_sort_vec(xxi,k,xi) ![xxi, k] = sort(xi);
        call sub_sort_vec(dum,j,(/x,xxi/)) ![dum, j] = sort([x;xxi]);
        ra(j) = 1 .colon. size(j) !r(j) = 1:length(j);
        rb = ra(size(x)+1:) - (1 .colon. size(xxi)); !r = r(length(x)+1:end) - (1:length(xxi));
        rb(k) = rb !r(k) = r;
        where (xi==x(size(x))) rb = size(x)-1 !r(xi==x(end)) = length(x)-1;
        
        allocate(ind(count((rb>0) .AND. (rb<size(x))))) !ind = find((r>0) & (r<length(x)));
        ind = find((rb>0) .AND. (rb<size(x))) 
                                            !ind = ind(:);
!         yi = -1.d0 !yi = NaN(length(xxi),size(y,2),superiorfloat(x,y,xi));
        mintmp=minloc(x)
        maxtmp=maxloc(x)
        where(xi<=minval(x)) 
            yi = y(mintmp(1))
        elsewhere(xi>=maxval(x))
            yi = y(maxtmp(1))
        endwhere

        allocate(rind(size(ind))) !rind = r(ind);
        rind = rb(ind) 

        allocate(xrind(size(rind))) !xrind = x(rind);
        xrind = x(rind)

        allocate(u(size(xrind))) !u = (xi(ind)-xrind)./(x(rind+1)-xrind);
        u = (xi(ind)-xrind)/(x(rind+1)-xrind) 

        allocate(yrind(size(rind))) !yrind = y(rind,:);
        yrind = y(rind)

        yi(ind) = yrind + (y(rind+1)-yrind)*u !yi(ind,:)=yrind + bsxfun(@times,y(rind+1,:)-yrind,u);
    else
        ! Special scalar xi case                
        allocate(rc(1:min(count(x<=xi(1)),1))) !r = find(x <= xi,1,'last');
        rc = find(x <= xi(1),1,'last') 
        where (xi==x(size(x))) rc = size(x)-1 !r(xi==x(end)) = length(x)-1;
        if (size(rc)==0 .OR. any(rc<=0) .OR. any(rc>=size(x))) then !if isempty(r) || r<=0 || r>=length(x)
            !yi = -1.d0 !yi = NaN(1,size(y,2),superiorfloat(x,y,xi));
            mintmp=minloc(x)
            maxtmp=maxloc(x)
            where(xi<=minval(x)) 
                yi = y(mintmp(1))
            elsewhere(xi>=maxval(x))
                yi = y(maxtmp(1))
            endwhere
        else
            allocate(u(1)) !u = (xi-x(r))./(x(r+1)-x(r));
            u = (xi-x(rc))/(x(rc+1)-x(rc))
            yr = y(rc) !yr = y(r,:);
            yi = yr(1) + (y(rc+1)-yr(1))*u !yi = yr + bsxfun(@times,y(r+1,:)-yr,u);
        end if
    end if

!     if min(size(yi)) == 1 && numel(xi) > 1
!         yi = reshape(yi,siz); 
!     end

end function interp1q_vec

function interp1q_scal(x,y,xi) result(yi)
    
    implicit none
    real(8), dimension(1:), intent(in) :: x
    real(8), intent(in) :: xi
    real(8), dimension(1:), intent(in) :: y
    real(8) :: yi
    !local
    real(8) :: xitmp(1),yitmp(1)
    xitmp = xi
    yitmp = interp1q_vec(x,y,xitmp)
    yi = yitmp(1)
end function interp1q_scal





function cumprod_dp(x) result(y)
    implicit none
    real(8), intent(in) :: x(1:)
    real(8)  :: y(1:size(x,1))
    !local
    integer :: i,n
    n = size(x,1)
    do i = 1,n
        y(i)=product(x(1:i))
    end do
end function cumprod_dp

function cumprod_i(x) result(y)
    implicit none
    integer, intent(in) :: x(1:)
    integer  :: y(1:size(x,1))
    !local
    integer :: i,n
    n = size(x,1)
    do i = 1,n
        y(i)=product(x(1:i))
    end do
end function cumprod_i


! %SUB2IND Linear index from multiple subscripts.
! %   SUB2IND is used to determine the equivalent single index
! %   corresponding to a given set of subscript values.
! %
! %   IND = SUB2IND(SIZ,I,J) returns the linear index equivalent to the
! %   row and column subscripts in the arrays I and J for a matrix of
! %   size SIZ. 
! %
! %   IND = SUB2IND(SIZ,I1,I2,...,IN) returns the linear index
! %   equivalent to the N subscripts in the arrays I1,I2,...,IN for an
! %   array of size SIZ.
! %
! %   I1,I2,...,IN must have the same size, and IND will have the same size
! %   as I1,I2,...,IN. For an array A, if IND = SUB2IND(SIZE(A),I1,...,IN)),
! %   then A(IND(k))=A(I1(k),...,IN(k)) for all k.
! %
! %   Class support for inputs I,J: 
! %      float: double, single
! %
! %   See also IND2SUB.
! 
! %   Copyright 1984-2008 The MathWorks, Inc.
! %   $Revision: 1.14.4.8 $  $Date: 2008/05/23 15:34:46 $
function sub2ind_3dim(siz,i1,i2,i3) result(ndx)
    implicit none
    integer, dimension(1:), intent(in) :: siz
    integer, dimension(1:,1:,1:), intent(in) :: i1,i2,i3
    integer, dimension(1:size(i1,1),1:size(i1,2),1:size(i1,3)) :: ndx
    !local
    integer, parameter :: rank = 3
    integer, dimension(1:size(i1,1),1:size(i1,2),1:size(i1,3)) :: v
    integer, dimension(1:rank) :: k
    integer :: i

    if (size(siz)/=rank) then
        write(*,*) 'sub2ind: error: incorrect rank ', rank
        STOP 
    end if

    !Compute linear indices
    k = (/1,cumprod(siz(1:rank-1))/)
    ndx = 1
    do i = 1,rank
        select case (i)
            case (1)
                v = i1
            case (2)
                v = i2
            case (3)
                v = i3
            case default
                write(*,*) 'sub2ind: error: case not found.  case: ', i
                STOP 
        end select
        
        if (any(v < 1) .OR. any(v > siz(i))) then            
            write(*,*) 'sub2ind: error: subscripts out of range. min max :', minval(v),maxval(v)
            STOP         
        end if
        ndx = ndx + (v-1)*k(i)
    end do

end function sub2ind_3dim

function sub2ind_2dim(siz,i1,i2) result(ndx)
    implicit none
    integer, dimension(1:), intent(in) :: siz
    integer, dimension(1:,1:), intent(in) :: i1,i2
    integer, dimension(1:size(i1,1),1:size(i1,2)) :: ndx
    !local
    integer, parameter :: rank = 2
    integer, dimension(1:size(i1,1),1:size(i1,2)) :: v
    integer, dimension(1:rank) :: k
    integer :: i

    if (size(siz)/=rank) then
        write(*,*) 'sub2ind: error: incorrect rank ', rank
        STOP 
    end if

    !Compute linear indices
    k = (/1,cumprod(siz(1:rank-1))/)
    ndx = 1
    do i = 1,rank
        select case (i)
            case (1)
                v = i1
            case (2)
                v = i2
            case default
                write(*,*) 'sub2ind: error: case not found.  case: ', i
                STOP 
        end select
        
        if (any(v < 1) .OR. any(v > siz(i))) then            
            write(*,*) 'sub2ind: error: subscripts out of range. min max :', minval(v),maxval(v)
            STOP         
        end if
        ndx = ndx + (v-1)*k(i)
    end do

end function sub2ind_2dim
    

!  REM    Remainder after division.
!     REM(x,y) is x - n.*y where n = fix(x./y) if y ~= 0.  If y is not an
!     integer and the quotient x./y is within roundoff error of an integer,
!     then n is that integer. The inputs x and y must be real arrays of the
!     same size, or real scalars.
!  
!     By convention:
!        REM(x,0) is NaN.
!        REM(x,x), for x~=0, is 0.
!        REM(x,y), for x~=y and y~=0, has the same sign as x.
!  
!     Note: MOD(x,y), for x~=y and y~=0, has the same sign as y.
!     REM(x,y) and MOD(x,y) are equal if x and y have the same sign, but
!     differ by y if x and y have different signs.
 
elemental function rem(x,y) 
    
    implicit none
    real(8), intent(in) :: x,y
    real(8) :: rem
    !local
    if (y==0.d0) then   
        rem = 1.d0/0.d0
    else
        rem = x-fix(x/y)*y
    end if
end function rem


! %IND2SUB Multiple subscripts from linear index.
! %   IND2SUB is used to determine the equivalent subscript values
! %   corresponding to a given single index into an array.
! %
! %   [I,J] = IND2SUB(SIZ,IND) returns the arrays I and J containing the
! %   equivalent row and column subscripts corresponding to the index
! %   matrix IND for a matrix of size SIZ.  
! %   For matrices, [I,J] = IND2SUB(SIZE(A),FIND(A>5)) returns the same
! %   values as [I,J] = FIND(A>5).
! %
! %   [I1,I2,I3,...,In] = IND2SUB(SIZ,IND) returns N subscript arrays
! %   I1,I2,..,In containing the equivalent N-D array subscripts
! %   equivalent to IND for an array of size SIZ.
! %
! %   Class support for input IND:
! %      float: double, single
! %
! %   See also SUB2IND, FIND.
!  
! %   Copyright 1984-2008 The MathWorks, Inc. 
! %   $Revision: 1.13.4.4 $  $Date: 2008/05/23 15:34:40 $

subroutine ind2sub_3dim(i1,i2,i3,siz,ndx_in)
    implicit none
    integer, dimension(1:), intent(in) :: siz
    integer, dimension(1:,1:,1:), intent(in) :: ndx_in
    integer, dimension(1:size(ndx_in,1),1:size(ndx_in,2),1:size(ndx_in,3)), intent(out) :: i1,i2,i3
    !local
    integer, parameter :: rank = 3
    integer :: n,i
    integer, dimension(1:rank) :: k
    integer, dimension(1:size(ndx_in,1),1:size(ndx_in,2),1:size(ndx_in,3)) :: vi,vj,ndx

    ndx = ndx_in

    if (size(siz)/=rank) then
        write(*,*) 'ind2sub: error: incorrect rank ', rank
        STOP 
    end if

    n = rank
    k = (/1,cumprod(siz(1:rank-1))/)
    do i = n,1,-1
        vi = rem(real(ndx-1,8),real(k(i),8)) + 1;         
        vj = (ndx - vi)/k(i) + 1; 
        select case (i)
            case (3)
                i3 = vj
            case (2)
                i2 = vj
            case (1)
                i1 = vj
            case default
                write(*,*) 'ind2sub: error: case not found. case: ',i
                STOP
        end select
        ndx = vi
    end do

end subroutine ind2sub_3dim

subroutine ind2sub_2dim(i1,i2,siz,ndx_in)
    implicit none
    integer, dimension(1:), intent(in) :: siz
    integer, dimension(1:,1:), intent(in) :: ndx_in
    integer, dimension(1:size(ndx_in,1),1:size(ndx_in,2)), intent(out) :: i1,i2
    !local
    integer, parameter :: rank = 2
    integer :: n,i
    integer, dimension(1:rank) :: k
    integer, dimension(1:size(ndx_in,1),1:size(ndx_in,2)) :: vi,vj,ndx

    ndx = ndx_in

    if (size(siz)/=rank) then
        write(*,*) 'ind2sub: error: incorrect rank ', rank
        STOP 
    end if

    n = rank
    k = (/1,cumprod(siz(1:rank-1))/)
    do i = n,1,-1
        vi = rem(real(ndx-1,8),real(k(i),8)) + 1;         
        vj = (ndx - vi)/k(i) + 1; 
        select case (i)
            case (2)
                i2 = vj
            case (1)
                i1 = vj
            case default
                write(*,*) 'ind2sub: error: case not found. case: ',i
                STOP
        end select
        ndx = vi
    end do

end subroutine ind2sub_2dim


!INTERP2 2-D interpolation (table lookup).
!    ZI = INTERP2(X,Y,Z,XI,YI) interpolates to find ZI, the values of the
!    underlying 2-D function Z at the points in vectors XI and YI.
!    Vectors X and Y specify the points at which the data Z is given.
! 
!    XI is a vector.  Similarly, YI can be a column vector and it 
!    specifies a matrix with constant rows. 
!
!Option specifies the following
! 0: (default) allow extrapolation
! 1: only interpolation
!
function interp2(x,y,z,xi,yi,option) result(zi)
    
    implicit none
    real(8), dimension(1:), intent(in) :: x, y, xi, yi
    real(8), dimension(1:,1:), intent(in) :: z 
    real(8), dimension(1:size(xi),1:size(yi)) :: zi
    integer, intent(in), optional :: option
    !local
    integer :: xi_ind, yi_ind
    
    !Check inputs
    if (size(z,1)/=size(x)) STOP 'interp2: z does not have #x rows'
    if (size(z,2)/=size(y)) STOP 'interp2: z does not have #y columns'

    !Interpolate for each point
    do xi_ind = 1,size(xi)
        do yi_ind = 1,size(yi)
            if (present(option)) then
                zi(xi_ind,yi_ind) = interp2_pt(x,y,z,xi(xi_ind),yi(yi_ind),option)
            else
                zi(xi_ind,yi_ind) = interp2_pt(x,y,z,xi(xi_ind),yi(yi_ind))
            end if
        end do
    end do

end function interp2

function interp2_pt(x,y,z,xi,yi,option) result(zi)
    
    implicit none
    real(8), dimension(1:), intent(in) :: x, y
    real(8), dimension(1:,1:), intent(in) :: z 
    real(8) :: xi, yi
    real(8) :: zi
    integer, intent(in), optional :: option
    !local
    integer :: len_x, len_y, xi_ilo, xi_ihi, yi_ilo, yi_ihi, xi_ind, yi_ind, x_ind, y_ind
    real(8) :: xi_mlo, xi_mhi, mu_x, yi_mlo, yi_mhi, mu_y
    integer, dimension(1:2) :: flags
    integer :: flag
    real(8) :: sum

    len_x = size(x)
    len_y = size(y)
    
    !Check inputs
    if (size(z,1)/=size(x)) STOP 'interp2: z does not have #x rows'
    if (size(z,2)/=size(y)) STOP 'interp2: z does not have #y columns'
   
    !Get the closest grid points for each moment on their respective grids
    call interp_get_interval(xi_ilo,xi_ihi,xi_mlo,xi_mhi,xi,x,len_x,flag); flags(1) = flag
    call interp_get_interval(yi_ilo,yi_ihi,yi_mlo,yi_mhi,yi,y,len_y,flag); flags(2) = flag

    if (present(option)) then                
        if (option==1) then
            !Don't allow extrapolation
            xi_mhi = max(min(xi_mhi,1.d0),0.d0)
            xi_mlo = max(min(xi_mlo,1.d0),0.d0)
            yi_mhi = max(min(yi_mhi,1.d0),0.d0)
            yi_mlo = max(min(yi_mlo,1.d0),0.d0)
        end if
    end if

    !Joint interpolation weight is just mu(x,y) = mu(x)*mu(y)
    sum = 0.d0
    do xi_ind = 1,2
        x_ind = merge(xi_ilo,xi_ihi,xi_ind==1)
        mu_x = merge(xi_mlo,xi_mhi,xi_ind==1)       
        do yi_ind = 1,2
            y_ind = merge(yi_ilo,yi_ihi,yi_ind==1)
            mu_y = merge(yi_mlo,yi_mhi,yi_ind==1)

            sum = sum + (mu_x*mu_y)*z(x_ind,y_ind)                        
        end do
    end do
    zi = sum

end function interp2_pt


!Matlab style wrapper for my interp subroutine
!INTERP3 3-D interpolation (table lookup).
!    VI = INTERP3(X1,X2,X3,V,X1I,X2I) interpolates to find VI, the values of the
!    underlying 3-D function V at the points in vectors X1I and Y2I.
! 
!    X1I and X2I are vectors.  
!
!Option specifies the following
! 0: (default) allow extrapolation
! 1: only interpolation
!
function interp3(x1,x2,x3,v,x1i,x2i,x3i,option) result(vi)
    
    implicit none
    real(8), dimension(1:), intent(in) :: x1, x2, x3, x1i, x2i, x3i
    real(8), dimension(1:,1:,1:), intent(in) :: v 
    real(8), dimension(1:size(x1i),1:size(x2i),1:size(x3i)) :: vi
    integer, intent(in), optional :: option
    !local
    integer :: x1i_ind, x2i_ind, x3i_ind
    
    !Check inputs
    if (size(v,1)/=size(x1)) STOP 'interp3: v does not have #x1 rows'
    if (size(v,2)/=size(x2)) STOP 'interp3: v does not have #x2 columns'
    if (size(v,3)/=size(x3)) STOP 'interp3: v does not have #x3 columns'

    !Interpolate for each point
    do x1i_ind = 1,size(x1i)
        do x2i_ind = 1,size(x2i)
            do x3i_ind = 1,size(x3i)
                if (present(option)) then
                    vi(x1i_ind,x2i_ind,x3i_ind) = interp3_pt(x1,x2,x3,v,x1i(x1i_ind),x2i(x2i_ind),x3i(x3i_ind),option)
                else
                    vi(x1i_ind,x2i_ind,x3i_ind) = interp3_pt(x1,x2,x3,v,x1i(x1i_ind),x2i(x2i_ind),x3i(x3i_ind))
                end if
            end do
        end do
    end do

end function interp3


function interp3_pt(x1,x2,x3,v,x1i,x2i,x3i,option) result(vi)
    
    implicit none
    real(8), dimension(1:), intent(in) :: x1, x2, x3
    real(8), dimension(1:,1:,1:), intent(in) :: v 
    real(8) :: x1i, x2i, x3i
    real(8) :: vi
    integer, intent(in), optional :: option
    !local
    integer :: len_x1, len_x2, len_x3, &
               x1i_ilo, x1i_ihi, x2i_ilo, x2i_ihi, x3i_ilo, x3i_ihi, &
               x1i_ind, x2i_ind, x3i_ind, &
               x1_ind, x2_ind, x3_ind
    real(8) :: x1i_mlo, x1i_mhi, mu_x1, &
                x2i_mlo, x2i_mhi, mu_x2, &
                x3i_mlo, x3i_mhi, mu_x3
    integer, dimension(1:3) :: flags
    integer :: flag
    real(8) :: sum

    len_x1 = size(x1)
    len_x2 = size(x2)
    len_x3 = size(x3)
    
    !Check inputs
    if (size(v,1)/=size(x1)) STOP 'interp3: z does not have #x1 rows'
    if (size(v,2)/=size(x2)) STOP 'interp3: z does not have #x2 columns'
    if (size(v,3)/=size(x3)) STOP 'interp3: z does not have #x3 columns'
   
    !Get the closest grid points for each moment on their respective grids
    call interp_get_interval(x1i_ilo,x1i_ihi,x1i_mlo,x1i_mhi,x1i,x1,len_x1,flag); flags(1) = flag
    call interp_get_interval(x2i_ilo,x2i_ihi,x2i_mlo,x2i_mhi,x2i,x2,len_x2,flag); flags(2) = flag
    call interp_get_interval(x3i_ilo,x3i_ihi,x3i_mlo,x3i_mhi,x3i,x3,len_x3,flag); flags(3) = flag

    if (present(option)) then                
        if (option==1) then
            !Don't allow extrapolation
            x1i_mhi = max(min(x1i_mhi,1.d0),0.d0)
            x1i_mlo = max(min(x1i_mlo,1.d0),0.d0)
            x2i_mhi = max(min(x2i_mhi,1.d0),0.d0)
            x2i_mlo = max(min(x2i_mlo,1.d0),0.d0)
            x3i_mhi = max(min(x3i_mhi,1.d0),0.d0)
            x3i_mlo = max(min(x3i_mlo,1.d0),0.d0)
        end if
    end if

    !Joint interpolation weight is just mu(x1,x2) = mu(x1)*mu(x2)
    sum = 0.d0
    do x1i_ind = 1,2
        x1_ind = merge(x1i_ilo,x1i_ihi,x1i_ind==1)
        mu_x1 = merge(x1i_mlo,x1i_mhi,x1i_ind==1)       
        do x2i_ind = 1,2
            x2_ind = merge(x2i_ilo,x2i_ihi,x2i_ind==1)
            mu_x2 = merge(x2i_mlo,x2i_mhi,x2i_ind==1)
            do x3i_ind = 1,2
                x3_ind = merge(x3i_ilo,x3i_ihi,x3i_ind==1)
                mu_x3 = merge(x3i_mlo,x3i_mhi,x3i_ind==1)

                sum = sum + (mu_x1*mu_x2*mu_x3)*v(x1_ind,x2_ind,x3_ind)                        
            end do
        end do
    end do
    vi = sum

end function interp3_pt

!Matlab style wrapper for my interp subroutine
!INTERP4 4-D interpolation (table lookup).
!    VI = INTERP3(X1,X2,X3,X4,V,X1I,X2I,X3I,X4I) interpolates to find VI, the values of the
!    underlying function V at the points in vectors X1I and Y2I.
! 
!    Note X1,X2,X3,X4 need not be sorted, but V must be structured V(a,b,c,d) = V(X1(a),X2(b),...)
!    X1I,X2I,X3I,X4I are vectors.  
!
!Option specifies the following
! 0: (default) allow extrapolation
! 1: only interpolation
!
function interp4(x1,x2,x3,x4,v,x1i,x2i,x3i,x4i,option) result(vi)
    
    implicit none
    real(8), dimension(1:), intent(in) :: x1, x2, x3, x4, x1i, x2i, x3i, x4i
    real(8), dimension(1:,1:,1:,1:), intent(in) :: v 
    real(8), dimension(1:size(x1i),1:size(x2i),1:size(x3i),1:size(x4i)) :: vi
    integer, intent(in), optional :: option
    !local
    integer :: x1i_ind, x2i_ind, x3i_ind, x4i_ind
    
    !Check inputs
    if (size(v,1)/=size(x1)) STOP 'interp4: v does not have #x1 rows'
    if (size(v,2)/=size(x2)) STOP 'interp4: v does not have #x2 columns'
    if (size(v,3)/=size(x3)) STOP 'interp4: v does not have #x3 columns'
    if (size(v,4)/=size(x4)) STOP 'interp4: v does not have #x4 columns'

    !Interpolate for each point
    do x1i_ind = 1,size(x1i)
        do x2i_ind = 1,size(x2i)
            do x3i_ind = 1,size(x3i)
                do x4i_ind = 1,size(x4i)
                    if (present(option)) then
                        vi(x1i_ind,x2i_ind,x3i_ind,x4i_ind) = &
                            interp4_pt(x1,x2,x3,x4,v,x1i(x1i_ind),x2i(x2i_ind),x3i(x3i_ind),x4i(x4i_ind),option)
                    else
                        vi(x1i_ind,x2i_ind,x3i_ind,x4i_ind) = &
                            interp4_pt(x1,x2,x3,x4,v,x1i(x1i_ind),x2i(x2i_ind),x3i(x3i_ind),x4i(x4i_ind))
                    end if
                end do
            end do
        end do
    end do

end function interp4

function interp4_pt(x1,x2,x3,x4,v,x1i,x2i,x3i,x4i,option) result(vi)
    
    implicit none
    real(8), dimension(1:), intent(in) :: x1, x2, x3, x4
    real(8), dimension(1:,1:,1:,1:), intent(in) :: v 
    real(8) :: x1i, x2i, x3i, x4i
    real(8) :: vi
    integer, intent(in), optional :: option
    !local
    integer :: len_x1, x1i_ilo, x1i_ihi, x1i_ind, x1_ind, &
               len_x2, x2i_ilo, x2i_ihi, x2i_ind, x2_ind, &
               len_x3, x3i_ilo, x3i_ihi, x3i_ind, x3_ind, &
               len_x4, x4i_ilo, x4i_ihi, x4i_ind, x4_ind               
    real(8) :: x1i_mlo, x1i_mhi, mu_x1, &
                x2i_mlo, x2i_mhi, mu_x2, &
                x3i_mlo, x3i_mhi, mu_x3, &
                x4i_mlo, x4i_mhi, mu_x4
    integer, dimension(1:4) :: flags
    integer :: flag
    real(8) :: sum

    len_x1 = size(x1)
    len_x2 = size(x2)
    len_x3 = size(x3)
    len_x4 = size(x4)
    
    !Check inputs
    if (size(v,1)/=size(x1)) STOP 'interp4: z does not have #x1 rows'
    if (size(v,2)/=size(x2)) STOP 'interp4: z does not have #x2 columns'
    if (size(v,3)/=size(x3)) STOP 'interp4: z does not have #x3 columns'
    if (size(v,4)/=size(x4)) STOP 'interp4: z does not have #x4 columns'
   
    !Get the closest grid points for each moment on their respective grids
    call interp_get_interval(x1i_ilo,x1i_ihi,x1i_mlo,x1i_mhi,x1i,x1,len_x1,flag); flags(1) = flag
    call interp_get_interval(x2i_ilo,x2i_ihi,x2i_mlo,x2i_mhi,x2i,x2,len_x2,flag); flags(2) = flag
    call interp_get_interval(x3i_ilo,x3i_ihi,x3i_mlo,x3i_mhi,x3i,x3,len_x3,flag); flags(3) = flag
    call interp_get_interval(x4i_ilo,x4i_ihi,x4i_mlo,x4i_mhi,x4i,x4,len_x4,flag); flags(4) = flag
    
    if (present(option)) then                
        if (option==1) then
            !Don't allow extrapolation
            x1i_mhi = max(min(x1i_mhi,1.d0),0.d0)
            x1i_mlo = max(min(x1i_mlo,1.d0),0.d0)
            x2i_mhi = max(min(x2i_mhi,1.d0),0.d0)
            x2i_mlo = max(min(x2i_mlo,1.d0),0.d0)
            x3i_mhi = max(min(x3i_mhi,1.d0),0.d0)
            x3i_mlo = max(min(x3i_mlo,1.d0),0.d0)
            x4i_mhi = max(min(x4i_mhi,1.d0),0.d0)
            x4i_mlo = max(min(x4i_mlo,1.d0),0.d0)
        end if
    end if

    !Joint interpolation weight is just mu(x1,x2,x3,x4) = mu(x1)*mu(x2)*mu(x3)*mu(x4)
    sum = 0.d0
    do x1i_ind = 1,2
        x1_ind = merge(x1i_ilo,x1i_ihi,x1i_ind==1)
        mu_x1 = merge(x1i_mlo,x1i_mhi,x1i_ind==1)       
        do x2i_ind = 1,2
            x2_ind = merge(x2i_ilo,x2i_ihi,x2i_ind==1)
            mu_x2 = merge(x2i_mlo,x2i_mhi,x2i_ind==1)
            do x3i_ind = 1,2
                x3_ind = merge(x3i_ilo,x3i_ihi,x3i_ind==1)
                mu_x3 = merge(x3i_mlo,x3i_mhi,x3i_ind==1)
                do x4i_ind = 1,2
                    x4_ind = merge(x4i_ilo,x4i_ihi,x4i_ind==1)
                    mu_x4 = merge(x4i_mlo,x4i_mhi,x4i_ind==1)

                    sum = sum + (mu_x1*mu_x2*mu_x3*mu_x4)*v(x1_ind,x2_ind,x3_ind,x4_ind)
                end do
            end do
        end do
    end do
    vi = sum

end function interp4_pt

pure subroutine interp_get_interval(ilo,ihi,mlo,mhi,x,grid_x,len_x,flag)
implicit none

integer, intent(in) :: len_x
real(8), dimension(1:len_x), intent(in) :: grid_x
real(8), intent(in) :: x
integer, intent(out) :: ilo, ihi
real(8), intent(out) :: mlo, mhi
integer, intent(out), optional :: flag

integer, dimension(1:1) :: loc_dummy
logical, dimension(1:len_x) :: check
real(8) :: dist

ilo = -1
ihi = -1
mlo = 1.d14
mhi = 1.d14

!Check if only one point on grid
if (len_x==1) then
    if (present(flag)) flag = 3
    ilo = 1 
    ihi = 1
    mlo = 1.d0
    mhi = 0.d0
else

    !Is this interpolation?
    if (any(grid_x<=x) .AND. any(grid_x>=x)) then 

        if (present(flag)) flag = 0

        check = grid_x<=x
        loc_dummy = minloc(abs(grid_x-x),mask=check)
        ilo = loc_dummy(1)

        check = grid_x>=x
        loc_dummy = minloc(abs(grid_x-x),mask=check)
        ihi = loc_dummy(1)   

    !Is this extrapolation downward?
    elseif (all(grid_x>x)) then

        if (present(flag)) flag = 1

        check = grid_x>x
        loc_dummy = minloc(abs(grid_x-x),mask=check)
        ilo = loc_dummy(1)

        check(ilo) = .FALSE.
        loc_dummy = minloc(abs(grid_x-x),mask=check)
        ihi = loc_dummy(1)    

    !Is this extrapolation upward
    elseif (all(grid_x<x)) then

        if (present(flag)) flag = 2

        check = grid_x<x
        loc_dummy = minloc(abs(grid_x-x),mask=check)
        ihi = loc_dummy(1)

        check(ihi) = .FALSE.
        loc_dummy = minloc(abs(grid_x-x),mask=check)
        ilo = loc_dummy(1)    
       
    else
        
        if (present(flag)) flag = 4

    end if

    !Get associated weights
    if (ilo/=-1 .AND. ihi /=-1) then

        dist = abs(grid_x(ilo)-grid_x(ihi))
        if(dist<=1d-40) then 
            mhi=0.d0
            mlo=1.d0
        else
            mhi=(x-grid_x(ilo))/dist
            mlo=1.d0-mhi
        end if

    end if

end if    


end subroutine interp_get_interval

end module mod_matlab


!TESTS FOR THE ABOVE MODULE
module test_mod_matlab
    use mod_matlab
    implicit none
contains
    subroutine test_sub2ind_and_ind2sub
        integer :: i1(3,4,2),i2(3,4,2),i3(3,4,2),ndx(3,4,2)
        integer :: j1(10,5),j2(10,5),jdx(10,5)

        !TEST SUB2IND
        !Test 3dim
        i1 = 1
        i2 = 4
        i2(1,:,2) = 85
        i3 = 2
        i1(2,:,:) = 2
        ndx = sub2ind((/2,86,3/),i1,i2,i3)
        call disp(vec(ndx))
        !SHOULD GIVE THE SAME AS FOLLOWING MATLAB CODE: i1 = ones(3,4,2); i2 = 4*i1; i2(1,:,2) = 85; i3=2*i1; i1(2,:,:) = 2; vec = sub2ind([2,86,3],i1,i2,i3); vec(:)
        !Test 2dim
        j1 = 3
        j1(:,3) = 1.colon.10        
        j2 = 5
        j2(3,3) = 7        
        jdx = sub2ind((/15,1000/),j1,j2)
        call disp(vec(jdx))
        !SHOULD GIVE SAME AS FOLLOWING MATLAB CODE: j1 = 3*ones(10,5); j1(:,3)=1:10;j2 = 5*ones(10,5);j2(3,3)=7;vec=sub2ind([15 1000],j1,j2); vec(:)    

        !TEST IND2SUB
        !Test 3dim
        i1 = 0
        i2 = 0
        i3 = 0
        call ind2sub(i1,i2,i3,(/2,86,3/),ndx)
        call disp(vec(i1))
        call disp(vec(i2))
        call disp(vec(i3))
        !SHOULD GIVE THE SAME AS FOLLOWING MATLAB CODE: i1 = ones(3,4,2); i2 = 4*i1; i2(1,:,2) = 85; i3=2*i1; i1(2,:,:) = 2; vec = sub2ind([2,86,3],i1,i2,i3); i1=0;i2=0;i3=0; [i1,i2,i3]=ind2sub([2,86,3],vec); disp(i1(:));disp(i2(:));disp(i3(:));
        
        !Test 2dim
        j1 = 0
        j2 = 0
        call ind2sub(j1,j2,(/15,1000/),jdx)
        call disp(j1)
        call disp(j2)
        !SHOULD GIVE SAME AS FOLLOWING MATLAB CODE: j1 = 3*ones(10,5); j1(:,3)=1:10;j2 = 5*ones(10,5);j2(3,3)=7;vec=sub2ind([15 1000],j1,j2); j1=0;j2=0; [j1 j2]= ind2sub([15,1000],vec)

    end subroutine test_sub2ind_and_ind2sub
    subroutine test_rem
        
        real(8) :: x(5),y(5)
        x = (/1.d0, 1.d0, 2.2d0, 5.d0,-3.d0/)
        y = (/0.d0, 1.d0, 12.d0,-3.d0, 5.d0/)
        write(*,*) rem(x(1),y(1))
        write(*,*) rem(x(2),y(2))
        write(*,*) rem(x(3),y(3))
        write(*,*) rem(x(4),y(4))
        write(*,*) rem(x(5),y(5))
        !SHOULD GIVE SAME AS FOLLOWING MATLAB CODE: x=[1,1,2.2,5,-3]; y= [0,1,12,-3,5]; rem(x,y)
    end subroutine test_rem

    subroutine test_interp1
        
        implicit none
        real(8) :: x(5),y(5,2), xi(10)
        x = (/1.d0, 1.d0, 2.2d0, 5.d0,-3.d0/)
        y(:,1) = .5d0*x !(/0.d0, 1.d0, 12.d0,-3.d0, 5.d0/)
        y(:,2) = 1.5d0*x !(y(:,1)+1.d0)*(-.5d0)
        xi(1:5) = x-1.d0
        xi(6:10) = x*1.5d0

        write(*,*) 'interpolated y'
        call disp(interp1(x,y,xi))
        write(*,*) 'interpolated y 1'
        call disp(interp1(x,y(:,1),xi))
        write(*,*) 'interpolated y 2'
        call disp(interp1(x,y(:,2),xi))
        write(*,*) 'interpolated pt'
        write(*,*) interp1((/1.d0,2.d0/),(/-1.d0,0.d0/),1.8d0)

        write(*,*) 'interpolated x descend'
        call disp(interp1((/3.d0,2.d0,2.d0,1.d0/),1.5d0*(/3.d0,2.d0,2.d0,1.d0/),.5d0*(/3.d0,2.d0,2.d0,1.d0/)))

    end subroutine test_interp1

    subroutine TestMergeSort
        
        implicit none
        integer, parameter :: N = 9
        real(8), dimension(N) :: A = real((/ 1, 5, 2, 7, 3, 9, 4, 6, 2 /),8), Aold
        integer, dimension(N) :: I 
        integer, parameter :: t = 100000
        real(8) :: rand01(t)
        integer :: J(t)
        Aold = A
        write(*,*) 'Original array :'
        call disp(A)
        call mergesort(A,I)       
        write(*,*) 'Sorted aray :'
        call disp(A)
        write(*,*) 'Ranking :'
        call disp(I)
        write(*,*) 'Sorted method 2 :'
        call disp(Aold(I))

        rand01 = rand(t)
        call mergesort(rand01,J)
        write(*,*) rand01
        write(*,*) issorted(rand01,'ascend')

    end subroutine TestMergeSort

    subroutine testeig
        
        implicit none
        real(8) :: A(4,4)
        A = rand(4,4)
        call disp(eig(A))

        A = eye(4,4)
        call disp(eig(A))

        A = 0.d0
        call disp(eig(A))

        A = 1.d0
        call disp(eig(A))

        A(1,:) = (/.9d0,.1d0,0.d0,0.d0/)
        A(2,:) = (/.1d0,.8d0,.1d0,0.d0/)
        A(3,:) = (/0.d0,.1d0,.8d0,.1d0/)
        A(4,:) = (/0.d0,0.d0,.1d0,.9d0/)
        call disp(eig(A))

    end subroutine testeig

    subroutine testregress
        
        implicit none
        real(8) :: A(4,4),y(4)

        y = rand(4)

        A = rand(4,4)
        call disp(regress(y,A))

        A = eye(4,4)
        call disp(regress(y,A))

        write(*,*) 'multicollinear example'
        A = 0.d0
        call disp(regress(y,A))

        write(*,*) 'nearly multicollinear example'
        A = 1.d0 + 1d-10*rand(4,4)
        call disp(regress(y,A))

        write(*,*) 'trans matrix example'
        A(1,:) = (/.9d0,.1d0,0.d0,0.d0/)
        A(2,:) = (/.1d0,.8d0,.1d0,0.d0/)
        A(3,:) = (/0.d0,.1d0,.8d0,.1d0/)
        A(4,:) = (/0.d0,0.d0,.1d0,.9d0/)
        call disp(regress(y,A))

    end subroutine testregress


    subroutine test_issorted
        implicit none
        real(8) :: x(4)
        write(*,*) 'ascend   descend'
        x = (/1.d0,2.d0,3.d0,4.d0/)
        write(*,*) issorted(x,'ascend'), issorted(x,'descend')
        x = (/1.d0,2.d0,2.d0,4.d0/)
        write(*,*) issorted(x,'ascend'), issorted(x,'descend')
        x = (/4.d0,1.d0,2.d0,3.d0/)
        write(*,*) issorted(x,'ascend'), issorted(x,'descend')
        x = (/4.d0,3.d0,2.d0,1.d0/)
        write(*,*) issorted(x,'ascend'), issorted(x,'descend')
        x = (/4.d0,2.d0,2.d0,1.d0/)
        write(*,*) issorted(x,'ascend'), issorted(x,'descend')
        x = (/4.d0,2.d0,2.d0,3.d0/)
        write(*,*) issorted(x,'ascend'), issorted(x,'descend')
        x = (/2.d0,2.d0,2.d0,2.d0/)
        write(*,*) issorted(x,'ascend'), issorted(x,'descend')
    end subroutine test_issorted

end module test_mod_matlab

