subroutine sub_aggstats_ratiomeans(iz,iagg,kstock,assets_out)

	use mod_globvar
	use mod_interpolate

	implicit none

	!This program computes aggregate statistics for for the housing model with borrowing and default - ratios of means
	!Financial statistics: 1998 SCF, all households with AGECL = 1 - 4 (data definitions in parentheses)
	
	integer, intent(in) :: iz, iagg
	real(8), intent(out) :: kstock, assets_out

	!Indices
	integer :: is, ie, ise, ise2, ig, ih, iu, igshock, im, iy, iyfine, imprime, isprime, ieprime
	
	!Preliminaries
	real(8) :: ownrate, mean_earn, mean_earnown, mean_earnrent, min_earn, meas_min_earn, med_earn, med_earnown
	real(8), dimension(ns*ne,2) :: earnmatrix
	real(8), dimension(ns) :: dist_earnown
	!Output, consumption, and investment
	real(8) :: yh, yc, assets, netmortpay, REOrevenue, mortcost, mortrev, svcorigcost, invest_k, hownstock, cons, hcons, hcons_rent, hcons_own_earn, hcons_impute_earn, &
	hcons_rent_earn, agg_xs, agg_xb, invest_h, lh, kh, pl, lc, kc, gdp, netexports, cons_gdp, hcons_gdp, invest_k_gdp, invest_h_gdp, govspend_gdp, netexports_gdp, kstock_gdp, &
	hownstock_gdp, w, rk
	!Financial portfolio statistics
	real(8) :: mean_assets, med_assets, networth, mean_networth, mean_networthown, mean_networthown_earnown, med_networth, med_networthown, med_networthown_earnown, mean_hwealth, &
	mean_hwealth_earnown, med_hwealth, med_hwealth_earnown, mortdebt, mean_mortdebt, mean_mortdebt_kpos, mean_mortdebt_earnown, mean_mortdebt_earnown_kpos, med_mortdebt, med_mortdebt_earnown
	real(8), dimension(nh) :: dist_housing
	real(8), dimension(nyfine) :: dist_assets, dist_networth, dist_mortdebt
	!Mortgage distribution statistics
	real(8) :: pct_mort, mean_ltv, mean_ltv_kpos, ltv80, ltv80_kpos, ltv90, ltv90_kpos, ltv95, ltv95_kpos
	!Housing sales and inventories statistics
	real(8) :: foreclosures, frecover, frate, REOdiscount, turnover, newsaleshare, existsales, reosales, reosaleshare, salesrate, inventories, months_supply, htot, mean_TOMREO, &
	meas_buy, mean_TOMbuy, meas_sell, meas_sellsuccess, mean_TOMsell, xsdiscount, mean_xsdiscount, mean_earndefault, mean_assetdefault
	real(8), dimension(nh) :: htot_reo_new
	!Other internal variables
	real(8) :: weight, yweight, y0weight, yprime_f0
	integer :: iylow, iyupp, iy0low, iy0upp
	
	!------------------------------------------------
	!Preliminaries
	!------------------------------------------------
	!Homeownership rate (at the end of subperiod 3)
	ownrate = sum(mu3own_f0) + sum(mu3own_f1)
	
	!Compute factor prices
	call sub_factorprices_sh(kagg(iagg),eavg,ph(iagg),zc(iz),zh(iz),w,rk)
	
	!Mean earnings
	mean_earn = 0.d0
	do is = 1, ns
		do ie = 1, ne
			mean_earn = mean_earn + labor(is,ie)*w*emeas(ie)*(sum(mu3own_f0(:,:,:,is)) + sum(mu3own_f1(:,:,is))	+ sum(mu3rent_f0(:,is)) + sum(mu3rent_f1(:,is)))
		enddo
	enddo

	!Mean earnings of homeowners
	if (sum(mu3own_f0) + sum(mu3own_f1) > 0.d0) then
		mean_earnown = 0.d0
		do is = 1, ns
			do ie = 1, ne
				mean_earnown = mean_earnown + labor(is,ie)*w*emeas(ie)*(sum(mu3own_f0(:,:,:,is)) + sum(mu3own_f1(:,:,is)))/(sum(mu3own_f0) + sum(mu3own_f1))
			enddo
		enddo
	else
		mean_earnown = mean_earn
	endif
	
	!Mean earnings of renters
	if (sum(mu3rent_f0) + sum(mu3rent_f1) > 0.d0) then
		mean_earnrent = 0.d0
		do is = 1, ns
			do ie = 1, ne
				mean_earnrent = mean_earnrent + labor(is,ie)*w*emeas(ie)*(sum(mu3rent_f0(:,is)) + sum(mu3rent_f1(:,is)))/(sum(mu3rent_f0) + sum(mu3rent_f1))
			enddo
		enddo
	else
		mean_earnrent = mean_earn
	endif
	
	!Median earnings
	!Create matrix of earnings levels and frequencies
	do is = 1, ns
		do ie = 1, ne
			earnmatrix(ie + (is - 1)*ne,1) = labor(is,ie)*w
			earnmatrix(ie + (is - 1)*ne,2) = emeas(ie)*(sum(mu3own_f0(:,:,:,is)) + sum(mu3own_f1(:,:,is)) + sum(mu3rent_f0(:,is)) + sum(mu3rent_f1(:,is)))
		enddo
	enddo
	!Sort earnings from lowest to highest
	do ise = 1, ns*ne
		!Find minimum of remaining values
		min_earn = minval(earnmatrix(ise:ns*ne,1))
		!Reorder
		do ise2 = 1, ns*ne
			if (earnmatrix(ise2,1) == min_earn) then
				meas_min_earn      = earnmatrix(ise2,2)
				earnmatrix(ise2,:) = earnmatrix(ise,:)
				earnmatrix(ise,1)  = min_earn
				earnmatrix(ise,2)  = meas_min_earn
				exit
			endif
		enddo
	enddo
	!Compute median
	do ise = 1, ns*ne
		if (sum(earnmatrix(1:ise,2)) > 0.5d0) then
			if (sum(earnmatrix(1:ise,2)) - sum(earnmatrix(1:max(1,ise - 1),2)) > 1e-6) then
				weight   = (0.5 - sum(earnmatrix(1:ise - 1,2)))/(sum(earnmatrix(1:ise,2)) - sum(earnmatrix(1:ise - 1,2)))
				med_earn = (1 - weight)*earnmatrix(max(1,ise - 1),1) + weight*earnmatrix(ise,1)
			else
				med_earn = earnmatrix(ise,1)
			endif
			exit
		endif
	enddo

	!Median earnings of homeowners
	if (sum(mu3own_f0) + sum(mu3own_f1) > 0.d0) then
		!Find distribution of homeowners over persistent income states
		do is = 1, ns
			dist_earnown(is) = sum(mu3own_f0(:,:,:,is)) + sum(mu3own_f1(:,:,is))
		enddo
		dist_earnown = dist_earnown/sum(dist_earnown)
		!Create matrix of earnings levels and frequencies
		do is = 1, ns
			do ie = 1, ne
				earnmatrix(ie + (is - 1)*ne,1) = labor(is,ie)*w
				earnmatrix(ie + (is - 1)*ne,2) = emeas(ie)*dist_earnown(is)
			enddo
		enddo
		!Sort earnings from lowest to highest
		do ise = 1, ns*ne
			!Find minimum of remaining values
			min_earn = minval(earnmatrix(ise:ns*ne,1))
			!Reorder
			do ise2 = 1, ns*ne
				if (earnmatrix(ise2,1) == min_earn) then
					meas_min_earn      = earnmatrix(ise2,2)
					earnmatrix(ise2,:) = earnmatrix(ise,:)
					earnmatrix(ise,1)  = min_earn
					earnmatrix(ise,2)  = meas_min_earn
					exit
				endif
			enddo
		enddo
		!Compute median
		do ise = 1, ns*ne
			if (sum(earnmatrix(1:ise,2)) > 0.5d0) then
				if (sum(earnmatrix(1:ise,2)) - sum(earnmatrix(1:max(1,ise - 1),2)) > 1e-6) then
					weight      = (0.5 - sum(earnmatrix(1:ise - 1,2)))/(sum(earnmatrix(1:ise,2)) - sum(earnmatrix(1:ise - 1,2)))
					med_earnown = (1 - weight)*earnmatrix(max(1,ise - 1),1) + weight*earnmatrix(ise,1)
				else
					med_earnown = earnmatrix(ise,1)
				endif
				exit
			endif
		enddo
	else
		med_earnown = med_earn
	endif
	
	!------------------------------------------------
	!Output, Consumption, and Investment
	!------------------------------------------------
	!Liquid assets
	assets = 0.d0
	do is = 1, ns
		!Renters
		do iy = ny_neg + 1, ny
			!Renters with good credit
			assets = assets + brent_f0(iy,is,iz,iagg)*mu3rent_f0(iy,is)
			!Renters with bad credit
			assets = assets + brent_f1(iy,is,iz,iagg)*mu3rent_f1(iy,is)
		enddo
		!Homeowners
		do ig = 1, ng
			!Homeowners with bad credit
			do iy = ny_neg + 1, ny
				assets = assets + bown_f1(iy,ig,is,iz,iagg)*mu3own_f1(iy,ig,is)
			enddo
			!Homeowners with good credit but no mortgage
			do iy = 1, ny
				assets = assets + bown_f0(iy,1,ig,is,iz,iagg)*mu3own_f0(iy,1,ig,is)
			enddo
			do im = 2, nm
				!Homeowners with mortgages
				do iy = ny_neg + 1, ny
					assets = assets + bown_f0(iy,im,ig,is,iz,iagg)*mu3own_f0(iy,im,ig,is)
				enddo
			enddo
		enddo
	enddo
	assets_out = assets
	
	!Net mortgage payments and REO revenues
	netmortpay = 0.d0
	REOrevenue = 0.d0
	mortcost    = 0.d0
	mortrev     = 0.d0
	svcorigcost = 0.d0
	do is = 1, ns
		do ig = 1, ng
			ih = hfun(ig)
			!Subperiod 1
			do im = 2, nm
				do iy = ny_neg + 1, ny
					!Homeowners who sell
					netmortpay = netmortpay + mm(im)*hh(ih)*psell_f0(iy,im,ig,is,iz,iagg)*mu1own_f0(iy,im,ig,is)
					mortrev    = mortrev + mm(im)*hh(ih)*psell_f0(iy,im,ig,is,iz,iagg)*mu1own_f0(iy,im,ig,is)
					!Homeowners who default
					netmortpay = netmortpay + min(fdeficiency*yy(iy),mm(im)*hh(ih) - Jreo(ih,iz,iagg))*(1 - psell_f0(iy,im,ig,is,iz,iagg))*d_f0(iy,im,ig,is,iz,iagg)*mu1own_f0(iy,im,ig,is)
					mortrev    = mortrev + min(fdeficiency*yy(iy),mm(im)*hh(ih) - Jreo(ih,iz,iagg))*(1 - psell_f0(iy,im,ig,is,iz,iagg))*d_f0(iy,im,ig,is,iz,iagg)*mu1own_f0(iy,im,ig,is)
					mortrev    = mortrev + Jreo(ih,iz,iagg)*(1 - psell_f0(iy,im,ig,is,iz,iagg))*d_f0(iy,im,ig,is,iz,iagg)*mu1own_f0(iy,im,ig,is)
					mortcost   = mortcost + (Jreo(ih,iz,iagg) - psell_reo(ih,iz,iagg)*xs_reo(ih,iz,iagg))*(1 - psell_f0(iy,im,ig,is,iz,iagg))*d_f0(iy,im,ig,is,iz,iagg)*mu1own_f0(iy,im,ig,is)
					!Revenue from sales of new REOs
					REOrevenue = REOrevenue + xs_reo(ih,iz,iagg)*(1 - sellcost)*(1 - psell_f0(iy,im,ig,is,iz,iagg))*d_f0(iy,im,ig,is,iz,iagg)*psell_reo(ih,iz,iagg)*mu1own_f0(iy,im,ig,is)
					!Homeowners who make a payment
					netmortpay = netmortpay + (mm(im) - qm(iz,iagg)*m_f0(iy,im,ig,is,iz,iagg))*hh(ih)*(1 - psell_f0(iy,im,ig,is,iz,iagg))*(1 - d_f0(iy,im,ig,is,iz,iagg))*mu1own_f0(iy,im,ig,is)
					mortrev     = mortrev + (mm(im) - (1 + rscost)*qm(iz,iagg)*m_f0(iy,im,ig,is,iz,iagg))*hh(ih)*(1 - psell_f0(iy,im,ig,is,iz,iagg))*(1 - d_f0(iy,im,ig,is,iz,iagg))*mu1own_f0(iy,im,ig,is)
					svcorigcost = svcorigcost + rscost*qm(iz,iagg)*m_f0(iy,im,ig,is,iz,iagg)*hh(ih)*(1 - psell_f0(iy,im,ig,is,iz,iagg))*(1 - d_f0(iy,im,ig,is,iz,iagg))*mu1own_f0(iy,im,ig,is)
				enddo
			enddo
			!Subperiod 3
			!New mortgages
			do iy = 1, ny
				netmortpay  = netmortpay - qm0(locm_f0(iy,1,ig,is,iz,iagg),locbown_f0(iy,1,ig,is,iz,iagg),ig,is,iz,iagg)*m_f0(iy,1,ig,is,iz,iagg)*hh(ih)*mu3own_f0(iy,1,ig,is)
				mortcost    = mortcost + qm0(locm_f0(iy,1,ig,is,iz,iagg),locbown_f0(iy,1,ig,is,iz,iagg),ig,is,iz,iagg)*(1 + rocost)*(1 + rscost)*m_f0(iy,1,ig,is,iz,iagg)*hh(ih)*mu3own_f0(iy,1,ig,is)
				svcorigcost = svcorigcost + qm0(locm_f0(iy,1,ig,is,iz,iagg),locbown_f0(iy,1,ig,is,iz,iagg),ig,is,iz,iagg)*((1 + rocost)*(1 + rscost) - 1)*m_f0(iy,1,ig,is,iz,iagg)*hh(ih)*mu3own_f0(iy,1,ig,is)
			enddo
			!Existing mortgages
			do im = 2, nm
				do iy = ny_neg + 1, ny
					mortrev     = mortrev + qm0(im,locbown_f0(iy,im,ig,is,iz,iagg),ig,is,iz,iagg)*(1 + rocost)*(1 + rscost)*mm(im)*hh(ih)*mu3own_f0(iy,im,ig,is)
					mortcost    = mortcost + qm0(im,locbown_f0(iy,im,ig,is,iz,iagg),ig,is,iz,iagg)*(1 + rocost)*(1 + rscost)*mm(im)*hh(ih)*mu3own_f0(iy,im,ig,is)
				enddo
			enddo
		enddo
	enddo
	!Revenue from sales of existing REOs
	do ih = 1, nh
		REOrevenue = REOrevenue + xs_reo(ih,iz,iagg)*psell_reo(ih,iz,iagg)*htot_reo(ih)
		mortrev    = mortrev + Jreo(ih,iz,iagg)*htot_reo(ih)
		mortcost   = mortcost + (Jreo(ih,iz,iagg) - psell_reo(ih,iz,iagg)*xs_reo(ih,iz,iagg))*htot_reo(ih)
	enddo
	
	!Steady state capital stock
	kstock   = qb(iz,iagg)*assets - mortcost
	invest_k = deprec_k*kstock
	
	!Housing stock
	hownstock = 0.d0
	do ig = 1, ng
		ih = hfun(ig)
		hownstock = hownstock + hh(ih)*(sum(mu3own_f0(:,:,ig,:)) + sum(mu3own_f1(:,ig,:)))
	enddo
	
	!Non-housing and housing consumption
	cons       = 0.d0
	hcons      = 0.d0
	hcons_rent = 0.d0
	do is = 1, ns
		!Renters
		do iy = ny_neg + 1, ny
			!Renters with good credit
			cons       = cons + crent_f0(iy,is,iz,iagg)*mu3rent_f0(iy,is)
			hcons      = hcons + hcrent_f0(iy,is,iz,iagg)*mu3rent_f0(iy,is)
			hcons_rent = hcons_rent + hcrent_f0(iy,is,iz,iagg)*mu3rent_f0(iy,is)
			!Renters with bad credit
			cons       = cons + crent_f1(iy,is,iz,iagg)*mu3rent_f1(iy,is)
			hcons      = hcons + hcrent_f1(iy,is,iz,iagg)*mu3rent_f1(iy,is)
			hcons_rent = hcons_rent + hcrent_f1(iy,is,iz,iagg)*mu3rent_f1(iy,is)
		enddo
		!Homeowners
		do ig = 1, ng
			!Homeowners with bad credit
			do iy = ny_neg + 1, ny
				cons  = cons + cown_f1(iy,ig,is,iz,iagg)*mu3own_f1(iy,ig,is)
				hcons = hcons + hcown_f1(iy,ig,is,iz,iagg)*mu3own_f1(iy,ig,is)
			enddo
			!Homeowners with good credit but no mortgage
			do iy = 1, ny
				cons  = cons + cown_f0(iy,1,ig,is,iz,iagg)*mu3own_f0(iy,1,ig,is)
				hcons = hcons + hcown_f0(iy,1,ig,is,iz,iagg)*mu3own_f0(iy,1,ig,is)
			enddo
			do im = 2, nm
				!Homeowners with mortgages
				do iy = ny_neg + 1, ny
					cons  = cons + cown_f0(iy,im,ig,is,iz,iagg)*mu3own_f0(iy,im,ig,is)
					hcons = hcons + hcown_f0(iy,im,ig,is,iz,iagg)*mu3own_f0(iy,im,ig,is)
				enddo
			enddo
		enddo
	enddo
	hcons_impute_earn = pa*(hownstock/(sum(mu3own_f0) + sum(mu3own_f1)))/mean_earnown
	hcons_rent_earn   = pa*(hcons_rent/(sum(mu3rent_f0) + sum(mu3rent_f1)))/mean_earnrent
	
	!Residential investment (including real estate agent costs)
	agg_xs  = 0.d0
	agg_xb  = 0.d0
	do is = 1, ns
		!Selling market costs
		do ig = 1, ng
			ih = hfun(ig)
			do iy = ny_neg + 1, ny
				agg_xs = agg_xs + xs_f1(iy,ig,is,iz,iagg)*psell_f1(iy,ig,is,iz,iagg)*mu1own_f1(iy,ig,is)
			enddo
			do im = 1, nm
				do iy = ny_neg + 1, ny
					agg_xs = agg_xs + xs_f0(iy,im,ig,is,iz,iagg)*psell_f0(iy,im,ig,is,iz,iagg)*mu1own_f0(iy,im,ig,is)
				enddo
			enddo
		enddo
		!Buying market costs
		do iy = ny_neg + 1, ny
			agg_xb = agg_xb + xb_f0(iy,is,iz,iagg)*pbuy_f0(iy,is,iz,iagg)*mu2rent_f0(iy,is)
			agg_xb = agg_xb + xb_f1(iy,is,iz,iagg)*pbuy_f1(iy,is,iz,iagg)*mu2rent_f1(iy,is)
		enddo
	enddo
	invest_h = agg_xb - (agg_xs + REOrevenue)
	
	!Output and gdp
	lh  = ((1 - lshare)*kshare_h*ph(iagg)*zh(iz)*(kshare_h/(1 - kshare_h)*w)**(kshare_h*(1 - lshare) - 1))**(1/lshare)
	kh  = kshare_h/(1 - kshare_h)*w*lh
	pl  = lshare*ph(iagg)*zh(iz)*(kh**kshare_h*lh**(1 - kshare_h))**(1 - lshare)
	yh  = zh(iz)*(kh**kshare_h*lh**(1 - kshare_h))**(1 - lshare)
	lc  = eavg - lh
	kc  = (kshare_c*zc(iz)/rk)**(1/(1 - kshare_c))*lc
	yc  = zc(iz)*kc**kshare_c*lc**(1 - kshare_c)
	gdp = yc - kh + ph(iagg)*yh + pa*hownstock
	
	!Net exports (= intermediary cash flow): kstock = kc in equilibrium --> zero net exports
	netexports = (1 - deprec_k + rk)*kc - assets + mortrev + qb(iz,iagg)*assets - mortcost - kc
	
	!GDP and GDP spending shares
	cons_gdp       = cons/gdp
	hcons_gdp      = pa*(hcons_rent + hownstock)/gdp
	invest_k_gdp   = invest_k/gdp
	invest_h_gdp   = invest_h/gdp
	govspend_gdp   = pl/gdp							!Wasteful government spending comes from revenues from land
	netexports_gdp = netexports/gdp
	kstock_gdp     = kstock/(4.0*gdp)				!Capital supplied by households (~ assets - mortgage debt)
	hownstock_gdp  = ph(iagg)*hownstock/(4.0*gdp)
	
	!Check consumption good market clearing
	print *, ' '
	print *, 'Total resources available to households'
	print *, (yc - rk*kc) + (ph(iagg)*yh - kh - pl) + assets
	print *, ' '
	print *, 'Total household spending'
	print *, (agg_xb - agg_xs) + netmortpay + qb(iz,iagg)*assets + cons + pa*hcons_rent
	print *, ' '
	print *, 'Net exports'
	print *, netexports
	print *, ' '
	print *, 'Goods market clearing'
	print *, yc + (1 - deprec_k)*kc - ((invest_h - ph(iagg)*yh + pl + kh) + kc + cons + pa*hcons + netexports + svcorigcost)	
	print *, ' '
	print *, 'GDP: Production'
	print *, gdp
	print *, ' '
	print *, 'GDP: Spending'
	print *, (invest_h + pl) + deprec_k*kc + cons + pa*(hcons + hownstock) + netexports + svcorigcost
	
	!------------------------------------------------
	!Financial portfolio statistics
	!------------------------------------------------
	!Mean financial assets over mean earnings (at the end of subperiod 3) (FIN - RETQLIQ + VEHIC - INSTALL - CCBAL - OTHLOC - ODEBT)
	mean_assets = qb(iz,iagg)*assets/(4.0*mean_earn)
	
	!Mean net worth over mean earnings (at the end of subperiod 3) (HOUSES - MRTHEL + FIN - RETQLIQ + VEHIC - INSTALL - CCBAL - OTHLOC - ODEBT)
	mean_networth = 0.d0
	do is = 1, ns
		!Renters
		do iy = ny_neg + 1, ny
			!Renters with good credit
			networth      = qb(iz,iagg)*brent_f0(iy,is,iz,iagg)
			mean_networth = mean_networth + networth*mu3rent_f0(iy,is)
			!Renters with bad credit
			networth      = qb(iz,iagg)*brent_f1(iy,is,iz,iagg)
			mean_networth = mean_networth + networth*mu3rent_f1(iy,is)
		enddo
		!Homeowners
		do ig = 1, ng
			ih = hfun(ig)
			!Homeowners with bad credit
			do iy = ny_neg + 1, ny
				networth      = qb(iz,iagg)*bown_f1(iy,ig,is,iz,iagg) + ph(iagg)*hh(ih)
				mean_networth = mean_networth + networth*mu3own_f1(iy,ig,is)
			enddo
			!Homeowners with good credit but no mortgage
			do iy = 1, ny
				networth      = qb(iz,iagg)*bown_f0(iy,1,ig,is,iz,iagg) + (ph(iagg) - m_f0(iy,1,ig,is,iz,iagg))*hh(ih)
				mean_networth = mean_networth + networth*mu3own_f0(iy,1,ig,is)
			enddo
			do im = 2, nm
				!Homeowners with mortgages
				do iy = ny_neg + 1, ny
					networth      = qb(iz,iagg)*bown_f0(iy,im,ig,is,iz,iagg) + (ph(iagg) - mm(im))*hh(ih)
					mean_networth = mean_networth + networth*mu3own_f0(iy,im,ig,is)
				enddo
			enddo
		enddo
	enddo
	mean_networth = mean_networth/(4.0*mean_earn)
	
	!Mean homeowner net worth over mean homeowner earnings (at the end of subperiod 3) (HOUSES - MRTHEL + FIN - RETQLIQ + VEHIC - INSTALL - CCBAL - OTHLOC - ODEBT)
	if (ownrate > 0.d0) then
		mean_networthown = 0.d0
		do is = 1, ns
			do ig = 1, ng
				ih = hfun(ig)
				!Homeowners with bad credit
				do iy = ny_neg + 1, ny
					networth         = qb(iz,iagg)*bown_f1(iy,ig,is,iz,iagg) + ph(iagg)*hh(ih)
					mean_networthown = mean_networthown + networth*mu3own_f1(iy,ig,is)
				enddo
				!Homeowners with good credit but no mortgage
				do iy = 1, ny
					networth         = qb(iz,iagg)*bown_f0(iy,1,ig,is,iz,iagg) + (ph(iagg) - m_f0(iy,1,ig,is,iz,iagg))*hh(ih)
					mean_networthown = mean_networthown + networth*mu3own_f0(iy,1,ig,is)
				enddo
				do im = 2, nm
					!Homeowners with mortgages
					do iy = ny_neg + 1, ny
						networth         = qb(iz,iagg)*bown_f0(iy,im,ig,is,iz,iagg) + (ph(iagg) - mm(im))*hh(ih)
						mean_networthown = mean_networthown + networth*mu3own_f0(iy,im,ig,is)
					enddo
				enddo
			enddo
		enddo
		mean_networthown_earnown = mean_networthown/(ownrate*4.0*mean_earnown)
		mean_networthown         = mean_networthown/(ownrate*4.0*mean_earn)
	else
		mean_networthown_earnown = mean_networth
		mean_networthown         = mean_networth
	endif
	
	!Mean housing wealth over mean homeowner earnings (at the end of subperiod 3) (HOUSES)
	if (ownrate > 0.d0) then
		mean_hwealth_earnown = ph(iagg)*hownstock/(ownrate*4.0*mean_earnown)
		mean_hwealth         = ph(iagg)*hownstock/(ownrate*4.0*mean_earn)
	else
		mean_hwealth_earnown = 0.d0
		mean_hwealth         = 0.d0
	endif
	
	!Percent of homeowners with mortgages (at the end of subperiod 3)
	!Existing mortgages
	pct_mort = sum(mu3own_f0(:,2:nm,:,:))
	do is = 1, ns
		do ig = 1, ng
			!New mortgages
			do iy = 1, ny
				if (locm_f0(iy,1,ig,is,iz,iagg) > 1) then
					pct_mort = pct_mort + mu3own_f0(iy,1,ig,is)
				endif
			enddo
		enddo
	enddo
	pct_mort = pct_mort/ownrate
	
	!Mean mortgage debt over mean homeowner earnings (at the end of subperiod 3) (MRTHEL)
	if (ownrate > 0.d0) then
		mean_mortdebt = 0.d0
		do is = 1, ns
			do ig = 1, ng
				ih = hfun(ig)
				!Homeowners with good credit but no mortgage
				do iy = 1, ny
					mortdebt      = m_f0(iy,1,ig,is,iz,iagg)*hh(ih)
					mean_mortdebt = mean_mortdebt + mortdebt*mu3own_f0(iy,1,ig,is)
				enddo
				do im = 2, nm
					!Homeowners with mortgages
					do iy = ny_neg + 1, ny
						mortdebt      = mm(im)*hh(ih)
						mean_mortdebt = mean_mortdebt + mortdebt*mu3own_f0(iy,im,ig,is)
					enddo
				enddo
			enddo
		enddo
		mean_mortdebt_earnown      = mean_mortdebt/(ownrate*4.0*mean_earnown)
		mean_mortdebt_earnown_kpos = mean_mortdebt_earnown/pct_mort
		mean_mortdebt              = mean_mortdebt/(ownrate*4.0*mean_earn)
		mean_mortdebt_kpos         = mean_mortdebt/pct_mort
	else
		mean_mortdebt_earnown      = 0.d0
		mean_mortdebt_earnown_kpos = 0.d0
		mean_mortdebt              = 0.d0
		mean_mortdebt_kpos         = 0.d0
	endif
	
	!------------------------------------------------
	!Financial portfolio statistics: medians
	!------------------------------------------------
	!Median financial assets over median earnings (at the end of subperiod 3) (FIN - RETQLIQ + VEHIC - INSTALL - CCBAL - OTHLOC - ODEBT)
	dist_assets = 0.d0
	do is = 1, ns
		!Renters
		do iy = ny_neg + 1, ny
			!Renters with good credit
			assets  = qb(iz,iagg)*brent_f0(iy,is,iz,iagg)
			iylow   = lyfine(assets)
			iyupp   = uyfine(assets)
			yweight = wyfine(assets)
			dist_assets(iylow) = dist_assets(iylow) + (1 - yweight)*mu3rent_f0(iy,is)
			dist_assets(iyupp) = dist_assets(iyupp) + yweight*mu3rent_f0(iy,is)
			!Renters with bad credit
			assets  = qb(iz,iagg)*brent_f1(iy,is,iz,iagg)
			iylow   = lyfine(assets)
			iyupp   = uyfine(assets)
			yweight = wyfine(assets)
			dist_assets(iylow) = dist_assets(iylow) + (1 - yweight)*mu3rent_f1(iy,is)
			dist_assets(iyupp) = dist_assets(iyupp) + yweight*mu3rent_f1(iy,is)
		enddo
		!Homeowners
		do ig = 1, ng
			!Homeowners with bad credit
			do iy = ny_neg + 1, ny
				assets  = qb(iz,iagg)*bown_f1(iy,ig,is,iz,iagg)
				iylow   = lyfine(assets)
				iyupp   = uyfine(assets)
				yweight = wyfine(assets)
				dist_assets(iylow) = dist_assets(iylow) + (1 - yweight)*mu3own_f1(iy,ig,is)
				dist_assets(iyupp) = dist_assets(iyupp) + yweight*mu3own_f1(iy,ig,is)
			enddo
			!Homeowners with good credit but no mortgage
			do iy = 1, ny
				assets  = qb(iz,iagg)*bown_f0(iy,1,ig,is,iz,iagg)
				iylow   = lyfine(assets)
				iyupp   = uyfine(assets)
				yweight = wyfine(assets)
				dist_assets(iylow) = dist_assets(iylow) + (1 - yweight)*mu3own_f0(iy,1,ig,is)
				dist_assets(iyupp) = dist_assets(iyupp) + yweight*mu3own_f0(iy,1,ig,is)
			enddo
			do im = 2, nm
				!Homeowners with mortgages
				do iy = ny_neg + 1, ny
					assets  = qb(iz,iagg)*bown_f0(iy,im,ig,is,iz,iagg)
					iylow   = lyfine(assets)
					iyupp   = uyfine(assets)
					yweight = wyfine(assets)
					dist_assets(iylow) = dist_assets(iylow) + (1 - yweight)*mu3own_f0(iy,im,ig,is)
					dist_assets(iyupp) = dist_assets(iyupp) + yweight*mu3own_f0(iy,im,ig,is)
				enddo
			enddo
		enddo
	enddo
	do iyfine = nyfine_neg + 1, nyfine
		if (sum(dist_assets(nyfine_neg + 1:iyfine)) > 0.5d0) then
			if (sum(dist_assets(nyfine_neg + 1:iyfine)) - sum(dist_assets(nyfine_neg + 1:max(nyfine_neg + 1,iyfine - 1))) > 1e-6) then
				weight     = (0.5 - sum(dist_assets(nyfine_neg + 1:iyfine - 1)))/(sum(dist_assets(nyfine_neg + 1:iyfine)) - sum(dist_assets(nyfine_neg + 1:iyfine - 1)))
				med_assets = (1 - weight)*yyfine(iyfine - 1) + weight*yyfine(iyfine)
			else
				med_assets = yyfine(iyfine)
			endif
			exit
		endif
	enddo
	med_assets = med_assets/(4.0*med_earn)
	
	!Median net worth over median earnings (at the end of subperiod 3) (HOUSES - MRTHEL + FIN - RETQLIQ + VEHIC - INSTALL - CCBAL - OTHLOC - ODEBT)
	dist_networth = 0.d0
	do is = 1, ns
		!Renters
		do iy = ny_neg + 1, ny
			!Renters with good credit
			networth = qb(iz,iagg)*brent_f0(iy,is,iz,iagg)
			iylow    = lyfine(networth)
			iyupp    = uyfine(networth)
			yweight  = wyfine(networth)
			dist_networth(iylow) = dist_networth(iylow) + (1 - yweight)*mu3rent_f0(iy,is)
			dist_networth(iyupp) = dist_networth(iyupp) + yweight*mu3rent_f0(iy,is)
			!Renters with bad credit
			networth = qb(iz,iagg)*brent_f1(iy,is,iz,iagg)
			iylow    = lyfine(networth)
			iyupp    = uyfine(networth)
			yweight  = wyfine(networth)
			dist_networth(iylow) = dist_networth(iylow) + (1 - yweight)*mu3rent_f1(iy,is)
			dist_networth(iyupp) = dist_networth(iyupp) + yweight*mu3rent_f1(iy,is)
		enddo
		!Homeowners
		do ig = 1, ng
			ih = hfun(ig)
			!Homeowners with bad credit
			do iy = ny_neg + 1, ny
				networth = qb(iz,iagg)*bown_f1(iy,ig,is,iz,iagg) + ph(iagg)*hh(ih)
				iylow    = lyfine(networth)
				iyupp    = uyfine(networth)
				yweight  = wyfine(networth)
				dist_networth(iylow) = dist_networth(iylow) + (1 - yweight)*mu3own_f1(iy,ig,is)
				dist_networth(iyupp) = dist_networth(iyupp) + yweight*mu3own_f1(iy,ig,is)
			enddo
			!Homeowners with good credit but no mortgage
			do iy = 1, ny
				networth = qb(iz,iagg)*bown_f0(iy,1,ig,is,iz,iagg) + (ph(iagg) - m_f0(iy,1,ig,is,iz,iagg))*hh(ih)
				iylow    = lyfine(networth)
				iyupp    = uyfine(networth)
				yweight  = wyfine(networth)
				dist_networth(iylow) = dist_networth(iylow) + (1 - yweight)*mu3own_f0(iy,1,ig,is)
				dist_networth(iyupp) = dist_networth(iyupp) + yweight*mu3own_f0(iy,1,ig,is)
			enddo
			do im = 2, nm
				!Homeowners with mortgages
				do iy = ny_neg + 1, ny
					networth = qb(iz,iagg)*bown_f0(iy,im,ig,is,iz,iagg) + (ph(iagg) - mm(im))*hh(ih)
					iylow    = lyfine(networth)
					iyupp    = uyfine(networth)
					yweight  = wyfine(networth)
					dist_networth(iylow) = dist_networth(iylow) + (1 - yweight)*mu3own_f0(iy,im,ig,is)
					dist_networth(iyupp) = dist_networth(iyupp) + yweight*mu3own_f0(iy,im,ig,is)
				enddo
			enddo
		enddo
	enddo
	do iyfine = 1, nyfine
		if (sum(dist_networth(1:iyfine)) > 0.5d0) then
			if (sum(dist_networth(1:iyfine)) - sum(dist_networth(1:max(1,iyfine - 1))) > 1e-6) then
				weight       = (0.5 - sum(dist_networth(1:iyfine - 1)))/(sum(dist_networth(1:iyfine)) - sum(dist_networth(1:iyfine - 1)))
				med_networth = (1 - weight)*yyfine(iyfine - 1) + weight*yyfine(iyfine)
			else
				med_networth = yyfine(iyfine)
			endif
			exit
		endif
	enddo
	med_networth = med_networth/(4.0*med_earn)
	
	!Median homeowner net worth over median homeowner earnings (at the end of subperiod 3) (HOUSES - MRTHEL + FIN - RETQLIQ + VEHIC - INSTALL - CCBAL - OTHLOC - ODEBT)
	if (ownrate > 0.d0) then
		dist_networth = 0.d0
		do is = 1, ns
			do ig = 1, ng
				ih = hfun(ig)
				!Homeowners with bad credit
				do iy = ny_neg + 1, ny
					networth = qb(iz,iagg)*bown_f1(iy,ig,is,iz,iagg) + ph(iagg)*hh(ih)
					iylow    = lyfine(networth)
					iyupp    = uyfine(networth)
					yweight  = wyfine(networth)
					dist_networth(iylow) = dist_networth(iylow) + (1 - yweight)*mu3own_f1(iy,ig,is)
					dist_networth(iyupp) = dist_networth(iyupp) + yweight*mu3own_f1(iy,ig,is)
				enddo
				!Homeowners with good credit but no mortgage
				do iy = 1, ny
					networth = qb(iz,iagg)*bown_f0(iy,1,ig,is,iz,iagg) + (ph(iagg) - m_f0(iy,1,ig,is,iz,iagg))*hh(ih)
					iylow    = lyfine(networth)
					iyupp    = uyfine(networth)
					yweight  = wyfine(networth)
					dist_networth(iylow) = dist_networth(iylow) + (1 - yweight)*mu3own_f0(iy,1,ig,is)
					dist_networth(iyupp) = dist_networth(iyupp) + yweight*mu3own_f0(iy,1,ig,is)
				enddo
				do im = 2, nm
					!Homeowners with mortgages
					do iy = ny_neg + 1, ny
						networth = qb(iz,iagg)*bown_f0(iy,im,ig,is,iz,iagg) + (ph(iagg) - mm(im))*hh(ih)
						iylow    = lyfine(networth)
						iyupp    = uyfine(networth)
						yweight  = wyfine(networth)
						dist_networth(iylow) = dist_networth(iylow) + (1 - yweight)*mu3own_f0(iy,im,ig,is)
						dist_networth(iyupp) = dist_networth(iyupp) + yweight*mu3own_f0(iy,im,ig,is)
					enddo
				enddo
			enddo
		enddo
		!Normalize distribution to be measure 1
		dist_networth = dist_networth/sum(dist_networth)
		do iyfine = 1, nyfine
			if (sum(dist_networth(1:iyfine)) > 0.5d0) then
				if (sum(dist_networth(1:iyfine)) - sum(dist_networth(1:max(1,iyfine - 1))) > 1e-6) then
					weight          = (0.5 - sum(dist_networth(1:iyfine - 1)))/(sum(dist_networth(1:iyfine)) - sum(dist_networth(1:iyfine - 1)))
					med_networthown = (1 - weight)*yyfine(iyfine - 1) + weight*yyfine(iyfine)
				else
					med_networthown = yyfine(iyfine)
				endif
				exit
			endif
		enddo
		med_networthown_earnown = med_networthown/(4.0*med_earnown)
		med_networthown         = med_networthown/(4.0*med_earn)
	else
		med_networthown_earnown = med_networth
		med_networthown         = med_networth
	endif
	
	!Median housing wealth over median homeowner earnings (at the end of subperiod 3) (HOUSES)
	if (ownrate > 0.d0) then
		dist_housing = 0.d0
		do ig = 1, ng
			ih = hfun(ig)
			dist_housing(ih) = dist_housing(ih) + sum(mu3own_f0(:,:,ig,:)) + sum(mu3own_f1(:,ig,:))
		enddo
		!Normalize distribution to be measure 1
		dist_housing = dist_housing/sum(dist_housing)
		do ih = 1, nh
			if (sum(dist_housing(1:ih)) > 0.5d0) then
				if (sum(dist_housing(1:ih)) - sum(dist_housing(1:max(1,ih - 1))) > 1e-6) then
					weight      = (0.5 - sum(dist_housing(1:ih - 1)))/(sum(dist_housing(1:ih)) - sum(dist_housing(1:ih - 1)))
					med_hwealth = ph(iagg)*((1 - weight)*hh(ih - 1) + weight*hh(ih))
				else
					med_hwealth = ph(iagg)*hh(ih)
				endif
				exit
			endif
		enddo
		med_hwealth_earnown = med_hwealth/(4.0*med_earnown)
		med_hwealth         = med_hwealth/(4.0*med_earn)
	else
		med_hwealth_earnown = 0.d0
		med_hwealth         = 0.d0
	endif
	
	!Median mortgage debt over median homeowner earnings (at the end of subperiod 3) (MRTHEL)
	if (ownrate > 0.d0) then
		dist_mortdebt = 0.d0
		do is = 1, ns
			do ig = 1, ng
				ih = hfun(ig)
				!Homeowners with bad credit
				do iy = ny_neg + 1, ny
					mortdebt = 0.d0
					iylow    = lyfine(mortdebt)
					iyupp    = uyfine(mortdebt)
					yweight  = wyfine(mortdebt)
					dist_mortdebt(iylow) = dist_mortdebt(iylow) + (1 - yweight)*mu3own_f1(iy,ig,is)
					dist_mortdebt(iyupp) = dist_mortdebt(iyupp) + yweight*mu3own_f1(iy,ig,is)
				enddo
				!Homeowners with good credit but no mortgage
				do iy = 1, ny
					mortdebt = m_f0(iy,1,ig,is,iz,iagg)*hh(ih)
					iylow    = lyfine(mortdebt)
					iyupp    = uyfine(mortdebt)
					yweight  = wyfine(mortdebt)
					dist_mortdebt(iylow) = dist_mortdebt(iylow) + (1 - yweight)*mu3own_f0(iy,1,ig,is)
					dist_mortdebt(iyupp) = dist_mortdebt(iyupp) + yweight*mu3own_f0(iy,1,ig,is)
				enddo
				do im = 2, nm
					!Homeowners with mortgages
					do iy = ny_neg + 1, ny
						mortdebt = mm(im)*hh(ih)
						iylow    = lyfine(mortdebt)
						iyupp    = uyfine(mortdebt)
						yweight  = wyfine(mortdebt)
						dist_mortdebt(iylow) = dist_mortdebt(iylow) + (1 - yweight)*mu3own_f0(iy,im,ig,is)
						dist_mortdebt(iyupp) = dist_mortdebt(iyupp) + yweight*mu3own_f0(iy,im,ig,is)
					enddo
				enddo
			enddo
		enddo
		!Normalize distribution to be measure 1
		dist_mortdebt = dist_mortdebt/sum(dist_mortdebt)
		do iyfine = nyfine_neg + 1, nyfine
			if (sum(dist_mortdebt(nyfine_neg + 1:iyfine)) > 0.5d0) then
				if (sum(dist_mortdebt(nyfine_neg + 1:iyfine)) - sum(dist_mortdebt(nyfine_neg + 1:max(nyfine_neg + 1,iyfine - 1))) > 1e-6) then
					weight       = (0.5 - sum(dist_mortdebt(nyfine_neg + 1:iyfine - 1)))/(sum(dist_mortdebt(nyfine_neg + 1:iyfine)) - sum(dist_mortdebt(nyfine_neg + 1:iyfine - 1)))
					med_mortdebt = (1 - weight)*yyfine(iyfine - 1) + weight*yyfine(iyfine)
				else
					med_mortdebt = yyfine(iyfine)
				endif
				exit
			endif
		enddo
		med_mortdebt_earnown = med_mortdebt/(4.0*med_earnown)
		med_mortdebt         = med_mortdebt/(4.0*med_earn)
	else
		med_mortdebt_earnown = 0.d0
		med_mortdebt         = 0.d0
	endif
	
	!------------------------------------------------
	!Mortgage distribution statistics
	!------------------------------------------------
	!Mean loan-to-value (MRTHEL/HOUSES)
	mean_ltv = 0.d0
	if (ownrate > 0.d0 .AND. pct_mort > 0.d0) then
		do is = 1, ns
			do ig = 1, ng
				!New mortgages
				do iy = 1, ny
					mean_ltv = mean_ltv + m_f0(iy,1,ig,is,iz,iagg)/ph(iagg)*mu3own_f0(iy,1,ig,is)
				enddo
				!Existing mortgages
				do im = 2, nm
					do iy = ny_neg + 1, ny
						mean_ltv = mean_ltv + mm(im)/ph(iagg)*mu3own_f0(iy,im,ig,is)
					enddo
				enddo
			enddo
		enddo
		mean_ltv      = mean_ltv/ownrate
		mean_ltv_kpos = mean_ltv/pct_mort
	else
		mean_ltv_kpos = 0.d0
	endif
	
	!Percentage of homeowners with ltv > 0.8
	ltv80 = 0.d0
	if (ownrate > 0.d0 .AND. pct_mort > 0.d0) then
		do is = 1, ns
			do ig = 1, ng
				do im = 2, nm
					if (mm(im)/ph(iagg) >= 0.8d0) then
						do iy = ny_neg + 1, ny
							ltv80 = ltv80 + mu3own_f0(iy,im,ig,is)
						enddo
					endif
				enddo
				do iy = 1, ny
					imprime = locm_f0(iy,1,ig,is,iz,iagg)
					if (mm(imprime)/ph(iagg) >= 0.8d0) then
						ltv80 = ltv80 + mu3own_f0(iy,1,ig,is)
					endif
				enddo
			enddo
		enddo
		ltv80      = ltv80/ownrate
		ltv80_kpos = ltv80/pct_mort
	else
		ltv80_kpos = 0.d0
	endif	
	
	!Percentage of homeowners with ltv > 0.9
	ltv90 = 0.d0
	if (ownrate > 0.d0 .AND. pct_mort > 0.d0) then
		do is = 1, ns
			do ig = 1, ng
				do im = 2, nm
					if (mm(im)/ph(iagg) >= 0.9d0) then
						do iy = ny_neg + 1, ny
							ltv90 = ltv90 + mu3own_f0(iy,im,ig,is)
						enddo
					endif
				enddo
				do iy = 1, ny
					imprime = locm_f0(iy,1,ig,is,iz,iagg)
					if (mm(imprime)/ph(iagg) >= 0.9d0) then
						ltv90 = ltv90 + mu3own_f0(iy,1,ig,is)
					endif
				enddo
			enddo
		enddo
		ltv90      = ltv90/ownrate
		ltv90_kpos = ltv90/pct_mort
	else
		ltv90_kpos = 0.d0
	endif
	
	!Percentage of homeowners with ltv > 0.95
	ltv95 = 0.d0
	if (ownrate > 0.d0 .AND. pct_mort > 0.d0) then
		do is = 1, ns
			do ig = 1, ng
				do im = 2, nm
					if (mm(im)/ph(iagg) >= 0.95d0) then
						do iy = ny_neg + 1, ny
							ltv95 = ltv95 + mu3own_f0(iy,im,ig,is)
						enddo
					endif
				enddo
				do iy = 1, ny
					imprime = locm_f0(iy,1,ig,is,iz,iagg)
					if (mm(imprime)/ph(iagg) >= 0.95d0) then
						ltv95 = ltv95 + mu3own_f0(iy,1,ig,is)
					endif
				enddo
			enddo
		enddo
		ltv95	   = ltv95/ownrate
		ltv95_kpos = ltv95/pct_mort
	else
		ltv95_kpos = 0.d0
	endif
	
	!------------------------------------------------
	!Foreclosure statistics
	!------------------------------------------------
	!Foreclosures, foreclosure rate, and average foreclosure recovery ratio
	if (sum(mu1own_f0(:,2:nm,:,:)) > 0.d0) then
		foreclosures = 0.d0
		frecover     = 0.d0
		do is = 1, ns
			do ig = 1, ng
				ih = hfun(ig)
				do im = 2, nm
					do iy = ny_neg + 1, ny
						foreclosures = foreclosures + (1 - psell_f0(iy,im,ig,is,iz,iagg))*d_f0(iy,im,ig,is,iz,iagg)*mu1own_f0(iy,im,ig,is)
						frecover     = frecover + min(1.d0,(Jreo(ih,iz,iagg) + fdeficiency*yy(iy))/(mm(im)*hh(ih)))*(1 - psell_f0(iy,im,ig,is,iz,iagg))&
						*d_f0(iy,im,ig,is,iz,iagg)*mu1own_f0(iy,im,ig,is)
					enddo
				enddo
			enddo
		enddo
		frate = foreclosures/sum(mu1own_f0(:,2:nm,:,:))
		if (foreclosures > 0.d0) then
			frecover = frecover/foreclosures
		endif
	else
		frate    = 0.d0
		frecover = 1.d0
	endif
	
	!------------------------------------------------
	!Housing sales and inventories statistics
	!------------------------------------------------
	!Turnover (measure of people buying a house divided by measure of homeowners at the end of subperiod 3)
	turnover = 0.d0
	do is = 1, ns
		do iy = ny_neg + 1, ny
			!Renters with good credit
			turnover = turnover + pbuy_f0(iy,is,iz,iagg)*mu2rent_f0(iy,is)
			!Renters with bad credit
			turnover = turnover + pbuy_f1(iy,is,iz,iagg)*mu2rent_f1(iy,is)
		enddo
	enddo
	turnover = turnover/ownrate
	
	!Sales volume, inventories, months of supply, REO sales share, and sales rate
	!Sales is the measure of housing traded, inventories is the measure of housing put up for sale but not sold,
	!months of supply is annualized inventories/sales, REO sales share is REO sales volume as a share of existing home
	!sales, and sales rate is existing sales as a share of the housing stock
	existsales  = 0.d0
	inventories = 0.d0
	reosales    = 0.d0
	do is = 1, ns
		do ig = 1, ng
			ih = hfun(ig)
			!Homeowners with bad credit
			do iy = ny_neg + 1, ny
				if (psell_f1(iy,ig,is,iz,iagg) > 0.d0) then
					existsales  = existsales + psell_f1(iy,ig,is,iz,iagg)*hh(ih)*mu1own_f1(iy,ig,is)
					inventories = inventories + (1 - psell_f1(iy,ig,is,iz,iagg))*hh(ih)*mu1own_f1(iy,ig,is)
				endif
			enddo
			!Homeowners with good credit
			do im = 1, nm
				do iy = ny_neg + 1, ny
					!Owners who voluntarily sell
					if (psell_f0(iy,im,ig,is,iz,iagg) > 0.d0) then
						existsales  = existsales + psell_f0(iy,im,ig,is,iz,iagg)*hh(ih)*mu1own_f0(iy,im,ig,is)
						inventories = inventories + (1 - psell_f0(iy,im,ig,is,iz,iagg))*hh(ih)*mu1own_f0(iy,im,ig,is)
					endif
					!Owners who default
					if (psell_reo(ih,iz,iagg) > 0.d0) then
						!Sales volume and inventories from new REOs
						existsales  = existsales + (1 - psell_f0(iy,im,ig,is,iz,iagg))*d_f0(iy,im,ig,is,iz,iagg)*mu1own_f0(iy,im,ig,is)&
						*hh(ih)*psell_reo(ih,iz,iagg)
						reosales    = reosales + (1 - psell_f0(iy,im,ig,is,iz,iagg))*d_f0(iy,im,ig,is,iz,iagg)*mu1own_f0(iy,im,ig,is)&
						*hh(ih)*psell_reo(ih,iz,iagg)
						inventories = inventories + (1 - psell_f0(iy,im,ig,is,iz,iagg))*d_f0(iy,im,ig,is,iz,iagg)*mu1own_f0(iy,im,ig,is)&
						*hh(ih)*(1 - psell_reo(ih,iz,iagg))
					endif
				enddo
			enddo
		enddo
	enddo
	!Sales volume and inventories from existing REOs
	do ih = 1, nh
		existsales  = existsales + htot_reo(ih)*hh(ih)*psell_reo(ih,iz,iagg)
		reosales    = reosales + htot_reo(ih)*hh(ih)*psell_reo(ih,iz,iagg)
		inventories = inventories + htot_reo(ih)*hh(ih)*(1 - psell_reo(ih,iz,iagg))
	enddo
	months_supply = inventories/(4.0*existsales)*12.0
	reosaleshare  = reosales/(yh + existsales)
	newsaleshare  = yh/(yh + existsales)
	!Compute beginning of period housing stock and sales rate
	htot  = 0.d0
	do ig = 1, ng
		ih   = hfun(ig)
		htot = htot + hh(ih)*(sum(mu1own_f0(:,:,ig,:)) + sum(mu1own_f1(:,ig,:)))
	enddo
	do ih = 1, nh
		htot = htot + hh(ih)*htot_reo(ih)
	enddo
	salesrate = 4.0*existsales/htot
	
	!Stock of new REO housing
	htot_reo_new = 0.d0
	do is = 1, ns
		do ig = 1, ng
			ih = hfun(ig)
			!Homeowners with a mortgage
			do im = 2, nm
				do iy = ny_neg + 1, ny
					htot_reo_new(ih) = htot_reo_new(ih) + (1 - psell_f0(iy,im,ig,is,iz,iagg))*d_f0(iy,im,ig,is,iz,iagg)*mu1own_f0(iy,im,ig,is)
				enddo
			enddo
		enddo
	enddo
	
	!REO sale discount and search duration
	REOdiscount  = 0.d0
	mean_TOMREO  = 0.d0
	if (sum(htot_reo) + sum(htot_reo_new) > 0.d0) then
		do ih = 1, nh
			REOdiscount = REOdiscount + (1 - xs_reo(ih,iz,iagg)/(ph(iagg)*hh(ih)))*(htot_reo(ih) + htot_reo_new(ih))/(sum(htot_reo) + sum(htot_reo_new))
			if (psell_reo(ih,iz,iagg) > 0.d0) then
				mean_TOMREO = mean_TOMREO + ((1 - psell_reo(ih,iz,iagg))/psell_reo(ih,iz,iagg) + 0.5)*12.0*htot_reo(ih)/sum(htot_reo)
			endif
		enddo
	endif
	
	!Mean buyer search duration
	mean_TOMbuy = 0.d0
	meas_buy    = 0.d0
	do is = 1, ns
		do iy = ny_neg + 1, ny
			if (pbuy_f0(iy,is,iz,iagg) > 0.d0) then
				mean_TOMbuy = mean_TOMbuy + ((1 - pbuy_f0(iy,is,iz,iagg))/pbuy_f0(iy,is,iz,iagg) + 0.5)*12.0*mu2rent_f0(iy,is)
				meas_buy    = meas_buy + mu2rent_f0(iy,is)
			endif
			if (pbuy_f1(iy,is,iz,iagg) > 0.d0) then
				mean_TOMbuy = mean_TOMbuy + ((1 - pbuy_f1(iy,is,iz,iagg))/pbuy_f1(iy,is,iz,iagg) + 0.5)*12.0*mu2rent_f1(iy,is)
				meas_buy    = meas_buy + mu2rent_f1(iy,is)
			endif				
		enddo
	enddo
	if (meas_buy > 0.d0) then
		mean_TOMbuy = mean_TOMbuy/meas_buy
	else
		mean_TOMbuy = 0.d0
	endif
	
	!Mean seller search duration
	mean_TOMsell = 0.d0
	meas_sell    = 0.d0
	do is = 1, ns
		do ig = 1, ng
			!Sellers with bad credit
			do iy = ny_neg + 1, ny
				if (psell_f1(iy,ig,is,iz,iagg) > 0.d0) then
					mean_TOMsell = mean_TOMsell + ((1 - psell_f1(iy,ig,is,iz,iagg))/psell_f1(iy,ig,is,iz,iagg) + 0.5)*12.0*mu1own_f1(iy,ig,is)
					meas_sell    = meas_sell + mu1own_f1(iy,ig,is)
				endif
			enddo
			!Sellers with good credit
			do im = 1, nm
				do iy =  ny_neg + 1, ny
					if (psell_f0(iy,im,ig,is,iz,iagg) > 0.d0) then
						mean_TOMsell = mean_TOMsell + ((1 - psell_f0(iy,im,ig,is,iz,iagg))/psell_f0(iy,im,ig,is,iz,iagg) + 0.5)*12.0*mu1own_f0(iy,im,ig,is)
						meas_sell    = meas_sell + mu1own_f0(iy,im,ig,is)
					endif
				enddo
			enddo
		enddo
	enddo
	if (meas_sell > 0.d0) then
		mean_TOMsell = mean_TOMsell/meas_sell
	else
		mean_TOMsell = 0.d0
	endif
	
	!Mean sales price discount (xs/(ph(iagg)*hh(ih))
	if (meas_sell > 0.d0) then
		meas_sellsuccess = 0.d0
		mean_xsdiscount  = 0.d0
		do is = 1, ns
			do ig = 1, ng
				ih = hfun(ig)
				!Homeowners with bad credit
				do iy = ny_neg + 1, ny
					if (psell_f1(iy,ig,is,iz,iagg) > 0.d0) then
						xsdiscount       = 1 - xsratio_f1(iy,ig,is,iz,iagg)
						mean_xsdiscount  = mean_xsdiscount + xsdiscount*psell_f1(iy,ig,is,iz,iagg)*mu1own_f1(iy,ig,is)
						meas_sellsuccess = meas_sellsuccess + psell_f1(iy,ig,is,iz,iagg)*mu1own_f1(iy,ig,is)
					endif
				enddo
				!Homeowners with good credit
				do im = 1, nm
					do iy = ny_neg + 1, ny
						if (psell_f0(iy,im,ig,is,iz,iagg) > 0.d0) then
							xsdiscount       = 1 - xsratio_f0(iy,im,ig,is,iz,iagg)
							mean_xsdiscount  = mean_xsdiscount + xsdiscount*psell_f0(iy,im,ig,is,iz,iagg)*mu1own_f0(iy,im,ig,is)
							meas_sellsuccess = meas_sellsuccess + psell_f0(iy,im,ig,is,iz,iagg)*mu1own_f0(iy,im,ig,is)
						endif
					enddo
				enddo
			enddo
		enddo
		mean_xsdiscount = mean_xsdiscount/meas_sellsuccess
	else
		mean_xsdiscount = 0.d0
	endif
	
	!Mean earnings and assets of defaulters
	if (foreclosures > 0.d0) then
		do is = 1, ns
			do ig = 1, ng
				ih      = hfun(ig)
				igshock = gfun(ih,2)
				!Homeowners taking out a new mortgage
				do iy = 1, ny
					imprime = locm_f0(iy,1,ig,is,iz,iagg)
					if (iu == 1 .AND. nu == 2) then
						do isprime = 1, ns
							do ieprime = 1, ne
								yprime_f0 = (labor(isprime,ieprime)*w + bown_f0(iy,1,ig,is,iz,iagg))*(1 - taxprime(iz,iagg,iz))
								iy0low    = ly(yprime_f0)
								iy0upp    = uy(yprime_f0)
								y0weight  = wy(yprime_f0)
								!No housing shock
								mean_earndefault  = mean_earndefault + labor(isprime,ieprime)*w*(1 - y0weight)*(1 - psell_f0(iy0low,imprime,ig,isprime,iz,iagg))&
								*d_f0(iy0low,imprime,ig,isprime,iz,iagg)*(1 - pseparate)*emeas(ieprime)*prob_ss(is,isprime,iz,iz)*mu3own_f0(iy,1,ig,is)
								mean_earndefault  = mean_earndefault + labor(isprime,ieprime)*w*y0weight*(1 - psell_f0(iy0upp,imprime,ig,isprime,iz,iagg))&
								*d_f0(iy0upp,imprime,ig,isprime,iz,iagg)*(1 - pseparate)*emeas(ieprime)*prob_ss(is,isprime,iz,iz)*mu3own_f0(iy,1,ig,is)
								mean_assetdefault = mean_assetdefault + bown_f0(iy,1,ig,is,iz,iagg)*(1 - y0weight)*(1 - psell_f0(iy0low,imprime,ig,isprime,iz,iagg))&
								*d_f0(iy0low,imprime,ig,isprime,iz,iagg)*(1 - pseparate)*emeas(ieprime)*prob_ss(is,isprime,iz,iz)*mu3own_f0(iy,1,ig,is)
								mean_assetdefault = mean_assetdefault + bown_f0(iy,1,ig,is,iz,iagg)*y0weight*(1 - psell_f0(iy0upp,imprime,ig,isprime,iz,iagg))&
								*d_f0(iy0upp,imprime,ig,isprime,iz,iagg)*(1 - pseparate)*emeas(ieprime)*prob_ss(is,isprime,iz,iz)*mu3own_f0(iy,1,ig,is)
								!Housing shock
								mean_earndefault  = mean_earndefault + labor(isprime,ieprime)*w*(1 - y0weight)*(1 - psell_f0(iy0low,imprime,igshock,isprime,iz,iagg))&
								*d_f0(iy0low,imprime,igshock,isprime,iz,iagg)*pseparate*emeas(ieprime)*prob_ss(is,isprime,iz,iz)*mu3own_f0(iy,1,ig,is)
								mean_earndefault  = mean_earndefault + labor(isprime,ieprime)*w*y0weight*(1 - psell_f0(iy0upp,imprime,igshock,isprime,iz,iagg))&
								*d_f0(iy0upp,imprime,igshock,isprime,iz,iagg)*pseparate*emeas(ieprime)*prob_ss(is,isprime,iz,iz)*mu3own_f0(iy,1,ig,is)
								mean_assetdefault = mean_assetdefault + bown_f0(iy,1,ig,is,iz,iagg)*(1 - y0weight)*(1 - psell_f0(iy0low,imprime,igshock,isprime,iz,iagg))&
								*d_f0(iy0low,imprime,igshock,isprime,iz,iagg)*pseparate*emeas(ieprime)*prob_ss(is,isprime,iz,iz)*mu3own_f0(iy,1,ig,is)
								mean_assetdefault = mean_assetdefault + bown_f0(iy,1,ig,is,iz,iagg)*y0weight*(1 - psell_f0(iy0upp,imprime,igshock,isprime,iz,iagg))&
								*d_f0(iy0upp,imprime,igshock,isprime,iz,iagg)*pseparate*emeas(ieprime)*prob_ss(is,isprime,iz,iz)*mu3own_f0(iy,1,ig,is)
							enddo
						enddo
					else
						do isprime = 1, ns
							do ieprime = 1, ne
								yprime_f0 = (labor(isprime,ieprime)*w + bown_f0(iy,1,ig,is,iz,iagg))*(1 - taxprime(iz,iagg,iz))
								iy0low    = ly(yprime_f0)
								iy0upp    = uy(yprime_f0)
								y0weight  = wy(yprime_f0)
								mean_earndefault  = mean_earndefault + labor(isprime,ieprime)*w*(1 - y0weight)*(1 - psell_f0(iy0low,imprime,ig,isprime,iz,iagg))&
								*d_f0(iy0low,imprime,ig,isprime,iz,iagg)*emeas(ieprime)*prob_ss(is,isprime,iz,iz)*mu3own_f0(iy,1,ig,is)
								mean_earndefault  = mean_earndefault + labor(isprime,ieprime)*w*y0weight*(1 - psell_f0(iy0upp,imprime,ig,isprime,iz,iagg))&
								*d_f0(iy0upp,imprime,ig,isprime,iz,iagg)*emeas(ieprime)*prob_ss(is,isprime,iz,iz)*mu3own_f0(iy,1,ig,is)
								mean_assetdefault = mean_assetdefault + bown_f0(iy,1,ig,is,iz,iagg)*(1 - y0weight)*(1 - psell_f0(iy0low,imprime,ig,isprime,iz,iagg))&
								*d_f0(iy0low,imprime,ig,isprime,iz,iagg)*emeas(ieprime)*prob_ss(is,isprime,iz,iz)*mu3own_f0(iy,1,ig,is)
								mean_assetdefault = mean_assetdefault + bown_f0(iy,1,ig,is,iz,iagg)*y0weight*(1 - psell_f0(iy0upp,imprime,ig,isprime,iz,iagg))&
								*d_f0(iy0upp,imprime,ig,isprime,iz,iagg)*emeas(ieprime)*prob_ss(is,isprime,iz,iz)*mu3own_f0(iy,1,ig,is)
							enddo
						enddo
					endif
				enddo
				do im = 2, nm
					do iy = ny_neg + 1, ny
						if (iu == 1 .AND. nu == 2) then
							do isprime = 1, ns
								do ieprime = 1, ne
									yprime_f0 = (labor(isprime,ieprime)*w + bown_f0(iy,im,ig,is,iz,iagg))*(1 - taxprime(iz,iagg,iz))
									iy0low    = ly(yprime_f0)
									iy0upp    = uy(yprime_f0)
									y0weight  = wy(yprime_f0)
									!No housing shock
									mean_earndefault  = mean_earndefault + labor(isprime,ieprime)*w*(1 - y0weight)*(1 - psell_f0(iy0low,im,ig,isprime,iz,iagg))&
									*d_f0(iy0low,im,ig,isprime,iz,iagg)*(1 - pseparate)*emeas(ieprime)*prob_ss(is,isprime,iz,iz)*mu3own_f0(iy,im,ig,is)
									mean_earndefault  = mean_earndefault + labor(isprime,ieprime)*w*y0weight*(1 - psell_f0(iy0upp,im,ig,isprime,iz,iagg))&
									*d_f0(iy0upp,im,ig,isprime,iz,iagg)*(1 - pseparate)*emeas(ieprime)*prob_ss(is,isprime,iz,iz)*mu3own_f0(iy,im,ig,is)
									mean_assetdefault = mean_assetdefault + bown_f0(iy,im,ig,is,iz,iagg)*(1 - y0weight)*(1 - psell_f0(iy0low,im,ig,isprime,iz,iagg))&
									*d_f0(iy0low,im,ig,isprime,iz,iagg)*(1 - pseparate)*emeas(ieprime)*prob_ss(is,isprime,iz,iz)*mu3own_f0(iy,im,ig,is)
									mean_assetdefault = mean_assetdefault + bown_f0(iy,im,ig,is,iz,iagg)*y0weight*(1 - psell_f0(iy0upp,im,ig,isprime,iz,iagg))&
									*d_f0(iy0upp,im,ig,isprime,iz,iagg)*(1 - pseparate)*emeas(ieprime)*prob_ss(is,isprime,iz,iz)*mu3own_f0(iy,im,ig,is)
									!Housing shock
									mean_earndefault  = mean_earndefault + labor(isprime,ieprime)*w*(1 - y0weight)*(1 - psell_f0(iy0low,im,igshock,isprime,iz,iagg))&
									*d_f0(iy0low,im,igshock,isprime,iz,iagg)*pseparate*emeas(ieprime)*prob_ss(is,isprime,iz,iz)*mu3own_f0(iy,im,ig,is)
									mean_earndefault  = mean_earndefault + labor(isprime,ieprime)*w*y0weight*(1 - psell_f0(iy0upp,im,igshock,isprime,iz,iagg))&
									*d_f0(iy0upp,im,igshock,isprime,iz,iagg)*pseparate*emeas(ieprime)*prob_ss(is,isprime,iz,iz)*mu3own_f0(iy,im,ig,is)
									mean_assetdefault = mean_assetdefault + bown_f0(iy,im,ig,is,iz,iagg)*(1 - y0weight)*(1 - psell_f0(iy0low,im,igshock,isprime,iz,iagg))&
									*d_f0(iy0low,im,igshock,isprime,iz,iagg)*pseparate*emeas(ieprime)*prob_ss(is,isprime,iz,iz)*mu3own_f0(iy,im,ig,is)
									mean_assetdefault = mean_assetdefault + bown_f0(iy,im,ig,is,iz,iagg)*y0weight*(1 - psell_f0(iy0upp,im,igshock,isprime,iz,iagg))&
									*d_f0(iy0upp,im,igshock,isprime,iz,iagg)*pseparate*emeas(ieprime)*prob_ss(is,isprime,iz,iz)*mu3own_f0(iy,im,ig,is)
								enddo
							enddo
						else
							do isprime = 1, ns
								do ieprime = 1, ne
									!!!Mean earnings!!!
									yprime_f0 = (labor(isprime,ieprime)*w + bown_f0(iy,im,ig,is,iz,iagg))*(1 - taxprime(iz,iagg,iz))
									iy0low    = ly(yprime_f0)
									iy0upp    = uy(yprime_f0)
									y0weight  = wy(yprime_f0)
									mean_earndefault  = mean_earndefault + labor(isprime,ieprime)*w*(1 - y0weight)*(1 - psell_f0(iy0low,im,ig,isprime,iz,iagg))&
									*d_f0(iy0low,im,ig,isprime,iz,iagg)*emeas(ieprime)*prob_ss(is,isprime,iz,iz)*mu3own_f0(iy,im,ig,is)
									mean_earndefault  = mean_earndefault + labor(isprime,ieprime)*w*y0weight*(1 - psell_f0(iy0upp,im,ig,isprime,iz,iagg))&
									*d_f0(iy0upp,im,ig,isprime,iz,iagg)*emeas(ieprime)*prob_ss(is,isprime,iz,iz)*mu3own_f0(iy,im,ig,is)
									mean_assetdefault = mean_assetdefault + bown_f0(iy,im,ig,is,iz,iagg)*(1 - y0weight)*(1 - psell_f0(iy0low,im,ig,isprime,iz,iagg))&
									*d_f0(iy0low,im,ig,isprime,iz,iagg)*emeas(ieprime)*prob_ss(is,isprime,iz,iz)*mu3own_f0(iy,im,ig,is)
									mean_assetdefault = mean_assetdefault + bown_f0(iy,im,ig,is,iz,iagg)*y0weight*(1 - psell_f0(iy0upp,im,ig,isprime,iz,iagg))&
									*d_f0(iy0upp,im,ig,isprime,iz,iagg)*emeas(ieprime)*prob_ss(is,isprime,iz,iz)*mu3own_f0(iy,im,ig,is)
								enddo
							enddo
						endif
					enddo
				enddo
			enddo
		enddo
		mean_earndefault  = 4.0*mean_earndefault/foreclosures
		mean_assetdefault = mean_assetdefault/foreclosures
	else
		mean_earndefault  = 0.d0
		mean_assetdefault = 0.d0
	endif
	
	!Print output
	open(unit=11,	file='modelstats_means.txt', status = 'replace')
	write(11, '(a1)'), ' '
	write(11, '(a50)'), ' ******Aggregate statistics: ratios of means******'
	write(11, '(a1)'), ' '
	write(11, '(a50)'), '--------Output, consumption, and investment--------'
	write(11, '(a1)'), ' '
	write(11, '(a41, f10.6, a9, f10.6)'), '             Homeownership rate: model = ' ,ownrate, ', data = ' ,0.64
	write(11, '(a41, f10.6, a9, f10.6)'), '    Imputed housing consumption: model = ' ,hcons_impute_earn, ', data = ' ,0.17 !AHS
	write(11, '(a41, f10.6, a9, f10.6)'), '     Renter housing consumption: model = ' ,hcons_rent_earn, ', data = ' ,0.29 !AHS
	write(11, '(a1)'), ' '
	write(11, '(a41, f10.6, a9, f10.6)'), '     Housing consumption to GDP: model = ' ,hcons_gdp, ', data = ' ,0.15
	write(11, '(a41, f10.6, a9, f10.6)'), ' Non-housing consumption to GDP: model = ' ,cons_gdp, ', data = ' ,0.68
	write(11, '(a41, f10.6, a9, f10.6)'), '  Non-housing investment to GDP: model = ' ,invest_k_gdp, ', data = ' ,0.14
	write(11, '(a41, f10.6, a9, f10.6)'), '  Residential investment to GDP: model = ' ,invest_h_gdp, ', data = ' ,0.045
	write(11, '(a41, f10.6)'), '     Government spending to GDP: model = ' ,govspend_gdp
	write(11, '(a41, f10.6)'), '             Net exports to GDP: model = ' ,netexports_gdp
	write(11, '(a41, f10.6, a9, f10.6)'), '     Non-housing capital to GDP: model = ' ,kstock_gdp, ', data = ' ,1.64
	write(11, '(a41, f10.6, a9, f10.6)'), '         Housing capital to GDP: model = ' ,hownstock_gdp, ', data = ' ,1.04
	write(11, '(a1)'), ' '
	write(11, '(a50)'), '------------------Interest rates------------------'
	write(11, '(a1)'), ' '
	write(11, '(a41, f10.6, a9, f10.6)'), '                 Risk-free rate: model = ' ,(1 + rb(iz,iagg))**4 - 1, ', data = ' ,0.04
	write(11, '(a41, f10.6)'), '                  Mortgage rate: model = ' ,(1/qm(iz,iagg))**4 - 1
	write(11, '(a1)'), ' '
	write(11, '(a50)'), '-----------Financial statistics: means------------'
	write(11, '(a1)'), ' '
	write(11, '(a41, f10.6, a9, f10.6)'), '           Earnings mean/median: model = ' ,mean_earn/med_earn, ', data = ' ,1.51
	write(11, '(a41, f10.6, a9, f10.6)'), '          Net worth mean/median: model = ' ,mean_networth/med_networth, ', data = ' ,3.96
	write(11, '(a41, f10.6, a9, f10.6)'), '             Assets mean/median: model = ' ,mean_assets/med_assets, ', data = ' ,7.65
	write(11, '(a41, f10.6, a9, f10.6)'), '            Housing mean/median: model = ' ,mean_hwealth/med_hwealth, ', data = ' ,1.39
	write(11, '(a41, f10.6, a9, f10.6)'), '      Mortgage debt mean/median: model = ' ,mean_mortdebt/med_mortdebt, ', data = ' ,1.87
	write(11, '(a1)'), ' '
	write(11, '(a41, f10.6, a9, f10.6)'), '    Relative homeowner earnings: model = ' ,mean_earnown/mean_earn, ', data = ' ,1.21
	write(11, '(a41, f10.6, a9, f10.6)'), '               Financial assets: model = ' ,mean_assets, ', data = ' ,5.89 !2.575 (different asset definition; agecl = 1 - 4)
	write(11, '(a41, f10.6, a9, f10.6)'), '                      Net worth: model = ' ,mean_networth, ', data = ' ,7.41 !3.609 (different NW definition; agecl = 1 - 4)
	write(11, '(a41, f10.6, a9, f10.6)'), '       Homeowner housing wealth: model = ' ,mean_hwealth, ', data = ' ,3.62
	write(11, '(a41, f10.6, a9, f10.6)'), '        Homeowner mortgage debt: model = ' ,mean_mortdebt, ', data = ' ,1.32
	write(11, '(a41, f10.6, a9, f10.6)'), '     Homeowner mortgage debt|>0: model = ' ,mean_mortdebt_kpos, ', data = ' ,2.03
	write(11, '(a41, f10.6, a9, f10.6)'), '            Homeowner net worth: model = ' ,mean_networthown, ', data = ' ,10.61
	write(11, '(a1)'), ' '
	write(11, '(a41, f10.6, a9, f10.6)'), '      Housing to owner earnings: model = ' ,mean_hwealth_earnown, ', data = ' ,2.99 !2.375 (agecl = 1 - 4)
	write(11, '(a41, f10.6, a9, f10.6)'), '     Mortgage to owner earnings: model = ' ,mean_mortdebt_earnown, ', data = ' ,1.09 !1.08 (agecl = 1 - 4)
	write(11, '(a41, f10.6, a9, f10.6)'), '  Mortgage to owner earnings|>0: model = ' ,mean_mortdebt_earnown_kpos, ', data = ' ,1.67
	write(11, '(a41, f10.6, a9, f10.6)'), '    Net worth to owner earnings: model = ' ,mean_networthown_earnown, ', data = ' ,8.76 !4.291 (different NW definition; agecl = 1 - 4)
	write(11, '(a1)'), ' '
	write(11, '(a50)'), '----------Financial statistics: medians-----------'
	write(11, '(a1)'), ' '
	write(11, '(a41, f10.6, a9, f10.6)'), '    Relative homeowner earnings: model = ' ,med_earnown/med_earn, ', data = ' ,1.36
	write(11, '(a41, f10.6, a9, f10.6)'), '               Financial assets: model = ' ,med_assets, ', data = ' ,1.16 !0.282 (different asset definition; agecl = 1 - 4)
	write(11, '(a41, f10.6, a9, f10.6)'), '                      Net worth: model = ' ,med_networth, ', data = ' ,2.83 !0.927 (different NW definition; agecl = 1 - 4)
	write(11, '(a41, f10.6, a9, f10.6)'), '       Homeowner housing wealth: model = ' ,med_hwealth, ', data = ' ,3.94
	write(11, '(a41, f10.6, a9, f10.6)'), '        Homeowner mortgage debt: model = ' ,med_mortdebt, ', data = ' ,1.07
	write(11, '(a41, f10.6, a9, f10.6)'), '            Homeowner net worth: model = ' ,med_networthown, ', data = ' ,5.22
	write(11, '(a1)'), ' '
	write(11, '(a41, f10.6, a9, f10.6)'), '      Housing to owner earnings: model = ' ,med_hwealth_earnown, ', data = ' ,2.90 !2.098 (agecl = 1 - 4)
	write(11, '(a41, f10.6, a9, f10.6)'), '     Mortgage to owner earnings: model = ' ,med_mortdebt_earnown, ', data = ' ,0.78 !1.08 (agecl = 1 - 4)
	write(11, '(a41, f10.6, a9, f10.6)'), '    Net worth to owner earnings: model = ' ,med_networthown_earnown, ', data = ' ,3.83 !1.587 (different NW definition; agecl = 1 - 4)
	write(11, '(a1)'), ' '
	write(11, '(a50)'), '---------Mortgage distribution statistics---------'
	write(11, '(a1)'), ' '
	write(11, '(a41, f10.6, a9, f10.6)'), '         Percent with mortgages: model = ' ,pct_mort, ', data = ' ,0.651 !0.791 (agecl = 1 - 4)
	write(11, '(a41, f10.6, a9, f10.6)'), '                       Mean ltv: model = ' ,mean_ltv, ', data = ' ,0.37 !0.468 (agecl = 1 - 4)
	write(11, '(a41, f10.6, a9, f10.6)'), '       Percentage with >80% ltv: model = ' ,ltv80, ', data = ' ,0.137 !0.176 (agecl = 1 - 4)
	write(11, '(a41, f10.6, a9, f10.6)'), '       Percentage with >90% ltv: model = ' ,ltv90, ', data = ' ,0.078 !0.102 (agecl = 1 - 4)
	write(11, '(a41, f10.6, a9, f10.6)'), '       Percentage with >95% ltv: model = ' ,ltv95, ', data = ' ,0.049 !0.065 (agecl = 1 - 4)
	write(11, '(a1)'), ' '
	write(11, '(a41, f10.6, a9, f10.6)'), '                 Mean ltv|ltv>0: model = ' ,mean_ltv_kpos, ', data = ' ,0.57 !0.59 (agecl = 1 - 4)
	write(11, '(a41, f10.6, a9, f10.6)'), ' Percentage with >80% ltv|ltv>0: model = ' ,ltv80_kpos, ', data = ' ,0.212 !0.222 (agecl = 1 - 4)
	write(11, '(a41, f10.6, a9, f10.6)'), ' Percentage with >90% ltv|ltv>0: model = ' ,ltv90_kpos, ', data = ' ,0.121 !0.129 (agecl = 1 - 4)
	write(11, '(a41, f10.6, a9, f10.6)'), ' Percentage with >95% ltv|ltv>0: model = ' ,ltv95_kpos, ', data = ' ,0.076 !0.082 (agecl = 1 - 4)
	write(11, '(a1)'), ' '
	write(11, '(a50)'), '--------------Foreclosure statistics--------------'
	write(11, '(a1)'), ' '
	write(11, '(a41, f10.6, a9, f10.6)'), '               Foreclosure rate: model = ' ,frate, ', data = ' ,0.00088
	write(11, '(a41, f10.6, a9, f10.6)'), '           Foreclosure discount: model = ' ,REOdiscount, ', data = ' ,0.22
	write(11, '(a41, f10.6, a9, f10.6)'), '     Foreclosure recovery ratio: model = ' ,frecover, ', data = ' ,0.65
	write(11, '(a41, f10.6)'), '        Annual earnings|default: model = ' ,mean_earndefault
	write(11, '(a41, f10.6)'), '                 Assets|default: model = ' ,mean_assetdefault
	write(11, '(a1)'), ' '
	write(11, '(a50)'), '-----Housing sales and inventories statistics-----'
	write(11, '(a1)'), ' '
	write(11, '(a41, f10.6, a9, f10.6)'), '                       Turnover: model = ' ,turnover, ', data = ' ,0.02
	write(11, '(a41, f10.6, a9, f10.6)'), '                     Sales rate: model = ' ,salesrate, ', data = ' ,0.06
	write(11, '(a41, f10.6, a9, f10.6)'), '                REO sales share: model = ' ,reosaleshare, ', data = ' ,0.05
	write(11, '(a41, f10.6, a9, f10.6)'), '                New sales share: model = ' ,newsaleshare, ', data = ' ,0.1583
	write(11, '(a41, f10.6, a9, f10.6)'), '               Months of supply: model = ' ,months_supply, ', data = ' ,6.0
	write(11, '(a41, f10.6, a9, f10.6)'), '         Mean buyer TOM (weeks): model = ' ,mean_TOMbuy, ', data = ' ,10.0
	write(11, '(a41, f10.6, a9, f10.6)'), '        Mean seller TOM (weeks): model = ' ,mean_TOMsell, ', data = ' ,17.33
	write(11, '(a41, f10.6, a9, f10.6)'), '           Mean REO TOM (weeks): model = ' ,mean_TOMREO, ', data = ' ,52.0
	write(11, '(a41, f10.6, a9, f10.6)'), '      Mean sales price discount: model = ' ,mean_xsdiscount, ', data = ' ,0.07
	close(11)
	
	!Initial capital for simulation
	kstock = kc
	
end subroutine sub_aggstats_ratiomeans