subroutine sub_buscyc_stats(series,nt,nseries,refvar,nlagcorr,sdvec,corrmat,autocorrmat,crosscorrmat,ph_id,yc_id,nlagreg,phregress,Rsquared)

	use mod_globvar
	use mod_matlab

	implicit none

	!This program computes business cycle statistics for the model
	!Refvar is the reference variable for comparisons of standard deviations and for lagged correlations (usually GDP)
	
	integer, intent(in) :: nt, nseries, refvar, nlagcorr, ph_id, yc_id, nlagreg
	real(8), dimension(nt,nseries), intent(in) :: series
	real(8), dimension(nseries), intent(out) :: sdvec
	real(8), dimension(nseries,nseries), intent(out) :: corrmat
	real(8), dimension(nseries,2*nlagcorr + 1), intent(out) :: autocorrmat, crosscorrmat
	real(8), dimension(2 + nlagreg,2), intent(out) :: phregress !first column coefficients, second column standard errors
	
	!Lagged time series for correlations (-nlagcorr, -nlagcorr + 1, ..., -1, 0, 1, ..., nlagcorr - 1, nlagcorr)
	real(8), dimension(:,:,:), allocatable :: series_lags
	
	!Price appreciation regression
	real(8), dimension(:,:), allocatable :: Xdata, idmat
	real(8), dimension(:), allocatable :: Ydata, Ypredict, Ymeans, SEtemp
	real(8) :: Rsquared, ssquared
	
	!HP-filtered series
	real(8), dimension(:,:), allocatable :: series_trend, series_detrend, junk

	!Indices
	integer :: iseries, ilag, it, irow, icolumn, iregressor
		
	!Other internal variables
	real(8) :: lambda
	integer :: itmod
	
	!Allocate variables
	allocate(series_trend(nt,nseries))
	allocate(series_detrend(nt,nseries))
	allocate(junk(nt,3))
	allocate(series_lags(nt,nseries,2*nlagcorr + 1))
	allocate(Xdata(nt - 1 - nlagreg,2 + nlagreg))
	allocate(Ydata(nt - 1 - nlagreg))
	allocate(Ypredict(nt - 1 - nlagreg))
	allocate(Ymeans(nt - 1 - nlagreg))
	allocate(idmat(2 + nlagreg,2 + nlagreg))
	allocate(SEtemp(2 + nlagreg))
	
	!Run HP filter
	lambda = 1e5
	do iseries = 1, nseries
		call HPFILT(series(:,iseries),series_detrend(:,iseries),junk,nt,lambda,1)
	enddo
	
	!Compute standard deviations and correlations
	sdvec   = std_mat(series_detrend)
	corrmat = corr_mat(series_detrend)
	
	!Compute lagged time series: x(t - nlagcorr), x(t - nlagcorr + 1), ..., x(t), x(t + 1), ..., x(t + nlagcorr)
	do ilag = 1, 2*nlagcorr + 1
		series_lags(:,:,ilag) = cshift(series,-nlagcorr + ilag - 1)
	enddo
	
	!Compute lagged correlations-- autocorrmat(iseries,ilag) = corr(x_iseries(t),x_iseries(t + (-nlagcorr + ilag - 1)))$
	!crosscorrmat(iseries,ilag) = corr(x_refvar(t),x_iseries(t + (-nlaggcorr + ilag - 1)))
	do iseries = 1, nseries
		do ilag = 1, 2*nlagcorr + 1
			autocorrmat(iseries,ilag)  = corr_vecs(series(1 + nlagcorr:nt - nlagcorr,iseries),&
			series_lags(1 + nlagcorr:nt - nlagcorr,iseries,ilag))
			crosscorrmat(iseries,ilag) = corr_vecs(series(1 + nlagcorr:nt - nlagcorr,refvar),&
			series_lags(1 + nlagcorr:nt - nlagcorr,iseries,ilag))
		enddo
	enddo
	
	!Run regression dlnph_t = alpha + sum_{l=1:L}beta_l dlnph_{t - l} + gamma dlnyc_t + eps_t
	!Assumes that house price appreciation is the last series and that ph_t and yc_t (annualized) are already in logs
	do it = 2 + nlagreg, nt
		itmod = it - 1 - nlagreg
		Ydata(itmod)   = 4.0*(series(it,ph_id) - series(it - 1,ph_id))
		Xdata(itmod,1) = 1.d0
		do ilag = 1, nlagreg
			Xdata(itmod,1 + ilag) = 4.0*(series(it - ilag,ph_id) - series(it - ilag - 1,ph_id))
		enddo
		Xdata(itmod,2 + nlagreg) = 4.0*(series(it,yc_id) - series(it - 1,yc_id))
	enddo
	phregress(:,1) = regress(Ydata,Xdata)
	!Compute standard errors and R^2
	Ypredict = matmul(Xdata,phregress(:,1))
	Ymeans   = sum(Ydata)/real(nt - 1 - nlagreg,8)
	Rsquared = 1 - dot_product(Ydata - Ypredict,Ydata - Ypredict)/dot_product(Ydata - Ymeans,Ydata - Ymeans)
	ssquared = dot_product(Ydata - Ypredict,Ydata - Ypredict)/((nt - 1 - nlagreg) - (2 + nlagreg))
	!Compute identity matrix
	do irow = 1, 2 + nlagreg
		do icolumn = 1, 2 + nlagreg
			if (irow == icolumn) then
				idmat(irow,icolumn) = 1.d0
			else
				idmat(irow,icolumn) = 0.d0
			endif
		enddo
	enddo
	do iregressor = 1, 2 + nlagreg
		SEtemp = ssquared*mldivide(matmul(transpose(Xdata),Xdata),idmat(:,iregressor))
		phregress(iregressor,2) = SEtemp(iregressor)**0.5
	enddo

end subroutine sub_buscyc_stats