subroutine sub_equilibrium

	use mod_globvar
	use mod_interpolate
	use mod_krusellsmith_regress

	implicit none

	!This program computes the stationary equilibrium for the housing model with borrowing and default

	!Indices-- individual states and choice variables
	integer :: im, imprime, ibprime, iy, ig, ih, iu, igshock, is, isprime, ieprime
	
	!Indices-- aggregate states and miscellaneous
	integer :: iter_coeff, iter_q, iter_qtemp, iz, izprime, iagg, it, iz0, iph0, ik0, index
	
	!Updated mortgage prices
	real(8), dimension(ne) :: qm0_e
	real(8), dimension(ns) :: qm0_s
	real(8), dimension(nu) :: qm0_u
	real(8), dimension(nz) :: qm0_z
	real(8), dimension(:,:,:,:,:,:), allocatable :: qm0new, qm0temp
	real(8), dimension(4) :: mrecov, d_low, d_upp, pxs_low, pxs_upp, qm0_low, qm0_upp
	integer, dimension(4) :: im_low, im_upp, ib_low, ib_upp
	real(8) :: yprime_f0, y0weight
	integer :: iy0low, iy0upp
	
	!Updated forecasting coefficients
	real(8), dimension(3,3,nz,nz) :: forecast_coeffnew
	
	!Simulation and regression
	real(8), dimension(:), allocatable :: zrand, ph_t, ph_tprime, kstock_t, assets_t, kstock_tprime, assets_tprime, tax_t, tax_tprime, yh_t, yc_t, ownrate_t, htot_t	
	integer, dimension(:), allocatable :: locz_t, locz_tprime
	real(8), dimension(nz,nz) :: forecast_coefftemp
	real(8), dimension(3) :: Rsquared                                       !R^2 for regression-- Rsquared(1) is for ph, Rsquared(2) is for K, and Rsquared(3) is for taxprime
	integer :: version_ph, version_k, version_tax                           !Regression versions
	integer :: T                                                            !Length of simulation
	integer :: T_ignore                                                     !Number of simulation periods to ignore in regression
	
	!Vector of configuration parameters for equilibrium subroutine
	integer, dimension(16) :: config_equil
	
	!Normed difference between successive iterations
	real(8) :: err_coeff                                                    !OLS coefficients
	real(8) :: err_qtemp                                                    !Inner qm0 loop
	real(8) :: err_q                                                        !Outer qm0 loop
	
	!Maximum iterations
	integer :: max_itercoeff                                                !Maximum number of OLS coefficient loop iterations
	integer :: max_iterqtemp                                                !Maximum number of inner qm0 loop iterations
	integer :: max_iterq                                                    !Maximum number of outer qm0 loop iterations
	integer :: max_iterq_1                                                  !Maximum number of outer qm0 loop iterations when iter_coeff = 1
	integer :: max_iterq_2                                                  !Maximum number of outer qm0 loop iterations when iter_coeff >= 2
	integer :: max_iterv_1                                                  !Maximum number of value function iterations when iter_coeff = 1 and iter_q = 1
	integer :: max_itervaccel_1                                             !Maximum number of accelerated value function iterations when iter_coeff = 1 and iter_q = 1
	integer :: max_iterv_2                                                  !Maximum number of value function iterations when iter_coeff >= 2 or iter_q >= 2
	integer :: max_itervaccel_2                                             !Maximum number of accelerated value function iterations when iter_coeff >=2 and iter_q >= 2
	
	!Other internal variables
	real(8) :: ylower, spending_f0, c_f0, hc_f0, spending_f1, c_f1, hc_f1, phe0, rk_expect
	
	!Load configuration
	open(unit=11,	file='Input\config_equil.txt')
	read (11, *) config_equil
	close(11)
	T                = config_equil(1)
	T_ignore         = config_equil(2)
	max_itercoeff    = config_equil(3)
	max_iterqtemp    = config_equil(4)
	max_iterq_1      = config_equil(5)
	max_iterq_2      = config_equil(6)
	max_iterv_1      = config_equil(7)
	max_itervaccel_1 = config_equil(8)
	max_iterv_2      = config_equil(9)
	max_itervaccel_2 = config_equil(10)
	iz0              = config_equil(11)
	iph0             = config_equil(12)
	ik0              = config_equil(13)
	version_ph       = config_equil(14)
	version_k        = config_equil(15)
	version_tax      = config_equil(16)
	
	!Allocate updated mortgage prices
	allocate(qm0new(nm,nb,ng,ns,nz,nagg))
	allocate(qm0temp(nm,nb,ng,ns,nz,nagg))
	
	!Initialize value functions, forecasting coefficients, and mortgage prices
	if (load_data == 1) then
		!Load value functions, forecasting coefficients, and mortgage prices
		call sub_loadfiles
		!Forecasting equations
		do iz = 1, nz
			do iagg = 1, nagg
				do izprime = 1, nz
					phprime(iz,iagg,izprime)     = dot_product(forecast_coeff(1,:,iz,izprime),forecast_vec(:,iagg))
					kaggprime(iz,iagg,izprime)	 = dot_product(forecast_coeff(2,:,iz,izprime),forecast_vec(:,iagg))
					taxprime(iz,iagg,izprime)    = dot_product(forecast_coeff(3,:,iz,izprime),forecast_vec(:,iagg))
					iaggprime(iz,iagg,izprime,1) = aggfun(lp(phprime(iz,iagg,izprime)),lk(kaggprime(iz,iagg,izprime)))
					iaggprime(iz,iagg,izprime,2) = aggfun(up(phprime(iz,iagg,izprime)),lk(kaggprime(iz,iagg,izprime)))
					iaggprime(iz,iagg,izprime,3) = aggfun(lp(phprime(iz,iagg,izprime)),uk(kaggprime(iz,iagg,izprime)))
					iaggprime(iz,iagg,izprime,4) = aggfun(up(phprime(iz,iagg,izprime)),uk(kaggprime(iz,iagg,izprime)))
					waggprime(iz,iagg,izprime,1) = (1 - wp(phprime(iz,iagg,izprime)))*(1 - wk(kaggprime(iz,iagg,izprime)))
					waggprime(iz,iagg,izprime,2) = wp(phprime(iz,iagg,izprime))*(1 - wk(kaggprime(iz,iagg,izprime)))
					waggprime(iz,iagg,izprime,3) = (1 - wp(phprime(iz,iagg,izprime)))*wk(kaggprime(iz,iagg,izprime))
					waggprime(iz,iagg,izprime,4) = wp(phprime(iz,iagg,izprime))*wk(kaggprime(iz,iagg,izprime))
					call sub_factorprices_sh(kaggprime(iz,iagg,izprime),eavg,phprime(iz,iagg,izprime),zc(izprime),zh(izprime),wprime(iz,iagg,izprime),rkprime(iz,iagg,izprime))
				enddo
				rk_expect   = dot_product(rkprime(iz,iagg,:),prob_zz(iz,:))
				qb(iz,iagg) = 1/(1 + rk_expect - deprec_k)
				rb(iz,iagg) = 1/qb(iz,iagg) - 1
				qm(iz,iagg) = (1 - deprec_h)/((1 + rb(iz,iagg))*(1 + rscost))
			enddo
		enddo
		!Next period mortgage prices given the aggregate forecasting equations
		do iz = 1, nz
			do iagg = 1, nagg
				do izprime = 1, nz
					qmprime(iz,iagg,izprime,1) = (1 - deprec_h)/((1 + rb(iz,iaggprime(iz,iagg,izprime,1)))*(1 + rscost))
					qmprime(iz,iagg,izprime,2) = (1 - deprec_h)/((1 + rb(iz,iaggprime(iz,iagg,izprime,2)))*(1 + rscost))
					qmprime(iz,iagg,izprime,3) = (1 - deprec_h)/((1 + rb(iz,iaggprime(iz,iagg,izprime,3)))*(1 + rscost))
					qmprime(iz,iagg,izprime,4) = (1 - deprec_h)/((1 + rb(iz,iaggprime(iz,iagg,izprime,4)))*(1 + rscost))
				enddo
			enddo
		enddo
		!Optimal REO behavior
		Jreo = -100.d0
		call sub_valfn_reo
	else
		!Guess
		d_f0           = 0.1d0
		psell_f0       = 0.d0
		locm_f0        = 1
		locbown_f0_sp1 = 1
		!Value functions
		do iagg = 1, nagg
			do iz = 1, nz
				do is = 1, ns
					do iy = 1, ny
						spending_f0 = max(min_spend,yy(iy))
						c_f0        = cshare(iz)*spending_f0
						hc_f0       = (1 - cshare(iz))*spending_f0/pa
						spending_f1 = max(min_spend,(1 - fincloss)*yy(iy))
						c_f1        = cshare(iz)*spending_f1
						hc_f1       = (1 - cshare(iz))*spending_f1/pa
						Vrent_f0(iy,is,iz,iagg) = c_f0**cshare(iz)*hc_f0**(1 - cshare(iz))
						Vrent_f1(iy,is,iz,iagg) = c_f1**cshare(iz)*hc_f1**(1 - cshare(iz))
					enddo
					do ig = 1, ng
						ih = hfun(ig)
						do iy = 1, ny
							spending_f1 = max(min_spend,(1 - fincloss)*yy(iy) + ph(iagg)*hh(ih))
							c_f1        = cshare(iz)*spending_f1
							hc_f1       = (1 - cshare(iz))*spending_f1/pa
							Vown_f1(iy,ig,is,iz,iagg) = c_f1**cshare(iz)*hc_f1**(1 - cshare(iz))
						enddo
						do im = 1, nm
							do iy = 1, ny
								spending_f0 = max(min_spend,yy(iy) + (ph(iagg) - mm(im))*hh(ih))
								c_f0        = cshare(iz)*spending_f0
								hc_f0       = (1 - cshare(iz))*spending_f0/pa
								Vown_f0(iy,im,ig,is,iz,iagg) = c_f0**cshare(iz)*hc_f0**(1 - cshare(iz))
							enddo
						enddo
					enddo
				enddo
			enddo
		enddo
		!Forecasting coefficients
		forecast_coeff(1,1,:,:) = 0.d0
		forecast_coeff(1,2,:,:) = 1.d0
		forecast_coeff(1,3,:,:) = 0.d0
		forecast_coeff(2,1,:,:) = 0.d0
		forecast_coeff(2,3,:,:) = 1.d0
		forecast_coeff(2,2,:,:) = 0.d0
		forecast_coeff(3,:,:,:) = 0.d0
		!Forecasting equations
		do iz = 1, nz
			do iagg = 1, nagg
				do izprime = 1, nz
					phprime(iz,iagg,izprime)     = dot_product(forecast_coeff(1,:,iz,izprime),forecast_vec(:,iagg))
					kaggprime(iz,iagg,izprime)   = dot_product(forecast_coeff(2,:,iz,izprime),forecast_vec(:,iagg))
					taxprime(iz,iagg,izprime)    = dot_product(forecast_coeff(3,:,iz,izprime),forecast_vec(:,iagg))
					iaggprime(iz,iagg,izprime,1) = aggfun(lp(phprime(iz,iagg,izprime)),lk(kaggprime(iz,iagg,izprime)))
					iaggprime(iz,iagg,izprime,2) = aggfun(up(phprime(iz,iagg,izprime)),lk(kaggprime(iz,iagg,izprime)))
					iaggprime(iz,iagg,izprime,3) = aggfun(lp(phprime(iz,iagg,izprime)),uk(kaggprime(iz,iagg,izprime)))
					iaggprime(iz,iagg,izprime,4) = aggfun(up(phprime(iz,iagg,izprime)),uk(kaggprime(iz,iagg,izprime)))
					waggprime(iz,iagg,izprime,1) = (1 - wp(phprime(iz,iagg,izprime)))*(1 - wk(kaggprime(iz,iagg,izprime)))
					waggprime(iz,iagg,izprime,2) = wp(phprime(iz,iagg,izprime))*(1 - wk(kaggprime(iz,iagg,izprime)))
					waggprime(iz,iagg,izprime,3) = (1 - wp(phprime(iz,iagg,izprime)))*wk(kaggprime(iz,iagg,izprime))
					waggprime(iz,iagg,izprime,4) = wp(phprime(iz,iagg,izprime))*wk(kaggprime(iz,iagg,izprime))
					call sub_factorprices_sh(kaggprime(iz,iagg,izprime),eavg,phprime(iz,iagg,izprime),zc(izprime),zh(izprime),wprime(iz,iagg,izprime),rkprime(iz,iagg,izprime))
				enddo
				rk_expect   = dot_product(rkprime(iz,iagg,:),prob_zz(iz,:))
				qb(iz,iagg) = 1/(1 + rk_expect - deprec_k)
				rb(iz,iagg) = 1/qb(iz,iagg) - 1
				qm(iz,iagg) = (1 - deprec_h)/((1 + rb(iz,iagg))*(1 + rscost))
			enddo
		enddo
		!Next period mortgage prices given the aggregate forecasting equations
		do iz = 1, nz
			do iagg = 1, nagg
				do izprime = 1, nz
					qmprime(iz,iagg,izprime,1) = (1 - deprec_h)/((1 + rb(iz,iaggprime(iz,iagg,izprime,1)))*(1 + rscost))
					qmprime(iz,iagg,izprime,2) = (1 - deprec_h)/((1 + rb(iz,iaggprime(iz,iagg,izprime,2)))*(1 + rscost))
					qmprime(iz,iagg,izprime,3) = (1 - deprec_h)/((1 + rb(iz,iaggprime(iz,iagg,izprime,3)))*(1 + rscost))
					qmprime(iz,iagg,izprime,4) = (1 - deprec_h)/((1 + rb(iz,iaggprime(iz,iagg,izprime,4)))*(1 + rscost))
				enddo
			enddo
		enddo
		!Optimal REO behavior
		Jreo = -100.d0
		call sub_valfn_reo
		!Mortgage prices
		do iagg = 1, nagg
			do iz = 1, nz
				qm0temp(:,:,:,:,iz,iagg) = qm(iz,iagg)/(1 + rocost)
			enddo
		enddo
		print *, ' '
		do iter_qtemp = 1, max_iterqtemp
			do iagg = 1, nagg
				do iz = 1, nz
					qm0(1,:,:,:,iz,iagg) = qm(iz,iagg)/(1 + rocost)
					do is = 1, ns
						do ig = 1, ng
							ih      = hfun(ig)
							iu      = ufun(ig)
							igshock = gfun(ih,2)
							call omp_set_num_threads(num_threads)
							!$omp parallel private(imprime,izprime,isprime,ieprime,yprime_f0,iy0low,iy0upp,y0weight,im_low,ib_low,d_low,pxs_low,im_upp,ib_upp,d_upp,pxs_upp,mrecov,qm0_low,qm0_upp,qm0_e,qm0_s,qm0_u,qm0_z)
							!$omp do
							do ibprime = 1, nb
								do imprime = 2, nm
									do izprime = 1, nz
										if (iu == 1 .AND. nu == 2) then
											do isprime = 1, ns
												!No housing shock
												do ieprime = 1, ne
													yprime_f0  = (labor(isprime,ieprime)*wprime(iz,iagg,izprime) + bb(ibprime))*(1 - taxprime(iz,iagg,izprime))
													iy0low     = ly(yprime_f0)
													iy0upp     = uy(yprime_f0)
													y0weight   = wy(yprime_f0)
													im_low     = locm_f0(iy0low,imprime,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))
													ib_low     = locbown_f0_sp1(iy0low,imprime,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))
													d_low      = d_f0(iy0low,imprime,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))
													pxs_low    = psell_f0(iy0low,imprime,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))
													im_upp     = locm_f0(iy0upp,imprime,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))
													ib_upp     = locbown_f0_sp1(iy0upp,imprime,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))
													d_upp      = d_f0(iy0upp,imprime,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))
													pxs_upp    = psell_f0(iy0upp,imprime,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))
													mrecov     = min(1.d0,(Jreo(ih,izprime,iaggprime(iz,iagg,izprime,:)) + fdeficiency*yprime_f0)/(mm(imprime)*hh(ih)))
													qm0_low(1) = qm0temp(im_low(1),ib_low(1),ig,isprime,izprime,iaggprime(iz,iagg,izprime,1))
													qm0_upp(1) = qm0temp(im_upp(1),ib_upp(1),ig,isprime,izprime,iaggprime(iz,iagg,izprime,1))
													qm0_low(2) = qm0temp(im_low(2),ib_low(2),ig,isprime,izprime,iaggprime(iz,iagg,izprime,2))
													qm0_upp(2) = qm0temp(im_upp(2),ib_upp(2),ig,isprime,izprime,iaggprime(iz,iagg,izprime,2))
													qm0_low(3) = qm0temp(im_low(3),ib_low(3),ig,isprime,izprime,iaggprime(iz,iagg,izprime,3))
													qm0_upp(3) = qm0temp(im_upp(3),ib_upp(3),ig,isprime,izprime,iaggprime(iz,iagg,izprime,3))
													qm0_low(4) = qm0temp(im_low(4),ib_low(4),ig,isprime,izprime,iaggprime(iz,iagg,izprime,4))
													qm0_upp(4) = qm0temp(im_upp(4),ib_upp(4),ig,isprime,izprime,iaggprime(iz,iagg,izprime,4))
													!Update mortgage prices
													qm0_e(ieprime) = dot_product(waggprime(iz,iagg,izprime,:),(1 - y0weight)*(pxs_low + (1 - pxs_low)*(d_low*mrecov&
													+ (1 - d_low)*(1 + (1 + rscost)*(qm0_low*(1 + rocost) - qmprime(iz,iagg,izprime,:))*mm(im_low)/mm(imprime)))) + y0weight&
													*(pxs_upp + (1 - pxs_upp)*(d_upp*mrecov + (1 - d_upp)*(1 + (1 + rscost)*(qm0_upp*(1 + rocost) - qmprime(iz,iagg,izprime,:))&
													*mm(im_upp)/mm(imprime)))))
												enddo
												qm0_u(1) = dot_product(qm0_e,emeas)
												!Housing shock
												do ieprime = 1, ne
													yprime_f0  = (labor(isprime,ieprime)*wprime(iz,iagg,izprime) + bb(ibprime))*(1 - taxprime(iz,iagg,izprime))
													iy0low     = ly(yprime_f0)
													iy0upp     = uy(yprime_f0)
													y0weight   = wy(yprime_f0)
													im_low     = locm_f0(iy0low,imprime,igshock,isprime,izprime,iaggprime(iz,iagg,izprime,:))
													ib_low     = locbown_f0_sp1(iy0low,imprime,igshock,isprime,izprime,iaggprime(iz,iagg,izprime,:))
													d_low      = d_f0(iy0low,imprime,igshock,isprime,izprime,iaggprime(iz,iagg,izprime,:))
													pxs_low    = psell_f0(iy0low,imprime,igshock,isprime,izprime,iaggprime(iz,iagg,izprime,:))
													im_upp     = locm_f0(iy0upp,imprime,igshock,isprime,izprime,iaggprime(iz,iagg,izprime,:))
													ib_upp     = locbown_f0_sp1(iy0upp,imprime,igshock,isprime,izprime,iaggprime(iz,iagg,izprime,:))
													d_upp      = d_f0(iy0upp,imprime,igshock,isprime,izprime,iaggprime(iz,iagg,izprime,:))
													pxs_upp    = psell_f0(iy0upp,imprime,igshock,isprime,izprime,iaggprime(iz,iagg,izprime,:))
													mrecov     = min(1.d0,(Jreo(ih,izprime,iaggprime(iz,iagg,izprime,:)) + fdeficiency*yprime_f0)/(mm(imprime)*hh(ih)))
													qm0_low(1) = qm0temp(im_low(1),ib_low(1),igshock,isprime,izprime,iaggprime(iz,iagg,izprime,1))
													qm0_upp(1) = qm0temp(im_upp(1),ib_upp(1),igshock,isprime,izprime,iaggprime(iz,iagg,izprime,1))
													qm0_low(2) = qm0temp(im_low(2),ib_low(2),igshock,isprime,izprime,iaggprime(iz,iagg,izprime,2))
													qm0_upp(2) = qm0temp(im_upp(2),ib_upp(2),igshock,isprime,izprime,iaggprime(iz,iagg,izprime,2))
													qm0_low(3) = qm0temp(im_low(3),ib_low(3),igshock,isprime,izprime,iaggprime(iz,iagg,izprime,3))
													qm0_upp(3) = qm0temp(im_upp(3),ib_upp(3),igshock,isprime,izprime,iaggprime(iz,iagg,izprime,3))
													qm0_low(4) = qm0temp(im_low(4),ib_low(4),igshock,isprime,izprime,iaggprime(iz,iagg,izprime,4))
													qm0_upp(4) = qm0temp(im_upp(4),ib_upp(4),igshock,isprime,izprime,iaggprime(iz,iagg,izprime,4))
													!Update mortgage prices
													qm0_e(ieprime) = dot_product(waggprime(iz,iagg,izprime,:),(1 - y0weight)*(pxs_low + (1 - pxs_low)*(d_low*mrecov&
													+ (1 - d_low)*(1 + (1 + rscost)*(qm0_low*(1 + rocost) - qmprime(iz,iagg,izprime,:))*mm(im_low)/mm(imprime)))) + y0weight&
													*(pxs_upp + (1 - pxs_upp)*(d_upp*mrecov + (1 - d_upp)*(1 + (1 + rscost)*(qm0_upp*(1 + rocost) - qmprime(iz,iagg,izprime,:))&
													*mm(im_upp)/mm(imprime)))))
												enddo
												qm0_u(2) = dot_product(qm0_e,emeas)
												qm0_s(isprime) = (1 - pseparate)*qm0_u(1) + pseparate*qm0_u(2)
											enddo
										else
											do isprime = 1, ns
												do ieprime = 1, ne
													yprime_f0  = (labor(isprime,ieprime)*wprime(iz,iagg,izprime) + bb(ibprime))*(1 - taxprime(iz,iagg,izprime))
													iy0low     = ly(yprime_f0)
													iy0upp     = uy(yprime_f0)
													y0weight   = wy(yprime_f0)
													im_low     = locm_f0(iy0low,imprime,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))
													ib_low     = locbown_f0_sp1(iy0low,imprime,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))
													d_low      = d_f0(iy0low,imprime,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))
													pxs_low    = psell_f0(iy0low,imprime,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))
													im_upp     = locm_f0(iy0upp,imprime,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))
													ib_upp     = locbown_f0_sp1(iy0upp,imprime,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))
													d_upp      = d_f0(iy0upp,imprime,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))
													pxs_upp    = psell_f0(iy0upp,imprime,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))
													mrecov     = min(1.d0,(Jreo(ih,izprime,iaggprime(iz,iagg,izprime,:)) + fdeficiency*yprime_f0)/(mm(imprime)*hh(ih)))
													qm0_low(1) = qm0temp(im_low(1),ib_low(1),ig,isprime,izprime,iaggprime(iz,iagg,izprime,1))
													qm0_upp(1) = qm0temp(im_upp(1),ib_upp(1),ig,isprime,izprime,iaggprime(iz,iagg,izprime,1))
													qm0_low(2) = qm0temp(im_low(2),ib_low(2),ig,isprime,izprime,iaggprime(iz,iagg,izprime,2))
													qm0_upp(2) = qm0temp(im_upp(2),ib_upp(2),ig,isprime,izprime,iaggprime(iz,iagg,izprime,2))
													qm0_low(3) = qm0temp(im_low(3),ib_low(3),ig,isprime,izprime,iaggprime(iz,iagg,izprime,3))
													qm0_upp(3) = qm0temp(im_upp(3),ib_upp(3),ig,isprime,izprime,iaggprime(iz,iagg,izprime,3))
													qm0_low(4) = qm0temp(im_low(4),ib_low(4),ig,isprime,izprime,iaggprime(iz,iagg,izprime,4))
													qm0_upp(4) = qm0temp(im_upp(4),ib_upp(4),ig,isprime,izprime,iaggprime(iz,iagg,izprime,4))
													!Update mortgage prices
													qm0_e(ieprime) = dot_product(waggprime(iz,iagg,izprime,:),(1 - y0weight)*(pxs_low + (1 - pxs_low)*(d_low*mrecov&
													+ (1 - d_low)*(1 + (1 + rscost)*(qm0_low*(1 + rocost) - qmprime(iz,iagg,izprime,:))*mm(im_low)/mm(imprime)))) + y0weight&
													*(pxs_upp + (1 - pxs_upp)*(d_upp*mrecov + (1 - d_upp)*(1 + (1 + rscost)*(qm0_upp*(1 + rocost) - qmprime(iz,iagg,izprime,:))&
													*mm(im_upp)/mm(imprime)))))
												enddo
												qm0_s(isprime) = dot_product(qm0_e,emeas)
											enddo
										endif
										qm0_z(izprime) = dot_product(qm0_s,prob_ss(is,:,iz,izprime))
									enddo
									qm0(imprime,ibprime,ig,is,iz,iagg) = qm(iz,iagg)/(1 + rocost)*dot_product(qm0_z,prob_zz(iz,:))
								enddo
							enddo
							!$omp end do
							!$omp end parallel
						enddo
					enddo
				enddo
			enddo
			err_qtemp = maxval(abs(qm0 - qm0temp))
			if (iter_qtemp - 1 == 10*floor((real(iter_qtemp) - 1)/10.0)) then
				write(*, '(a27 i4, a14, f8.5)'), '     Inner mortgage loop = ',iter_qtemp,', err_qtemp = ',err_qtemp
			endif
			if (err_qtemp < 1e-5 .OR. iter_qtemp == max_iterqtemp) then
				exit
			endif
			qm0temp = qm0
		enddo
		!Initialize distribution of agents
		mu1rent_f0 = 0.d0
		mu1rent_f1 = 0.d0
		mu1own_f1  = 0.d0
		mu1own_f0  = 0.d0
		do is = 1, ns
			do ig = 1, ng
				do im = 1, nm
					do iy = ny_neg + 1, ny
						if (iy == ny_neg + 1 .AND. im == 1 .AND. ig == 1) then
							mu1own_f0(iy,im,ig,is) = 1.d0/real(ns,8)
						endif
					enddo
				enddo
			enddo
		enddo
	endif
	
	!Allocate variables for simulation and regression
	!Regression matrices
	allocate(zrand(T + 1))
	allocate(locz_t(T + 1))
	allocate(locz_tprime(T))
	!Equilibrium prices
	allocate(ph_t(T))
	allocate(ph_tprime(T))
	!Output and capital stock
	allocate(yh_t(T))
	allocate(yc_t(T))
	allocate(kstock_t(T))
	allocate(assets_t(T))
	allocate(kstock_tprime(T))
	allocate(assets_tprime(T))
	!Homeownership rate, and housing stock
	allocate(ownrate_t(T))
	allocate(htot_t(T))
	!Taxes
	allocate(tax_t(T))
	allocate(tax_tprime(T))
	!Generate sequence of aggregate shocks
	do it = 1, T + 1
		call random_number(zrand(it))
		if (it == 1) then
			do iz = 1, nz
				if (zrand(it) <= sum(stationary_z(1:iz))) then
					locz_t(it) = iz
					exit
				endif
			enddo
		else
			do iz = 1, nz
				if (zrand(it) <= sum(prob_zz(locz_t(it - 1),1:iz))) then
					locz_t(it) = iz
					exit
				endif
			enddo
		endif
	enddo
	!Generate {z(t+1)}
	do it = 1, T
		locz_tprime(it) = locz_t(it + 1)
	enddo
	
	!!!Compute stochastic equilibrium!!!
	!Outer loop -- forecasting coefficients
	err_coeff = 999.d0
	Rsquared  = 0.d0
	do iter_coeff = 1, max_itercoeff
		!Load configuration
		open(unit=11,	file='Input\config_equil.txt')
		read (11, *) config_equil
		close(11)		
		max_iterqtemp    = config_equil(4)
		max_iterq_1      = config_equil(5)
		max_iterq_2      = config_equil(6)
		max_iterv_1      = config_equil(7)
		max_itervaccel_1 = config_equil(8)
		max_iterv_2      = config_equil(9)
		max_itervaccel_2 = config_equil(10)
		iz0              = config_equil(11)
		iph0             = config_equil(12)
		ik0              = config_equil(13)
		version_ph       = config_equil(14)
		version_k        = config_equil(15)
		version_tax      = config_equil(16)
		!Inner loop -- mortgage prices
		if (iter_coeff == 1) then
			max_iterq = max_iterq_1
		else
			max_iterq = max_iterq_2
		endif
		do iter_q = 1, max_iterq
			!Calculate lower bound of y at mortgage origination
			do iz = 1, nz
				do iagg = 1, nagg
					do is = 1, ns
						do ig = 1, ng
							ih = hfun(ig)
							ylb(ig,is,iz,iagg) = max_y
							do imprime = 1, nm
								do ibprime = 1, nb
									if (qm0(imprime,ibprime,ig,is,iz,iagg)*mm(imprime) <= max_ltv*ph(iagg)) then
										ylower = qb(iz,iagg)*bb(ibprime) - qm0(imprime,ibprime,ig,is,iz,iagg)*mm(imprime)*hh(ih)
										if (ylower < ylb(ig,is,iz,iagg)) then
											ylb(ig,is,iz,iagg) = ylower
										endif
									endif
								enddo
							enddo
						enddo
					enddo
				enddo
			enddo
			!Compute value function iteration
			if (iter_coeff == 1 .AND. iter_q == 1) then
				itermax       = max_iterv_1
				itermax_accel = max_itervaccel_1
			else
				itermax       = max_iterv_2
				itermax_accel = max_itervaccel_2
			endif
			toler = 1e-6
			if (valfn_type == 1) then
				call sub_valfn_ces
			elseif (valfn_type == 2) then
				call sub_valfn_ez_ces
			endif
			!Print files
			if (print_input == 2) then
				call sub_print_input
			endif
			if (print_output == 2) then
				call sub_print_readable
				call sub_print_final
			endif
			!Calculate updated mortgage prices
			qm0temp = qm0
			print *, ' '
			do iter_qtemp = 1, max_iterqtemp
				do iagg = 1, nagg
					do iz = 1, nz
						qm0new(1,:,:,:,iz,iagg) = qm(iz,iagg)/(1 + rocost)
						do is = 1, ns
							do ig = 1, ng
								ih      = hfun(ig)
								iu      = ufun(ig)
								igshock = gfun(ih,2)
								call omp_set_num_threads(num_threads)
								!$omp parallel private(imprime,izprime,isprime,ieprime,yprime_f0,iy0low,iy0upp,y0weight,im_low,ib_low,d_low,pxs_low,im_upp,ib_upp,d_upp,pxs_upp,mrecov,qm0_low,qm0_upp,qm0_e,qm0_s,qm0_u,qm0_z)
								!$omp do
								do ibprime = 1, nb
									do imprime = 2, nm
										do izprime = 1, nz
											if (iu == 1 .AND. nu == 2) then
												do isprime = 1, ns
													!No housing shock
													do ieprime = 1, ne
														yprime_f0  = (labor(isprime,ieprime)*wprime(iz,iagg,izprime) + bb(ibprime))*(1 - taxprime(iz,iagg,izprime))
														iy0low     = ly(yprime_f0)
														iy0upp     = uy(yprime_f0)
														y0weight   = wy(yprime_f0)
														im_low     = locm_f0(iy0low,imprime,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))
														ib_low     = locbown_f0_sp1(iy0low,imprime,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))
														d_low      = d_f0(iy0low,imprime,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))
														pxs_low    = psell_f0(iy0low,imprime,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))
														im_upp     = locm_f0(iy0upp,imprime,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))
														ib_upp     = locbown_f0_sp1(iy0upp,imprime,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))
														d_upp      = d_f0(iy0upp,imprime,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))
														pxs_upp    = psell_f0(iy0upp,imprime,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))
														mrecov     = min(1.d0,(Jreo(ih,izprime,iaggprime(iz,iagg,izprime,:)) + fdeficiency*yprime_f0)/(mm(imprime)*hh(ih)))
														qm0_low(1) = qm0temp(im_low(1),ib_low(1),ig,isprime,izprime,iaggprime(iz,iagg,izprime,1))
														qm0_upp(1) = qm0temp(im_upp(1),ib_upp(1),ig,isprime,izprime,iaggprime(iz,iagg,izprime,1))
														qm0_low(2) = qm0temp(im_low(2),ib_low(2),ig,isprime,izprime,iaggprime(iz,iagg,izprime,2))
														qm0_upp(2) = qm0temp(im_upp(2),ib_upp(2),ig,isprime,izprime,iaggprime(iz,iagg,izprime,2))
														qm0_low(3) = qm0temp(im_low(3),ib_low(3),ig,isprime,izprime,iaggprime(iz,iagg,izprime,3))
														qm0_upp(3) = qm0temp(im_upp(3),ib_upp(3),ig,isprime,izprime,iaggprime(iz,iagg,izprime,3))
														qm0_low(4) = qm0temp(im_low(4),ib_low(4),ig,isprime,izprime,iaggprime(iz,iagg,izprime,4))
														qm0_upp(4) = qm0temp(im_upp(4),ib_upp(4),ig,isprime,izprime,iaggprime(iz,iagg,izprime,4))
														!Update mortgage prices
														qm0_e(ieprime) = dot_product(waggprime(iz,iagg,izprime,:),(1 - y0weight)*(pxs_low + (1 - pxs_low)*(d_low*mrecov&
														+ (1 - d_low)*(1 + (1 + rscost)*(qm0_low*(1 + rocost) - qmprime(iz,iagg,izprime,:))*mm(im_low)/mm(imprime)))) + y0weight&
														*(pxs_upp + (1 - pxs_upp)*(d_upp*mrecov + (1 - d_upp)*(1 + (1 + rscost)*(qm0_upp*(1 + rocost) - qmprime(iz,iagg,izprime,:))&
														*mm(im_upp)/mm(imprime)))))
													enddo
													qm0_u(1) = dot_product(qm0_e,emeas)
													!Housing shock
													do ieprime = 1, ne
														yprime_f0  = (labor(isprime,ieprime)*wprime(iz,iagg,izprime) + bb(ibprime))*(1 - taxprime(iz,iagg,izprime))
														iy0low     = ly(yprime_f0)
														iy0upp     = uy(yprime_f0)
														y0weight   = wy(yprime_f0)
														im_low     = locm_f0(iy0low,imprime,igshock,isprime,izprime,iaggprime(iz,iagg,izprime,:))
														ib_low     = locbown_f0_sp1(iy0low,imprime,igshock,isprime,izprime,iaggprime(iz,iagg,izprime,:))
														d_low      = d_f0(iy0low,imprime,igshock,isprime,izprime,iaggprime(iz,iagg,izprime,:))
														pxs_low    = psell_f0(iy0low,imprime,igshock,isprime,izprime,iaggprime(iz,iagg,izprime,:))
														im_upp     = locm_f0(iy0upp,imprime,igshock,isprime,izprime,iaggprime(iz,iagg,izprime,:))
														ib_upp     = locbown_f0_sp1(iy0upp,imprime,igshock,isprime,izprime,iaggprime(iz,iagg,izprime,:))
														d_upp      = d_f0(iy0upp,imprime,igshock,isprime,izprime,iaggprime(iz,iagg,izprime,:))
														pxs_upp    = psell_f0(iy0upp,imprime,igshock,isprime,izprime,iaggprime(iz,iagg,izprime,:))
														mrecov     = min(1.d0,(Jreo(ih,izprime,iaggprime(iz,iagg,izprime,:)) + fdeficiency*yprime_f0)/(mm(imprime)*hh(ih)))
														qm0_low(1) = qm0temp(im_low(1),ib_low(1),igshock,isprime,izprime,iaggprime(iz,iagg,izprime,1))
														qm0_upp(1) = qm0temp(im_upp(1),ib_upp(1),igshock,isprime,izprime,iaggprime(iz,iagg,izprime,1))
														qm0_low(2) = qm0temp(im_low(2),ib_low(2),igshock,isprime,izprime,iaggprime(iz,iagg,izprime,2))
														qm0_upp(2) = qm0temp(im_upp(2),ib_upp(2),igshock,isprime,izprime,iaggprime(iz,iagg,izprime,2))
														qm0_low(3) = qm0temp(im_low(3),ib_low(3),igshock,isprime,izprime,iaggprime(iz,iagg,izprime,3))
														qm0_upp(3) = qm0temp(im_upp(3),ib_upp(3),igshock,isprime,izprime,iaggprime(iz,iagg,izprime,3))
														qm0_low(4) = qm0temp(im_low(4),ib_low(4),igshock,isprime,izprime,iaggprime(iz,iagg,izprime,4))
														qm0_upp(4) = qm0temp(im_upp(4),ib_upp(4),igshock,isprime,izprime,iaggprime(iz,iagg,izprime,4))
														!Update mortgage prices
														qm0_e(ieprime) = dot_product(waggprime(iz,iagg,izprime,:),(1 - y0weight)*(pxs_low + (1 - pxs_low)*(d_low*mrecov&
														+ (1 - d_low)*(1 + (1 + rscost)*(qm0_low*(1 + rocost) - qmprime(iz,iagg,izprime,:))*mm(im_low)/mm(imprime)))) + y0weight&
														*(pxs_upp + (1 - pxs_upp)*(d_upp*mrecov + (1 - d_upp)*(1 + (1 + rscost)*(qm0_upp*(1 + rocost) - qmprime(iz,iagg,izprime,:))&
														*mm(im_upp)/mm(imprime)))))
													enddo
													qm0_u(2) = dot_product(qm0_e,emeas)
													qm0_s(isprime) = (1 - pseparate)*qm0_u(1) + pseparate*qm0_u(2)
												enddo
											else
												do isprime = 1, ns
													do ieprime = 1, ne
														yprime_f0  = (labor(isprime,ieprime)*wprime(iz,iagg,izprime) + bb(ibprime))*(1 - taxprime(iz,iagg,izprime))
														iy0low     = ly(yprime_f0)
														iy0upp     = uy(yprime_f0)
														y0weight   = wy(yprime_f0)
														im_low     = locm_f0(iy0low,imprime,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))
														ib_low     = locbown_f0_sp1(iy0low,imprime,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))
														d_low      = d_f0(iy0low,imprime,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))
														pxs_low    = psell_f0(iy0low,imprime,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))
														im_upp     = locm_f0(iy0upp,imprime,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))
														ib_upp     = locbown_f0_sp1(iy0upp,imprime,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))
														d_upp      = d_f0(iy0upp,imprime,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))
														pxs_upp    = psell_f0(iy0upp,imprime,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))
														mrecov     = min(1.d0,(Jreo(ih,izprime,iaggprime(iz,iagg,izprime,:)) + fdeficiency*yprime_f0)/(mm(imprime)*hh(ih)))
														qm0_low(1) = qm0temp(im_low(1),ib_low(1),ig,isprime,izprime,iaggprime(iz,iagg,izprime,1))
														qm0_upp(1) = qm0temp(im_upp(1),ib_upp(1),ig,isprime,izprime,iaggprime(iz,iagg,izprime,1))
														qm0_low(2) = qm0temp(im_low(2),ib_low(2),ig,isprime,izprime,iaggprime(iz,iagg,izprime,2))
														qm0_upp(2) = qm0temp(im_upp(2),ib_upp(2),ig,isprime,izprime,iaggprime(iz,iagg,izprime,2))
														qm0_low(3) = qm0temp(im_low(3),ib_low(3),ig,isprime,izprime,iaggprime(iz,iagg,izprime,3))
														qm0_upp(3) = qm0temp(im_upp(3),ib_upp(3),ig,isprime,izprime,iaggprime(iz,iagg,izprime,3))
														qm0_low(4) = qm0temp(im_low(4),ib_low(4),ig,isprime,izprime,iaggprime(iz,iagg,izprime,4))
														qm0_upp(4) = qm0temp(im_upp(4),ib_upp(4),ig,isprime,izprime,iaggprime(iz,iagg,izprime,4))
														!Update mortgage prices
														qm0_e(ieprime) = dot_product(waggprime(iz,iagg,izprime,:),(1 - y0weight)*(pxs_low + (1 - pxs_low)*(d_low*mrecov&
														+ (1 - d_low)*(1 + (1 + rscost)*(qm0_low*(1 + rocost) - qmprime(iz,iagg,izprime,:))*mm(im_low)/mm(imprime)))) + y0weight&
														*(pxs_upp + (1 - pxs_upp)*(d_upp*mrecov + (1 - d_upp)*(1 + (1 + rscost)*(qm0_upp*(1 + rocost) - qmprime(iz,iagg,izprime,:))&
														*mm(im_upp)/mm(imprime)))))
													enddo
													qm0_s(isprime) = dot_product(qm0_e,emeas)
												enddo
											endif
											qm0_z(izprime) = dot_product(qm0_s,prob_ss(is,:,iz,izprime))
										enddo
										qm0new(imprime,ibprime,ig,is,iz,iagg) = qm(iz,iagg)/(1 + rocost)*dot_product(qm0_z,prob_zz(iz,:))
									enddo
								enddo
								!$omp end do
								!$omp end parallel
							enddo
						enddo
					enddo
				enddo
				err_qtemp = maxval(abs(qm0new - qm0temp))
				if (iter_qtemp - 1 == 10*floor((real(iter_qtemp) - 1)/10.0)) then
					write(*, '(a27 i4, a14, f8.5)'), '     Inner mortgage loop = ',iter_qtemp,', err_qtemp = ',err_qtemp
				endif
				if (err_qtemp < 1e-5 .OR. iter_qtemp == max_iterqtemp) then
					exit
				endif
				qm0temp = qm0new
			enddo
			err_q = maxval(abs(qm0new - qm0))
			print *, ' '
			write(*, '(a12, i4, a10, f8.5, a14, f10.5)'), '   Iter_q = ' ,iter_q, ', err_q = ' ,err_q, ', err_coeff = ' ,err_coeff
			if (err_q < 1e-5 .OR. iter_q == max_iterq) then
				exit
			endif
			if (err_q >= 1e-2) then
				qm0 = qm0 + 0.5*(qm0new - qm0)
			else
				qm0 = qm0 + 0.9*(qm0new - qm0)
			endif
		enddo
		!Initialize the distribution of agents
		call sub_stationary(iz0,aggfun(iph0,ik0))
		call sub_aggstats_ratiomeans(iz0,aggfun(iph0,ik0),kstock_t(1),assets_t(1))
		if (print_input >= 1) then
			call sub_print_input
		endif
		if (print_output >= 1) then
			call sub_print_readable
			call sub_print_final
		endif
		!Simulate economy
		print *, ' '
		print *, '     Simulate economy'
		phe0  = ph(aggfun(iph0,ik0))
		do it = 1, T
			call sub_simulate(it,0,locz_t(it),locz_tprime(it),kstock_t(it),assets_t(it),phe0,ph_t(it),kstock_tprime(it),assets_tprime(it),tax_t(it),yh_t(it),yc_t(it),ownrate_t(it),htot_t(it))
			if (it >= 2) then
				ph_tprime(it - 1)  = ph_t(it)
				tax_tprime(it - 1) = tax_t(it)
			endif
			if (it <= T - 1) then
				kstock_t(it + 1) = kstock_tprime(it)
				assets_t(it + 1) = assets_tprime(it)
			endif
			phe0 = ph_t(it)
			if (it - 1 == 50*floor((real(it)-1)/50.0)) then
				print *, ' '
				write(*, '(a11, f8.5, a14, f10.5)'), '   err_q = ' ,err_q, ', err_coeff = ' ,err_coeff
				print *, ' '
			endif
!			write(*, '(a9, i4, a7, i2, a7, f8.5, a6, f7.4, a8, f7.4)'), '     t = ' ,it, ', iz = ' ,locz_t(it), ', ph = ' ,ph_t(it), ', K = ' ,kstock_t(it), ', own = ' ,ownrate_t(it)
		enddo
		!Print files
		if (print_input >= 1) then
			call sub_print_input
		endif
		if (print_output >= 1) then
			call sub_print_readable
			call sub_print_final
		endif
		open(unit=11,	file='simul.txt', status = 'replace')
		write(11, '(i4, i2, f10.5, f10.5, f10.5, f10.5, f10.5, f10.5, f10.5)'), (index, locz_t(index), ph_t(index), kstock_t(index), yh_t(index), yc_t(index), ownrate_t(index), htot_t(index), tax_t(index), index = 1,T)
		close(11)
		open(unit=11,	file='Output\simul.txt', status = 'replace')
		write(11, '(a9, i4, a7, i2, a7, f10.5, a6, f10.5, a7, f10.5, a7, f10.5, a12, f10.5, a6, f10.5, a8, f10.5)'), ('     t = ' ,index, ', iz = ' ,locz_t(index), ', ph = ' ,ph_t(index), ', K = ' ,kstock_t(index), ', yh = ' ,yh_t(index), ', yc = ' ,yc_t(index), ', ownrate = ' ,ownrate_t(index), ', H = ' ,htot_t(index), ', tax = ' ,tax_t(index), index = 1,T)
		close(11)
		!Run regression and update forecasting coefficients
		!Regression for phprime
		call sub_krusellsmith_regress(ph_tprime(T_ignore + 1:T - 1),ph_t(T_ignore + 1:T - 1),kstock_t(T_ignore + 1:T - 1),locz_t(T_ignore + 1:T - 1),locz_tprime(T_ignore + 1:T - 1),version_ph,forecast_coeffnew(1,:,:,:),Rsquared(1))
		!Regression for Kprime
		call sub_krusellsmith_regress(kstock_tprime(T_ignore + 1:T - 1),kstock_t(T_ignore + 1:T - 1),ph_t(T_ignore + 1:T - 1),locz_t(T_ignore + 1:T - 1),locz_tprime(T_ignore + 1:T - 1),version_k,forecast_coeffnew(2,:,:,:),Rsquared(2))
		!Regression for taxprime
		call sub_krusellsmith_regress(tax_tprime(T_ignore + 1:T - 1),ph_t(T_ignore + 1:T - 1),kstock_t(T_ignore + 1:T - 1),locz_t(T_ignore + 1:T - 1),locz_tprime(T_ignore + 1:T - 1),version_tax,forecast_coeffnew(3,:,:,:),Rsquared(3))
		forecast_coefftemp         = forecast_coeffnew(2,2,:,:)
		forecast_coeffnew(2,2,:,:) = forecast_coeffnew(2,3,:,:)
		forecast_coeffnew(2,3,:,:) = forecast_coefftemp
		err_coeff = maxval(abs(forecast_coeffnew - forecast_coeff))
		print *, ' '
		write(*, '(a16, i4, a10, f8.5, a14, f10.5, a8, f10.7, f10.7, f10.7)'), '   Iter_coeff = ' ,iter_coeff, ', err_q = ' ,err_q, ', err_coeff = ' ,err_coeff, ', R^2 = ' ,Rsquared(1), Rsquared(2), Rsquared(3)
		open(unit=11,	file='converge_summary.txt', access = 'append')
		write(11, '(a16, i4, a10, f8.5, a14, f10.5, a8, f10.7, f10.7, f10.7)'), '   Iter_coeff = ' ,iter_coeff, ', err_q = ' ,err_q, ', err_coeff = ' ,err_coeff, ', R^2 = ' ,Rsquared(1), Rsquared(2), Rsquared(3)
		close(11)
		open(unit=11,	file='coeff_log.txt', access = 'append')
		if (iter_coeff == 1) then	
			write(11, '(a1)'), ' '
			write(11, '(a50)'), '--------------------------------------------------'
		endif
		write(11, '(a1)'), ' '
		write(11, '(a16, i4)'), '   Iter_coeff = ' ,iter_coeff
		write(11, '(a1)'), ' '
		write(11, '(a17)'), '   forecast_coeff'
		do iz = 1, nz
			do izprime = 1, nz
				write (11, '(3f20.10)') forecast_coeff(1,:,iz,izprime)
			enddo
		enddo
		do iz = 1, nz
			do izprime = 1, nz
				write (11, '(3f20.10)') forecast_coeff(2,:,iz,izprime)
			enddo
		enddo
		do iz = 1, nz
			do izprime = 1, nz
				write (11, '(3f20.10)') forecast_coeff(3,:,iz,izprime)
			enddo
		enddo
		write(11, '(a1)'), ' '
		write(11, '(a20)'), '   forecast_coeffnew'
		do iz = 1, nz
			do izprime = 1, nz
				write (11, '(3f20.10)') forecast_coeffnew(1,:,iz,izprime)
			enddo
		enddo
		do iz = 1, nz
			do izprime = 1, nz
				write (11, '(3f20.10)') forecast_coeffnew(2,:,iz,izprime)
			enddo
		enddo
		do iz = 1, nz
			do izprime = 1, nz
				write (11, '(3f20.10)') forecast_coeffnew(3,:,iz,izprime)
			enddo
		enddo
		close(11)
		if (err_coeff < 0.001d0 .AND. err_q < 1e-5 .OR. iter_coeff == max_itercoeff) then
			print *, ' '
			print *, '***CONVERGENCE!!!***'
			exit
		endif
		!Update forecasting coefficients
		forecast_coeff = forecast_coeff + 0.2*(forecast_coeffnew - forecast_coeff)
		open(unit=11,	file='Input\forecast_coeffnew.txt', status = 'replace')
		do iz = 1, nz
			do izprime = 1, nz
				write (11, '(3f20.10)') forecast_coeff(1,:,iz,izprime)
			enddo
		enddo
		do iz = 1, nz
			do izprime = 1, nz
				write (11, '(3f20.10)') forecast_coeff(2,:,iz,izprime)
			enddo
		enddo
		do iz = 1, nz
			do izprime = 1, nz
				write (11, '(3f20.10)') forecast_coeff(3,:,iz,izprime)
			enddo
		enddo
		close(11)
		!Update forecasting equations
		do iz = 1, nz
			do iagg = 1, nagg
				do izprime = 1, nz
					phprime(iz,iagg,izprime)     = dot_product(forecast_coeff(1,:,iz,izprime),forecast_vec(:,iagg))
					kaggprime(iz,iagg,izprime)   = dot_product(forecast_coeff(2,:,iz,izprime),forecast_vec(:,iagg))
					taxprime(iz,iagg,izprime)    = dot_product(forecast_coeff(3,:,iz,izprime),forecast_vec(:,iagg))
					iaggprime(iz,iagg,izprime,1) = aggfun(lp(phprime(iz,iagg,izprime)),lk(kaggprime(iz,iagg,izprime)))
					iaggprime(iz,iagg,izprime,2) = aggfun(up(phprime(iz,iagg,izprime)),lk(kaggprime(iz,iagg,izprime)))
					iaggprime(iz,iagg,izprime,3) = aggfun(lp(phprime(iz,iagg,izprime)),uk(kaggprime(iz,iagg,izprime)))
					iaggprime(iz,iagg,izprime,4) = aggfun(up(phprime(iz,iagg,izprime)),uk(kaggprime(iz,iagg,izprime)))
					waggprime(iz,iagg,izprime,1) = (1 - wp(phprime(iz,iagg,izprime)))*(1 - wk(kaggprime(iz,iagg,izprime)))
					waggprime(iz,iagg,izprime,2) = wp(phprime(iz,iagg,izprime))*(1 - wk(kaggprime(iz,iagg,izprime)))
					waggprime(iz,iagg,izprime,3) = (1 - wp(phprime(iz,iagg,izprime)))*wk(kaggprime(iz,iagg,izprime))
					waggprime(iz,iagg,izprime,4) = wp(phprime(iz,iagg,izprime))*wk(kaggprime(iz,iagg,izprime))
					call sub_factorprices_sh(kaggprime(iz,iagg,izprime),eavg,phprime(iz,iagg,izprime),zc(izprime),zh(izprime),wprime(iz,iagg,izprime),rkprime(iz,iagg,izprime))
				enddo
				rk_expect   = dot_product(rkprime(iz,iagg,:),prob_zz(iz,:))
				qb(iz,iagg) = 1/(1 + rk_expect - deprec_k)
				rb(iz,iagg) = 1/qb(iz,iagg) - 1
				qm(iz,iagg) = (1 - deprec_h)/((1 + rb(iz,iagg))*(1 + rscost))
			enddo
		enddo
		!Next period mortgage prices given the aggregate forecasting equations
		do iz = 1, nz
			do iagg = 1, nagg
				do izprime = 1, nz
					qmprime(iz,iagg,izprime,1) = (1 - deprec_h)/((1 + rb(iz,iaggprime(iz,iagg,izprime,1)))*(1 + rscost))
					qmprime(iz,iagg,izprime,2) = (1 - deprec_h)/((1 + rb(iz,iaggprime(iz,iagg,izprime,2)))*(1 + rscost))
					qmprime(iz,iagg,izprime,3) = (1 - deprec_h)/((1 + rb(iz,iaggprime(iz,iagg,izprime,3)))*(1 + rscost))
					qmprime(iz,iagg,izprime,4) = (1 - deprec_h)/((1 + rb(iz,iaggprime(iz,iagg,izprime,4)))*(1 + rscost))
				enddo
			enddo
		enddo
		!Compute optimal REO behavior
		call sub_valfn_reo
	enddo
	!Print files
	if (print_input >= 1) then
		call sub_print_input
	endif
	if (print_output >= 1) then
		call sub_print_readable
		call sub_print_final
	endif
	
	!Generate time series and business cycle statistics
	load_data   = 0
	load_series = 0
	call sub_aggresults

end subroutine sub_equilibrium