subroutine sub_impulseresponse

	use mod_globvar
	use mod_interpolate

	implicit none

	!This program computes aggregate statistics for the equilibrium of the model

	!Indices-- individual states and choice variables
	integer :: imprime, ibprime, ig, ih, is
	
	!Indices-- aggregate states and miscellaneous
	integer :: iz, izprime, iagg, it, iz0, iz1, iph0, ik0
	
	!Vector of configuration parameters for impulse response subroutine
	integer, dimension(6) :: config_irf
	
	!Other internal variables
	real(8) :: ylower, phe0, rk_expect
	integer :: index, T, T_initialize, nseries, shocktype, Tshock
	real(8), dimension(:), allocatable :: ph_t, ph_tprime, qb_t, yh_t, yc_t, gdp_t, invest_h_t, invest_k_t, cons_t, hcons_t, kstock_t, kstock_tprime, hownstock_t, networth_t, &
	assets_t, assets_tprime, hwealth_t, mortdebt_t, ownrate_t, turnover_t, tradevol_t, existsales_t, reosales_t, reosaleshare_t, newsaleshare_t, salesrate_t, inventories_t, &
	months_supply_t, frate_t, htot_t, tax_t, tax_tprime, mean_TOMbuy_t, mean_TOMsell_t, defaultprem_t, defaultprem_ltv80_t, defaultprem_ltv90_t, defaultprem_ltv95_t, num_orig_ltv80_t, &
	num_orig_ltv90_t, num_orig_ltv95_t
	integer, dimension(:), allocatable :: locz_t, locz_tprime
	
	!Load configuration
	open(unit=11,	file='Input\config_irf.txt')
	read (11, *) config_irf
	close(11)
	T             = config_irf(1)
	T_initialize  = config_irf(2)
	itermax       = config_irf(3)
	itermax_accel = config_irf(4)
	iph0          = config_irf(5)
	ik0           = config_irf(6)
	nseries       = 35
	
	write(*,*), 'Initial z?'
	read(*,*) iz0
	write(*,*), 'Final z?'
	read(*,*) iz1
	write(*,*), 'Permanent or transitory shock? (1 = permanent, 2 = transitory)'
	read(*,*) shocktype
	if (shocktype == 2) then
		write(*,*), 'Duration of shock? (in quarters)'
		read(*,*) Tshock
	endif
	
	!Allocate variables for simulation and time series generation
	!Aggregate shocks
	allocate(locz_t(T + 1))
	allocate(locz_tprime(T))
	!Equilibrium prices
	allocate(ph_t(T))
	allocate(ph_tprime(T))
	allocate(qb_t(T))
	!Output and capital stock
	allocate(yh_t(T))
	allocate(yc_t(T))
	allocate(gdp_t(T))
	allocate(kstock_t(T))
	allocate(kstock_tprime(T))
	!Consumption and investment
	allocate(cons_t(T))
	allocate(hcons_t(T))
	allocate(invest_h_t(T))
	allocate(invest_k_t(T))
	!Homeownership rate, and housing stock
	allocate(ownrate_t(T))
	allocate(htot_t(T))
	allocate(hownstock_t(T))
	!Assets, debt, and net worth
	allocate(networth_t(T))
	allocate(assets_t(T))
	allocate(assets_tprime(T))
	allocate(hwealth_t(T))
	allocate(mortdebt_t(T))
	!Sales, inventories, and foreclosures
	allocate(turnover_t(T))
	allocate(tradevol_t(T))
	allocate(existsales_t(T))
	allocate(reosales_t(T))
	allocate(reosaleshare_t(T))
	allocate(newsaleshare_t(T))
	allocate(salesrate_t(T))
	allocate(inventories_t(T))
	allocate(months_supply_t(T))
	allocate(frate_t(T))
	allocate(mean_TOMbuy_t(T))
	allocate(mean_TOMsell_t(T))
	!Taxes
	allocate(tax_t(T))
	allocate(tax_tprime(T))
	!Misc
	allocate(defaultprem_t(T))
	allocate(defaultprem_ltv80_t(T))
	allocate(defaultprem_ltv90_t(T))
	allocate(defaultprem_ltv95_t(T))
	allocate(num_orig_ltv80_t(T))
	allocate(num_orig_ltv90_t(T))
	allocate(num_orig_ltv95_t(T))
	
	!Combined time series
	allocate(timeseries(T,nseries))
	
	!Generate time series
	!Initialize value functions, forecasting coefficients, and mortgage prices
	if (load_data == 1) then
		!Load value functions, forecasting coefficients, and mortgage prices
		call sub_loadfiles
		!Forecasting equations
		do iz = 1, nz
			do iagg = 1, nagg
				do izprime = 1, nz
					phprime(iz,iagg,izprime)     = dot_product(forecast_coeff(1,:,iz,izprime),forecast_vec(:,iagg))
					kaggprime(iz,iagg,izprime)   = dot_product(forecast_coeff(2,:,iz,izprime),forecast_vec(:,iagg))
					taxprime(iz,iagg,izprime)    = dot_product(forecast_coeff(3,:,iz,izprime),forecast_vec(:,iagg))
					iaggprime(iz,iagg,izprime,1) = aggfun(lp(phprime(iz,iagg,izprime)),lk(kaggprime(iz,iagg,izprime)))
					iaggprime(iz,iagg,izprime,2) = aggfun(up(phprime(iz,iagg,izprime)),lk(kaggprime(iz,iagg,izprime)))
					iaggprime(iz,iagg,izprime,3) = aggfun(lp(phprime(iz,iagg,izprime)),uk(kaggprime(iz,iagg,izprime)))
					iaggprime(iz,iagg,izprime,4) = aggfun(up(phprime(iz,iagg,izprime)),uk(kaggprime(iz,iagg,izprime)))
					waggprime(iz,iagg,izprime,1) = (1 - wp(phprime(iz,iagg,izprime)))*(1 - wk(kaggprime(iz,iagg,izprime)))
					waggprime(iz,iagg,izprime,2) = wp(phprime(iz,iagg,izprime))*(1 - wk(kaggprime(iz,iagg,izprime)))
					waggprime(iz,iagg,izprime,3) = (1 - wp(phprime(iz,iagg,izprime)))*wk(kaggprime(iz,iagg,izprime))
					waggprime(iz,iagg,izprime,4) = wp(phprime(iz,iagg,izprime))*wk(kaggprime(iz,iagg,izprime))
					call sub_factorprices_sh(kaggprime(iz,iagg,izprime),eavg,phprime(iz,iagg,izprime),zc(izprime),zh(izprime),wprime(iz,iagg,izprime),rkprime(iz,iagg,izprime))
				enddo
				rk_expect   = dot_product(rkprime(iz,iagg,:),prob_zz(iz,:))
				qb(iz,iagg) = 1/(1 + rk_expect - deprec_k)
				rb(iz,iagg) = 1/qb(iz,iagg) - 1
				qm(iz,iagg) = (1 - deprec_h)/((1 + rb(iz,iagg))*(1 + rscost))
			enddo
		enddo
		!Optimal REO behavior
		Jreo = -100.d0
		call sub_valfn_reo
		!Calculate lower bound of y at mortgage origination
		do iz = 1, nz
			do iagg = 1, nagg
				do is = 1, ns
					do ig = 1, ng
						ih = hfun(ig)
						ylb(ig,is,iz,iagg) = max_y
						do imprime = 1, nm
							do ibprime = 1, nb
								if (qm0(imprime,ibprime,ig,is,iz,iagg)*mm(imprime) <= max_ltv*ph(iagg)) then
									ylower = qb(iz,iagg)*bb(ibprime) - qm0(imprime,ibprime,ig,is,iz,iagg)*mm(imprime)*hh(ih)
									if (ylower < ylb(ig,is,iz,iagg)) then
										ylb(ig,is,iz,iagg) = ylower
									endif
								endif
							enddo
						enddo
					enddo
				enddo
			enddo
		enddo
		!Compute value function iteration
		toler = 1e-6
		if (valfn_type == 1) then
            call sub_valfn_ces
        elseif (valfn_type == 2) then
			call sub_valfn_ez_ces
		endif
	endif
	!Generate sequence of aggregate shocks		
	do it = 1, T + 1
		if (shocktype == 1) then
			if (it <= T_initialize) then
				locz_t(it) = iz0
			else
				locz_t(it) = iz1
			endif
		elseif (shocktype == 2) then
			if (it <= T_initialize) then
				locz_t(it) = iz0
			elseif (it > T_initialize .AND. it <= T_initialize + Tshock) then
				locz_t(it) = iz1
			else
				locz_t(it) = iz0
			endif
		endif
	enddo
	!Generate {z(t+1)}
	do it = 1, T
		locz_tprime(it) = locz_t(it + 1)
	enddo
	!Simulate economy
	print *, ' '
	print *, '     Generate time series'
	call sub_stationary(iz0,aggfun(iph0,ik0))
	call sub_aggstats_ratiomeans(iz0,aggfun(iph0,ik0),kstock_t(1),assets_t(1))
	phe0 = ph(aggfun(iph0,ik0))
	do it = 1, T
		call sub_simulate(it,1,locz_t(it),locz_tprime(it),kstock_t(it),assets_t(it),phe0,ph_t(it),kstock_tprime(it),assets_tprime(it),tax_t(it),yh_t(it),yc_t(it),ownrate_t(it),htot_t(it))
		if (it <= T - 1) then
			kstock_t(it + 1) = kstock_tprime(it)
			assets_t(it + 1) = assets_tprime(it)
		endif
		phe0 = ph_t(it)
!		write(*, '(a9, i4, a7, i2, a7, f8.5, a6, f7.4, a8, f6.3)'), '     t = ' ,it, ', iz = ' ,locz_t(it), ', ph = ' ,ph_t(it), ', K = ' ,kstock_t(it), ', own = ' ,ownrate_t(it)
	enddo
	ph_t            = timeseries(:,1)
	qb_t            = timeseries(:,2)
	yh_t            = timeseries(:,3)
	yc_t            = timeseries(:,4)
	gdp_t           = timeseries(:,5)
	invest_h_t      = timeseries(:,6)
	invest_k_t      = timeseries(:,7)
	cons_t          = timeseries(:,8)
	hcons_t         = timeseries(:,9)
	kstock_t        = timeseries(:,10)
	hownstock_t     = timeseries(:,11)
	networth_t      = timeseries(:,12)
	assets_t        = timeseries(:,13)
	hwealth_t       = timeseries(:,14)
	mortdebt_t      = timeseries(:,15)
	ownrate_t       = timeseries(:,16)
	turnover_t      = timeseries(:,17)
	tradevol_t      = timeseries(:,18)
	existsales_t    = timeseries(:,19)
	reosales_t      = timeseries(:,20)
	reosaleshare_t  = timeseries(:,21)
	newsaleshare_t  = timeseries(:,22)
	salesrate_t     = timeseries(:,23)
	inventories_t   = timeseries(:,24)
	months_supply_t = timeseries(:,25)
	frate_t         = timeseries(:,26)
	mean_TOMbuy_t   = timeseries(:,27)
	mean_TOMsell_t  = timeseries(:,28)
	defaultprem_t   = timeseries(:,29)
	defaultprem_ltv80_t = timeseries(:,30)
	defaultprem_ltv90_t = timeseries(:,31)
	defaultprem_ltv95_t = timeseries(:,32)
	num_orig_ltv80_t    = timeseries(:,33)
	num_orig_ltv90_t    = timeseries(:,34)
	num_orig_ltv95_t    = timeseries(:,35)
	
	!Print files
	open(unit=11,	file='Time series\IRF\timeseries.txt', status = 'replace')
	write(11, '(35f15.10)'), (timeseries(index,:), index = 1,T)
	close(11)
	open(unit=11,	file='Time series\IRF\z_t.txt', status = 'replace')
	write(11, '(i2)'), locz_t
	close(11)
	open(unit=11,	file='Time series\IRF\ph_t.txt', status = 'replace')
	write(11, '(f15.10)'), ph_t
	close(11)
	open(unit=11,	file='Time series\IRF\yh_t.txt', status = 'replace')
	write(11, '(f15.10)'), yh_t
	close(11)
	open(unit=11,	file='Time series\IRF\yc_t.txt', status = 'replace')
	write(11, '(f15.10)'), yc_t
	close(11)
	open(unit=11,	file='Time series\IRF\gdp_t.txt', status = 'replace')
	write(11, '(f15.10)'), gdp_t
	close(11)
	open(unit=11,	file='Time series\IRF\invest_h_t.txt', status = 'replace')
	write(11, '(f15.10)'), invest_h_t
	close(11)
	open(unit=11,	file='Time series\IRF\invest_k_t.txt', status = 'replace')
	write(11, '(f15.10)'), invest_k_t
	close(11)
	open(unit=11,	file='Time series\IRF\cons_t.txt', status = 'replace')
	write(11, '(f15.10)'), cons_t
	close(11)
	open(unit=11,	file='Time series\IRF\hcons_t.txt', status = 'replace')
	write(11, '(f15.10)'), hcons_t
	close(11)
	open(unit=11,	file='Time series\IRF\kstock_t.txt', status = 'replace')
	write(11, '(f15.10)'), kstock_t
	close(11)
	open(unit=11,	file='Time series\IRF\hownstock_t.txt', status = 'replace')
	write(11, '(f15.10)'), hownstock_t
	close(11)
	open(unit=11,	file='Time series\IRF\networth_t.txt', status = 'replace')
	write(11, '(f15.10)'), networth_t
	close(11)
	open(unit=11,	file='Time series\IRF\assets_t.txt', status = 'replace')
	write(11, '(f15.10)'), assets_t
	close(11)
	open(unit=11,	file='Time series\IRF\hwealth_t.txt', status = 'replace')
	write(11, '(f15.10)'), hwealth_t
	close(11)
	open(unit=11,	file='Time series\IRF\mortdebt_t.txt', status = 'replace')
	write(11, '(f15.10)'), mortdebt_t
	close(11)
	open(unit=11,	file='Time series\IRF\ownrate_t.txt', status = 'replace')
	write(11, '(f15.10)'), ownrate_t
	close(11)	
	open(unit=11,	file='Time series\IRF\turnover_t.txt', status = 'replace')
	write(11, '(f15.10)'), turnover_t
	close(11)
	open(unit=11,	file='Time series\IRF\tradevol_t.txt', status = 'replace')
	write(11, '(f15.10)'), tradevol_t
	close(11)
	open(unit=11,	file='Time series\IRF\existsales_t.txt', status = 'replace')
	write(11, '(f15.10)'), existsales_t
	close(11)
	open(unit=11,	file='Time series\IRF\reosales_t.txt', status = 'replace')
	write(11, '(f15.10)'), reosales_t
	close(11)
	open(unit=11,	file='Time series\IRF\reosaleshare_t.txt', status = 'replace')
	write(11, '(f15.10)'), reosaleshare_t
	close(11)
	open(unit=11,	file='Time series\IRF\newsaleshare_t.txt', status = 'replace')
	write(11, '(f15.10)'), newsaleshare_t
	close(11)
	open(unit=11,	file='Time series\IRF\salesrate_t.txt', status = 'replace')
	write(11, '(f15.10)'), salesrate_t
	close(11)
	open(unit=11,	file='Time series\IRF\inventories_t.txt', status = 'replace')
	write(11, '(f15.10)'), inventories_t
	close(11)
	open(unit=11,	file='Time series\IRF\months_supply_t.txt', status = 'replace')
	write(11, '(f15.10)'), months_supply_t
	close(11)
	open(unit=11,	file='Time series\IRF\frate_t.txt', status = 'replace')
	write(11, '(f15.10)'), frate_t
	close(11)
	open(unit=11,	file='Time series\IRF\mean_TOMbuy_t.txt', status = 'replace')
	write(11, '(f15.10)'), mean_TOMbuy_t
	close(11)
	open(unit=11,	file='Time series\IRF\mean_TOMsell_t.txt', status = 'replace')
	write(11, '(f15.10)'), mean_TOMsell_t
	close(11)
	open(unit=11,	file='Time series\IRF\defaultprem_t.txt', status = 'replace')
	write(11, '(f15.10)'), defaultprem_t
	close(11)
	open(unit=11,	file='Time series\IRF\defaultprem_ltv80_t.txt', status = 'replace')
	write(11, '(f15.10)'), defaultprem_ltv80_t
	close(11)
	open(unit=11,	file='Time series\IRF\defaultprem_ltv90_t.txt', status = 'replace')
	write(11, '(f15.10)'), defaultprem_ltv90_t
	close(11)
	open(unit=11,	file='Time series\IRF\defaultprem_ltv95_t.txt', status = 'replace')
	write(11, '(f15.10)'), defaultprem_ltv95_t
	close(11)
	open(unit=11,	file='Time series\IRF\num_orig_ltv80_t.txt', status = 'replace')
	write(11, '(f15.10)'), num_orig_ltv80_t
	close(11)
	open(unit=11,	file='Time series\IRF\num_orig_ltv90_t.txt', status = 'replace')
	write(11, '(f15.10)'), num_orig_ltv90_t
	close(11)
	open(unit=11,	file='Time series\IRF\num_orig_ltv95_t.txt', status = 'replace')
	write(11, '(f15.10)'), num_orig_ltv95_t
	close(11)
	open(unit=11,	file='Time series\IRF\pa.txt', status = 'replace')
	write (11, '(f15.10)') pa
	close(11)

end subroutine sub_impulseresponse