subroutine sub_initialize

	use mod_globvar
	use mod_toolbox

	implicit none

	!This program generates grids, transition probabilities, etc.
	
	!Indices-- individual states
	integer :: im, iy, ib, ih, ix, is, isprime, ie, iu, ig, iyfine
	
	!Indices-- aggregate states
	integer :: izc, izcprime, izh, izhprime, ihu, ihuprime, iz, izprime, iph, ik, iagg

	!Other internal variables
	integer :: printcorr, printgrids, rouwenhorst_s, rouwenhorst_zc, rouwenhorst_zh, rouwenhorst_hu, ns_temp
	real(8) :: mmod, ymod, bmod, hmod, xmod, phmod, kmod, mu, upp, low, sig_s, ph0
	real(8), dimension(:), allocatable :: ss_temp
	real(8), dimension(:,:,:,:), allocatable :: prob_ss_temp
	real(8), dimension(ns) :: stationary_s
	real(8), dimension(nzc) :: stationary_zc
	real(8), dimension(nzh) :: stationary_zh
	real(8), dimension(nhu) :: stationary_hu
	real(8), dimension(nzc,nzc) :: prob_zczc
	real(8), dimension(nzh,nzh) :: prob_zhzh
	real(8), dimension(nhu,nhu) :: prob_huhu

	!******* External functions to be called *******
	real(8), external :: evalue, pmatchbuy, pmatchsell, normcdf, xb, xs

	!------------------------------------------------
	!State variable correspondences
	!------------------------------------------------
	
	!Generate link between g and (h,u) values
	do iu = 1, nu
		do ih = 1, nh
			ig = nh*(iu - 1) + ih
			gfun(ih,iu) = ig
			hfun(ig)    = ih
			ufun(ig)    = iu
		enddo
	enddo
	
	!Generate link between z and (zc,zh,hu) values
	do ihu = 1, nhu
		do izh = 1, nzh
			do izc = 1, nzc
				iz = nzh*nzc*(ihu - 1) + nzc*(izh - 1) + izc
				zfun(izc,izh,ihu) = iz
				zcfun(iz)         = izc
				zhfun(iz)         = izh
				hufun(iz)		  = ihu
			enddo
		enddo
	enddo
	
	!Generate link between agg and (ph,kagg) values
	do ik = 1, nk
		do iph = 1, nph
			iagg = nph*(ik - 1) + iph
			aggfun(iph,ik) = iagg
			phfun(iagg)    = iph
			kfun(iagg)	   = ik
		enddo
	enddo
	
	printcorr = 0
	if (printcorr == 1) then
		print *, 'g --> (h,u)'
		do ig = 1, ng
			write(*, '(a3, i4, a5, i2, a5, i2)'), 'g =',ig, ', h =',hfun(ig), ', u =' ,ufun(ig)
		enddo
		print *, ' '
		print *, '(h,u) --> g'
		do iu = 1, nu
			do ih = 1, nh
				write(*, '(a3, i2, a5, i2, a5, i2)'), 'h =',ih, ', u =',iu, ', g =' ,gfun(ih,iu)
			enddo
		enddo
		print *, ' '
		print *, 'z --> (zc,zh,hu)'
		do iz = 1, nz
			write(*, '(a3, i3, a6, i2, a6, i2, a6, i2)'), 'z =' ,iz, ', zc =',zcfun(iz), ', zh =' ,zhfun(iz), ', hu =' ,hufun(iz)
		enddo
		print *, ' '
		print *, '(zc,zh,hu) --> z'
		do ihu = 1, nhu
			do izh = 1, nzh
				do izc = 1, nzc
					write(*, '(a4, i2, a6, i2, a6, i2, a6, i3)'), 'zc =',izc, ', zh =',izh, ', hu =' ,ihu, ', z =' ,zfun(izc,izh,ihu)
				enddo
			enddo
		enddo
	endif
	
	!------------------------------------------------
	!Grids and transition probabilities for individual states
	!------------------------------------------------

	!Generate grid for leverage values
	do im = 1, nm
		mmod   = dm*real(im - 1,8)
		mm(im) = mmod**(1/curv_m)
	enddo
	
	!Generate grid for cash at hand
	do iy = 1, ny
		if (iy <= ny_neg) then
			ymod   = -(-min_y)**curv_yneg + dy_neg*real(iy - 1,8)
			ymod   = -ymod
			yy(iy) = -ymod**(1/curv_yneg)
		else
			ymod   = dy_pos*real(iy - ny_neg - 1,8)
			yy(iy) = ymod**(1/curv_ypos)
		endif
	enddo
	
	!Generate extra fine grid for cash at hand
	do iyfine = 1, nyfine
		if (iyfine <= nyfine_neg) then
			ymod   = -(-min_yfine)**curv_yfineneg + dyfine_neg*real(iyfine - 1,8)
			ymod   = -ymod
			yyfine(iyfine) = -ymod**(1/curv_yfineneg)
		else
			ymod   = dyfine_pos*real(iyfine - nyfine_neg - 1,8)
			yyfine(iyfine) = ymod**(1/curv_yfinepos)
		endif
	enddo

	!Generate grid for bonds
	do ib = 1, nb
		bmod   = db*real(ib - 1,8)
		bb(ib) = bmod**(1/curv_b)
	enddo

	!Generate grid for housing
	do ih = 1, nh
!		hmod   = dh*real(ih - 1,8)
!		hh(ih) = hmod**(1/curv_h) + min_h
		hh(ih) = min_h*(1 + hincrement)**real(ih - 1,8)
	enddo

	!Generate grid for housing preference shocks
	uu(1) = 1.0
	if (nu == 2) then
		uu(2) = useparate
	endif

	!Generate grid for housing submarkets
	do ix = 1, nx
		xmod   = dx*real(ix - 1,8)
		xx(ix) = xmod**(1/curv_x)
	enddo
	
	!Generate submarket match probabilities
	do ix = 1, nx
		prob_xb(ix) = pmatchbuy(xx(ix))
		prob_xs(ix) = pmatchsell(xx(ix))
		if (monthly_quarterly == 1) then
			prob_xb(ix) = 1 - (1 - prob_xb(ix))**3.0
			prob_xs(ix) = 1 - (1 - prob_xs(ix))**3.0
		endif
	enddo
	
	!Generate grid for transitory labor productivity shocks
	ee(ne) = cover_e*sig_e
	ee(1)  = -ee(ne)	
	do ie = 2, ne - 1
		ee(ie) = ee(1) + (ee(ne) - ee(1))*real(ie - 1,8)/real(ne - 1,8)
	enddo
	
	!Pdf for transitory labor productivity shocks
	upp = (ee(2) + ee(1))/2.0
	upp = upp/sig_e
	emeas(1) = normcdf(upp)
	low = (ee(ne) + ee(ne - 1))/2.0
	low = low/sig_e
	emeas(ne) = 1.0 - normcdf(low)
	do ie = 2, ne - 1
		low = (ee(ie - 1) + ee(ie))/2.0
		upp = (ee(ie + 1) + ee(ie))/2.0
		low = low/sig_e
		upp = upp/sig_e
		emeas(ie) = normcdf(upp) - normcdf(low)
	enddo
	
	!Discretize AR(1) process for permanent labor productivity shock
	rouwenhorst_s = 1
	if (rich_shock == 1) then
		ns_temp = ns - 1
	else
		ns_temp = ns
	endif
	allocate(ss_temp(ns_temp))
	allocate(prob_ss_temp(ns_temp,ns_temp,nz,nz))
	if (rouwenhorst_s == 1) then
		!Rouwenhorst method
		call rouwenhorst(rho_s,sig_s_avg/(1 - rho_s**2.0)**0.5,prob_ss_temp(:,:,1,1),ss_temp)
		do iz = 1, nz
			do izprime = 1, nz
				prob_ss_temp(:,:,iz,izprime) = prob_ss_temp(:,:,1,1)
			enddo
		enddo
	else
		!Tauchen method
		ss_temp(ns_temp) = cover_s*sig_s_e/sqrt(1.0 - rho_s**2.0)
		ss_temp(1)       = -ss_temp(ns_temp)	
		do is = 2, ns_temp - 1
			ss_temp(is) = ss_temp(1) + (ss_temp(ns_temp) - ss_temp(1))*real(is - 1,8)/real(ns_temp - 1,8)
		enddo
		do iz = 1, nz
			do izprime = 1, nz
				izcprime = zcfun(izprime)
				if (izcprime >= ceiling((real(nzc,8) + 1)/2)) then
					sig_s = sig_s_e
				else
					sig_s = sig_s_r
				endif
				do is = 1, ns_temp
					mu  = rho_s*ss_temp(is)
					upp = (ss_temp(2) + ss_temp(1))/2.0
					upp = (upp - mu)/sig_s
					prob_ss_temp(is,1,iz,izprime) = normcdf(upp)
					low = (ss_temp(ns_temp) + ss_temp(ns_temp - 1))/2.0
					low = (low - mu)/sig_s
					prob_ss_temp(is,ns_temp,iz,izprime) = 1.0 - normcdf(low)
					do isprime = 2, ns_temp - 1		
						low = (ss_temp(isprime - 1) + ss_temp(isprime))/2.0
						upp = (ss_temp(isprime + 1) + ss_temp(isprime))/2.0
						low = (low - mu)/sig_s
						upp = (upp - mu)/sig_s
						prob_ss_temp(is,isprime,iz,izprime) = normcdf(upp) - normcdf(low)
					enddo
				enddo
			enddo
		enddo
	endif
	
	!Adjust transition probabilities to quarterly frequency
	do iz = 1, nz
		do izprime = 1, nz
			do is = 1, ns_temp
				do isprime = 1, ns_temp
					if (is == isprime) then
						prob_ss_temp(is,isprime,iz,izprime) = 0.25d0*prob_ss_temp(is,isprime,iz,izprime) + 0.75d0
					else
						prob_ss_temp(is,isprime,iz,izprime) = 0.25d0*prob_ss_temp(is,isprime,iz,izprime)
					endif
				enddo
			enddo
		enddo
	enddo
	
	if (rich_shock == 1) then
		!Add lucky high shock
		do is = 1, ns - 1
			ss(is) = ss_temp(is)
		enddo
		ss(ns) = ss(ns - 1) + log(s_rich)
		do is = 1, ns - 2
			do isprime = 1, ns - 1
				prob_ss(is,isprime,:,:) = prob_ss_temp(is,isprime,:,:)
			enddo
			prob_ss(is,ns,:,:) = 0.d0
		enddo
		do isprime = 1, ns - 1
			prob_ss(ns - 1,isprime,:,:) = prob_ss_temp(ns - 1,isprime,:,:)*(1 - prob_getrich)
		enddo
		prob_ss(ns - 1,ns,:,:) = prob_getrich
		do isprime = 1, ns - 2
			prob_ss(ns,isprime,:,:) = 0.d0
		enddo
		prob_ss(ns,ns,:,:)     = prob_stayrich
		prob_ss(ns,ns - 1,:,:) = 1 - prob_stayrich
	else
		prob_ss = prob_ss_temp
		ss      = ss_temp
	endif

	!Calculate stationary distribution of household characteristics during expansions
	call ergodist(ns,prob_ss(:,:,nz,nz),stationary_s)
	
	!Calculate average labor productivity
	enorm = 1.d0
	eavg  = 0.d0
	do is = 1, ns
		do ie = 1,ne
			labor(is,ie) = evalue(ss(is),ee(ie))
			eavg = eavg + labor(is,ie)*emeas(ie)*stationary_s(is)
		enddo
	enddo
!	enorm = eavg
!	eavg  = 0.d0
!	do is = 1, ns
!		do ie = 1,ne
!			eavg = eavg + evalue(ss(is),ee(ie))*emeas(ie)*stationary_s(is)
!		enddo
!	enddo
	
	!------------------------------------------------
	!Grids and transition probabilities for aggregate states
	!------------------------------------------------
	
	!Discretize AR(1) process for composite consumption TFP
	rouwenhorst_zc = 1
	if (nzc == 1) then
		zzc(1)         = zc_steady
		prob_zczc(1,1) = 1.d0
	else
		if (rouwenhorst_zc == 1) then
			!Rouwenhorst method
			call rouwenhorst(rho_zc,sig_zc/(1 - rho_zc**2.0)**0.5,prob_zczc,zzc)
		else
			!Tauchen method
			zzc(nzc) = cover_zc*sig_zc/sqrt(1.0 - rho_zc**2.0)
			zzc(1)   = -zzc(nzc)	
			do izc = 2, nzc - 1
				zzc(izc) = zzc(1) + (zzc(nzc) - zzc(1))*real(izc - 1,8)/real(nzc - 1,8)
			enddo
			do izc = 1, nzc
				mu  = rho_zc*zzc(izc)
				upp = (zzc(2) + zzc(1))/2.0
				upp = (upp - mu)/sig_zc
				prob_zczc(izc,1) = normcdf(upp)
				low = (zzc(nzc) + zzc(nzc - 1))/2.0
				low = (low - mu)/sig_zc	
				prob_zczc(izc,nzc) = 1.0 - normcdf(low)
				do izcprime = 2, nzc - 1		
					low = (zzc(izcprime - 1) + zzc(izcprime))/2.0
					upp = (zzc(izcprime + 1) + zzc(izcprime))/2.0
					low = (low - mu)/sig_zc
					upp = (upp - mu)/sig_zc
					prob_zczc(izc,izcprime) = normcdf(upp) - normcdf(low)
				enddo
			enddo
		endif
		zzc = exp(zzc)
		!Calculate stationary distribution of TFP shocks
		call ergodist(nzc,prob_zczc,stationary_zc)
		!Normalize
		zzc = zzc*zc_steady/dot_product(zzc,stationary_zc)
	endif
	
	!Discretize AR(1) process for housing construction TFP
	rouwenhorst_zh = 1
	if (nzh == 1) then
		zzh(1)         = zh_steady
		prob_zhzh(1,1) = 1.d0
	else
		if (rouwenhorst_zh == 1) then
			!Rouwenhorst method
			call rouwenhorst(rho_zh,sig_zh/(1 - rho_zh**2.0)**0.5,prob_zhzh,zzh)
		else
			!Tauchen method
			zzh(nzh) = cover_zh*sig_zh/sqrt(1.0 - rho_zh**2.0)
			zzh(1)   = -zzh(nzh)	
			do izh = 2, nzh - 1
				zzh(izh) = zzh(1) + (zzh(nzh) - zzh(1))*real(izh - 1,8)/real(nzh - 1,8)
			enddo
			do izh = 1, nzh
				mu  = rho_zh*zzh(izh)
				upp = (zzh(2) + zzh(1))/2.0
				upp = (upp - mu)/sig_zh
				prob_zhzh(izh,1) = normcdf(upp)
				low = (zzh(nzh) + zzh(nzh - 1))/2.0
				low = (low - mu)/sig_zh	
				prob_zhzh(izh,nzh) = 1.0 - normcdf(low)
				do izhprime = 2, nzh - 1		
					low = (zzh(izhprime - 1) + zzh(izhprime))/2.0
					upp = (zzh(izhprime + 1) + zzh(izhprime))/2.0
					low = (low - mu)/sig_zh
					upp = (upp - mu)/sig_zh
					prob_zhzh(izh,izhprime) = normcdf(upp) - normcdf(low)
				enddo
			enddo
		endif
		zzh = exp(zzh)
		!Calculate stationary distribution of housing TFP shocks
		call ergodist(nzh,prob_zhzh,stationary_zh)
		!Normalize
		zzc = zzh*zh_steady/dot_product(zzh,stationary_zh)
	endif
	
	!Discretize AR(1) process for preference shocks to utility share of housing
	rouwenhorst_hu = 1
	if (nhu == 1) then
		hhu(1)         = 1
		prob_huhu(1,1) = 1.d0
	else
		if (rouwenhorst_hu == 1) then
			!Rouwenhorst method
			call rouwenhorst(rho_hu,sig_hu/(1 - rho_hu**2.0)**0.5,prob_huhu,hhu)
		else
			!Tauchen method
			hhu(nhu) = cover_hu*sig_hu/sqrt(1.0 - rho_hu**2.0)
			hhu(1)   = -hhu(nhu)	
			do ihu = 2, nhu - 1
				hhu(ihu) = hhu(1) + (hhu(nhu) - hhu(1))*real(ihu - 1,8)/real(nhu - 1,8)
			enddo
			do ihu = 1, nhu
				mu  = rho_hu*hhu(ihu)
				upp = (hhu(2) + hhu(1))/2.0
				upp = (upp - mu)/sig_hu
				prob_huhu(ihu,1) = normcdf(upp)
				low = (hhu(nhu) + hhu(nhu - 1))/2.0
				low = (low - mu)/sig_hu	
				prob_huhu(ihu,nhu) = 1.0 - normcdf(low)
				do ihuprime = 2, nhu - 1		
					low = (hhu(ihuprime - 1) + hhu(ihuprime))/2.0
					upp = (hhu(ihuprime + 1) + hhu(ihuprime))/2.0
					low = (low - mu)/sig_hu
					upp = (upp - mu)/sig_hu
					prob_huhu(ihu,ihuprime) = normcdf(upp) - normcdf(low)
				enddo
			enddo
		endif
		hhu = exp(hhu)
		!Calculate stationary distribution of preference shocks to utility share of housing
		call ergodist(nhu,prob_huhu,stationary_hu)
		!Normalize
		hhu = hhu/dot_product(hhu,stationary_hu)
	endif

	!Transition probabilities for aggregate shocks
	do iz = 1, nz
		do izprime = 1, nz
			prob_zz(iz,izprime) = prob_zczc(zcfun(iz),zcfun(izprime))*prob_zhzh(zhfun(iz),zhfun(izprime))*prob_huhu(hufun(iz),hufun(izprime))
		enddo
	enddo
	
	!Calculate stationary distribution of aggregate shocks
	call ergodist(nz,prob_zz,stationary_z)

	!Generate grid for the shadow price of housing
	if (nph == 1) then
		pph(1) = min_ph
	else
		do iph = 1, nph
			phmod    = dph*real(iph - 1,8)
			pph(iph) = phmod**(1/curv_ph) + min_ph
		enddo
	endif
	
	!Generate grid for aggregate capital
	if (nk == 1) then
		kk(1) = min_k
	else
		do ik = 1, nk
			kmod   = dk*real(ik - 1,8)
			kk(ik) = kmod**(1/curv_k) + min_k
		enddo
	endif
	
	!Generate ph(iagg) and kagg(iagg)
	do iagg = 1, nagg
		iph = phfun(iagg)
		ik  = kfun(iagg)
		ph(iagg)   = pph(iph)
		kagg(iagg) = kk(ik)
	enddo

	!Vector of exogenous variables for Krusell-Smith regression
	do iagg = 1, nagg
		forecast_vec(1,iagg) = 1.d0
		forecast_vec(2,iagg) = ph(iagg)
		forecast_vec(3,iagg) = kagg(iagg)
	enddo
	
	!Market prices and earnings
	ph0 = 1.d0
	do iz = 1, nz
		cshare(iz) = 1.d0/(1.d0 + (((1 - cshare0)*hhu(hufun(iz)))/(1 - (1 - cshare0)*hhu(hufun(iz)))*(rentratio*ph0)**(ies - 1))**(1/ies))
		if (cshare(iz) < 0.d0) then
			print *, 'Negative consumption share!'
			stop
		elseif (cshare(iz) > 1.d0) then
			print *, 'Negative housing share!'
			stop
		endif
		zc(iz) = zzc(zcfun(iz))
		zh(iz) = zzh(zhfun(iz))
	enddo
			
	!Submarket revenue
	do iagg = 1, nagg
		do ih = 1, nh
			do ix = 1, nx
				xbuy(ix,ih,iagg)  = xb(xx(ix),hh(ih),ph(iagg))
				xsell(ix,ih,iagg) = xs(xx(ix),hh(ih),ph(iagg))
			enddo
		enddo
	enddo
	
	!Display grids (to make sure nothing is obviously wrong)
	printgrids = 0
	if (printgrids == 1) then
		print *, 'Leverage'
		print *, mm
		print *, ' '
		print *, 'Cash at hand'
		print *, yy
		print *, ' '
		print *, 'Bonds'
		print *, bb
		print *, ' '
		print *, 'Housing'
		print *, hh
		print *, ' '
		print *, 'Submarkets'
		print *, xx
	endif

end subroutine sub_initialize