subroutine sub_params

	use mod_globvar

	implicit none

	!This program sets parameters automatically or using a text input file
	
	!Variables for parameter vectors
	integer :: nparams_integers, nparams_reals
	integer, dimension(:), allocatable :: params_integers
	integer, dimension(:,:), allocatable :: params_integers_temp
	real(8), dimension(:), allocatable :: params_reals
	real(8), dimension(:,:), allocatable :: params_reals_temp
	
	!Other internal variables
	integer :: index, iparam, index_integer, index_real
	real(8) :: rmspread
	
	!Allocate parameter vectors
	nparams_integers = 16
	nparams_reals    = 76
	allocate(params_integers(nparams_integers))
	allocate(params_reals(nparams_reals))
	allocate(params_integers_temp(nparams_integers,2))
	allocate(params_reals_temp(nparams_reals,2))
		
	if (load_params == 0) then
		!Set parameters manually (74 real parameters, 16 integer parameters)
		!!!Parameters common to steady state model!!!
		!Cash at hand						!Real #		!Integer #
		ny_neg         = 50								!i1
		ny_pos         = 175							!i2
		min_y          = -1.d0				!r1
		max_y          = 15.d0				!r2
		curv_yneg      = 1.d0				!r3
		curv_ypos      = 0.45d0				!r4
		!Cash at hand (fine grid)
		nyfine_neg     = 500							!i3
		nyfine_pos     = 10000							!i4
		min_yfine      = -1.d0				!r5
		max_yfine      = 30.d0				!r6
		curv_yfineneg  = 1.d0				!r7
		curv_yfinepos  = 0.45d0				!r8
		!Bonds
		nb             = 350							!i5
		max_b          = 15.d0				!r9
		curv_b         = 0.55d0				!r10
		!Housing and apartment parameters
		nh             = 3								!i6
		min_h          = 2.05d0				!r11
		curv_h         = 0.65d0				!r12
		nu             = 1								!i7
		useparate      = 0.d0				!r13
		pseparate	   = 0.d0				!r14
		utilown        = 1.8d0				!r15
		rentratio      = 0.0125d0			!r16
		!Housing search
		nx			   = 51								!i8
		curv_x         = 1.d0				!r17
		min_xs         = 0.8d0				!r18
		max_xs         = 1.d0				!r19
		min_xb		   = 1.d0				!r20
		max_xb         = 1.025d0			!r21
		curv_ms        = 0.755d0			!r22
		curv_mb        = 0.355d0			!r23
		searchcost     = 0.0062d0			!r24
		!Mortgage parameters
		nm             = 51								!i9
		max_ltv        = 1.d0				!r25
		max_m          = 1.1d0				!r26
		curv_m         = 1.d0				!r27
		!Foreclosure and REO parameters
		pflag          = 0.95d0				!r28
		fincloss       = 0.d0				!r29
		fdeficiency    = 0.d0				!r30
		fdiscount	   = 0.1017d0			!r31
		psearch		   = 0.3213d0			!r32
		!Preference parameters
		beta		   = 0.9798d0			!r33
		gamma		   = 5.1d0				!r34
		eis			   = 1.73d0				!r35
		ies			   = 0.13d0				!r36
		cshare0        = 0.7975d0			!r37
		min_spend      = 1e-6				!r38
		!Idiosyncratic labor productivity parameters (see Storesletten et al (2004))
		ns             = 2								!i10
		mean_s         = 0.d0				!r39
		rho_s          = 0.952d0			!r40
		sig_s_avg      = 0.17d0				!r41
		cover_s        = 1.d0				!r42
		s_rich         = 20.d0				!r43
		prob_getrich   = 0.001d0			!r44
		prob_stayrich  = 0.01d0				!r45
		ne			   = 10								!i11
		sig_e          = 0.49d0				!r46
		cover_e		   = 2.5d0				!r47
		!Depreciation
		deprec_k       = 0.025d0			!r48
		deprec_h       = 0.00625d0			!r49
		!Production function parameters
		kshare_c	   = 0.26d0				!r50
		kshare_h	   = 0.3d0				!r51
		lshare         = 0.33d0				!r52
		zh_steady	   = 0.25d0				!r53
		!Mortgage costs
		rocost         = 0.02d0				!r54
		rmspread       = 0.041d0			!r55
		!Government policy
		sellcost       = 0.d0				!r56
		!!!Parameters unique to model with aggregate shocks!!!
		!Mortgage costs
		rscost         = 0.d0				!r57
		!Housing and apartment parameters
		phbase         = 0.85455d0			!r58
		!Idiosyncratic labor productivity parameters (see Storesletten et al (2004))
		sig_s_e        = 0.125d0			!r59
		sig_s_r        = 0.211d0			!r60
		!Shadow price of housing
		nph            = 5								!i12
		min_ph         = 0.68364d0			!r61
		max_ph         = 1.02546d0			!r62
		curv_ph        = 1.d0				!r63
		!Aggregate capital
		nk             = 3								!i13
		min_k          = 2.25d0				!r64
		max_k          = 2.8d0				!r65
		curv_k         = 1.d0				!r66
		!Total factor productivity of composite good production
		nzc            = 2								!i14
		zc_steady      = 0.218033d0			!r67
		sig_zc		   = 0.007d0			!r68
		rho_zc         = 0.95d0				!r69
		cover_zc       = 1.d0				!r70
		!Total factor productivity of housing construction
		nzh            = 1								!i15
		rho_zh         = 0.99d0				!r71
		sig_zh         = 0.01d0				!r72
		cover_zh       = 1.d0				!r73
		!Aggregate shocks to utility share of housing
		nhu		       = 1								!i16
		sig_hu		   = 0.015d0			!r74
		rho_hu         = 0.95d0				!r75
		cover_hu       = 1.d0				!r76
	else
		!Load parameters
		open(unit=11,	file='Input\params_reals_in.txt')
		do iparam = 1, nparams_reals
			read(11, *) (params_reals_temp(iparam,index), index = 1,2)
		enddo
		close(11)		
		open(unit=11,	file='Input\params_integers_in.txt')
		do iparam = 1, nparams_integers
			read(11, *) (params_integers_temp(iparam,index), index = 1,2)
		enddo
		close(11)
		params_reals    = params_reals_temp(:,2)
		params_integers = params_integers_temp(:,2)
		index_integer   = 1
		index_real      = 1
		!!!Parameters common to steady state model!!!
		!Cash at hand
		ny_neg         = params_integers(index_integer)
																		index_integer = index_integer + 1
		ny_pos         = params_integers(index_integer)
																		index_integer = index_integer + 1
		min_y          = params_reals(index_real)
																		index_real    = index_real + 1
		max_y          = params_reals(index_real)
																		index_real    = index_real + 1
		curv_yneg      = params_reals(index_real)
																		index_real    = index_real + 1
		curv_ypos      = params_reals(index_real)
																		index_real    = index_real + 1
		!Cash at hand (fine grid)
		nyfine_neg     = params_integers(index_integer)
																		index_integer = index_integer + 1
		nyfine_pos     = params_integers(index_integer)
																		index_integer = index_integer + 1
		min_yfine      = params_reals(index_real)
																		index_real    = index_real + 1
		max_yfine      = params_reals(index_real)
																		index_real    = index_real + 1
		curv_yfineneg  = params_reals(index_real)
																		index_real    = index_real + 1
		curv_yfinepos  = params_reals(index_real)
																		index_real    = index_real + 1
		!Bonds
		nb             = params_integers(index_integer)
																		index_integer = index_integer + 1
		max_b          = params_reals(index_real)
																		index_real    = index_real + 1
		curv_b         = params_reals(index_real)
																		index_real    = index_real + 1
		!Housing and apartment parameters
		nh             = params_integers(index_integer)
																		index_integer = index_integer + 1
		min_h          = params_reals(index_real)
																		index_real    = index_real + 1
		curv_h         = params_reals(index_real)
																		index_real    = index_real + 1
		nu             = params_integers(index_integer)
																		index_integer = index_integer + 1
		useparate      = params_reals(index_real)
																		index_real    = index_real + 1
		pseparate	   = params_reals(index_real)
																		index_real    = index_real + 1
		utilown        = params_reals(index_real)
																		index_real    = index_real + 1
		rentratio      = params_reals(index_real)/4.d0
																		index_real    = index_real + 1
		!Housing search
		nx			   = params_integers(index_integer)
																		index_integer = index_integer + 1
		curv_x         = params_reals(index_real)
																		index_real    = index_real + 1
		min_xs         = params_reals(index_real)
																		index_real    = index_real + 1
		max_xs         = params_reals(index_real)
																		index_real    = index_real + 1
		min_xb         = params_reals(index_real)
																		index_real    = index_real + 1
		max_xb         = params_reals(index_real)
																		index_real    = index_real + 1
		curv_ms        = params_reals(index_real)
																		index_real    = index_real + 1
		curv_mb        = params_reals(index_real)
																		index_real    = index_real + 1
		searchcost     = params_reals(index_real)
																		index_real    = index_real + 1
		!Mortgage pseudo-leverage ratios
		nm             = params_integers(index_integer)
																		index_integer = index_integer + 1
		max_ltv		   = params_reals(index_real)
																		index_real    = index_real + 1
		max_m          = params_reals(index_real)
																		index_real    = index_real + 1
		curv_m         = params_reals(index_real)
																		index_real    = index_real + 1
		!Foreclosure and REO parameters
		pflag          = params_reals(index_real)
																		index_real    = index_real + 1
		fincloss       = params_reals(index_real)
																		index_real    = index_real + 1
		fdeficiency    = params_reals(index_real)
																		index_real    = index_real + 1
		fdiscount	   = params_reals(index_real)
																		index_real    = index_real + 1
		psearch		   = params_reals(index_real)
																		index_real    = index_real + 1
		!Preference parameters (CRRA specification)
		beta		   = params_reals(index_real)
																		index_real    = index_real + 1
		gamma		   = params_reals(index_real)
																		index_real    = index_real + 1
		eis			   = params_reals(index_real)
																		index_real    = index_real + 1
		ies			   = params_reals(index_real)
																		index_real    = index_real + 1
		cshare0        = params_reals(index_real)
																		index_real    = index_real + 1
		min_spend      = params_reals(index_real)
																		index_real    = index_real + 1
		!Idiosyncratic labor productivity parameters (see Storesletten et al (2004))
		ns             = params_integers(index_integer)
																		index_integer = index_integer + 1
		mean_s         = params_reals(index_real)
																		index_real    = index_real + 1
		rho_s          = params_reals(index_real)
																		index_real    = index_real + 1
		sig_s_avg      = params_reals(index_real)
																		index_real    = index_real + 1
		cover_s        = params_reals(index_real)
																		index_real    = index_real + 1
		s_rich         = params_reals(index_real)
																		index_real    = index_real + 1
		prob_getrich   = params_reals(index_real)
																		index_real    = index_real + 1
		prob_stayrich  = params_reals(index_real)
																		index_real    = index_real + 1
		ne			   = params_integers(index_integer)
																		index_integer = index_integer + 1
		sig_e          = params_reals(index_real)
																		index_real    = index_real + 1
		cover_e		   = params_reals(index_real)
																		index_real    = index_real + 1
		!Depreciation
		deprec_k       = params_reals(index_real)/4.d0
																		index_real    = index_real + 1
		deprec_h       = params_reals(index_real)/4.d0
																		index_real    = index_real + 1
		!Production function parameters
		kshare_c	   = params_reals(index_real)
																		index_real    = index_real + 1
		kshare_h	   = params_reals(index_real)
																		index_real    = index_real + 1
		lshare         = params_reals(index_real)
																		index_real    = index_real + 1
		zh_steady      = params_reals(index_real)
																		index_real    = index_real + 1
		!Mortgage costs and interest rates
		rocost         = params_reals(index_real)
																		index_real    = index_real + 1
		rmspread	   = params_reals(index_real)
																		index_real    = index_real + 1
		!Government policy
		sellcost       = params_reals(index_real)
																		index_real    = index_real + 1
		!!!Parameters unique to model with aggregate shocks!!!
		!Mortgage costs
		rscost         = params_reals(index_real)
																		index_real    = index_real + 1
		!Housing and apartment parameters
		phbase         = params_reals(index_real)
																		index_real    = index_real + 1
		!Idiosyncratic labor productivity parameters (see Storesletten et al (2004))
		sig_s_e        = params_reals(index_real)
																		index_real    = index_real + 1
		sig_s_r        = params_reals(index_real)
																		index_real    = index_real + 1
		!Shadow price of housing
		nph            = params_integers(index_integer)
																		index_integer = index_integer + 1
		min_ph         = params_reals(index_real)
																		index_real    = index_real + 1
		max_ph         = params_reals(index_real)
																		index_real    = index_real + 1
		curv_ph        = params_reals(index_real)
																		index_real    = index_real + 1
		!Aggregate capital
		nk             = params_integers(index_integer)
																		index_integer = index_integer + 1
		min_k          = params_reals(index_real)
																		index_real    = index_real + 1
		max_k          = params_reals(index_real)
																		index_real    = index_real + 1
		curv_k         = params_reals(index_real)
																		index_real    = index_real + 1
		!Total factor productivity of composite good production
		nzc            = params_integers(index_integer)
																		index_integer = index_integer + 1
		zc_steady      = params_reals(index_real)
																		index_real    = index_real + 1
		sig_zc		   = params_reals(index_real)
																		index_real    = index_real + 1
		rho_zc         = params_reals(index_real)
																		index_real    = index_real + 1
		cover_zc       = params_reals(index_real)
																		index_real    = index_real + 1
		!Total factor productivity of housing construction
		nzh            = params_integers(index_integer)
																		index_integer = index_integer + 1
		rho_zh         = params_reals(index_real)
																		index_real    = index_real + 1
		sig_zh         = params_reals(index_real)
																		index_real    = index_real + 1
		cover_zh       = params_reals(index_real)
																		index_real    = index_real + 1
		!Aggregate shocks to utility share of housing
		nhu		       = params_integers(index_integer)
		sig_hu		   = params_reals(index_real)
																		index_real    = index_real + 1
		rho_hu         = params_reals(index_real)
																		index_real    = index_real + 1
		cover_hu       = params_reals(index_real)
	endif
	
	if (index_real .NE. nparams_reals .OR. index_integer .NE. nparams_integers) then
		print *, 'Error with parameters file!'
		stop
	endif
	
	!!!Finish setting parameters!!!
	!Cash at hand
	ny     = ny_neg + ny_pos
	dy_neg = (-min_y)**curv_yneg/real(ny_neg,8)
	dy_pos = max_y**curv_ypos/real(ny_pos - 1,8)
	
	!Cash at hand (fine grid)
	nyfine     = nyfine_neg + nyfine_pos
	dyfine_neg = (-min_yfine)**curv_yfineneg/real(nyfine_neg,8)
	dyfine_pos = max_yfine**curv_yfinepos/real(nyfine_pos - 1,8)
	
	!Bonds
	db = max_b**curv_b/real(nb - 1,8)
	
	!Housing and apartment parameters
	max_h      = 2.d0*min_h
	dh		   = (max_h - min_h)**curv_h/real(nh - 1,8)
	hincrement = (max_h/min_h)**(1.d0/real(nh - 1)) - 1.d0
	ng         = nh*nu
	max_a      = min_h
	pa         = rentratio*phbase
	
	!Housing search
	dx = 1.d0/real(nx - 1,8)
	
	!Mortgage pseudo-leverage ratios
	max_m = max_m*max_ph
	dm    = max_m**curv_m/real(nm - 1,8)
	
	!Preference parameters
	theta = (1 - gamma)/(1 - 1/eis)
	
	!Total number of aggregate shocks
	nz = nzc*nzh*nhu
	
	!Shadow price of housing
	dph = (max_ph - min_ph)**curv_ph/max(1.d0,real(nph - 1,8))
	
	!Aggregate capital
	dk = (max_k - min_k)**curv_k/max(1.d0,real(nk - 1,8))
	
	!Number of interpolation points for endogenous aggregate variables -- 2^(number of endogenous aggregate variables)
	ninterp = 4
	
	!Total number of endogenous aggregate grid points
	nagg = nph*nk
	
	!------------------------------------------------
	!Prices and trade probabilities
	!------------------------------------------------
	allocate(prob_xs(nx))
	allocate(prob_xb(nx))
	allocate(xbuy(nx,nh,nagg))
	allocate(xsell(nx,nh,nagg))
	allocate(labor(ns,ne))
	allocate(ph(nagg))
	allocate(kagg(nagg))
	allocate(rkprime(nz,nagg,nz))
	allocate(wprime(nz,nagg,nz))
	allocate(qb(nz,nagg))
	allocate(rb(nz,nagg))
	allocate(qm(nz,nagg))
	allocate(qm0(nm,nb,ng,ns,nz,nagg))
	allocate(zc(nz))
	allocate(zh(nz))
	allocate(cshare(nz))	

	!------------------------------------------------
	!Grids
	!------------------------------------------------
	allocate(mm(nm))
	allocate(yy(ny))
	allocate(yyfine(nyfine))
	allocate(bb(nb))
	allocate(hh(nh))
	allocate(uu(nu))
	allocate(xx(nx))
	allocate(ss(ns))
	allocate(ee(ne))
	allocate(zzc(nzc))
	allocate(zzh(nzh))
	allocate(hhu(nhu))
	allocate(pph(nph))
	allocate(kk(nk))

	!------------------------------------------------
	!Transition probabilities and state variable correspondences
	!------------------------------------------------
	allocate(zcfun(nz))
	allocate(zhfun(nz))
	allocate(hufun(nz))
	allocate(zfun(nzc,nzh,nhu))
	allocate(phfun(nagg))
	allocate(kfun(nagg))
	allocate(aggfun(nph,nk))
	allocate(hfun(ng))
	allocate(ufun(ng))
	allocate(gfun(nh,nu))
	allocate(prob_ss(ns,ns,nz,nz))
	allocate(prob_zz(nz,nz))
	allocate(stationary_z(nz))
	allocate(emeas(ne))
	
	!------------------------------------------------
	!Simulation
	!------------------------------------------------
	allocate(forecast_coeff(3,3,nz,nz))
	allocate(forecast_vec(3,nagg))
	allocate(phprime(nz,nagg,nz))
	allocate(kaggprime(nz,nagg,nz))
	allocate(taxprime(nz,nagg,nz))
	allocate(qmprime(nz,nagg,nz,ninterp))
	allocate(iaggprime(nz,nagg,nz,ninterp))
	allocate(waggprime(nz,nagg,nz,ninterp))
	
	!------------------------------------------------
	!Endogenous objects for optimization
	!------------------------------------------------
	!Allocate value and policy functions
	allocate(Vown_f0(ny,nm,ng,ns,nz,nagg))
	allocate(m_f0(ny,nm,ng,ns,nz,nagg))
	allocate(locm_f0(ny,nm,ng,ns,nz,nagg))
	allocate(bown_f0(ny,nm,ng,ns,nz,nagg))
	allocate(bown_f0_sp1(ny,nm,ng,ns,nz,nagg))
	allocate(locbown_f0(ny,nm,ng,ns,nz,nagg))
	allocate(locbown_f0_sp1(ny,nm,ng,ns,nz,nagg))
	allocate(cown_f0(ny,nm,ng,ns,nz,nagg))
	allocate(hcown_f0(ny,nm,ng,ns,nz,nagg))
	allocate(spendown_f0(ny,nm,ng,ns,nz,nagg))
	allocate(Wown_f0(ny,nm,ng,ns,nz,nagg))
	allocate(d_f0(ny,nm,ng,ns,nz,nagg))
	allocate(xs_f0(ny,nm,ng,ns,nz,nagg))
	allocate(locxs_f0(ny,nm,ng,ns,nz,nagg))
	allocate(xsratio_f0(ny,nm,ng,ns,nz,nagg))
	allocate(psell_f0(ny,nm,ng,ns,nz,nagg))
	allocate(Vown_f1(ny,ng,ns,nz,nagg))
	allocate(bown_f1(ny,ng,ns,nz,nagg))
	allocate(locbown_f1(ny,ng,ns,nz,nagg))
	allocate(cown_f1(ny,ng,ns,nz,nagg))
	allocate(hcown_f1(ny,ng,ns,nz,nagg))
	allocate(spendown_f1(ny,ng,ns,nz,nagg))
	allocate(Wown_f1(ny,ng,ns,nz,nagg))
	allocate(xs_f1(ny,ng,ns,nz,nagg))
	allocate(locxs_f1(ny,ng,ns,nz,nagg))
	allocate(xsratio_f1(ny,ng,ns,nz,nagg))
	allocate(psell_f1(ny,ng,ns,nz,nagg))
	allocate(Vrent_f0(ny,ns,nz,nagg))
	allocate(brent_f0(ny,ns,nz,nagg))
	allocate(locbrent_f0(ny,ns,nz,nagg))
	allocate(crent_f0(ny,ns,nz,nagg))
	allocate(hcrent_f0(ny,ns,nz,nagg))
	allocate(spendrent_f0(ny,ns,nz,nagg))
	allocate(Vrent_f1(ny,ns,nz,nagg))
	allocate(brent_f1(ny,ns,nz,nagg))
	allocate(locbrent_f1(ny,ns,nz,nagg))
	allocate(crent_f1(ny,ns,nz,nagg))
	allocate(hcrent_f1(ny,ns,nz,nagg))
	allocate(spendrent_f1(ny,ns,nz,nagg))
	allocate(Wrent_f0(ny,ns,nz,nagg))
	allocate(xb_f0(ny,ns,nz,nagg))
	allocate(locxb_f0(ny,ns,nz,nagg))
	allocate(xbratio_f0(ny,ns,nz,nagg))
	allocate(h_f0(ny,ns,nz,nagg))
	allocate(loch_f0(ny,ns,nz,nagg))
	allocate(pbuy_f0(ny,ns,nz,nagg))
	allocate(Wrent_f1(ny,ns,nz,nagg))
	allocate(xb_f1(ny,ns,nz,nagg))
	allocate(locxb_f1(ny,ns,nz,nagg))
	allocate(xbratio_f1(ny,ns,nz,nagg))
	allocate(h_f1(ny,ns,nz,nagg))
	allocate(loch_f1(ny,ns,nz,nagg))
	allocate(pbuy_f1(ny,ns,nz,nagg))

	!Allocate REO objects
	allocate(Jreo(nh,nz,nagg))
	allocate(psell_reo(nh,nz,nagg))
	allocate(xs_reo(nh,nz,nagg))
	allocate(htot_reo(nh))
	
	!------------------------------------------------
	!Distributions and REO housing stock (format: subperiod, owner/renter, good credit/bad credit)
	!------------------------------------------------
	!Allocate distributions
	allocate(mu1own_f0(ny,nm,ng,ns))
	allocate(mu2own_f0(ny,nm,ng,ns))
	allocate(mu3own_f0(ny,nm,ng,ns))
	allocate(mu1own_f1(ny,ng,ns))
	allocate(mu2own_f1(ny,ng,ns))
	allocate(mu3own_f1(ny,ng,ns))
	allocate(mu1rent_f0(ny,ns))
	allocate(mu2rent_f0(ny,ns))
	allocate(mu3rent_f0(ny,ns))
	allocate(mu1rent_f1(ny,ns))
	allocate(mu2rent_f1(ny,ns))
	allocate(mu3rent_f1(ny,ns))
	
	!------------------------------------------------
	!Other
	!------------------------------------------------
	allocate(ylb(ng,ns,nz,nagg))
	
	!Print parameters files
	index_integer = 1
	index_real    = 1
	!!!Parameters common to steady state model!!!
	!Cash at hand
	params_integers(index_integer)  = ny_neg
																	index_integer = index_integer + 1
	params_integers(index_integer)  = ny_pos
																	index_integer = index_integer + 1
	params_reals(index_real)		= min_y
																	index_real    = index_real + 1
	params_reals(index_real)		= max_y
																	index_real    = index_real + 1
	params_reals(index_real)		= curv_yneg
																	index_real    = index_real + 1
	params_reals(index_real)		= curv_ypos
																	index_real    = index_real + 1
	!Cash at hand (fine grid)
	params_integers(index_integer)  = nyfine_neg
																	index_integer = index_integer + 1
	params_integers(index_integer)  = nyfine_pos
																	index_integer = index_integer + 1
	params_reals(index_real)		= min_yfine
																	index_real    = index_real + 1
	params_reals(index_real)		= max_yfine
																	index_real    = index_real + 1
	params_reals(index_real)		= curv_yfineneg
																	index_real    = index_real + 1
	params_reals(index_real)		= curv_yfinepos
																	index_real    = index_real + 1
	!Bonds
	params_integers(index_integer)  = nb
																	index_integer = index_integer + 1
	params_reals(index_real)		= max_b
																	index_real    = index_real + 1
	params_reals(index_real)		= curv_b
																	index_real    = index_real + 1
	!Housing and apartment parameters
	params_integers(index_integer)  = nh
																	index_integer = index_integer + 1
	params_reals(index_real)		= min_h
																	index_real    = index_real + 1
	params_reals(index_real)		= curv_h
																	index_real    = index_real + 1
	params_integers(index_integer)  = nu
																	index_integer = index_integer + 1
	params_reals(index_real)		= useparate
																	index_real    = index_real + 1
	params_reals(index_real)		= pseparate
																	index_real    = index_real + 1
	params_reals(index_real)		= utilown
																	index_real    = index_real + 1
	params_reals(index_real)		= rentratio*4.d0
																	index_real    = index_real + 1
	!Housing search
	params_integers(index_integer)  = nx
																	index_integer = index_integer + 1
	params_reals(index_real)		= curv_x
																	index_real    = index_real + 1
	params_reals(index_real)		= min_xs
																	index_real    = index_real + 1
	params_reals(index_real)		= max_xs
																	index_real    = index_real + 1
	params_reals(index_real)		= min_xb
																	index_real    = index_real + 1
	params_reals(index_real)		= max_xb
																	index_real    = index_real + 1
	params_reals(index_real)		= curv_ms
																	index_real    = index_real + 1
	params_reals(index_real)		= curv_mb
																	index_real    = index_real + 1
	params_reals(index_real)		= searchcost
																	index_real    = index_real + 1
	!Mortgage pseudo-leverage ratios
	params_integers(index_integer)  = nm
																	index_integer = index_integer + 1
	params_reals(index_real)		= max_ltv
																	index_real    = index_real + 1
	params_reals(index_real)		= max_m
																	index_real    = index_real + 1
	params_reals(index_real)		= curv_m
																	index_real    = index_real + 1
	!Foreclosure and REO parameters
	params_reals(index_real)		= pflag
																	index_real    = index_real + 1
	params_reals(index_real)		= fincloss
																	index_real    = index_real + 1
	params_reals(index_real)		= fdeficiency
																	index_real    = index_real + 1
	params_reals(index_real)		= fdiscount
																	index_real    = index_real + 1
	params_reals(index_real)		= psearch
																	index_real    = index_real + 1
	!Preference parameters (CRRA specification)
	params_reals(index_real)		= beta
																	index_real    = index_real + 1
	params_reals(index_real)		= gamma
																	index_real    = index_real + 1
	params_reals(index_real)		= eis
																	index_real    = index_real + 1
	params_reals(index_real)		= ies
																	index_real    = index_real + 1
	params_reals(index_real)		= cshare0
																	index_real    = index_real + 1
	params_reals(index_real)		= min_spend
																	index_real    = index_real + 1
	!Idiosyncratic labor productivity parameters (see Storesletten et al (2004))
	params_integers(index_integer) = ns
																	index_integer = index_integer + 1
	params_reals(index_real)	   = mean_s
																	index_real    = index_real + 1
	params_reals(index_real)	   = rho_s
																	index_real    = index_real + 1
	params_reals(index_real)       = sig_s_avg
																	index_real    = index_real + 1
	params_reals(index_real)       = cover_s
																	index_real    = index_real + 1
	params_reals(index_real)       = s_rich
																	index_real    = index_real + 1
	params_reals(index_real)       = prob_getrich
																	index_real    = index_real + 1
	params_reals(index_real)       = prob_stayrich
																	index_real    = index_real + 1
	params_integers(index_integer) = ne
																	index_integer = index_integer + 1
	params_reals(index_real)	   = sig_e
																	index_real    = index_real + 1
	params_reals(index_real)       = cover_e
																	index_real    = index_real + 1
	!Depreciation
	params_reals(index_real)	   = deprec_k*4.d0
																	index_real    = index_real + 1
	params_reals(index_real)       = deprec_h*4.d0
																	index_real    = index_real + 1
	!Production function parameters
	params_reals(index_real)	   = kshare_c
																	index_real    = index_real + 1
	params_reals(index_real)       = kshare_h
																	index_real    = index_real + 1
	params_reals(index_real)       = lshare
																	index_real    = index_real + 1
	params_reals(index_real)       = zh_steady
																	index_real    = index_real + 1
	!Mortgage costs and interest rates
	params_reals(index_real)       = rocost
																	index_real    = index_real + 1
	params_reals(index_real)	   = rmspread
																	index_real    = index_real + 1
	!Government policy
	params_reals(index_real)       = sellcost
																	index_real    = index_real + 1
	!!!Parameters unique to model with aggregate shocks!!!
	!Mortgage costs
	params_reals(index_real)       = rscost
																	index_real    = index_real + 1
	!Housing and apartment parameters
	params_reals(index_real)       = phbase
																	index_real    = index_real + 1
	!Idiosyncratic labor productivity parameters (see Storesletten et al (2004))
	params_reals(index_real)       = sig_s_e
																	index_real    = index_real + 1
	params_reals(index_real)       = sig_s_r
																	index_real    = index_real + 1
	!Shadow price of housing
	params_integers(index_integer) = nph
																	index_integer = index_integer + 1
	params_reals(index_real)       = min_ph
																	index_real    = index_real + 1
	params_reals(index_real)       = max_ph
																	index_real    = index_real + 1
	params_reals(index_real)       = curv_ph
																	index_real    = index_real + 1
	!Aggregate capital
	params_integers(index_integer) = nk
																	index_integer = index_integer + 1
	params_reals(index_real)       = min_k
																	index_real    = index_real + 1
	params_reals(index_real)       = max_k
																	index_real    = index_real + 1
	params_reals(index_real)       = curv_k
																	index_real    = index_real + 1
	!Total factor productivity of composite good production
	params_integers(index_integer) = nzc
																	index_integer = index_integer + 1
	params_reals(index_real)       = zc_steady
																	index_real    = index_real + 1
	params_reals(index_real)       = sig_zc
																	index_real    = index_real + 1
	params_reals(index_real)       = rho_zc
																	index_real    = index_real + 1
	params_reals(index_real)       = cover_zc
																	index_real    = index_real + 1
	!Total factor productivity of housing construction
	params_integers(index_integer) = nzh
																	index_integer = index_integer + 1
	params_reals(index_real)       = rho_zh
																	index_real    = index_real + 1
	params_reals(index_real)       = sig_zh
																	index_real    = index_real + 1
	params_reals(index_real)       = cover_zh
																	index_real    = index_real + 1
	!Aggregate shocks to utility share of housing
	params_integers(index_integer) = nhu
	params_reals(index_real)       = sig_hu
																	index_real    = index_real + 1
	params_reals(index_real)       = rho_hu
																	index_real    = index_real + 1
	params_reals(index_real)       = cover_hu
	
	open(unit=11,	file='params_reals_out.txt', status = 'replace')
	write(11, '(i4, 1f20.10)') (index, params_reals(index), index = 1, nparams_reals)
	close(11)
	
	open(unit=11,	file='params_integers_out.txt', status = 'replace')
	write(11, '(i4, i10)') (index, params_integers(index), index = 1, nparams_integers)
	close(11)
	
	open(unit=11,	file='params_summary.txt', status = 'replace')
	write(11, '(a1)'), ' '
	write(11, '(a50)'), ' ****************Model parameters*****************'
	write(11, '(a1)'), ' '
	write(11, '(a50)'), '-------------------Cash at hand-------------------'
	write(11, '(a1)'), ' '
	write(11, '(a20, i10)'), '           ny_neg = ' ,ny_neg
	write(11, '(a20, i10)'), '           ny_pos = ' ,ny_pos
	write(11, '(a20, i10)'), '               ny = ' ,ny
	write(11, '(a20, f20.10)'), '            min_y = ' ,min_y
	write(11, '(a20, f20.10)'), '            max_y = ' ,max_y
	write(11, '(a20, f20.10)'), '        curv_yneg = ' ,curv_yneg
	write(11, '(a20, f20.10)'), '        curv_ypos = ' ,curv_ypos
	write(11, '(a20, f20.10)'), '           dy_neg = ' ,dy_neg
	write(11, '(a20, f20.10)'), '           dy_pos = ' ,dy_pos
	write(11, '(a1)'), ' '
	write(11, '(a50)'), '----------Cash at hand (extra fine grid)----------'
	write(11, '(a1)'), ' '
	write(11, '(a20, i10)'), '       nyfine_neg = ' ,nyfine_neg
	write(11, '(a20, i10)'), '       nyfine_pos = ' ,nyfine_pos
	write(11, '(a20, i10)'), '           nyfine = ' ,nyfine
	write(11, '(a20, f20.10)'), '        min_yfine = ' ,min_yfine
	write(11, '(a20, f20.10)'), '        max_yfine = ' ,max_yfine
	write(11, '(a20, f20.10)'), '    curv_yfineneg = ' ,curv_yfineneg
	write(11, '(a20, f20.10)'), '    curv_yfinepos = ' ,curv_yfinepos
	write(11, '(a20, f20.10)'), '       dyfine_neg = ' ,dyfine_neg
	write(11, '(a20, f20.10)'), '       dyfine_pos = ' ,dyfine_pos
	write(11, '(a1)'), ' '
	write(11, '(a50)'), '----------------------Bonds-----------------------'
	write(11, '(a1)'), ' '
	write(11, '(a20, i10)'), '               nb = ' ,nb
	write(11, '(a20, f20.10)'), '            max_b = ' ,max_b
	write(11, '(a20, f20.10)'), '           curv_b = ' ,curv_b
	write(11, '(a20, f20.10)'), '               db = ' ,db
	write(11, '(a1)'), ' '
	write(11, '(a50)'), '---------Housing and apartment parameters---------'
	write(11, '(a1)'), ' '
	write(11, '(a20, i10)'), '               nh = ' ,nh
	write(11, '(a20, f20.10)'), '            min_h = ' ,min_h
	write(11, '(a20, f20.10)'), '            max_h = ' ,max_h
	write(11, '(a20, f20.10)'), '           curv_h = ' ,curv_h
	write(11, '(a20, f20.10)'), '               dh = ' ,dh
	write(11, '(a20, f20.10)'), '       hincrement = ' ,hincrement
	write(11, '(a20, i10)'), '               nu = ' ,nu
	write(11, '(a20, i10)'), '               ng = ' ,ng
	write(11, '(a20, f20.10)'), '        useparate = ' ,useparate
	write(11, '(a20, f20.10)'), '        pseparate = ' ,pseparate
	write(11, '(a20, f20.10)'), '          utilown = ' ,utilown
	write(11, '(a20, f20.10)'), '            max_a = ' ,max_a
	write(11, '(a20, f20.10)'), '        rentratio = ' ,rentratio*4.d0
	write(11, '(a1)'), ' '
	write(11, '(a50)'), '------------------Housing search------------------'
	write(11, '(a1)'), ' '
	write(11, '(a20, i10)'), '               nx = ' ,nx
	write(11, '(a20, f20.10)'), '           curv_x = ' ,curv_x
	write(11, '(a20, f20.10)'), '           phbase = ' ,phbase
	write(11, '(a20, f20.10)'), '           min_xs = ' ,min_xs
	write(11, '(a20, f20.10)'), '           max_xs = ' ,max_xs
	write(11, '(a20, f20.10)'), '           min_xb = ' ,min_xb
	write(11, '(a20, f20.10)'), '           max_xb = ' ,max_xb
	write(11, '(a20, f20.10)'), '               dx = ' ,dx
	write(11, '(a20, f20.10)'), '          curv_ms = ' ,curv_ms
	write(11, '(a20, f20.10)'), '          curv_mb = ' ,curv_mb
	write(11, '(a20, f20.10)'), '       searchcost = ' ,searchcost
	write(11, '(a1)'), ' '
	write(11, '(a50)'), '----------Mortgage pseudo-leverage ratios----------'
	write(11, '(a1)'), ' '
	write(11, '(a20, i10)'), '               nm = ' ,nm
	write(11, '(a20, f20.10)'), '		   max_ltv = ' ,max_ltv
	write(11, '(a20, f20.10)'), '            max_m = ' ,max_m/max_ph
	write(11, '(a20, f20.10)'), '           curv_m = ' ,curv_m
	write(11, '(a20, f20.10)'), '               dm = ' ,dm
	write(11, '(a1)'), ' '
	write(11, '(a50)'), '-----------Foreclosure and REO parameters----------'
	write(11, '(a1)'), ' '
	write(11, '(a20, f20.10)'), '            pflag = ' ,pflag
	write(11, '(a20, f20.10)'), '         fincloss = ' ,fincloss
	write(11, '(a20, f20.10)'), '      fdeficiency = ' ,fdeficiency
	write(11, '(a20, f20.10)'), '        fdiscount = ' ,fdiscount
	write(11, '(a20, f20.10)'), '          psearch = ' ,psearch
	write(11, '(a1)'), ' '
	write(11, '(a50)'), '---------------Preference parameters---------------'
	write(11, '(a1)'), ' '
	write(11, '(a20, f20.10)'), '             beta = ' ,beta
	write(11, '(a20, f20.10)'), '            gamma = ' ,gamma
	write(11, '(a20, f20.10)'), '              eis = ' ,eis
	write(11, '(a20, f20.10)'), '            theta = ' ,theta
	write(11, '(a20, f20.10)'), '              ies = ' ,ies
	write(11, '(a20, f20.10)'), '           cshare = ' ,cshare0
	write(11, '(a20, f20.10)'), '        min_spend = ' ,min_spend
	write(11, '(a1)'), ' '
	write(11, '(a50)'), '-----------Labor productivity parameters-----------'
	write(11, '(a1)'), ' '
	write(11, '(a20, i10)'), '               ns = ' ,ns
	write(11, '(a20, f20.10)'), '           mean_s = ' ,mean_s
	write(11, '(a20, f20.10)'), '            rho_s = ' ,rho_s
	write(11, '(a20, f20.10)'), '          sig_s_e = ' ,sig_s_e
	write(11, '(a20, f20.10)'), '          sig_s_r = ' ,sig_s_r
	write(11, '(a20, f20.10)'), '        sig_s_avg = ' ,sig_s_avg
	write(11, '(a20, f20.10)'), '          cover_s = ' ,cover_s
	write(11, '(a20, f20.10)'), '           s_rich = ' ,s_rich
	write(11, '(a20, f20.10)'), '     prob_getrich = ' ,prob_getrich
	write(11, '(a20, f20.10)'), '    prob_stayrich = ' ,prob_stayrich
	write(11, '(a20, i10)'), '               ne = ' ,ne
	write(11, '(a20, f20.10)'), '            sig_e = ' ,sig_e
	write(11, '(a20, f20.10)'), '          cover_e = ' ,cover_e
	write(11, '(a1)'), ' '
	write(11, '(a50)'), '--------------------Depreciation-------------------'
	write(11, '(a1)'), ' '
	write(11, '(a20, f20.10)'), '         deprec_k = ' ,deprec_k*4.d0
	write(11, '(a20, f20.10)'), '         deprec_h = ' ,deprec_h*4.d0
	write(11, '(a1)'), ' '
	write(11, '(a50)'), '----------Production function parameters-----------'
	write(11, '(a1)'), ' '
	write(11, '(a20, f20.10)'), '         kshare_c = ' ,kshare_c
	write(11, '(a20, f20.10)'), '         kshare_h = ' ,kshare_h
	write(11, '(a20, f20.10)'), '           lshare = ' ,lshare
	write(11, '(a1)'), ' '
	write(11, '(a50)'), '-------------------Mortgage costs------------------'
	write(11, '(a1)'), ' '
	write(11, '(a20, f20.10)'), '         rmspread = ' ,rmspread
	write(11, '(a20, f20.10)'), '           rocost = ' ,rocost
	write(11, '(a20, f20.10)'), '           rscost = ' ,rscost
	write(11, '(a1)'), ' '
	write(11, '(a50)'), '-----------------Government policy-----------------'
	write(11, '(a1)'), ' '
	write(11, '(a20, f20.10)'), '         sellcost = ' ,sellcost
	write(11, '(a1)'), ' '
	write(11, '(a50)'), '--------------Shadow price of housing--------------'
	write(11, '(a1)'), ' '
	write(11, '(a20, i10)'), '              nph = ' ,nph
	write(11, '(a20, f20.10)'), '           min_ph = ' ,min_ph
	write(11, '(a20, f20.10)'), '           max_ph = ' ,max_ph
	write(11, '(a20, f20.10)'), '          curv_ph = ' ,curv_ph
	write(11, '(a20, f20.10)'), '              dph = ' ,dph
	write(11, '(a1)'), ' '
	write(11, '(a50)'), '-----------------Aggregate capital-----------------'
	write(11, '(a1)'), ' '
	write(11, '(a20, i10)'), '               nk = ' ,nk
	write(11, '(a20, f20.10)'), '            min_k = ' ,min_k
	write(11, '(a20, f20.10)'), '            max_k = ' ,max_k
	write(11, '(a20, f20.10)'), '           curv_k = ' ,curv_k
	write(11, '(a20, f20.10)'), '               dk = ' ,dk
	write(11, '(a1)'), ' '
	write(11, '(a50)'), '-----Number of aggregate interpolation points------'
	write(11, '(a1)'), ' '
	write(11, '(a20, i10)'), '          ninterp = ' ,ninterp
	write(11, '(a1)'), ' '
	write(11, '(a50)'), '----Number of endogenous aggregate grid points-----'
	write(11, '(a1)'), ' '
	write(11, '(a20, i10)'), '             nagg = ' ,nagg
	write(11, '(a1)'), ' '
	write(11, '(a50)'), '---------TFP of composite good production----------'
	write(11, '(a1)'), ' '
	write(11, '(a20, i10)'), '              nzc = ' ,nzc
	write(11, '(a20, f20.10)'), '        zc_steady = ' ,zc_steady
	write(11, '(a20, f20.10)'), '           sig_zc = ' ,sig_zc
	write(11, '(a20, f20.10)'), '           rho_zc = ' ,rho_zc
	write(11, '(a20, f20.10)'), '         cover_zc = ' ,cover_zc
	write(11, '(a1)'), ' '
	write(11, '(a50)'), '------------TFP of housing construction------------'
	write(11, '(a1)'), ' '
	write(11, '(a20, i10)'), '              nzh = ' ,nzh
	write(11, '(a20, f20.10)'), '        zh_steady = ' ,zh_steady
	write(11, '(a20, f20.10)'), '           sig_zh = ' ,sig_zh
	write(11, '(a20, f20.10)'), '           rho_zh = ' ,rho_zh
	write(11, '(a20, f20.10)'), '         cover_zh = ' ,cover_zh
	write(11, '(a1)'), ' '
	write(11, '(a50)'), '--------Aggregate housing preference shocks--------'
	write(11, '(a1)'), ' '
	write(11, '(a20, i10)'), '              nhu = ' ,nhu
	write(11, '(a20, f20.10)'), '           sig_hu = ' ,sig_hu
	write(11, '(a20, f20.10)'), '           rho_hu = ' ,rho_hu
	write(11, '(a20, f20.10)'), '         cover_hu = ' ,cover_hu
	write(11, '(a1)'), ' '
	write(11, '(a50)'), '---------Total number of aggregate shocks----------'
	write(11, '(a1)'), ' '
	write(11, '(a20, i10)'), '               nz = ' ,nz
	close(11)

end subroutine sub_params