subroutine sub_simulate(it,genseries,iz,izprime,kstock,assets,phe0,phe,kstockprime,assetsprime,taxrate,yh,yc,ownrate,htot)

	use mod_globvar
	use omp_lib
	use mod_interpolate

	implicit none

	!This program computes the equilibrium price of housing and then updates the distribution of agents over (y',k',g',s')
	
	integer, intent(in) :: it, genseries, iz, izprime
	real(8), intent(in) :: kstock, assets, phe0
	real(8), intent(out) :: phe, kstockprime, assetsprime, taxrate, yh, yc, ownrate, htot
	
	!Indices for today's state
	integer :: is, ig, ih, iu, im, iy, ie
	!Indices for tomorrow's state
	integer :: igshock, isprime, ieprime, iztomorrow
	
	!Beginning of period distributions
	real(8), dimension(:,:,:,:), allocatable :: mu1own_f0_orig
	real(8), dimension(:,:,:), allocatable :: mu1own_f1_orig
	real(8), dimension(:,:), allocatable :: mu1rent_f0_orig, mu1rent_f1_orig
	
	!Temporary distributions for parallelization
	real(8), dimension(:,:,:,:,:), allocatable :: mu1own_f0_temp, mu2own_f0_temp, mu3own_f0_temp
	real(8), dimension(:,:,:,:), allocatable :: mu1own_f1_temp, mu2own_f1_temp, mu3own_f1_temp
	real(8), dimension(:,:,:), allocatable :: mu1rent_f0_temp, mu1rent_f1_temp, mu2rent_f0_temp, mu2rent_f1_temp

	!Other internal variables
	real(8) :: min_dphe, toler_excessdemand, pctdiff, qbe, ysell, ydefault, ypay, ybuy, yprime_f0, yprime_f1, supply, demand, excess_demand1, excess_demand0, dphe, kprime_forecast, &
	meas_agents, yweight, y0weight, y1weight, w, rk, w_z, rk_expect, REOrevenue, phprime_forecast, invest_h, pl, kh, lh, cons, hcons, agg_xs, agg_xb, mortrev, mortcost, svcorigcost, &
	phe_prev, maxrev, ynew
	integer :: iter, id, max_iter, impay, ihbuy, igbuy, imprime, sign_excessdemand1, sign_excessdemand0, iylow, iyupp, iy0low, iy0upp, iy1low, iy1upp, iinterp, exitloop
	real(8), dimension(3) :: aggstate_vec
	real(8), dimension(nz) :: rk_z
	real(8), dimension(num_threads_simul) :: supply_temp, agg_xs_temp, REOrevenue_temp, mortrev_temp, mortcost_temp, svcorigcost_temp, assetsprime_temp, cons_temp, hcons_temp
	real(8), dimension(nh,num_threads_simul) :: htot_reo_temp
	real(8), dimension(nh) :: htot_reo_new
	real(8), dimension(ninterp) :: wagg
	integer, dimension(ninterp) :: iagg
	
	!Allocate distributions for parallelization
	allocate(mu1own_f0_temp(ny,nm,ng,ns,num_threads_simul))
	allocate(mu2own_f0_temp(ny,nm,ng,ns,num_threads_simul))
	allocate(mu3own_f0_temp(ny,nm,ng,ns,num_threads_simul))
	allocate(mu1own_f1_temp(ny,ng,ns,num_threads_simul))
	allocate(mu2own_f1_temp(ny,ng,ns,num_threads_simul))
	allocate(mu3own_f1_temp(ny,ng,ns,num_threads_simul))
	allocate(mu1rent_f0_temp(ny,ns,num_threads_simul))
	allocate(mu2rent_f0_temp(ny,ns,num_threads_simul))
	allocate(mu1rent_f1_temp(ny,ns,num_threads_simul))
	allocate(mu2rent_f1_temp(ny,ns,num_threads_simul))
	
	!Allocate beginning of period distributions
	allocate(mu1rent_f0_orig(ny,ns))
	allocate(mu1rent_f1_orig(ny,ns))
	allocate(mu1own_f1_orig(ny,ng,ns))
	allocate(mu1own_f0_orig(ny,nm,ng,ns))

	!Initialize
	min_dphe           = 1e-6
	toler_excessdemand = 1e-4
	phe                = phe0
	max_iter	       = 100
	taxrate            = 0.d0
	maxrev             = 0.d0
	do is = 1, ns
		do iy = ny_neg + 1, ny
			maxrev = maxrev + yy(iy)*(mu1rent_f0(iy,is) + mu1rent_f1(iy,is))
		enddo
		do ig = 1, ng
			do iy = ny_neg + 1, ny
				maxrev = maxrev + yy(iy)*mu1own_f1(iy,ig,is)
			enddo
			do im = 1, nm
				do iy = ny_neg + 1, ny
					maxrev = maxrev + yy(iy)*mu1own_f0(iy,im,ig,is)
				enddo
			enddo
		enddo
	enddo
	mu1rent_f0_orig = mu1rent_f0
	mu1rent_f1_orig = mu1rent_f1
	mu1own_f0_orig  = mu1own_f0
	mu1own_f1_orig  = mu1own_f1
	!Find market clearing ph (using bisection method) and find tomorrow's distribution of agents
	do iter = 1, max_iter
		!Tax household wealth to cover intermediary losses
		mu1rent_f0 = 0.d0
		mu1rent_f1 = 0.d0
		mu1own_f0  = 0.d0
		mu1own_f1  = 0.d0
		do is = 1, ns
			do iy = ny_neg + 1, ny
				ynew    = yy(iy)*(1 - taxrate)
				iylow   = ly(ynew)
				iyupp   = uy(ynew)
				yweight = wy(ynew)
				mu1rent_f0(iylow,is) = mu1rent_f0(iylow,is) + (1 - yweight)*mu1rent_f0_orig(iy,is)
				mu1rent_f0(iyupp,is) = mu1rent_f0(iyupp,is) + yweight*mu1rent_f0_orig(iy,is)
				mu1rent_f1(iylow,is) = mu1rent_f1(iylow,is) + (1 - yweight)*mu1rent_f1_orig(iy,is)
				mu1rent_f1(iyupp,is) = mu1rent_f1(iyupp,is) + yweight*mu1rent_f1_orig(iy,is)
			enddo
			do ig = 1, ng
				do iy = ny_neg + 1, ny
					ynew    = yy(iy)*(1 - taxrate)
					iylow   = ly(ynew)
					iyupp   = uy(ynew)
					yweight = wy(ynew)
					mu1own_f1(iylow,ig,is) = mu1own_f1(iylow,ig,is) + (1 - yweight)*mu1own_f1_orig(iy,ig,is)
					mu1own_f1(iyupp,ig,is) = mu1own_f1(iyupp,ig,is) + yweight*mu1own_f1_orig(iy,ig,is)
				enddo
				do im = 1, nm
					do iy = ny_neg + 1, ny
						ynew    = yy(iy)*(1 - taxrate)
						iylow   = ly(ynew)
						iyupp   = uy(ynew)
						yweight = wy(ynew)
						mu1own_f0(iylow,im,ig,is) = mu1own_f0(iylow,im,ig,is) + (1 - yweight)*mu1own_f0_orig(iy,im,ig,is)
						mu1own_f0(iyupp,im,ig,is) = mu1own_f0(iyupp,im,ig,is) + yweight*mu1own_f0_orig(iy,im,ig,is)
					enddo
				enddo
			enddo
		enddo
		!Aggregate state
		iagg(1) = aggfun(lp(phe),lk(kstock))
		iagg(2) = aggfun(up(phe),lk(kstock))
		iagg(3) = aggfun(lp(phe),uk(kstock))
		iagg(4) = aggfun(up(phe),uk(kstock))
		wagg(1) = (1 - wp(phe))*(1 - wk(kstock))
		wagg(2) = wp(phe)*(1 - wk(kstock))
		wagg(3) = (1 - wp(phe))*wk(kstock)
		wagg(4) = wp(phe)*wk(kstock)
		!Find bond price that satisfies no arbitrage condition for banks
		aggstate_vec(1)  = 1.d0
		aggstate_vec(2)  = phe
		aggstate_vec(3)  = kstock
		do iztomorrow = 1, nz
			phprime_forecast = dot_product(forecast_coeff(1,:,iz,iztomorrow),aggstate_vec)
			kprime_forecast  = dot_product(forecast_coeff(2,:,iz,iztomorrow),aggstate_vec)
			call sub_factorprices_sh(kprime_forecast,eavg,phprime_forecast,zc(iztomorrow),zh(iztomorrow),w_z,rk_z(iztomorrow))	!NOTE: this isn't robust to fluctuations in aggregate labor
		enddo
		rk_expect = dot_product(rk_z,prob_zz(iz,:))
		qbe       = 1/(1 + rk_expect - deprec_k)
		!Compute factor prices and output
		call sub_factorprices_sh(kstock,eavg,phe,zc(iz),zh(iz),w,rk)
		call sub_output_sh(kstock,eavg,phe,zc(iz),zh(iz),yc,yh)
		!Compute supply and demand of housing
		!Supply of new housing
		supply = yh
		!Supply of existing homes and new REOs
		supply_temp   = 0.d0
		htot_reo_new  = 0.d0
		htot_reo_temp = 0.d0
		do is = 1, ns
			!Homeowners
			call omp_set_num_threads(num_threads_simul)
			!$omp parallel private(id,ih,iy)
			!$omp do
			do ig = 1, ng
				id = omp_get_thread_num() + 1
				ih = hfun(ig)
				do iy = ny_neg + 1, ny
					!Homeowners with bad credit
					supply_temp(id) = supply_temp(id) + dot_product(wagg,psell_f1(iy,ig,is,iz,iagg))*hh(ih)*mu1own_f1(iy,ig,is)
					!Homeowners with good credit but no mortgage
					supply_temp(id) = supply_temp(id) + dot_product(wagg,psell_f0(iy,1,ig,is,iz,iagg))*hh(ih)*mu1own_f0(iy,1,ig,is)
				enddo
			enddo
			!$omp end do
			!$omp end parallel
			do ig = 1, ng
				ih = hfun(ig)
				!Homeowners with a mortgage
				call omp_set_num_threads(num_threads_simul)
				!$omp parallel private(id,iy)
				!$omp do
				do im = 2, nm
					id = omp_get_thread_num() + 1
					do iy = ny_neg + 1, ny
						!Owners who sell voluntarily
						supply_temp(id) = supply_temp(id) + dot_product(wagg,psell_f0(iy,im,ig,is,iz,iagg))*hh(ih)*mu1own_f0(iy,im,ig,is)
						!Owners who default
						supply_temp(id) = supply_temp(id) + dot_product(wagg,(1 - psell_f0(iy,im,ig,is,iz,iagg))&
						*d_f0(iy,im,ig,is,iz,iagg)*psell_reo(ih,iz,iagg))*hh(ih)*mu1own_f0(iy,im,ig,is)
						htot_reo_temp(ih,id) = htot_reo_temp(ih,id) + dot_product(wagg,(1 - psell_f0(iy,im,ig,is,iz,iagg))&
						*d_f0(iy,im,ig,is,iz,iagg))*mu1own_f0(iy,im,ig,is)
					enddo
				enddo
				!$omp end do
				!$omp end parallel
			enddo
		enddo
		do id = 1, num_threads_simul
			supply       = supply + supply_temp(id)
			htot_reo_new = htot_reo_new + htot_reo_temp(:,id)
		enddo
		!Supply of housing from existing REOs
		do ih = 1, nh
			supply = supply + dot_product(wagg,psell_reo(ih,iz,iagg))*hh(ih)*htot_reo(ih)
		enddo
		!Transition from subperiod 1 to subperiod 2
		mu2rent_f0      = 0.d0
		mu2rent_f1      = 0.d0
		mu2own_f0       = 0.d0
		mu2own_f1       = 0.d0
		mu2rent_f0_temp = 0.d0
		mu2rent_f1_temp = 0.d0
		mu2own_f0_temp  = 0.d0
		mu2own_f1_temp  = 0.d0
		do is = 1, ns
			!Renters
			do iy = ny_neg + 1, ny
				!Renters with good credit
				mu2rent_f0(iy,is) = mu2rent_f0(iy,is) + mu1rent_f0(iy,is)
				!Renters with bad credit
				mu2rent_f1(iy,is) = mu2rent_f1(iy,is) + mu1rent_f1(iy,is)
			enddo
			!Homeowners
			call omp_set_num_threads(num_threads_simul)
			!$omp parallel private(id,ih,iy,iinterp,ysell,iylow,iyupp,yweight)
			!$omp do
			do ig = 1, ng
				id = omp_get_thread_num() + 1
				ih = hfun(ig)
				do iy = ny_neg + 1, ny
					do iinterp = 1, ninterp
						!Homeowners with bad credit
						!Successful sale
						ysell   = yy(iy) + xs_f1(iy,ig,is,iz,iagg(iinterp))*(1 - sellcost)
						iylow   = ly(ysell)
						iyupp   = uy(ysell)
						yweight = wy(ysell)
						mu2rent_f1_temp(iylow,is,id) = mu2rent_f1_temp(iylow,is,id) + wagg(iinterp)*(1 - yweight)&
						*psell_f1(iy,ig,is,iz,iagg(iinterp))*mu1own_f1(iy,ig,is)
						mu2rent_f1_temp(iyupp,is,id) = mu2rent_f1_temp(iyupp,is,id) + wagg(iinterp)*yweight&
						*psell_f1(iy,ig,is,iz,iagg(iinterp))*mu1own_f1(iy,ig,is)
						!No sale
						mu2own_f1_temp(iy,ig,is,id) = mu2own_f1_temp(iy,ig,is,id) + wagg(iinterp)*(1 - psell_f1(iy,ig,is,iz,iagg(iinterp)))&
						*mu1own_f1(iy,ig,is)
						!Homeowners with good credit but no mortgage
						!Successful sale
						ysell   = yy(iy) + xs_f0(iy,1,ig,is,iz,iagg(iinterp))*(1 - sellcost)
						iylow   = ly(ysell)
						iyupp   = uy(ysell)
						yweight = wy(ysell)
						mu2rent_f0_temp(iylow,is,id) = mu2rent_f0_temp(iylow,is,id) + wagg(iinterp)*(1 - yweight)&
						*psell_f0(iy,1,ig,is,iz,iagg(iinterp))*mu1own_f0(iy,1,ig,is)
						mu2rent_f0_temp(iyupp,is,id) = mu2rent_f0_temp(iyupp,is,id) + wagg(iinterp)*yweight&
						*psell_f0(iy,1,ig,is,iz,iagg(iinterp))*mu1own_f0(iy,1,ig,is)
						!No sale
						mu2own_f0_temp(iy,1,ig,is,id) = mu2own_f0_temp(iy,1,ig,is,id) + wagg(iinterp)&
						*(1 - psell_f0(iy,1,ig,is,iz,iagg(iinterp)))*mu1own_f0(iy,1,ig,is)
					enddo
				enddo
			enddo
			!$omp end do
			!$omp end parallel
			do ig = 1, ng
				ih = hfun(ig)
				!Homeowners with a mortgage
				call omp_set_num_threads(num_threads_simul)
				!$omp parallel private(id,iy,iinterp,ysell,iylow,iyupp,yweight,ydefault,impay,ypay)
				!$omp do
				do im = 2, nm
					id = omp_get_thread_num() + 1
					do iy = ny_neg + 1, ny
						do iinterp = 1, ninterp
							!Successful sale
							ysell   = yy(iy) - mm(im)*hh(ih) + xs_f0(iy,im,ig,is,iz,iagg(iinterp))*(1 - sellcost)
							iylow   = ly(ysell)
							iyupp   = uy(ysell)
							yweight = wy(ysell)
							mu2rent_f0_temp(iylow,is,id) = mu2rent_f0_temp(iylow,is,id) + wagg(iinterp)*(1 - yweight)&
							*psell_f0(iy,im,ig,is,iz,iagg(iinterp))*mu1own_f0(iy,im,ig,is)
							mu2rent_f0_temp(iyupp,is,id) = mu2rent_f0_temp(iyupp,is,id) + wagg(iinterp)*yweight&
							*psell_f0(iy,im,ig,is,iz,iagg(iinterp))*mu1own_f0(iy,im,ig,is)
							!No sale, default
							ydefault = (1 - fdeficiency)*yy(iy) + max(0.d0,Jreo(ih,iz,iagg(iinterp)) - mm(im)*hh(ih))
							iylow    = ly(ydefault)
							iyupp    = uy(ydefault)
							yweight  = wy(ydefault)
							mu2rent_f1_temp(iylow,is,id) = mu2rent_f1_temp(iylow,is,id) + wagg(iinterp)*(1 - yweight)&
							*(1 - psell_f0(iy,im,ig,is,iz,iagg(iinterp)))*d_f0(iy,im,ig,is,iz,iagg(iinterp))*mu1own_f0(iy,im,ig,is)
							mu2rent_f1_temp(iyupp,is,id) = mu2rent_f1_temp(iyupp,is,id) + wagg(iinterp)*yweight&
							*(1 - psell_f0(iy,im,ig,is,iz,iagg(iinterp)))*d_f0(iy,im,ig,is,iz,iagg(iinterp))*mu1own_f0(iy,im,ig,is)
							!No sale, make payment
							impay    = locm_f0(iy,im,ig,is,iz,iagg(iinterp))
							ypay     = yy(iy) - (mm(im) - qm(iz,iagg(iinterp))*m_f0(iy,im,ig,is,iz,iagg(iinterp)))*hh(ih)
							if (ypay < 0.d0) then
								ypay = ypay/abs(ylb(ig,is,iz,iagg(iinterp)))
							endif
							iylow   = ly(ypay)
							iyupp   = uy(ypay)
							yweight = wy(ypay)
							mu2own_f0_temp(iylow,impay,ig,is,id) = mu2own_f0_temp(iylow,impay,ig,is,id) + wagg(iinterp)*(1 - yweight)&
							*(1 - psell_f0(iy,im,ig,is,iz,iagg(iinterp)))*(1 - d_f0(iy,im,ig,is,iz,iagg(iinterp)))*mu1own_f0(iy,im,ig,is)
							mu2own_f0_temp(iyupp,impay,ig,is,id) = mu2own_f0_temp(iyupp,impay,ig,is,id) + wagg(iinterp)*yweight&
							*(1 - psell_f0(iy,im,ig,is,iz,iagg(iinterp)))*(1 - d_f0(iy,im,ig,is,iz,iagg(iinterp)))*mu1own_f0(iy,im,ig,is)
						enddo
					enddo
				enddo
				!$omp end do
				!$omp end parallel
			enddo
		enddo
		do id = 1, num_threads_simul
			mu2rent_f0 = mu2rent_f0 + mu2rent_f0_temp(:,:,id)
			mu2rent_f1 = mu2rent_f1 + mu2rent_f1_temp(:,:,id)
			mu2own_f0  = mu2own_f0 + mu2own_f0_temp(:,:,:,:,id)
			mu2own_f1  = mu2own_f1 + mu2own_f1_temp(:,:,:,id)
		enddo
		!Total demand for housing
		demand = 0.d0
		do is = 1, ns
			do iy = ny_neg + 1, ny
				!Renters with good credit
				demand = demand + dot_product(wagg,pbuy_f0(iy,is,iz,iagg)*h_f0(iy,is,iz,iagg))*mu2rent_f0(iy,is)
				!Renters with bad credit
				demand = demand + dot_product(wagg,pbuy_f1(iy,is,iz,iagg)*h_f1(iy,is,iz,iagg))*mu2rent_f1(iy,is)
			enddo
		enddo
		!Transition from subperiod 2 to subperiod 3
		mu3rent_f0 = 0.d0
		mu3rent_f1 = 0.d0
		mu3own_f0  = 0.d0
		mu3own_f1  = 0.d0
		do is = 1, ns
			!Renters
			do iy = ny_neg + 1, ny
				do iinterp = 1, ninterp
					!Renters with good credit
					ihbuy = loch_f0(iy,is,iz,iagg(iinterp))
					igbuy = gfun(ihbuy,1)
					ybuy  = yy(iy) - xb_f0(iy,is,iz,iagg(iinterp))
					if (ybuy < 0.d0) then
						ybuy = ybuy/abs(ylb(igbuy,is,iz,iagg(iinterp)))
					endif
					iylow   = ly(ybuy)
					iyupp   = uy(ybuy)
					yweight = wy(ybuy)
					!Successful purchase
					mu3own_f0(iylow,1,igbuy,is) = mu3own_f0(iylow,1,igbuy,is) + wagg(iinterp)*(1 - yweight)&
					*pbuy_f0(iy,is,iz,iagg(iinterp))*mu2rent_f0(iy,is)
					mu3own_f0(iyupp,1,igbuy,is) = mu3own_f0(iyupp,1,igbuy,is) + wagg(iinterp)*yweight*pbuy_f0(iy,is,iz,iagg(iinterp))&
					*mu2rent_f0(iy,is)
					!No purchase
					mu3rent_f0(iy,is) = mu3rent_f0(iy,is) + wagg(iinterp)*(1 - pbuy_f0(iy,is,iz,iagg(iinterp)))*mu2rent_f0(iy,is)
					!Renters with bad credit
					ihbuy   = loch_f1(iy,is,iz,iagg(iinterp))
					igbuy   = gfun(ihbuy,1)
					ybuy    = yy(iy) - xb_f1(iy,is,iz,iagg(iinterp))
					iylow   = ly(ybuy)
					iyupp   = uy(ybuy)
					yweight = wy(ybuy)
					!Successful purchase
					mu3own_f1(iylow,igbuy,is) = mu3own_f1(iylow,igbuy,is) + wagg(iinterp)*(1 - yweight)*pbuy_f1(iy,is,iz,iagg(iinterp))&
					*mu2rent_f1(iy,is)
					mu3own_f1(iyupp,igbuy,is) = mu3own_f1(iyupp,igbuy,is) + wagg(iinterp)*yweight*pbuy_f1(iy,is,iz,iagg(iinterp))&
					*mu2rent_f1(iy,is)
					!No purchase
					mu3rent_f1(iy,is) = mu3rent_f1(iy,is) + wagg(iinterp)*(1 - pbuy_f1(iy,is,iz,iagg(iinterp)))*mu2rent_f1(iy,is)
				enddo
			enddo
		enddo
		!Homeowners
		mu3own_f1 = mu3own_f1 + mu2own_f1
		mu3own_f0 = mu3own_f0 + mu2own_f0
		!Calculate residential investment and housing production inputs
		agg_xb           = 0.d0
		agg_xs           = 0.d0
		REOrevenue	     = 0.d0
		mortrev          = 0.d0
		mortcost         = 0.d0
		svcorigcost      = 0.d0
		assetsprime      = 0.d0
		cons             = 0.d0
		hcons		     = 0.d0
		agg_xs_temp      = 0.d0
		REOrevenue_temp  = 0.d0
		mortrev_temp     = 0.d0
		mortcost_temp    = 0.d0
		svcorigcost_temp = 0.d0
		assetsprime_temp = 0.d0
		cons_temp        = 0.d0
		hcons_temp       = 0.d0
		do is = 1, ns
			!!!Subperiod 1!!!
			!Homeowners
			call omp_set_num_threads(num_threads_simul)
			!$omp parallel private(id,ih,iy)
			!$omp do
			do ig = 1, ng
				id = omp_get_thread_num() + 1
				ih = hfun(ig)
				do iy = ny_neg + 1, ny
					!Homeowners with bad credit
					agg_xs_temp(id) = agg_xs_temp(id) + dot_product(wagg,xs_f1(iy,ig,is,iz,iagg)*psell_f1(iy,ig,is,iz,iagg))*mu1own_f1(iy,ig,is)
					!Homeowners with good credit and no mortgage
					agg_xs_temp(id) = agg_xs_temp(id) + dot_product(wagg,xs_f0(iy,1,ig,is,iz,iagg)*psell_f0(iy,1,ig,is,iz,iagg))*mu1own_f0(iy,1,ig,is)
				enddo
			enddo
			!$omp end do
			!$omp end parallel
			!Homeowners with good credit and a mortgage
			do ig = 1, ng
				ih = hfun(ig)
				call omp_set_num_threads(num_threads_simul)
				!$omp parallel private(id,iy)
				!$omp do
				do im = 2, nm
					id = omp_get_thread_num() + 1
					do iy = ny_neg + 1, ny
						!Homeowners who sell
						mortrev_temp(id) = mortrev_temp(id) + dot_product(wagg,mm(im)*hh(ih)*psell_f0(iy,im,ig,is,iz,iagg))*mu1own_f0(iy,im,ig,is)
						agg_xs_temp(id)  = agg_xs_temp(id) + dot_product(wagg,xs_f0(iy,im,ig,is,iz,iagg)*psell_f0(iy,im,ig,is,iz,iagg))*mu1own_f0(iy,im,ig,is)
						!Homeowners who default
						mortrev_temp(id)  = mortrev_temp(id) + dot_product(wagg,min(fdeficiency*yy(iy),mm(im)*hh(ih) - Jreo(ih,iz,iagg))*(1 - psell_f0(iy,im,ig,is,iz,iagg))&
						*d_f0(iy,im,ig,is,iz,iagg))*mu1own_f0(iy,im,ig,is)
						mortrev_temp(id)  = mortrev_temp(id) + dot_product(wagg,Jreo(ih,iz,iagg)*(1 - psell_f0(iy,im,ig,is,iz,iagg))*d_f0(iy,im,ig,is,iz,iagg))*mu1own_f0(iy,im,ig,is)
						mortcost_temp(id) = mortcost_temp(id) + dot_product(wagg,(Jreo(ih,iz,iagg) - psell_reo(ih,iz,iagg)*xs_reo(ih,iz,iagg))*(1 - psell_f0(iy,im,ig,is,iz,iagg))&
						*d_f0(iy,im,ig,is,iz,iagg))*mu1own_f0(iy,im,ig,is)
						!Homeowners who make a payment
						mortrev_temp(id)     = mortrev_temp(id) + dot_product(wagg,(mm(im) - (1 + rscost)*qm(iz,iagg)*m_f0(iy,im,ig,is,iz,iagg))*hh(ih)*(1 - psell_f0(iy,im,ig,is,iz,iagg))&
						*(1 - d_f0(iy,im,ig,is,iz,iagg)))*mu1own_f0(iy,im,ig,is)
						svcorigcost_temp(id) = svcorigcost_temp(id) + dot_product(wagg,rscost*qm(iz,iagg)*m_f0(iy,im,ig,is,iz,iagg)*hh(ih)*(1 - psell_f0(iy,im,ig,is,iz,iagg))&
						*(1 - d_f0(iy,im,ig,is,iz,iagg)))*mu1own_f0(iy,im,ig,is)
						!Revenue from sales of new REOs
						REOrevenue_temp(id) = REOrevenue_temp(id) + dot_product(wagg,xs_reo(ih,iz,iagg)*(1 - psell_f0(iy,im,ig,is,iz,iagg))*d_f0(iy,im,ig,is,iz,iagg)*psell_reo(ih,iz,iagg))&
						*mu1own_f0(iy,im,ig,is)
					enddo
				enddo
				!$omp end do
				!$omp end parallel
			enddo
			!!!Subperiod 2!!!
			do iy = ny_neg + 1, ny
				agg_xb = agg_xb + dot_product(wagg,xb_f0(iy,is,iz,iagg)*pbuy_f0(iy,is,iz,iagg))*mu2rent_f0(iy,is)
				agg_xb = agg_xb + dot_product(wagg,xb_f1(iy,is,iz,iagg)*pbuy_f1(iy,is,iz,iagg))*mu2rent_f1(iy,is)
			enddo
			!!!Subperiod 3!!!
			!Renters
			do iy = ny_neg + 1, ny
				!Renters with good credit
				cons        = cons + dot_product(wagg,crent_f0(iy,is,iz,iagg))*mu3rent_f0(iy,is)
				hcons       = hcons + dot_product(wagg,hcrent_f0(iy,is,iz,iagg))*mu3rent_f0(iy,is)
				assetsprime = assetsprime + dot_product(wagg,brent_f0(iy,is,iz,iagg))*mu3rent_f0(iy,is)
				!Renters with bad credit
				cons        = cons + dot_product(wagg,crent_f1(iy,is,iz,iagg))*mu3rent_f1(iy,is)
				hcons       = hcons + dot_product(wagg,hcrent_f1(iy,is,iz,iagg))*mu3rent_f1(iy,is)
				assetsprime = assetsprime + dot_product(wagg,brent_f1(iy,is,iz,iagg))*mu3rent_f1(iy,is)
			enddo
			!Homeowners
			call omp_set_num_threads(num_threads_simul)
			!$omp parallel private(id,ih,iy,iinterp)
			!$omp do
			do ig = 1, ng
				id = omp_get_thread_num() + 1
				ih = hfun(ig)
				!Homeowners with bad credit
				do iy = ny_neg + 1, ny
					cons_temp(id)        = cons_temp(id) + dot_product(wagg,cown_f1(iy,ig,is,iz,iagg))*mu3own_f1(iy,ig,is)
					hcons_temp(id)       = hcons_temp(id) + dot_product(wagg,hcown_f1(iy,ig,is,iz,iagg))*mu3own_f1(iy,ig,is)
					assetsprime_temp(id) = assetsprime_temp(id) + dot_product(wagg,bown_f1(iy,ig,is,iz,iagg))*mu3own_f1(iy,ig,is)
				enddo
				!Homeowners with good credit but no mortgage
				do iy = 1, ny
					cons_temp(id)        = cons_temp(id) + dot_product(wagg,cown_f0(iy,1,ig,is,iz,iagg))*mu3own_f0(iy,1,ig,is)
					hcons_temp(id)       = hcons_temp(id) + dot_product(wagg,hcown_f0(iy,1,ig,is,iz,iagg))*mu3own_f0(iy,1,ig,is)
					assetsprime_temp(id) = assetsprime_temp(id) + dot_product(wagg,bown_f0(iy,1,ig,is,iz,iagg))*mu3own_f0(iy,1,ig,is)
					do iinterp = 1, ninterp
						mortcost_temp(id)    = mortcost_temp(id) + wagg(iinterp)*qm0(locm_f0(iy,1,ig,is,iz,iagg(iinterp)),locbown_f0(iy,1,ig,is,iz,iagg(iinterp)),ig,is,iz,iagg(iinterp))&
						*(1 + rocost)*(1 + rscost)*m_f0(iy,1,ig,is,iz,iagg(iinterp))*hh(ih)*mu3own_f0(iy,1,ig,is)
						svcorigcost_temp(id) = svcorigcost_temp(id) + wagg(iinterp)*qm0(locm_f0(iy,1,ig,is,iz,iagg(iinterp)),locbown_f0(iy,1,ig,is,iz,iagg(iinterp)),ig,is,iz,iagg(iinterp))&
						*((1 + rocost)*(1 + rscost) - 1)*m_f0(iy,1,ig,is,iz,iagg(iinterp))*hh(ih)*mu3own_f0(iy,1,ig,is)
					enddo
				enddo
			enddo
			!$omp end do
			!$omp end parallel
			do ig = 1, ng
				ih = hfun(ig)
				call omp_set_num_threads(num_threads_simul)
				!$omp parallel private(id,iy,iinterp)
				!$omp do
				do im = 2, nm
					id = omp_get_thread_num() + 1
					!Homeowners with mortgages
					do iy = ny_neg + 1, ny
						cons_temp(id)        = cons_temp(id) + dot_product(wagg,cown_f0(iy,im,ig,is,iz,iagg))*mu3own_f0(iy,im,ig,is)
						hcons_temp(id)       = hcons_temp(id) + dot_product(wagg,hcown_f0(iy,im,ig,is,iz,iagg))*mu3own_f0(iy,im,ig,is)
						assetsprime_temp(id) = assetsprime_temp(id) + dot_product(wagg,bown_f0(iy,im,ig,is,iz,iagg))*mu3own_f0(iy,im,ig,is)
						do iinterp = 1, ninterp
							mortrev_temp(id)  = mortrev_temp(id) + wagg(iinterp)*qm0(im,locbown_f0(iy,im,ig,is,iz,iagg(iinterp)),ig,is,iz,iagg(iinterp))&
							*(1 + rocost)*(1 + rscost)*mm(im)*hh(ih)*mu3own_f0(iy,im,ig,is)
							mortcost_temp(id) = mortcost_temp(id) + wagg(iinterp)*qm0(im,locbown_f0(iy,im,ig,is,iz,iagg(iinterp)),ig,is,iz,iagg(iinterp))&
							*(1 + rocost)*(1 + rscost)*mm(im)*hh(ih)*mu3own_f0(iy,im,ig,is)
						enddo
					enddo
				enddo
				!$omp end do
				!$omp end parallel
			enddo
		enddo
		do id = 1, num_threads_simul
			agg_xs      = agg_xs + agg_xs_temp(id)
			REOrevenue  = REOrevenue + REOrevenue_temp(id)
			mortrev     = mortrev + mortrev_temp(id)
			mortcost    = mortcost + mortcost_temp(id)
			svcorigcost = svcorigcost + svcorigcost_temp(id)
			assetsprime = assetsprime + assetsprime_temp(id)
			cons        = cons + cons_temp(id)
			hcons       = hcons + hcons_temp(id)
		enddo
		!Revenue from sales of existing REOs
		do ih = 1, nh
			REOrevenue = REOrevenue + dot_product(wagg,xs_reo(ih,iz,iagg)*psell_reo(ih,iz,iagg))*htot_reo(ih)
			mortrev    = mortrev + dot_product(wagg,Jreo(ih,iz,iagg))*htot_reo(ih)
			mortcost   = mortcost + dot_product(wagg,(Jreo(ih,iz,iagg) - psell_reo(ih,iz,iagg)*xs_reo(ih,iz,iagg)))*htot_reo(ih)
		enddo
		invest_h = agg_xb - (agg_xs + REOrevenue)
		lh       = ((1 - lshare)*kshare_h*phe*zh(iz)*(kshare_h/(1 - kshare_h)*w)**(kshare_h*(1 - lshare) - 1))**(1/lshare)
		kh       = kshare_h/(1 - kshare_h)*w*lh
		pl       = lshare*phe*zh(iz)*(kh**kshare_h*lh**(1 - kshare_h))**(1 - lshare)
		!Calculate kstockprime to clear the goods market
		kstockprime = qbe*assetsprime - mortcost
!		print *, (1 - deprec_k + rk)*kstock - assets + mortrev + taxrate*maxrev + qbe*assetsprime - mortcost - kstockprime
!		print *, yc + (1 - deprec_k)*kstock - (invest_h - phe*yh + pl + kh + cons + pa*hcons + svcorigcost + kstockprime)
		!Compute housing excess demand and update ph
		excess_demand1 = demand - supply
		if (excess_demand1 >= 0.d0) then
			sign_excessdemand1 = 1
		else
			sign_excessdemand1 = -1
		endif
		if (iter >=2 ) then
			if (excess_demand0 >= 0.d0) then
				sign_excessdemand0 = 1
			else
				sign_excessdemand0 = -1
			endif
		endif
		if (iter == 1) then
			if (demand > 0.d0 .AND. supply > 0.d0) then
				pctdiff = abs(excess_demand1)/(0.5*demand + 0.5*supply)
				dphe    = max(-0.5d0*phe,min(1.d0,pctdiff)*phe*sign_excessdemand1)
			else
				dphe = 0.1d0*phe*sign_excessdemand1
			endif
		elseif (sign_excessdemand1*sign_excessdemand0 == -1) then
			dphe    = max(-0.5d0*phe,-0.5d0*dphe)
			pctdiff = abs(excess_demand1)/(0.5*demand + 0.5*supply)
			if (abs(dphe) > min(1.d0,pctdiff)*phe) then
				dphe = dphe*min(1.d0,pctdiff)*phe/abs(dphe)
			endif
		else
			dphe    = max(-0.5d0*phe,dphe)
			pctdiff = abs(excess_demand1)/(0.5*demand + 0.5*supply)
			if (abs(dphe) > min(1.d0,pctdiff)*phe) then
				dphe = dphe*min(1.d0,pctdiff)*phe/abs(dphe)
			endif
		endif
		!Results
!  	    write(*, '(a6, i3, a6, f8.5, a6, f8.5, a6, f8.5, a7, f8.5)'), 'Iter =' ,iter, ', ph =' ,phe, ', Dh =' ,demand, ', Sh =' ,supply, ', dph =' ,dphe
		if (abs(dphe) < min_dphe .OR. (demand > 0.d0 .AND. supply > 0.d0 .AND. abs(excess_demand1)/(0.5*demand + 0.5*supply) < toler_excessdemand) .OR. iter == max_iter .OR. exitloop == 1) then
			if (abs((1 - deprec_k + rk)*kstock - assets + mortrev + taxrate*maxrev) < 1e-5) then
				exit
			endif
		endif
		!Update phe and the tax rate to distribute profits/losses
		taxrate = -((1 - deprec_k + rk)*kstock - assets + mortrev)/maxrev
		if (abs(phe - min_ph) < 1e-6 .AND. excess_demand1 < 0.d0 .AND. abs(excess_demand1)/(0.5*demand + 0.5*supply) > toler_excessdemand) then
			print *, 'Lowest price on ph grid is too high'
			exit
		elseif (abs(phe - max_ph) < 1e-6 .AND. excess_demand1 > 0.d0 .AND. abs(excess_demand1)/(0.5*demand + 0.5*supply) > toler_excessdemand) then
			print *, 'Highest price on ph grid is too low'
			exit
		endif
		if (iter >= 2 .AND. sign_excessdemand1*sign_excessdemand0 == 1 .AND. abs(excess_demand1) > abs(excess_demand0)) then
			phe      = phe_prev
			exitloop = 1
		else
			phe_prev = phe
			if (demand > supply) then
				phe = min(max_ph,phe + dphe)
			else
				phe = max(min_ph,phe + dphe)
			endif
		endif
		excess_demand0 = excess_demand1
	enddo
	write(*, '(a4, i5, a5, i2, a6, f7.4, a5, f7.4, a5, f7.4, a5, f6.3, a7, f8.5)'), ' t =' ,it, ', z =' ,iz, ', ph =' ,phe, ', D =' ,demand, ', S =' ,supply, ', K =' ,kstock, ', tax =' ,taxrate
	!Calculate end of period homeownership rate
	ownrate = sum(mu3own_f0) + sum(mu3own_f1)
	!Calculate end of period REO housing stock
	do ih = 1, nh
		htot_reo(ih) = (htot_reo(ih) + htot_reo_new(ih))*(1 - dot_product(wagg,psell_reo(ih,iz,iagg)))
	enddo
	!Calculate end of period housing stock
	htot = 0.d0
	do ig = 1, ng
		ih   = hfun(ig)
		htot = htot + hh(ih)*(sum(mu3own_f0(:,:,ig,:)) + sum(mu3own_f1(:,ig,:)))
	enddo
	do ih = 1, nh
		htot = htot + htot_reo(ih)*hh(ih)
	enddo
	!Generate time series variables
	if (genseries == 1) then
		call sub_timeseries(it,iz,phe,kstock,qbe,kstockprime,yh,yc,ownrate,assetsprime,invest_h,cons,hcons)
	endif
	!Transition from subperiod 3 to subperiod 1 tomorrow
	mu1rent_f0      = 0.d0
	mu1rent_f1      = 0.d0
	mu1own_f0       = 0.d0
	mu1own_f1       = 0.d0
	mu1rent_f0_temp = 0.d0
	mu1rent_f1_temp = 0.d0
	mu1own_f0_temp  = 0.d0
	mu1own_f1_temp  = 0.d0
	do is = 1, ns
		!Renters
		do iy = ny_neg + 1, ny
			do isprime = 1, ns
				do ieprime = 1, ne
					do iinterp = 1, ninterp
						!Renters with good credit
						yprime_f0 = labor(isprime,ieprime)*wprime(iz,iagg(iinterp),izprime) + brent_f0(iy,is,iz,iagg(iinterp))
						iy0low    = ly(yprime_f0)
						iy0upp    = uy(yprime_f0)
						y0weight  = wy(yprime_f0)
						mu1rent_f0(iy0low,isprime) = mu1rent_f0(iy0low,isprime) + wagg(iinterp)*(1 - y0weight)&
						*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3rent_f0(iy,is)
						mu1rent_f0(iy0upp,isprime) = mu1rent_f0(iy0upp,isprime) + wagg(iinterp)*y0weight&
						*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3rent_f0(iy,is)
						!Renters with bad credit
						yprime_f0 = labor(isprime,ieprime)*wprime(iz,iagg(iinterp),izprime) + brent_f1(iy,is,iz,iagg(iinterp))
						iy0low    = ly(yprime_f0)
						iy0upp    = uy(yprime_f0)
						y0weight  = wy(yprime_f0)
						yprime_f1 = (1 - fincloss)*labor(isprime,ieprime)*wprime(iz,iagg(iinterp),izprime) + brent_f1(iy,is,iz,iagg(iinterp))
						iy1low    = ly(yprime_f1)
						iy1upp    = uy(yprime_f1)
						y1weight  = wy(yprime_f1)
						!Credit status remains bad
						mu1rent_f1(iy1low,isprime) = mu1rent_f1(iy1low,isprime) + wagg(iinterp)*(1 - y1weight)&
						*pflag*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3rent_f1(iy,is)
						mu1rent_f1(iy1upp,isprime) = mu1rent_f1(iy1upp,isprime) + wagg(iinterp)*y1weight&
						*pflag*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3rent_f1(iy,is)
						!Credit status changes to good
						mu1rent_f0(iy0low,isprime) = mu1rent_f0(iy0low,isprime) + wagg(iinterp)*(1 - y0weight)&
						*(1 - pflag)*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3rent_f1(iy,is)
						mu1rent_f0(iy0upp,isprime) = mu1rent_f0(iy0upp,isprime) + wagg(iinterp)*y0weight&
						*(1 - pflag)*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3rent_f1(iy,is)
					enddo
				enddo
			enddo
		enddo
		!Homeowners
		call omp_set_num_threads(num_threads_simul)
		!$omp parallel private(id,ih,iu,igshock,iy,isprime,ieprime,iinterp,yprime_f0,iy0low,iy0upp,y0weight,yprime_f1,iy1low,iy1upp,y1weight,imprime)
		!$omp do
		do ig = 1, ng
			id      = omp_get_thread_num() + 1
			ih      = hfun(ig)
			iu      = ufun(ig)
			igshock = gfun(ih,2)
			!Homeowners with bad credit
			do iy = ny_neg + 1, ny
				if (iu == 1 .AND. nu == 2) then
					do isprime = 1, ns
						do ieprime = 1, ne
							do iinterp = 1, ninterp
								yprime_f0 = labor(isprime,ieprime)*wprime(iz,iagg(iinterp),izprime) + bown_f1(iy,ig,is,iz,iagg(iinterp))
								iy0low    = ly(yprime_f0)
								iy0upp    = uy(yprime_f0)
								y0weight  = wy(yprime_f0)
								yprime_f1 = (1 - fincloss)*labor(isprime,ieprime)*wprime(iz,iagg(iinterp),izprime) + bown_f1(iy,ig,is,iz,iagg(iinterp))
								iy1low    = ly(yprime_f1)
								iy1upp    = uy(yprime_f1)
								y1weight  = wy(yprime_f1)								
								!Credit status remains bad, house fully depreciates
								mu1rent_f1_temp(iy1low,isprime,id) = mu1rent_f1_temp(iy1low,isprime,id) + wagg(iinterp)*(1 - y1weight)&
								*deprec_h*pflag*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3own_f1(iy,ig,is)
								mu1rent_f1_temp(iy1upp,isprime,id) = mu1rent_f1_temp(iy1upp,isprime,id) + wagg(iinterp)*y1weight&
								*deprec_h*pflag*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3own_f1(iy,ig,is)
								!Credit status remains bad, house does not depreciate
								!No housing shock
								mu1own_f1_temp(iy1low,ig,isprime,id) = mu1own_f1_temp(iy1low,ig,isprime,id)&
								+ wagg(iinterp)*(1 - y1weight)*(1 - deprec_h)*pflag*prob_ss(is,isprime,iz,iz)*(1 - pseparate)*emeas(ieprime)&
								*mu3own_f1(iy,ig,is)
								mu1own_f1_temp(iy1upp,ig,isprime,id) = mu1own_f1_temp(iy1upp,ig,isprime,id)&
								+ wagg(iinterp)*y1weight*(1 - deprec_h)*pflag*prob_ss(is,isprime,iz,iz)*(1 - pseparate)*emeas(ieprime)*mu3own_f1(iy,ig,is)
								!Housing shock
								mu1own_f1_temp(iy1low,igshock,isprime,id) = mu1own_f1_temp(iy1low,igshock,isprime,id)&
								+ wagg(iinterp)*(1 - y1weight)*(1 - deprec_h)*pflag*prob_ss(is,isprime,iz,iz)*pseparate*emeas(ieprime)&
								*mu3own_f1(iy,ig,is)
								mu1own_f1_temp(iy1upp,igshock,isprime,id) = mu1own_f1_temp(iy1upp,igshock,isprime,id)&
								+ wagg(iinterp)*y1weight*(1 - deprec_h)*pflag*prob_ss(is,isprime,iz,iz)*pseparate*emeas(ieprime)*mu3own_f1(iy,ig,is)
								!Credit status changes to good, house fully depreciates
								mu1rent_f0_temp(iy0low,isprime,id) = mu1rent_f0_temp(iy0low,isprime,id) + wagg(iinterp)*(1 - y0weight)&
								*deprec_h*(1 - pflag)*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3own_f1(iy,ig,is)
								mu1rent_f0_temp(iy0upp,isprime,id) = mu1rent_f0_temp(iy0upp,isprime,id) + wagg(iinterp)*y0weight&
								*deprec_h*(1 - pflag)*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3own_f1(iy,ig,is)
								!Credit status changes to good, house does not depreciate
								!No housing shock
								mu1own_f0_temp(iy0low,1,ig,isprime,id) = mu1own_f0_temp(iy0low,1,ig,isprime,id)&
								+ wagg(iinterp)*(1 - y0weight)*(1 - deprec_h)*(1 - pflag)*prob_ss(is,isprime,iz,iz)*(1 - pseparate)*emeas(ieprime)&
								*mu3own_f1(iy,ig,is)
								mu1own_f0_temp(iy0upp,1,ig,isprime,id) = mu1own_f0_temp(iy0upp,1,ig,isprime,id)&
								+ wagg(iinterp)*y0weight*(1 - deprec_h)*(1 - pflag)*prob_ss(is,isprime,iz,iz)*(1 - pseparate)*emeas(ieprime)&
								*mu3own_f1(iy,ig,is)
								!Housing shock
								mu1own_f0_temp(iy0low,1,igshock,isprime,id) = mu1own_f0_temp(iy0low,1,igshock,isprime,id)&
								+ wagg(iinterp)*(1 - y0weight)*(1 - deprec_h)*(1 - pflag)*prob_ss(is,isprime,iz,iz)*pseparate*emeas(ieprime)&
								*mu3own_f1(iy,ig,is)
								mu1own_f0_temp(iy0upp,1,igshock,isprime,id) = mu1own_f0_temp(iy0upp,1,igshock,isprime,id)&
								+ wagg(iinterp)*y0weight*(1 - deprec_h)*(1 - pflag)*prob_ss(is,isprime,iz,iz)*pseparate*emeas(ieprime)&
								*mu3own_f1(iy,ig,is)
							enddo
						enddo
					enddo
				else
					do isprime = 1, ns
						do ieprime = 1, ne
							do iinterp = 1, ninterp
								yprime_f0 = labor(isprime,ieprime)*wprime(iz,iagg(iinterp),izprime) + bown_f1(iy,ig,is,iz,iagg(iinterp))
								iy0low    = ly(yprime_f0)
								iy0upp    = uy(yprime_f0)
								y0weight  = wy(yprime_f0)
								yprime_f1 = (1 - fincloss)*labor(isprime,ieprime)*wprime(iz,iagg(iinterp),izprime) + bown_f1(iy,ig,is,iz,iagg(iinterp))
								iy1low    = ly(yprime_f1)
								iy1upp    = uy(yprime_f1)
								y1weight  = wy(yprime_f1)								
								!Credit status remains bad, house fully depreciates
								mu1rent_f1_temp(iy1low,isprime,id) = mu1rent_f1_temp(iy1low,isprime,id) + wagg(iinterp)*(1 - y1weight)&
								*deprec_h*pflag*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3own_f1(iy,ig,is)
								mu1rent_f1_temp(iy1upp,isprime,id) = mu1rent_f1_temp(iy1upp,isprime,id) + wagg(iinterp)*y1weight&
								*deprec_h*pflag*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3own_f1(iy,ig,is)
								!Credit status remains bad, house does not depreciate
								mu1own_f1_temp(iy1low,ig,isprime,id) = mu1own_f1_temp(iy1low,ig,isprime,id)&
								+ wagg(iinterp)*(1 - y1weight)*(1 - deprec_h)*pflag*prob_ss(is,isprime,iz,iz)*emeas(ieprime)&
								*mu3own_f1(iy,ig,is)
								mu1own_f1_temp(iy1upp,ig,isprime,id) = mu1own_f1_temp(iy1upp,ig,isprime,id)&
								+ wagg(iinterp)*y1weight*(1 - deprec_h)*pflag*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3own_f1(iy,ig,is)								
								!Credit status changes to good, house fully depreciates
								mu1rent_f0_temp(iy0low,isprime,id) = mu1rent_f0_temp(iy0low,isprime,id) + wagg(iinterp)*(1 - y0weight)&
								*deprec_h*(1 - pflag)*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3own_f1(iy,ig,is)
								mu1rent_f0_temp(iy0upp,isprime,id) = mu1rent_f0_temp(iy0upp,isprime,id) + wagg(iinterp)*y0weight&
								*deprec_h*(1 - pflag)*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3own_f1(iy,ig,is)
								!Credit status changes to good, house does not depreciate
								mu1own_f0_temp(iy0low,1,ig,isprime,id) = mu1own_f0_temp(iy0low,1,ig,isprime,id)&
								+ wagg(iinterp)*(1 - y0weight)*(1 - deprec_h)*(1 - pflag)*prob_ss(is,isprime,iz,iz)*emeas(ieprime)&
								*mu3own_f1(iy,ig,is)
								mu1own_f0_temp(iy0upp,1,ig,isprime,id) = mu1own_f0_temp(iy0upp,1,ig,isprime,id)&
								+ wagg(iinterp)*y0weight*(1 - deprec_h)*(1 - pflag)*prob_ss(is,isprime,iz,iz)*emeas(ieprime)&
								*mu3own_f1(iy,ig,is)
							enddo
						enddo
					enddo
				endif
			enddo
			!Homeowners without a mortgage
			do iy = 1, ny
				if (iu == 1 .AND. nu == 2) then
					do isprime = 1, ns
						do ieprime = 1, ne
							do iinterp = 1, ninterp
								imprime   = locm_f0(iy,1,ig,is,iz,iagg(iinterp))
								yprime_f0 = labor(isprime,ieprime)*wprime(iz,iagg(iinterp),izprime) + bown_f0(iy,1,ig,is,iz,iagg(iinterp))
								iy0low    = ly(yprime_f0)
								iy0upp    = uy(yprime_f0)
								y0weight  = wy(yprime_f0)
								!House fully depreciates
								mu1rent_f0_temp(iy0low,isprime,id) = mu1rent_f0_temp(iy0low,isprime,id) + wagg(iinterp)*(1 - y0weight)&
								*deprec_h*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3own_f0(iy,1,ig,is)
								mu1rent_f0_temp(iy0upp,isprime,id) = mu1rent_f0_temp(iy0upp,isprime,id) + wagg(iinterp)*y0weight&
								*deprec_h*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3own_f0(iy,1,ig,is)
								!House does not depreciate
								!No housing shock
								mu1own_f0_temp(iy0low,imprime,ig,isprime,id) = mu1own_f0_temp(iy0low,imprime,ig,isprime,id)&
								+ wagg(iinterp)*(1 - y0weight)*(1 - deprec_h)*prob_ss(is,isprime,iz,iz)*(1 - pseparate)*emeas(ieprime)*mu3own_f0(iy,1,ig,is)
								mu1own_f0_temp(iy0upp,imprime,ig,isprime,id) = mu1own_f0_temp(iy0upp,imprime,ig,isprime,id)&
								+ wagg(iinterp)*y0weight*(1 - deprec_h)*prob_ss(is,isprime,iz,iz)*(1 - pseparate)*emeas(ieprime)*mu3own_f0(iy,1,ig,is)
								!Housing shock
								mu1own_f0_temp(iy0low,imprime,igshock,isprime,id) = mu1own_f0_temp(iy0low,imprime,igshock,isprime,id)&
								+ wagg(iinterp)*(1 - y0weight)*(1 - deprec_h)*prob_ss(is,isprime,iz,iz)*pseparate*emeas(ieprime)*mu3own_f0(iy,1,ig,is)
								mu1own_f0_temp(iy0upp,imprime,igshock,isprime,id) = mu1own_f0_temp(iy0upp,imprime,igshock,isprime,id)&
								+ wagg(iinterp)*y0weight*(1 - deprec_h)*prob_ss(is,isprime,iz,iz)*pseparate*emeas(ieprime)*mu3own_f0(iy,1,ig,is)
							enddo
						enddo
					enddo
				else
					do isprime = 1, ns
						do ieprime = 1, ne
							do iinterp = 1, ninterp
								imprime = locm_f0(iy,1,ig,is,iz,iagg(iinterp))
								yprime_f0 = labor(isprime,ieprime)*wprime(iz,iagg(iinterp),izprime) + bown_f0(iy,1,ig,is,iz,iagg(iinterp))
								iy0low    = ly(yprime_f0)
								iy0upp    = uy(yprime_f0)
								y0weight  = wy(yprime_f0)
								!House fully depreciates
								mu1rent_f0_temp(iy0low,isprime,id) = mu1rent_f0_temp(iy0low,isprime,id) + wagg(iinterp)*(1 - y0weight)&
								*deprec_h*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3own_f0(iy,1,ig,is)
								mu1rent_f0_temp(iy0upp,isprime,id) = mu1rent_f0_temp(iy0upp,isprime,id) + wagg(iinterp)*y0weight&
								*deprec_h*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3own_f0(iy,1,ig,is)
								!House does not depreciate
								mu1own_f0_temp(iy0low,imprime,ig,isprime,id) = mu1own_f0_temp(iy0low,imprime,ig,isprime,id)&
								+ wagg(iinterp)*(1 - y0weight)*(1 - deprec_h)*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3own_f0(iy,1,ig,is)
								mu1own_f0_temp(iy0upp,imprime,ig,isprime,id) = mu1own_f0_temp(iy0upp,imprime,ig,isprime,id)&
								+ wagg(iinterp)*y0weight*(1 - deprec_h)*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3own_f0(iy,1,ig,is)
							enddo
						enddo
					enddo
				endif
			enddo
		enddo
		!$omp end do
		!$omp end parallel
		do ig = 1, ng
			ih      = hfun(ig)
			iu      = ufun(ig)
			igshock = gfun(ih,2)
			!Homeowners with a mortgage
			call omp_set_num_threads(num_threads_simul)
			!$omp parallel private(id,iy,isprime,ieprime,iinterp,yprime_f0,iy0low,iy0upp,y0weight)
			!$omp do
			do im = 2, nm
				id = omp_get_thread_num() + 1
				do iy = ny_neg + 1, ny
					if (iu == 1 .AND. nu == 2) then
						do isprime = 1, ns
							do ieprime = 1, ne
								do iinterp = 1, ninterp
									yprime_f0 = labor(isprime,ieprime)*wprime(iz,iagg(iinterp),izprime) + bown_f0(iy,im,ig,is,iz,iagg(iinterp))
									iy0low    = ly(yprime_f0)
									iy0upp    = uy(yprime_f0)
									y0weight  = wy(yprime_f0)
									!House fully depreciates
									mu1rent_f0_temp(iy0low,isprime,id) = mu1rent_f0_temp(iy0low,isprime,id)&
									+ wagg(iinterp)*(1 - y0weight)*deprec_h*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3own_f0(iy,im,ig,is)
									mu1rent_f0_temp(iy0upp,isprime,id) = mu1rent_f0_temp(iy0upp,isprime,id) + wagg(iinterp)*y0weight&
									*deprec_h*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3own_f0(iy,im,ig,is)
									!House does not depreciate
									!No housing shock
									mu1own_f0_temp(iy0low,im,ig,isprime,id) = mu1own_f0_temp(iy0low,im,ig,isprime,id)&
									+ wagg(iinterp)*(1 - y0weight)*(1 - deprec_h)*prob_ss(is,isprime,iz,iz)*(1 - pseparate)*emeas(ieprime)&
									*mu3own_f0(iy,im,ig,is)
									mu1own_f0_temp(iy0upp,im,ig,isprime,id) = mu1own_f0_temp(iy0upp,im,ig,isprime,id)&
									+ wagg(iinterp)*y0weight*(1 - deprec_h)*prob_ss(is,isprime,iz,iz)*(1 - pseparate)*emeas(ieprime)*mu3own_f0(iy,im,ig,is)
									!Housing shock
									mu1own_f0_temp(iy0low,im,igshock,isprime,id) = mu1own_f0_temp(iy0low,im,igshock,isprime,id)&
									+ wagg(iinterp)*(1 - y0weight)*(1 - deprec_h)*prob_ss(is,isprime,iz,iz)*pseparate*emeas(ieprime)&
									*mu3own_f0(iy,im,ig,is)
									mu1own_f0_temp(iy0upp,im,igshock,isprime,id) = mu1own_f0_temp(iy0upp,im,igshock,isprime,id)&
									+ wagg(iinterp)*y0weight*(1 - deprec_h)*prob_ss(is,isprime,iz,iz)*pseparate*emeas(ieprime)*mu3own_f0(iy,im,ig,is)
								enddo
							enddo
						enddo
					else
						do isprime = 1, ns
							do ieprime = 1, ne
								do iinterp = 1, ninterp
									yprime_f0 = labor(isprime,ieprime)*wprime(iz,iagg(iinterp),izprime) + bown_f0(iy,im,ig,is,iz,iagg(iinterp))
									iy0low    = ly(yprime_f0)
									iy0upp    = uy(yprime_f0)
									y0weight  = wy(yprime_f0)
									!House fully depreciates
									mu1rent_f0_temp(iy0low,isprime,id) = mu1rent_f0_temp(iy0low,isprime,id)&
									+ wagg(iinterp)*(1 - y0weight)*deprec_h*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3own_f0(iy,im,ig,is)
									mu1rent_f0_temp(iy0upp,isprime,id) = mu1rent_f0_temp(iy0upp,isprime,id) + wagg(iinterp)*y0weight&
									*deprec_h*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3own_f0(iy,im,ig,is)
									!House does not depreciate
									mu1own_f0_temp(iy0low,im,ig,isprime,id) = mu1own_f0_temp(iy0low,im,ig,isprime,id)&
									+ wagg(iinterp)*(1 - y0weight)*(1 - deprec_h)*prob_ss(is,isprime,iz,iz)*emeas(ieprime)&
									*mu3own_f0(iy,im,ig,is)
									mu1own_f0_temp(iy0upp,im,ig,isprime,id) = mu1own_f0_temp(iy0upp,im,ig,isprime,id)&
									+ wagg(iinterp)*y0weight*(1 - deprec_h)*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3own_f0(iy,im,ig,is)
								enddo
							enddo
						enddo
					endif
				enddo  
			enddo
			!$omp end do
			!$omp end parallel
		enddo
	enddo
	do id = 1, num_threads_simul
		mu1rent_f0 = mu1rent_f0 + mu1rent_f0_temp(:,:,id)
		mu1rent_f1 = mu1rent_f1 + mu1rent_f1_temp(:,:,id)
		mu1own_f0  = mu1own_f0 + mu1own_f0_temp(:,:,:,:,id)
		mu1own_f1  = mu1own_f1 + mu1own_f1_temp(:,:,:,id)
	enddo
	!Calculate tomorrow's REO housing stock
	htot_reo = htot_reo*(1 - deprec_h)
	!Rescale distributions so that the total measure of agents is exactly 1
	meas_agents = sum(mu1own_f0) + sum(mu1own_f1) + sum(mu1rent_f0) + sum(mu1rent_f1)
!	print *, meas_agents
	mu1own_f0   = mu1own_f0/meas_agents
	mu1own_f1   = mu1own_f1/meas_agents
	mu1rent_f0  = mu1rent_f0/meas_agents
	mu1rent_f1  = mu1rent_f1/meas_agents

end subroutine sub_simulate