subroutine sub_stationary(iz,iagg)

	use mod_globvar
	use omp_lib
	use mod_interpolate

	implicit none

	!This program computes the stationary distribution of agents over (y,m,g,s,f)
	
	integer, intent(in) :: iz, iagg
	
	!Indices for today's state
	integer :: is, ig, ih, iu, im, iy
	!Indices for tomorrow's state
	integer :: igshock, isprime, ieprime

	!Updated distributions
	real(8), dimension(:,:,:,:), allocatable :: mu1own_f0new
	real(8), dimension(:,:,:), allocatable :: mu1own_f1new
	real(8), dimension(:,:), allocatable :: mu1rent_f0new, mu1rent_f1new
	
	!Temporary distributions for parallelization
	real(8), dimension(:,:,:,:,:), allocatable :: mu1own_f0new_temp, mu2own_f0_temp
	real(8), dimension(:,:,:,:), allocatable :: mu1own_f1new_temp, mu2own_f1_temp
	real(8), dimension(:,:,:), allocatable :: mu1rent_f0new_temp, mu1rent_f1new_temp, mu2rent_f0_temp, mu2rent_f1_temp
		
	!Other internal variables
	real(8) :: err_mu, meas_agents, ysell, ydefault, ypay, ybuy, yprime_f0, yprime_f1, yweight, y0weight, y1weight
	real(8), dimension(nh,num_threads_simul) :: htot_reo_temp
	real(8), dimension(nh) :: htot_reo_new
	integer :: iter_mu, impay, ihbuy, igbuy, imprime, id, iylow, iyupp, iy0low, iy0upp, iy1low, iy1upp
	
	!Allocate updated distributions
	allocate(mu1own_f0new(ny,nm,ng,ns))
	allocate(mu1own_f1new(ny,ng,ns))
	allocate(mu1rent_f0new(ny,ns))
	allocate(mu1rent_f1new(ny,ns))
	
	!Allocate distributions for parallelization
	allocate(mu1own_f0new_temp(ny,nm,ng,ns,num_threads_simul))
	allocate(mu2own_f0_temp(ny,nm,ng,ns,num_threads_simul))
	allocate(mu1own_f1new_temp(ny,ng,ns,num_threads_simul))
	allocate(mu2own_f1_temp(ny,ng,ns,num_threads_simul))
	allocate(mu1rent_f0new_temp(ny,ns,num_threads_simul))
	allocate(mu2rent_f0_temp(ny,ns,num_threads_simul))
	allocate(mu1rent_f1new_temp(ny,ns,num_threads_simul))
	allocate(mu2rent_f1_temp(ny,ns,num_threads_simul))
	
	print *, ' '
	print *, '     Compute Stationary Distribution'
	print *, ' '
	do iter_mu = 1, 2500
		mu2rent_f0    = 0.d0
		mu2rent_f1    = 0.d0
		mu2own_f0     = 0.d0
		mu2own_f1     = 0.d0
		mu3rent_f0    = 0.d0
		mu3rent_f1    = 0.d0
		mu3own_f0     = 0.d0
		mu3own_f1     = 0.d0
		mu1rent_f0new = 0.d0
		mu1rent_f1new = 0.d0
		mu1own_f0new  = 0.d0
		mu1own_f1new  = 0.d0
		!Transition from subperiod 1 to subperiod 2
		mu2rent_f0_temp = 0.d0
		mu2rent_f1_temp = 0.d0
		mu2own_f0_temp  = 0.d0
		mu2own_f1_temp  = 0.d0
		do is = 1, ns
			!Renters
			do iy = ny_neg + 1, ny
				!Renters with good credit
				mu2rent_f0(iy,is) = mu2rent_f0(iy,is) + mu1rent_f0(iy,is)
				!Renters with bad credit
				mu2rent_f1(iy,is) = mu2rent_f1(iy,is) + mu1rent_f1(iy,is)
			enddo
			!Homeowners
			call omp_set_num_threads(num_threads_simul)
			!$omp parallel private(id,ih,iy,ysell,iylow,iyupp,yweight)
			!$omp do
			do ig = 1, ng
				id = omp_get_thread_num() + 1
				ih = hfun(ig)
				do iy = ny_neg + 1, ny
					!Homeowners with bad credit
					ysell   = yy(iy) + xs_f1(iy,ig,is,iz,iagg)*(1 - sellcost)
					iylow   = ly(ysell)
					iyupp   = uy(ysell)
					yweight = wy(ysell)
					!Successful sale
					mu2rent_f1_temp(iylow,is,id) = mu2rent_f1_temp(iylow,is,id) + (1 - yweight)*psell_f1(iy,ig,is,iz,iagg)&
					*mu1own_f1(iy,ig,is)
					mu2rent_f1_temp(iyupp,is,id) = mu2rent_f1_temp(iyupp,is,id) + yweight*psell_f1(iy,ig,is,iz,iagg)&
					*mu1own_f1(iy,ig,is)
					!No sale
					mu2own_f1_temp(iy,ig,is,id) = mu2own_f1_temp(iy,ig,is,id) + (1 - psell_f1(iy,ig,is,iz,iagg))*mu1own_f1(iy,ig,is)
					!Homeowners with good credit but no mortgage
					ysell   = yy(iy) + xs_f0(iy,1,ig,is,iz,iagg)*(1 - sellcost)
					iylow   = ly(ysell)
					iyupp   = uy(ysell)
					yweight = wy(ysell)
					!Successful sale
					mu2rent_f0_temp(iylow,is,id) = mu2rent_f0_temp(iylow,is,id) + (1 - yweight)*psell_f0(iy,1,ig,is,iz,iagg)&
					*mu1own_f0(iy,1,ig,is)
					mu2rent_f0_temp(iyupp,is,id) = mu2rent_f0_temp(iyupp,is,id) + yweight*psell_f0(iy,1,ig,is,iz,iagg)&
					*mu1own_f0(iy,1,ig,is)
					!No sale
					mu2own_f0_temp(iy,1,ig,is,id) = mu2own_f0_temp(iy,1,ig,is,id) + (1 - psell_f0(iy,1,ig,is,iz,iagg))*mu1own_f0(iy,1,ig,is)
				enddo
			enddo
			!$omp end do
			!$omp end parallel
			do ig = 1, ng
				ih = hfun(ig)
				!Homeowners with a mortgage
				call omp_set_num_threads(num_threads_simul)
				!$omp parallel private(id,iy,ysell,iylow,iyupp,yweight,ydefault,impay,ypay)
				!$omp do
				do im = 2, nm
					id = omp_get_thread_num() + 1
					do iy = ny_neg + 1, ny
						!Successful sale
						ysell   = yy(iy) - mm(im)*hh(ih) + xs_f0(iy,im,ig,is,iz,iagg)*(1 - sellcost)
						iylow   = ly(ysell)
						iyupp   = uy(ysell)
						yweight = wy(ysell)
						mu2rent_f0_temp(iylow,is,id) = mu2rent_f0_temp(iylow,is,id) + (1 - yweight)*psell_f0(iy,im,ig,is,iz,iagg)&
						*mu1own_f0(iy,im,ig,is)
						mu2rent_f0_temp(iyupp,is,id) = mu2rent_f0_temp(iyupp,is,id) + yweight*psell_f0(iy,im,ig,is,iz,iagg)&
						*mu1own_f0(iy,im,ig,is)
						!No sale, default
						ydefault = yy(iy) - max(0.d0,min(fdeficiency*yy(iy),mm(im)*hh(ih) - Jreo(ih,iz,iagg)))
						iylow    = ly(ydefault)
						iyupp    = uy(ydefault)
						yweight  = wy(ydefault)
						mu2rent_f1_temp(iylow,is,id) = mu2rent_f1_temp(iylow,is,id) + (1 - yweight)&
						*(1 - psell_f0(iy,im,ig,is,iz,iagg))*d_f0(iy,im,ig,is,iz,iagg)*mu1own_f0(iy,im,ig,is)
						mu2rent_f1_temp(iyupp,is,id) = mu2rent_f1_temp(iyupp,is,id) + yweight&
						*(1 - psell_f0(iy,im,ig,is,iz,iagg))*d_f0(iy,im,ig,is,iz,iagg)*mu1own_f0(iy,im,ig,is)
						!No sale, make payment
						impay    = locm_f0(iy,im,ig,is,iz,iagg)
						ypay     = yy(iy) - (mm(im) - qm(iz,iagg)*m_f0(iy,im,ig,is,iz,iagg))*hh(ih)
						if (ypay < 0.d0) then
							ypay = ypay/abs(ylb(ig,is,iz,iagg))
						endif
						iylow   = ly(ypay)
						iyupp   = uy(ypay)
						yweight = wy(ypay)
						mu2own_f0_temp(iylow,impay,ig,is,id) = mu2own_f0_temp(iylow,impay,ig,is,id) + (1 - yweight)&
						*(1 - psell_f0(iy,im,ig,is,iz,iagg))*(1 - d_f0(iy,im,ig,is,iz,iagg))*mu1own_f0(iy,im,ig,is)
						mu2own_f0_temp(iyupp,impay,ig,is,id) = mu2own_f0_temp(iyupp,impay,ig,is,id) + yweight&
						*(1 - psell_f0(iy,im,ig,is,iz,iagg))*(1 - d_f0(iy,im,ig,is,iz,iagg))*mu1own_f0(iy,im,ig,is)
					enddo
				enddo
				!$omp end do
				!$omp end parallel
			enddo
		enddo
		do id = 1, num_threads_simul
			mu2rent_f0 = mu2rent_f0 + mu2rent_f0_temp(:,:,id)
			mu2rent_f1 = mu2rent_f1 + mu2rent_f1_temp(:,:,id)
			mu2own_f0  = mu2own_f0 + mu2own_f0_temp(:,:,:,:,id)
			mu2own_f1  = mu2own_f1 + mu2own_f1_temp(:,:,:,id)
		enddo
		!Transition from subperiod 2 to subperiod 3
		do is = 1, ns
			!Renters
			do iy = ny_neg + 1, ny
				!Renters with good credit
				ihbuy = loch_f0(iy,is,iz,iagg)
				igbuy = gfun(ihbuy,1)
				ybuy  = yy(iy) - xb_f0(iy,is,iz,iagg)
				if (ybuy < 0.d0) then
					ybuy = ybuy/abs(ylb(igbuy,is,iz,iagg))
				endif
				iylow   = ly(ybuy)
				iyupp   = uy(ybuy)
				yweight = wy(ybuy)
				!Successful purchase
				mu3own_f0(iylow,1,igbuy,is) = mu3own_f0(iylow,1,igbuy,is) + (1 - yweight)*pbuy_f0(iy,is,iz,iagg)&
				*mu2rent_f0(iy,is)
				mu3own_f0(iyupp,1,igbuy,is) = mu3own_f0(iyupp,1,igbuy,is) + yweight*pbuy_f0(iy,is,iz,iagg)&
				*mu2rent_f0(iy,is)
				!No purchase
				mu3rent_f0(iy,is) = mu3rent_f0(iy,is) + (1 - pbuy_f0(iy,is,iz,iagg))*mu2rent_f0(iy,is)
				!Renters with bad credit
				ihbuy   = loch_f1(iy,is,iz,iagg)
				igbuy   = gfun(ihbuy,1)
				ybuy    = yy(iy) - xb_f1(iy,is,iz,iagg)
				iylow   = ly(ybuy)
				iyupp   = uy(ybuy)
				yweight = wy(ybuy)
				!Successful purchase
				mu3own_f1(iylow,igbuy,is) = mu3own_f1(iylow,igbuy,is) + (1 - yweight)*pbuy_f1(iy,is,iz,iagg)&
				*mu2rent_f1(iy,is)
				mu3own_f1(iyupp,igbuy,is) = mu3own_f1(iyupp,igbuy,is) + yweight*pbuy_f1(iy,is,iz,iagg)&
				*mu2rent_f1(iy,is)
				!No purchase
				mu3rent_f1(iy,is) = mu3rent_f1(iy,is) + (1 - pbuy_f1(iy,is,iz,iagg))*mu2rent_f1(iy,is)
			enddo
		enddo
		!Homeowners
		mu3own_f1 = mu3own_f1 + mu2own_f1
		mu3own_f0 = mu3own_f0 + mu2own_f0
		!Transition from subperiod 3 to subperiod 1 tomorrow
		mu1rent_f0new_temp = 0.d0
		mu1rent_f1new_temp = 0.d0
		mu1own_f0new_temp  = 0.d0
		mu1own_f1new_temp  = 0.d0
		do is = 1, ns
			!Renters
			do iy = ny_neg + 1, ny
				do isprime = 1, ns
					do ieprime = 1, ne
						!Renters with good credit
						yprime_f0 = labor(isprime,ieprime)*wprime(iz,iagg,iz) + brent_f0(iy,is,iz,iagg)
						iy0low    = ly(yprime_f0)
						iy0upp    = uy(yprime_f0)
						y0weight  = wy(yprime_f0)
						mu1rent_f0new(iy0low,isprime) = mu1rent_f0new(iy0low,isprime) + (1 - y0weight)&
						*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3rent_f0(iy,is)
						mu1rent_f0new(iy0upp,isprime) = mu1rent_f0new(iy0upp,isprime) + y0weight&
						*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3rent_f0(iy,is)
						!Renters with bad credit
						yprime_f0 = labor(isprime,ieprime)*wprime(iz,iagg,iz) + brent_f1(iy,is,iz,iagg)
						iy0low    = ly(yprime_f0)
						iy0upp    = uy(yprime_f0)
						y0weight  = wy(yprime_f0)
						yprime_f1 = (1 - fincloss)*labor(isprime,ieprime)*wprime(iz,iagg,iz) + brent_f1(iy,is,iz,iagg)
						iy1low    = ly(yprime_f1)
						iy1upp    = uy(yprime_f1)
						y1weight  = wy(yprime_f1)
						!Credit status remains bad
						mu1rent_f1new(iy1low,isprime) = mu1rent_f1new(iy1low,isprime) + (1 - y1weight)&
						*pflag*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3rent_f1(iy,is)
						mu1rent_f1new(iy1upp,isprime) = mu1rent_f1new(iy1upp,isprime) + y1weight&
						*pflag*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3rent_f1(iy,is)
						!Credit status changes to good
						mu1rent_f0new(iy0low,isprime) = mu1rent_f0new(iy0low,isprime) + (1 - y0weight)&
						*(1 - pflag)*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3rent_f1(iy,is)
						mu1rent_f0new(iy0upp,isprime) = mu1rent_f0new(iy0upp,isprime) + y0weight&
						*(1 - pflag)*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3rent_f1(iy,is)
					enddo
				enddo
			enddo
			!Homeowners
			call omp_set_num_threads(num_threads_simul)
			!$omp parallel private(id,ih,iu,igshock,iy,isprime,ieprime,yprime_f0,iy0low,iy0upp,y0weight,yprime_f1,iy1low,iy1upp,y1weight,imprime)
			!$omp do
			do ig = 1, ng
				id      = omp_get_thread_num() + 1
				ih      = hfun(ig)
				iu      = ufun(ig)
				igshock = gfun(ih,2)
				!Homeowners with bad credit
				do iy = ny_neg + 1, ny
					if (iu == 1 .AND. nu == 2) then
						do isprime = 1, ns
							do ieprime = 1, ne
								yprime_f0 = labor(isprime,ieprime)*wprime(iz,iagg,iz) + bown_f1(iy,ig,is,iz,iagg)
								iy0low    = ly(yprime_f0)
								iy0upp    = uy(yprime_f0)
								y0weight  = wy(yprime_f0)
								yprime_f1 = (1 - fincloss)*labor(isprime,ieprime)*wprime(iz,iagg,iz) + bown_f1(iy,ig,is,iz,iagg)
								iy1low    = ly(yprime_f1)
								iy1upp    = uy(yprime_f1)
								y1weight  = wy(yprime_f1)								
								!Credit status remains bad, house fully depreciates
								mu1rent_f1new_temp(iy1low,isprime,id) = mu1rent_f1new_temp(iy1low,isprime,id) + (1 - y1weight)&
								*deprec_h*pflag*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3own_f1(iy,ig,is)
								mu1rent_f1new_temp(iy1upp,isprime,id) = mu1rent_f1new_temp(iy1upp,isprime,id) + y1weight&
								*deprec_h*pflag*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3own_f1(iy,ig,is)
								!Credit status remains bad, house does not depreciate
								!No housing shock
								mu1own_f1new_temp(iy1low,ig,isprime,id) = mu1own_f1new_temp(iy1low,ig,isprime,id)&
								+ (1 - y1weight)*(1 - deprec_h)*pflag*prob_ss(is,isprime,iz,iz)*(1 - pseparate)*emeas(ieprime)&
								*mu3own_f1(iy,ig,is)
								mu1own_f1new_temp(iy1upp,ig,isprime,id) = mu1own_f1new_temp(iy1upp,ig,isprime,id)&
								+ y1weight*(1 - deprec_h)*pflag*prob_ss(is,isprime,iz,iz)*(1 - pseparate)*emeas(ieprime)*mu3own_f1(iy,ig,is)
								!Housing shock
								mu1own_f1new_temp(iy1low,igshock,isprime,id) = mu1own_f1new_temp(iy1low,igshock,isprime,id)&
								+ (1 - y1weight)*(1 - deprec_h)*pflag*prob_ss(is,isprime,iz,iz)*pseparate*emeas(ieprime)&
								*mu3own_f1(iy,ig,is)
								mu1own_f1new_temp(iy1upp,igshock,isprime,id) = mu1own_f1new_temp(iy1upp,igshock,isprime,id)&
								+ y1weight*(1 - deprec_h)*pflag*prob_ss(is,isprime,iz,iz)*pseparate*emeas(ieprime)*mu3own_f1(iy,ig,is)
								!Credit status changes to good, house fully depreciates
								mu1rent_f0new_temp(iy0low,isprime,id) = mu1rent_f0new_temp(iy0low,isprime,id) + (1 - y0weight)&
								*deprec_h*(1 - pflag)*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3own_f1(iy,ig,is)
								mu1rent_f0new_temp(iy0upp,isprime,id) = mu1rent_f0new_temp(iy0upp,isprime,id) + y0weight&
								*deprec_h*(1 - pflag)*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3own_f1(iy,ig,is)
								!Credit status changes to good, house does not depreciate
								!No housing shock
								mu1own_f0new_temp(iy0low,1,ig,isprime,id) = mu1own_f0new_temp(iy0low,1,ig,isprime,id)&
								+ (1 - y0weight)*(1 - deprec_h)*(1 - pflag)*prob_ss(is,isprime,iz,iz)*(1 - pseparate)*emeas(ieprime)&
								*mu3own_f1(iy,ig,is)
								mu1own_f0new_temp(iy0upp,1,ig,isprime,id) = mu1own_f0new_temp(iy0upp,1,ig,isprime,id)&
								+ y0weight*(1 - deprec_h)*(1 - pflag)*prob_ss(is,isprime,iz,iz)*(1 - pseparate)*emeas(ieprime)&
								*mu3own_f1(iy,ig,is)
								!Housing shock
								mu1own_f0new_temp(iy0low,1,igshock,isprime,id) = mu1own_f0new_temp(iy0low,1,igshock,isprime,id)&
								+ (1 - y0weight)*(1 - deprec_h)*(1 - pflag)*prob_ss(is,isprime,iz,iz)*pseparate*emeas(ieprime)&
								*mu3own_f1(iy,ig,is)
								mu1own_f0new_temp(iy0upp,1,igshock,isprime,id) = mu1own_f0new_temp(iy0upp,1,igshock,isprime,id)&
								+ y0weight*(1 - deprec_h)*(1 - pflag)*prob_ss(is,isprime,iz,iz)*pseparate*emeas(ieprime)&
								*mu3own_f1(iy,ig,is)
							enddo
						enddo
					else
						do isprime = 1, ns
							do ieprime = 1, ne
								yprime_f0 = labor(isprime,ieprime)*wprime(iz,iagg,iz) + bown_f1(iy,ig,is,iz,iagg)
								iy0low    = ly(yprime_f0)
								iy0upp    = uy(yprime_f0)
								y0weight  = wy(yprime_f0)
								yprime_f1 = (1 - fincloss)*labor(isprime,ieprime)*wprime(iz,iagg,iz) + bown_f1(iy,ig,is,iz,iagg)
								iy1low    = ly(yprime_f1)
								iy1upp    = uy(yprime_f1)
								y1weight  = wy(yprime_f1)								
								!Credit status remains bad, house fully depreciates
								mu1rent_f1new_temp(iy1low,isprime,id) = mu1rent_f1new_temp(iy1low,isprime,id) + (1 - y1weight)&
								*deprec_h*pflag*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3own_f1(iy,ig,is)
								mu1rent_f1new_temp(iy1upp,isprime,id) = mu1rent_f1new_temp(iy1upp,isprime,id) + y1weight&
								*deprec_h*pflag*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3own_f1(iy,ig,is)
								!Credit status remains bad, house does not depreciate
								mu1own_f1new_temp(iy1low,ig,isprime,id) = mu1own_f1new_temp(iy1low,ig,isprime,id)&
								+ (1 - y1weight)*(1 - deprec_h)*pflag*prob_ss(is,isprime,iz,iz)*emeas(ieprime)&
								*mu3own_f1(iy,ig,is)
								mu1own_f1new_temp(iy1upp,ig,isprime,id) = mu1own_f1new_temp(iy1upp,ig,isprime,id)&
								+ y1weight*(1 - deprec_h)*pflag*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3own_f1(iy,ig,is)								
								!Credit status changes to good, house fully depreciates
								mu1rent_f0new_temp(iy0low,isprime,id) = mu1rent_f0new_temp(iy0low,isprime,id) + (1 - y0weight)&
								*deprec_h*(1 - pflag)*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3own_f1(iy,ig,is)
								mu1rent_f0new_temp(iy0upp,isprime,id) = mu1rent_f0new_temp(iy0upp,isprime,id) + y0weight&
								*deprec_h*(1 - pflag)*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3own_f1(iy,ig,is)
								!Credit status changes to good, house does not depreciate
								mu1own_f0new_temp(iy0low,1,ig,isprime,id) = mu1own_f0new_temp(iy0low,1,ig,isprime,id)&
								+ (1 - y0weight)*(1 - deprec_h)*(1 - pflag)*prob_ss(is,isprime,iz,iz)*emeas(ieprime)&
								*mu3own_f1(iy,ig,is)
								mu1own_f0new_temp(iy0upp,1,ig,isprime,id) = mu1own_f0new_temp(iy0upp,1,ig,isprime,id)&
								+ y0weight*(1 - deprec_h)*(1 - pflag)*prob_ss(is,isprime,iz,iz)*emeas(ieprime)&
								*mu3own_f1(iy,ig,is)
							enddo
						enddo
					endif
				enddo
				!Homeowners without a mortgage
				do iy = 1, ny
					imprime = locm_f0(iy,1,ig,is,iz,iagg)
					if (iu == 1 .AND. nu == 2) then
						do isprime = 1, ns
							do ieprime = 1, ne
								yprime_f0 = labor(isprime,ieprime)*wprime(iz,iagg,iz) + bown_f0(iy,1,ig,is,iz,iagg)
								iy0low    = ly(yprime_f0)
								iy0upp    = uy(yprime_f0)
								y0weight  = wy(yprime_f0)
								!House fully depreciates
								mu1rent_f0new_temp(iy0low,isprime,id) = mu1rent_f0new_temp(iy0low,isprime,id) + (1 - y0weight)&
								*deprec_h*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3own_f0(iy,1,ig,is)
								mu1rent_f0new_temp(iy0upp,isprime,id) = mu1rent_f0new_temp(iy0upp,isprime,id) + y0weight&
								*deprec_h*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3own_f0(iy,1,ig,is)
								!House does not depreciate
								!No housing shock
								mu1own_f0new_temp(iy0low,imprime,ig,isprime,id) = mu1own_f0new_temp(iy0low,imprime,ig,isprime,id)&
								+ (1 - y0weight)*(1 - deprec_h)*prob_ss(is,isprime,iz,iz)*(1 - pseparate)*emeas(ieprime)*mu3own_f0(iy,1,ig,is)
								mu1own_f0new_temp(iy0upp,imprime,ig,isprime,id) = mu1own_f0new_temp(iy0upp,imprime,ig,isprime,id)&
								+ y0weight*(1 - deprec_h)*prob_ss(is,isprime,iz,iz)*(1 - pseparate)*emeas(ieprime)*mu3own_f0(iy,1,ig,is)
								!Housing shock
								mu1own_f0new_temp(iy0low,imprime,igshock,isprime,id) = mu1own_f0new_temp(iy0low,imprime,igshock,isprime,id)&
								+ (1 - y0weight)*(1 - deprec_h)*prob_ss(is,isprime,iz,iz)*pseparate*emeas(ieprime)*mu3own_f0(iy,1,ig,is)
								mu1own_f0new_temp(iy0upp,imprime,igshock,isprime,id) = mu1own_f0new_temp(iy0upp,imprime,igshock,isprime,id)&
								+ y0weight*(1 - deprec_h)*prob_ss(is,isprime,iz,iz)*pseparate*emeas(ieprime)*mu3own_f0(iy,1,ig,is)
							enddo
						enddo
					else
						do isprime = 1, ns
							do ieprime = 1, ne
								yprime_f0 = labor(isprime,ieprime)*wprime(iz,iagg,iz) + bown_f0(iy,1,ig,is,iz,iagg)
								iy0low    = ly(yprime_f0)
								iy0upp    = uy(yprime_f0)
								y0weight  = wy(yprime_f0)
								!House fully depreciates
								mu1rent_f0new_temp(iy0low,isprime,id) = mu1rent_f0new_temp(iy0low,isprime,id) + (1 - y0weight)&
								*deprec_h*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3own_f0(iy,1,ig,is)
								mu1rent_f0new_temp(iy0upp,isprime,id) = mu1rent_f0new_temp(iy0upp,isprime,id) + y0weight&
								*deprec_h*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3own_f0(iy,1,ig,is)
								!House does not depreciate
								mu1own_f0new_temp(iy0low,imprime,ig,isprime,id) = mu1own_f0new_temp(iy0low,imprime,ig,isprime,id)&
								+ (1 - y0weight)*(1 - deprec_h)*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3own_f0(iy,1,ig,is)
								mu1own_f0new_temp(iy0upp,imprime,ig,isprime,id) = mu1own_f0new_temp(iy0upp,imprime,ig,isprime,id)&
								+ y0weight*(1 - deprec_h)*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3own_f0(iy,1,ig,is)
							enddo
						enddo
					endif
				enddo
			enddo
			!$omp end do
			!$omp end parallel
			do ig = 1, ng
				ih      = hfun(ig)
				iu      = ufun(ig)
				igshock = gfun(ih,2)
				!Homeowners with a mortgage
				call omp_set_num_threads(num_threads_simul)
				!$omp parallel private(id,iy,isprime,ieprime,yprime_f0,iy0low,iy0upp,y0weight)
				!$omp do
				do im = 2, nm
					id = omp_get_thread_num() + 1
					do iy = ny_neg + 1, ny
						if (iu == 1 .AND. nu == 2) then
							do isprime = 1, ns
								do ieprime = 1, ne
									yprime_f0 = labor(isprime,ieprime)*wprime(iz,iagg,iz) + bown_f0(iy,im,ig,is,iz,iagg)
									iy0low    = ly(yprime_f0)
									iy0upp    = uy(yprime_f0)
									y0weight  = wy(yprime_f0)
									!House fully depreciates
									mu1rent_f0new_temp(iy0low,isprime,id) = mu1rent_f0new_temp(iy0low,isprime,id)&
									+ (1 - y0weight)*deprec_h*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3own_f0(iy,im,ig,is)
									mu1rent_f0new_temp(iy0upp,isprime,id) = mu1rent_f0new_temp(iy0upp,isprime,id) + y0weight&
									*deprec_h*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3own_f0(iy,im,ig,is)
									!House does not depreciate
									!No housing shock
									mu1own_f0new_temp(iy0low,im,ig,isprime,id) = mu1own_f0new_temp(iy0low,im,ig,isprime,id)&
									+ (1 - y0weight)*(1 - deprec_h)*prob_ss(is,isprime,iz,iz)*(1 - pseparate)*emeas(ieprime)&
									*mu3own_f0(iy,im,ig,is)
									mu1own_f0new_temp(iy0upp,im,ig,isprime,id) = mu1own_f0new_temp(iy0upp,im,ig,isprime,id)&
									+ y0weight*(1 - deprec_h)*prob_ss(is,isprime,iz,iz)*(1 - pseparate)*emeas(ieprime)*mu3own_f0(iy,im,ig,is)
									!Housing shock
									mu1own_f0new_temp(iy0low,im,igshock,isprime,id) = mu1own_f0new_temp(iy0low,im,igshock,isprime,id)&
									+ (1 - y0weight)*(1 - deprec_h)*prob_ss(is,isprime,iz,iz)*pseparate*emeas(ieprime)&
									*mu3own_f0(iy,im,ig,is)
									mu1own_f0new_temp(iy0upp,im,igshock,isprime,id) = mu1own_f0new_temp(iy0upp,im,igshock,isprime,id)&
									+ y0weight*(1 - deprec_h)*prob_ss(is,isprime,iz,iz)*pseparate*emeas(ieprime)*mu3own_f0(iy,im,ig,is)
								enddo
							enddo
						else
							do isprime = 1, ns
								do ieprime = 1, ne
									yprime_f0 = labor(isprime,ieprime)*wprime(iz,iagg,iz) + bown_f0(iy,im,ig,is,iz,iagg)
									iy0low    = ly(yprime_f0)
									iy0upp    = uy(yprime_f0)
									y0weight  = wy(yprime_f0)
									!House fully depreciates
									mu1rent_f0new_temp(iy0low,isprime,id) = mu1rent_f0new_temp(iy0low,isprime,id)&
									+ (1 - y0weight)*deprec_h*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3own_f0(iy,im,ig,is)
									mu1rent_f0new_temp(iy0upp,isprime,id) = mu1rent_f0new_temp(iy0upp,isprime,id) + y0weight&
									*deprec_h*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3own_f0(iy,im,ig,is)
									!House does not depreciate
									mu1own_f0new_temp(iy0low,im,ig,isprime,id) = mu1own_f0new_temp(iy0low,im,ig,isprime,id)&
									+ (1 - y0weight)*(1 - deprec_h)*prob_ss(is,isprime,iz,iz)*emeas(ieprime)&
									*mu3own_f0(iy,im,ig,is)
									mu1own_f0new_temp(iy0upp,im,ig,isprime,id) = mu1own_f0new_temp(iy0upp,im,ig,isprime,id)&
									+ y0weight*(1 - deprec_h)*prob_ss(is,isprime,iz,iz)*emeas(ieprime)*mu3own_f0(iy,im,ig,is)
								enddo
							enddo
						endif
					enddo  
				enddo
				!$omp end do
				!$omp end parallel
			enddo
		enddo
		do id = 1, num_threads_simul
			mu1rent_f0new = mu1rent_f0new + mu1rent_f0new_temp(:,:,id)
			mu1rent_f1new = mu1rent_f1new + mu1rent_f1new_temp(:,:,id)
			mu1own_f0new  = mu1own_f0new + mu1own_f0new_temp(:,:,:,:,id)
			mu1own_f1new  = mu1own_f1new + mu1own_f1new_temp(:,:,:,id)
		enddo
		err_mu = max(maxval(abs(mu1own_f0new - mu1own_f0)),maxval(abs(mu1own_f1new - mu1own_f1)),&
		maxval(abs(mu1rent_f0new - mu1rent_f0)),maxval(abs(mu1rent_f1new - mu1rent_f1)))
		if (iter_mu - 1 == 50*floor((real(iter_mu)-1)/50.0)) then
			write(*, '(a19, i4, a10, f11.8)'), '          Iter_mu =', iter_mu, ', err_mu =', err_mu
		endif
		mu1own_f0  = mu1own_f0new
		mu1own_f1  = mu1own_f1new
		mu1rent_f0 = mu1rent_f0new
		mu1rent_f1 = mu1rent_f1new
		!Rescale distributions so that the total measure of agents is exactly 1
		meas_agents = sum(mu1own_f0) + sum(mu1own_f1) + sum(mu1rent_f0) + sum(mu1rent_f1)
!	    print *, meas_agents
		mu1own_f0   = mu1own_f0/meas_agents
		mu1own_f1   = mu1own_f1/meas_agents
		mu1rent_f0  = mu1rent_f0/meas_agents
		mu1rent_f1  = mu1rent_f1/meas_agents
		if (err_mu < 1e-3/real(ny*nm*ng*ns)) then
			exit
		endif
	enddo
	
	!Compute stationary REO housing stock
	htot_reo_new  = 0.d0
	htot_reo_temp = 0.d0
	do is = 1, ns
		do ig = 1, ng
			ih = hfun(ig)
			!Homeowners with a mortgage
			call omp_set_num_threads(num_threads_simul)
			!$omp parallel private(id,iy)
			!$omp do
			do im = 2, nm
				id = omp_get_thread_num() + 1
				do iy = ny_neg + 1, ny
					htot_reo_temp(ih,id) = htot_reo_temp(ih,id) + (1 - psell_f0(iy,im,ig,is,iz,iagg))&
					*d_f0(iy,im,ig,is,iz,iagg)*mu1own_f0(iy,im,ig,is)
				enddo
			enddo
			!$omp end do
			!$omp end parallel
		enddo
	enddo	
	do id = 1, num_threads_simul
		htot_reo_new = htot_reo_new + htot_reo_temp(:,id)
	enddo
	do ih = 1, nh
		htot_reo(ih) = htot_reo_new(ih)*(1 - psell_reo(ih,iz,iagg))*(1 - deprec_h)/(1 - (1 - psell_reo(ih,iz,iagg))*(1 - deprec_h))
	enddo

end subroutine sub_stationary