subroutine sub_timeseries(it,iz,phe,kstock,qbe,kstockprime,yh,yc,ownrate,assetsprime,invest_h,cons,hcons)
	
	use mod_globvar
	use mod_interpolate

	implicit none
	
	!This subroutine computes aggregate variables for the time series
	!Note: financial variables are not continually re-normalized
	!Note: variables with tot_ in front are simply unconditional means (e.g. tot_mortdebt is mean mortgage debt,
	!*unconditional* on whether the household has a mortgage or is even a homeowner)
	
	integer, intent(in) :: it, iz
	real(8), intent(in) :: phe, kstock, qbe, kstockprime, yh, yc, ownrate, assetsprime, invest_h, cons, hcons
		
	!Indices
	integer :: is, ig, ih, im, iy, iinterp
	
	!Other internal variables
	real(8) :: hownstock, invest_k, kh, gdp, networth, hwealth, mortdebt, foreclosures, frate, turnover, existsales, inventories, reosales, months_supply, tradevol, &
	reosaleshare, newsaleshare, salesrate, mean_TOMbuy, meas_buy, mean_TOMsell, meas_sell, qm0_orig, num_orig, defaultprem, defaultprem_ltv80, defaultprem_ltv90, defaultprem_ltv95, &
	num_orig_overall, num_orig_ltv80, num_orig_ltv90, num_orig_ltv95
	integer, dimension(ninterp) :: iagg
	real(8), dimension(ninterp) :: wagg
	
	!!!Welfare analysis of recourse policy
	real(8) :: welfare_change_all, welfare_change_renters, welfare_change_lowltv, welfare_change_ltv80, welfare_change_ltv90, welfare_change_ltv95, meas
	
	!!!Buyer wealth quintiles
	real(8) :: meas_buyers, buyerwealth_q1, buyerwealth_q2, buyerwealth_q3, buyerwealth_q4, buyerwealth_q5
	
	!Linearly interpolate aggregate state
	iagg(1) = aggfun(lp(phe),lk(kstock))
	iagg(2) = aggfun(up(phe),lk(kstock))
	iagg(3) = aggfun(lp(phe),uk(kstock))
	iagg(4) = aggfun(up(phe),uk(kstock))
	wagg(1) = (1 - wp(phe))*(1 - wk(kstock))
	wagg(2) = wp(phe)*(1 - wk(kstock))
	wagg(3) = (1 - wp(phe))*wk(kstock)
	wagg(4) = wp(phe)*wk(kstock)
	
	!------------------------------------------------
	!Output, consumption, and investment
	!------------------------------------------------
	!Owner-occupied housing stock
	hownstock = 0.d0
	do ig = 1, ng
		ih = hfun(ig)
		hownstock = hownstock + hh(ih)*(sum(mu3own_f0(:,:,ig,:)) + sum(mu3own_f1(:,ig,:)))
	enddo
	
	!Capital investment
	invest_k = kstockprime - (1 - deprec_k)*kstock
	
	!Output
	kh  = ((yh/zh(iz))**(1/(1 - lshare))/((eavg - (yc/(zc(iz)*kstock**kshare_c))**(1/(1 - kshare_c)))**(1 - kshare_h)))**(1/kshare_h)
	gdp = yc - kh + phe*yh + pa*hownstock
	
	!------------------------------------------------
	!Financial portfolio statistics
	!------------------------------------------------
	!Mean net worth over mean earnings (at the end of subperiod 3) (HOUSES - MRTHEL + FIN - RETQLIQ + VEHIC - INSTALL - CCBAL - OTHLOC - ODEBT)
	networth = 0.d0
	do is = 1, ns
		!Renters
		do iy = ny_neg + 1, ny
			!Renters with good credit
			networth = networth + dot_product(wagg,qbe*brent_f0(iy,is,iz,iagg))*mu3rent_f0(iy,is)
			!Renters with bad credit
			networth = networth + dot_product(wagg,qbe*brent_f1(iy,is,iz,iagg))*mu3rent_f1(iy,is)
		enddo
		!Homeowners
		do ig = 1, ng
			ih = hfun(ig)
			!Homeowners with bad credit
			do iy = ny_neg + 1, ny
				networth = networth + dot_product(wagg,qbe*bown_f1(iy,ig,is,iz,iagg) + phe*hh(ih))*mu3own_f1(iy,ig,is)
			enddo
			!Homeowners with good credit but no mortgage
			do iy = 1, ny
				networth = networth + dot_product(wagg,qbe*bown_f0(iy,1,ig,is,iz,iagg) + (phe - m_f0(iy,1,ig,is,iz,iagg))*hh(ih))*mu3own_f0(iy,1,ig,is)
			enddo
			do im = 2, nm
				!Homeowners with mortgages
				do iy = ny_neg + 1, ny
					networth = networth + dot_product(wagg,qbe*bown_f0(iy,im,ig,is,iz,iagg) + (phe - mm(im))*hh(ih))*mu3own_f0(iy,im,ig,is)
				enddo
			enddo
		enddo
	enddo
	
	!Housing wealth
	hwealth = phe*hownstock
	
	!Mortgage debt
	mortdebt = 0.d0
	do is = 1, ns
		do ig = 1, ng
			ih = hfun(ig)
			!Homeowners with good credit but no mortgage
			do iy = 1, ny
				mortdebt = mortdebt + dot_product(wagg,m_f0(iy,1,ig,is,iz,iagg)*hh(ih))*mu3own_f0(iy,1,ig,is)
			enddo
			do im = 2, nm
				!Homeowners with mortgages
				do iy = ny_neg + 1, ny
					mortdebt = mortdebt + mm(im)*hh(ih)*mu3own_f0(iy,im,ig,is)
				enddo
			enddo
		enddo
	enddo
	
	!------------------------------------------------
	!Sales, inventories, and foreclosures
	!------------------------------------------------
	!Foreclosures, foreclosure rate, and average foreclosure recovery ratio
	if (sum(mu1own_f0(:,2:nm,:,:)) > 0.d0) then
		foreclosures = 0.d0
		do is = 1, ns
			do ig = 1, ng
				ih = hfun(ig)
				do im = 2, nm
					do iy = ny_neg + 1, ny
						foreclosures = foreclosures + dot_product(wagg,(1 - psell_f0(iy,im,ig,is,iz,iagg))*d_f0(iy,im,ig,is,iz,iagg))*mu1own_f0(iy,im,ig,is)
					enddo
				enddo
			enddo
		enddo
		frate = foreclosures/sum(mu1own_f0(:,2:nm,:,:))
	else
		frate = 0.d0
	endif
	
	!Turnover (measure of people buying a house divided by measure of homeowners at the end of subperiod 3)
	turnover = 0.d0
	do is = 1, ns
		do iy = ny_neg + 1, ny
			!Renters with good credit
			turnover = turnover + dot_product(wagg,pbuy_f0(iy,is,iz,iagg))*mu2rent_f0(iy,is)
			!Renters with bad credit
			turnover = turnover + dot_product(wagg,pbuy_f1(iy,is,iz,iagg))*mu2rent_f1(iy,is)
		enddo
	enddo
	turnover = turnover/ownrate
	
	!Sales volume, inventories, months of supply, REO sales share, and sales rate
	!Sales is the measure of housing traded, inventories is the measure of housing put up for sale but not sold,
	!months of supply is annualized inventories/sales, REO sales share is REO sales volume as a share of existing home
	!sales, and sales rate is existing sales as a share of the housing stock
	existsales  = 0.d0
	inventories = 0.d0
	reosales    = 0.d0
	do is = 1, ns
		do ig = 1, ng
			ih = hfun(ig)
			!Homeowners with bad credit
			do iy = ny_neg + 1, ny
				do iinterp = 1, ninterp
					if (psell_f1(iy,ig,is,iz,iagg(iinterp)) > 0.d0) then
						existsales  = existsales + wagg(iinterp)*psell_f1(iy,ig,is,iz,iagg(iinterp))*hh(ih)*mu1own_f1(iy,ig,is)
						inventories = inventories + wagg(iinterp)*(1 - psell_f1(iy,ig,is,iz,iagg(iinterp)))*hh(ih)*mu1own_f1(iy,ig,is)
					endif
				enddo
			enddo
			!Homeowners with good credit
			do im = 1, nm
				do iy = ny_neg + 1, ny
					do iinterp = 1, ninterp
						!Owners who voluntarily sell
						if (psell_f0(iy,im,ig,is,iz,iagg(iinterp)) > 0.d0) then
							existsales  = existsales + wagg(iinterp)*psell_f0(iy,im,ig,is,iz,iagg(iinterp))*hh(ih)*mu1own_f0(iy,im,ig,is)
							inventories = inventories + wagg(iinterp)*(1 - psell_f0(iy,im,ig,is,iz,iagg(iinterp)))*hh(ih)*mu1own_f0(iy,im,ig,is)
						endif
						!Owners who default
						if (psell_reo(ih,iz,iagg(iinterp)) > 0.d0) then
							!Sales volume and inventories from new REOs
							existsales  = existsales + wagg(iinterp)*(1 - psell_f0(iy,im,ig,is,iz,iagg(iinterp)))*d_f0(iy,im,ig,is,iz,iagg(iinterp))*mu1own_f0(iy,im,ig,is)&
							*hh(ih)*psell_reo(ih,iz,iagg(iinterp))
							reosales    = reosales + wagg(iinterp)*(1 - psell_f0(iy,im,ig,is,iz,iagg(iinterp)))*d_f0(iy,im,ig,is,iz,iagg(iinterp))*mu1own_f0(iy,im,ig,is)&
							*hh(ih)*psell_reo(ih,iz,iagg(iinterp))
							inventories = inventories + wagg(iinterp)*(1 - psell_f0(iy,im,ig,is,iz,iagg(iinterp)))*d_f0(iy,im,ig,is,iz,iagg(iinterp))*mu1own_f0(iy,im,ig,is)&
							*hh(ih)*(1 - psell_reo(ih,iz,iagg(iinterp)))
						endif
					enddo
				enddo
			enddo
		enddo
	enddo
	!Sales volume and inventories from existing REOs
	do ih = 1, nh
		do iinterp = 1, ninterp
			existsales  = existsales + wagg(iinterp)*htot_reo(ih)*hh(ih)*psell_reo(ih,iz,iagg(iinterp))
			reosales    = reosales + wagg(iinterp)*htot_reo(ih)*hh(ih)*psell_reo(ih,iz,iagg(iinterp))
			inventories = inventories + wagg(iinterp)*htot_reo(ih)*hh(ih)*(1 - psell_reo(ih,iz,iagg(iinterp)))
		enddo
	enddo
	months_supply = inventories/(4.0*existsales)*12.0
	tradevol      = existsales + yh
	reosaleshare  = reosales/tradevol
	newsaleshare  = yh/tradevol
	salesrate     = 4.0*existsales/hownstock
	
	!Mean buyer search duration
	mean_TOMbuy = 0.d0
	meas_buy    = 0.d0
	do is = 1, ns
		do iy = ny_neg + 1, ny
			do iinterp = 1, ninterp
				if (pbuy_f0(iy,is,iz,iagg(iinterp)) > 0.d0) then
					mean_TOMbuy = mean_TOMbuy + wagg(iinterp)*((1 - pbuy_f0(iy,is,iz,iagg(iinterp)))/pbuy_f0(iy,is,iz,iagg(iinterp)) + 0.5)*12.0*mu2rent_f0(iy,is)
					meas_buy    = meas_buy + wagg(iinterp)*mu2rent_f0(iy,is)
				endif
				if (pbuy_f1(iy,is,iz,iagg(iinterp)) > 0.d0) then
					mean_TOMbuy = mean_TOMbuy + wagg(iinterp)*((1 - pbuy_f1(iy,is,iz,iagg(iinterp)))/pbuy_f1(iy,is,iz,iagg(iinterp)) + 0.5)*12.0*mu2rent_f1(iy,is)
					meas_buy    = meas_buy + wagg(iinterp)*mu2rent_f1(iy,is)
				endif				
			enddo
		enddo
	enddo
	if (meas_buy > 0.d0) then
		mean_TOMbuy = mean_TOMbuy/meas_buy
	else
		mean_TOMbuy = 0.d0
	endif
	
	!Mean seller search duration
	mean_TOMsell = 0.d0
	meas_sell    = 0.d0
	do is = 1, ns
		do ig = 1, ng
			!Sellers with bad credit
			do iy = ny_neg + 1, ny
				do iinterp = 1, ninterp
					if (psell_f1(iy,ig,is,iz,iagg(iinterp)) > 0.d0) then
						mean_TOMsell = mean_TOMsell + wagg(iinterp)*((1 - psell_f1(iy,ig,is,iz,iagg(iinterp)))/psell_f1(iy,ig,is,iz,iagg(iinterp)) + 0.5)*12.0*mu1own_f1(iy,ig,is)
						meas_sell    = meas_sell + wagg(iinterp)*mu1own_f1(iy,ig,is)
					endif
				enddo
			enddo
			!Sellers with good credit
			do im = 1, nm
				do iy =  ny_neg + 1, ny
					do iinterp = 1, ninterp
						if (psell_f0(iy,im,ig,is,iz,iagg(iinterp)) > 0.d0) then
							mean_TOMsell = mean_TOMsell + wagg(iinterp)*((1 - psell_f0(iy,im,ig,is,iz,iagg(iinterp)))/psell_f0(iy,im,ig,is,iz,iagg(iinterp)) + 0.5)*12.0*mu1own_f0(iy,im,ig,is)
							meas_sell    = meas_sell + wagg(iinterp)*mu1own_f0(iy,im,ig,is)
						endif
					enddo
				enddo
			enddo
		enddo
	enddo
	if (meas_sell > 0.d0) then
		mean_TOMsell = mean_TOMsell/meas_sell
	else
		mean_TOMsell = 0.d0
	endif
	
	!Compute average mortgage prices
	!Overall, at origination
	qm0_orig = 0.d0
	num_orig = 0.d0
	do is = 1, ns
		do ig = 1, ng
			do iy = 1, ny
				do iinterp = 1, ninterp
					qm0_orig = qm0_orig + wagg(iinterp)*(qm(iz,iagg(iinterp))/(qm0(locm_f0(iy,1,ig,is,iz,iagg(iinterp)),locbown_f0(iy,1,ig,is,iz,iagg(iinterp)),ig,is,iz,iagg(iinterp))*(1 + rocost)) - 1)*mu3own_f0(iy,1,ig,is)
					num_orig = num_orig + wagg(iinterp)*mu3own_f0(iy,1,ig,is)
				enddo
			enddo
		enddo
	enddo
	defaultprem = qm0_orig/num_orig
	num_orig_overall = num_orig
	
	!For mortgages with ltv > 80%, at origination
	qm0_orig = 0.d0
	num_orig = 0.d0
	do is = 1, ns
		do ig = 1, ng
			do iy = 1, ny
				do iinterp = 1, ninterp
					if (qm0(locm_f0(iy,1,ig,is,iz,iagg(iinterp)),locbown_f0(iy,1,ig,is,iz,iagg(iinterp)),ig,is,iz,iagg(iinterp))*m_f0(iy,1,ig,is,iz,iagg(iinterp))/ph(iagg(iinterp)) >= 0.8d0) then
						qm0_orig = qm0_orig + wagg(iinterp)*(qm(iz,iagg(iinterp))/(qm0(locm_f0(iy,1,ig,is,iz,iagg(iinterp)),locbown_f0(iy,1,ig,is,iz,iagg(iinterp)),ig,is,iz,iagg(iinterp))*(1 + rocost)) - 1)*mu3own_f0(iy,1,ig,is)
						num_orig = num_orig + wagg(iinterp)*mu3own_f0(iy,1,ig,is)
					endif
				enddo
			enddo
		enddo
	enddo
	if (num_orig > 0.d0) then
		defaultprem_ltv80 = qm0_orig/num_orig
	else
		defaultprem_ltv80 = 0.d0
	endif
	num_orig_ltv80 = num_orig/num_orig_overall
	
	!For mortgages with ltv > 90%, at origination
	qm0_orig = 0.d0
	num_orig = 0.d0
	do is = 1, ns
		do ig = 1, ng
			do iy = 1, ny
				do iinterp = 1, ninterp
					if (qm0(locm_f0(iy,1,ig,is,iz,iagg(iinterp)),locbown_f0(iy,1,ig,is,iz,iagg(iinterp)),ig,is,iz,iagg(iinterp))*m_f0(iy,1,ig,is,iz,iagg(iinterp))/ph(iagg(iinterp)) >= 0.90d0) then
						qm0_orig = qm0_orig + wagg(iinterp)*(qm(iz,iagg(iinterp))/(qm0(locm_f0(iy,1,ig,is,iz,iagg(iinterp)),locbown_f0(iy,1,ig,is,iz,iagg(iinterp)),ig,is,iz,iagg(iinterp))*(1 + rocost)) - 1)*mu3own_f0(iy,1,ig,is)
						num_orig = num_orig + wagg(iinterp)*mu3own_f0(iy,1,ig,is)
					endif
				enddo
			enddo
		enddo
	enddo
	if (num_orig > 0.d0) then
		defaultprem_ltv90 = qm0_orig/num_orig
	else
		defaultprem_ltv90 = 0.d0
	endif
	num_orig_ltv90 = num_orig/num_orig_overall
	
	!For mortgages with ltv > 95%, at origination
	qm0_orig = 0.d0
	num_orig = 0.d0
	do is = 1, ns
		do ig = 1, ng
			do iy = 1, ny
				do iinterp = 1, ninterp
					if (qm0(locm_f0(iy,1,ig,is,iz,iagg(iinterp)),locbown_f0(iy,1,ig,is,iz,iagg(iinterp)),ig,is,iz,iagg(iinterp))*m_f0(iy,1,ig,is,iz,iagg(iinterp))/ph(iagg(iinterp)) >= 0.95d0) then
						qm0_orig = qm0_orig + wagg(iinterp)*(qm(iz,iagg(iinterp))/(qm0(locm_f0(iy,1,ig,is,iz,iagg(iinterp)),locbown_f0(iy,1,ig,is,iz,iagg(iinterp)),ig,is,iz,iagg(iinterp))*(1 + rocost)) - 1)*mu3own_f0(iy,1,ig,is)
						num_orig = num_orig + wagg(iinterp)*mu3own_f0(iy,1,ig,is)
					endif
				enddo
			enddo
		enddo
	enddo
	if (num_orig > 0.d0) then
		defaultprem_ltv95 = qm0_orig/num_orig
	else
		defaultprem_ltv95 = 0.d0
	endif
	num_orig_ltv95 = num_orig/num_orig_overall
	
	!!!----------------------------------- Welfare effect of recourse law ----------------------------------- !!!
	!Average welfare gain -- all agents
	welfare_change_all = 0.d0
	do is = 1, ns
		do ig = 1, ng
			!Owners with bad credit
			do iy = ny_neg + 1, ny
				do iinterp = 1, ninterp
					welfare_change_all = welfare_change_all + wagg(iinterp)*100.d0*(Wown_f1_new(iy,ig,is,iz,iagg(iinterp))/Wown_f1(iy,ig,is,iz,iagg(iinterp)) - 1.d0)*mu1own_f1(iy,ig,is)
				enddo
			enddo
			do im = 1, nm
				!Owners with good credit
				do iy = ny_neg + 1, ny
					do iinterp = 1, ninterp
						welfare_change_all = welfare_change_all + wagg(iinterp)*100.d0*(Wown_f0_new(iy,im,ig,is,iz,iagg(iinterp))/Wown_f0(iy,im,ig,is,iz,iagg(iinterp)) - 1.d0)*mu1own_f0(iy,im,ig,is)
					enddo
				enddo
			enddo
		enddo
		!Renters
		do iy = ny_neg + 1, ny
			do iinterp = 1, ninterp
				welfare_change_all = welfare_change_all + wagg(iinterp)*100.d0*(Wrent_f0_new(iy,is,iz,iagg(iinterp))/Wrent_f0(iy,is,iz,iagg(iinterp)) - 1.d0)*mu1rent_f0(iy,is)
				welfare_change_all = welfare_change_all + wagg(iinterp)*100.d0*(Wrent_f1_new(iy,is,iz,iagg(iinterp))/Wrent_f1(iy,is,iz,iagg(iinterp)) - 1.d0)*mu1rent_f1(iy,is)
			enddo
		enddo
	enddo
	
	!Average welfare gain -- renters with good credit
	welfare_change_renters = 0.d0
	meas				   = 0.d0
	do is = 1, ns
		!Renters
		do iy = ny_neg + 1, ny
			do iinterp = 1, ninterp
				welfare_change_renters = welfare_change_renters + wagg(iinterp)*100.d0*(Wrent_f0_new(iy,is,iz,iagg(iinterp))/Wrent_f0(iy,is,iz,iagg(iinterp)) - 1.d0)*mu1rent_f0(iy,is)
			enddo
			meas = meas + mu1rent_f0(iy,is)
		enddo
	enddo
	if (meas > 0.d0) then
		welfare_change_renters = welfare_change_renters/meas
	endif
	
	!Average welfare gain -- homeowners with LTV < 80%
	welfare_change_lowltv = 0.d0
	meas                  = 0.d0
	do is = 1, ns
		do ig = 1, ng
			do im = 1, nm
				!Owners with good credit
				do iy = ny_neg + 1, ny
					do iinterp = 1, ninterp
						if (mm(im)/ph(iagg(iinterp)) < 0.8d0) then
							welfare_change_lowltv = welfare_change_lowltv + wagg(iinterp)*100.d0*(Wown_f0_new(iy,im,ig,is,iz,iagg(iinterp))/Wown_f0(iy,im,ig,is,iz,iagg(iinterp)) - 1.d0)*mu1own_f0(iy,im,ig,is)
							meas                  = meas + wagg(iinterp)*mu1own_f0(iy,im,ig,is)
						endif
					enddo
				enddo
			enddo
		enddo
	enddo
	if (meas > 0.d0) then
		welfare_change_lowltv = welfare_change_lowltv/meas
	endif
	
	!Average welfare gain -- homeowners with LTV > 80%
	welfare_change_ltv80 = 0.d0
	meas                 = 0.d0
	do is = 1, ns
		do ig = 1, ng
			do im = 1, nm
				!Owners with good credit
				do iy = ny_neg + 1, ny
					do iinterp = 1, ninterp
						if (mm(im)/ph(iagg(iinterp)) >= 0.8d0) then
							welfare_change_ltv80 = welfare_change_ltv80 + wagg(iinterp)*100.d0*(Wown_f0_new(iy,im,ig,is,iz,iagg(iinterp))/Wown_f0(iy,im,ig,is,iz,iagg(iinterp)) - 1.d0)*mu1own_f0(iy,im,ig,is)
							meas                 = meas + wagg(iinterp)*mu1own_f0(iy,im,ig,is)
						endif
					enddo
				enddo
			enddo
		enddo
	enddo
	if (meas > 0.d0) then
		welfare_change_ltv80 = welfare_change_ltv80/meas
	endif
	
	!Average welfare gain -- homeowners with LTV > 90%
	welfare_change_ltv90 = 0.d0
	meas                 = 0.d0
	do is = 1, ns
		do ig = 1, ng
			do im = 1, nm
				!Owners with good credit
				do iy = ny_neg + 1, ny
					do iinterp = 1, ninterp
						if (mm(im)/ph(iagg(iinterp)) >= 0.9d0) then
							welfare_change_ltv90 = welfare_change_ltv90 + wagg(iinterp)*100.d0*(Wown_f0_new(iy,im,ig,is,iz,iagg(iinterp))/Wown_f0(iy,im,ig,is,iz,iagg(iinterp)) - 1.d0)*mu1own_f0(iy,im,ig,is)
							meas                 = meas + wagg(iinterp)*mu1own_f0(iy,im,ig,is)
						endif
					enddo
				enddo
			enddo
		enddo
	enddo
	if (meas > 0.d0) then
		welfare_change_ltv90 = welfare_change_ltv90/meas
	endif
	
	!Average welfare gain -- homeowners with LTV > 95%
	welfare_change_ltv95 = 0.d0
	meas                 = 0.d0
	do is = 1, ns
		do ig = 1, ng
			do im = 1, nm
				!Owners with good credit
				do iy = ny_neg + 1, ny
					do iinterp = 1, ninterp
						if (mm(im)/ph(iagg(iinterp)) >= 0.95d0) then
							welfare_change_ltv95 = welfare_change_ltv95 + wagg(iinterp)*100.d0*(Wown_f0_new(iy,im,ig,is,iz,iagg(iinterp))/Wown_f0(iy,im,ig,is,iz,iagg(iinterp)) - 1.d0)*mu1own_f0(iy,im,ig,is)
							meas                 = meas + wagg(iinterp)*mu1own_f0(iy,im,ig,is)
						endif
					enddo
				enddo
			enddo
		enddo
	enddo
	if (meas > 0.d0) then
		welfare_change_ltv95 = welfare_change_ltv95/meas
	endif
	
	!!!----------------------------------- Buyer wealth quintiles ----------------------------------- !!!
	!Total number of buyers
	meas_buyers = 0.d0
	do is = 1, ns
		do iy = ny_neg + 1, ny
			do iinterp = 1, ninterp
				meas_buyers = meas_buyers + wagg(iinterp)*pbuy_f0(iy,is,iz,iagg(iinterp))*mu1rent_f0(iy,is)
				meas_buyers = meas_buyers + wagg(iinterp)*pbuy_f1(iy,is,iz,iagg(iinterp))*mu1rent_f1(iy,is)
			enddo
		enddo
	enddo
	
	!Buyer wealth quintiles	
	buyerwealth_q1 = 0.d0
	meas           = 0.d0
	do iy = ny_neg + 1, ny
		do iinterp = 1, ninterp
			meas = meas + wagg(iinterp)*(dot_product(pbuy_f0(iy,:,iz,iagg(iinterp)),mu1rent_f0(iy,:)) + dot_product(pbuy_f1(iy,:,iz,iagg(iinterp)),mu1rent_f1(iy,:)))
		enddo
		if (meas > (0.1d0 - 1e-6)*meas_buyers) then
			buyerwealth_q1 = yy(iy)
			exit
		endif
	enddo
	buyerwealth_q2 = 0.d0
	meas           = 0.d0
	do iy = ny_neg + 1, ny
		do iinterp = 1, ninterp
			meas = meas + wagg(iinterp)*(dot_product(pbuy_f0(iy,:,iz,iagg(iinterp)),mu1rent_f0(iy,:)) + dot_product(pbuy_f1(iy,:,iz,iagg(iinterp)),mu1rent_f1(iy,:)))
		enddo
		if (meas > (0.3d0 - 1e-6)*meas_buyers) then
			buyerwealth_q2 = yy(iy)
			exit
		endif
	enddo
	buyerwealth_q3 = 0.d0
	meas           = 0.d0
	do iy = ny_neg + 1, ny
		do iinterp = 1, ninterp
			meas = meas + wagg(iinterp)*(dot_product(pbuy_f0(iy,:,iz,iagg(iinterp)),mu1rent_f0(iy,:)) + dot_product(pbuy_f1(iy,:,iz,iagg(iinterp)),mu1rent_f1(iy,:)))
		enddo
		if (meas > (0.5d0 - 1e-6)*meas_buyers) then
			buyerwealth_q3 = yy(iy)
			exit
		endif
	enddo
	buyerwealth_q4 = 0.d0
	meas           = 0.d0
	do iy = ny_neg + 1, ny
		do iinterp = 1, ninterp
			meas = meas + wagg(iinterp)*(dot_product(pbuy_f0(iy,:,iz,iagg(iinterp)),mu1rent_f0(iy,:)) + dot_product(pbuy_f1(iy,:,iz,iagg(iinterp)),mu1rent_f1(iy,:)))
		enddo
		if (meas > (0.7d0 - 1e-6)*meas_buyers) then
			buyerwealth_q4 = yy(iy)
			exit
		endif
	enddo
	buyerwealth_q5 = 0.d0
	meas           = 0.d0
	do iy = ny_neg + 1, ny
		do iinterp = 1, ninterp
			meas = meas + wagg(iinterp)*(dot_product(pbuy_f0(iy,:,iz,iagg(iinterp)),mu1rent_f0(iy,:)) + dot_product(pbuy_f1(iy,:,iz,iagg(iinterp)),mu1rent_f1(iy,:)))
		enddo
		if (meas > (0.9d0 - 1e-6)*meas_buyers) then
			buyerwealth_q5 = yy(iy)
			exit
		endif
	enddo
	
	!Output
	!!!Make sure number of series equals numseries!!!
	timeseries(it,1)  = phe
	timeseries(it,2)  = qbe
	timeseries(it,3)  = yh
	timeseries(it,4)  = yc
	timeseries(it,5)  = gdp
	timeseries(it,6)  = invest_h
	timeseries(it,7)  = invest_k
	timeseries(it,8)  = cons
	timeseries(it,9)  = pa*(hcons + hownstock)
	timeseries(it,10) = kstock
	timeseries(it,11) = hownstock
	timeseries(it,12) = networth
	timeseries(it,13) = assetsprime
	timeseries(it,14) = hwealth
	timeseries(it,15) = mortdebt
	timeseries(it,16) = ownrate
	timeseries(it,17) = turnover
	timeseries(it,18) = tradevol
	timeseries(it,19) = existsales
	timeseries(it,20) = reosales
	timeseries(it,21) = reosaleshare
	timeseries(it,22) = newsaleshare
	timeseries(it,23) = salesrate
	timeseries(it,24) = inventories
	timeseries(it,25) = months_supply
	timeseries(it,26) = frate
	timeseries(it,27) = mean_TOMbuy
	timeseries(it,28) = mean_TOMsell
	timeseries(it,29) = defaultprem
	timeseries(it,30) = defaultprem_ltv80
	timeseries(it,31) = defaultprem_ltv90
	timeseries(it,32) = defaultprem_ltv95
	timeseries(it,33) = num_orig_ltv80
	timeseries(it,34) = num_orig_ltv90
	timeseries(it,35) = num_orig_ltv95
	timeseries(it,36) = welfare_change_all
	timeseries(it,37) = welfare_change_renters
	timeseries(it,38) = welfare_change_lowltv
	timeseries(it,39) = welfare_change_ltv80
	timeseries(it,40) = welfare_change_ltv90
	timeseries(it,41) = welfare_change_ltv95
	timeseries(it,42) = buyerwealth_q1
	timeseries(it,43) = buyerwealth_q2
	timeseries(it,44) = buyerwealth_q3
	timeseries(it,45) = buyerwealth_q4
	timeseries(it,46) = buyerwealth_q5
    
end subroutine sub_timeseries