subroutine sub_valfn_ez_ces

	use mod_globvar
	use mod_interpolate

	implicit none

	!This program computes value function iteration
	!Features: 1) Owners choose between living in their house or renting an apartment; 2) There is a utility cost to failing to sell.
	!Note 1: Since all value functions are > 0, owners do not attempt to sell if searchcost > Vown.
	!Note 2: It is easy to modify the setup to have a utility cost to entering the selling market, as was previously the case.

	!Indices for today's state
	integer :: iz, iagg, is, ig, ih, iu, im, iy
	!Indices for tomorrow's state
	integer :: izprime, igshock, isprime, ieprime
	!Indices for grid search during optimization
	integer :: imprime, ibprime, ix

	!Continuation utilities
	real(8), dimension(ne) :: Vcont_e_f0, Vcont_e_f1
	real(8), dimension(ns) :: Vcont_s_f0, Vcont_s_f1
	real(8), dimension(nz) :: Vcont_z_f0, Vcont_z_f1
	real(8), dimension(nb,nm) :: Vcont_own_f0
	real(8), dimension(nb) :: Vcont_rent_f0, Vcont_own_f1, Vcont_rent_f1
		
	!Updated value functions
	real(8), dimension(:,:,:,:), allocatable :: Vnew_rent_f0, Vnew_rent_f1
	real(8), dimension(:,:,:,:,:), allocatable :: Vnew_own_f1
	real(8), dimension(:,:,:,:,:,:), allocatable :: Vnew_own_f0, Wnosell_f0
		
	!Other internal variables
	real(8) :: err_valfn, yprime_f0, yprime_f1, V_maxsofar, c_maxsofar, hc_maxsofar, spend_maxsofar, spending, cstay, hcstay, ustay, cmove, hcmove, umove, c, hc, util, &
	val, cash_at_hand, ybuy, ydefault, wdefault, W_maxsofar, ypay, wval, ysell, y0weight, y1weight, yweight, minerr_noaccel, minerr_accel
	integer :: iter_valfn, b_maxsofar, m_maxsofar, ibprime_start, h_maxsofar, x_maxsofar, iy0low, iy0upp, iy1low, iy1upp, iylow, iyupp
	
	!Allocate updated value functions
	allocate(Vnew_rent_f0(ny,ns,nz,nagg))
	allocate(Vnew_rent_f1(ny,ns,nz,nagg))
	allocate(Vnew_own_f1(ny,ng,ns,nz,nagg))
	allocate(Vnew_own_f0(ny,nm,ng,ns,nz,nagg))
	allocate(Wnosell_f0(ny,nm,ng,ns,nz,nagg))
	
	print *, ' '
	print *, 'Compute value function iteration'
	print *, ' '
	minerr_noaccel = 1e8
	minerr_accel   = 1e8
	do iter_valfn = 1, itermax
		if (iter_valfn - 1 == 10*floor((real(iter_valfn)-1)/10.0) .OR. iter_valfn >= itermax_accel .OR. (minerr_noaccel < 10.0*toler .AND. minerr_accel < toler) .OR. minerr_accel < 0.5*toler) then
			!House buying subperiod
			do iagg = 1, nagg
				do iz = 1, nz
					do is = 1, ns
						call omp_set_num_threads(num_threads)
						!$omp parallel private(W_maxsofar,h_maxsofar,x_maxsofar,ih,ig,ix,ybuy,iylow,iyupp,yweight,wval)
						!$omp do
						do iy = ny_neg + 1, ny
							!Households with good credit
							W_maxsofar = -1e9
							h_maxsofar = 1
							x_maxsofar = 1
							do ih = 1, nh
								ig = gfun(ih,1)
								do ix = 2, nx
									ybuy = yy(iy) - xbuy(ix,ih,iagg)
									if (ybuy < 0.d0) then
										ybuy = ybuy/abs(ylb(ig,is,iz,iagg))
									endif
									iylow   = ly(ybuy)
									iyupp   = uy(ybuy)
									yweight = wy(ybuy)
									if (ybuy >= -1.d0) then
										wval = ((1 - prob_xb(ix))*Vrent_f0(iy,is,iz,iagg)**(1 - gamma) + prob_xb(ix)*((1 - yweight)*Vown_f0(iylow,1,ig,is,iz,iagg)**(1 - gamma)&
										+ yweight*Vown_f0(iyupp,1,ig,is,iz,iagg)**(1 - gamma)))**(1/(1 - gamma))
										if (wval > W_maxsofar) then
											W_maxsofar = wval
											h_maxsofar = ih
											x_maxsofar = ix
										endif
									else
										exit
									endif
								enddo
							enddo
							if (W_maxsofar > Vrent_f0(iy,is,iz,iagg)) then
								Wrent_f0(iy,is,iz,iagg)   = W_maxsofar
								loch_f0(iy,is,iz,iagg)    = h_maxsofar
								h_f0(iy,is,iz,iagg)       = hh(h_maxsofar)
								locxb_f0(iy,is,iz,iagg)   = x_maxsofar
								xb_f0(iy,is,iz,iagg)      = xbuy(x_maxsofar,h_maxsofar,iagg)
								xbratio_f0(iy,is,iz,iagg) = xbuy(x_maxsofar,h_maxsofar,iagg)/(ph(iagg)*hh(h_maxsofar))
								pbuy_f0(iy,is,iz,iagg)    = prob_xb(x_maxsofar)
							else
								Wrent_f0(iy,is,iz,iagg)   = Vrent_f0(iy,is,iz,iagg)
								loch_f0(iy,is,iz,iagg)    = 1			!Renters who don't buy any house choose ih = 1 and pbuy = 0
								h_f0(iy,is,iz,iagg)       = 0.d0
								locxb_f0(iy,is,iz,iagg)   = 1
								xb_f0(iy,is,iz,iagg)      = 0.d0
								xbratio_f0(iy,is,iz,iagg) = 0.d0
								pbuy_f0(iy,is,iz,iagg)    = 0.d0
							endif
							!Households with bad credit
							W_maxsofar = -1e9
							h_maxsofar = 1
							x_maxsofar = 1
							do ih = 1, nh
								ig = gfun(ih,1)
								do ix = 2, nx
									ybuy = yy(iy) - xbuy(ix,ih,iagg)
									iylow   = ly(ybuy)
									iyupp   = uy(ybuy)
									yweight = wy(ybuy)
									if (ybuy >= 0.d0) then
										wval = ((1 - prob_xb(ix))*Vrent_f1(iy,is,iz,iagg)**(1 - gamma) + prob_xb(ix)*((1 - yweight)*Vown_f1(iylow,ig,is,iz,iagg)**(1 - gamma)&
										+ yweight*Vown_f1(iyupp,ig,is,iz,iagg)**(1 - gamma)))**(1/(1 - gamma))
										if (wval > W_maxsofar) then
											W_maxsofar = wval
											h_maxsofar = ih
											x_maxsofar = ix
										endif
									else
										exit
									endif
								enddo
							enddo
							if (W_maxsofar > Vrent_f1(iy,is,iz,iagg)) then
								Wrent_f1(iy,is,iz,iagg)   = W_maxsofar
								loch_f1(iy,is,iz,iagg)    = h_maxsofar
								h_f1(iy,is,iz,iagg)       = hh(h_maxsofar)
								locxb_f1(iy,is,iz,iagg)   = x_maxsofar
								xb_f1(iy,is,iz,iagg)      = xbuy(x_maxsofar,h_maxsofar,iagg)
								xbratio_f1(iy,is,iz,iagg) = xbuy(x_maxsofar,h_maxsofar,iagg)/(ph(iagg)*hh(h_maxsofar))
								pbuy_f1(iy,is,iz,iagg)    = prob_xb(x_maxsofar)
							else
								Wrent_f1(iy,is,iz,iagg)   = Vrent_f1(iy,is,iz,iagg)
								loch_f1(iy,is,iz,iagg)    = 1			!Renters who don't buy any house choose ih = 1 and pbuy = 0
								h_f1(iy,is,iz,iagg)       = 0.d0
								locxb_f1(iy,is,iz,iagg)   = 1
								xb_f1(iy,is,iz,iagg)      = 0.d0
								xbratio_f1(iy,is,iz,iagg) = 0.d0
								pbuy_f1(iy,is,iz,iagg)    = 0.d0
							endif
						enddo
						!$omp end do
						!$omp end parallel
						do iy = 1, ny_neg
							Wrent_f0(iy,is,iz,iagg)	  = Wrent_f0(ny_neg + 1,is,iz,iagg)
							loch_f0(iy,is,iz,iagg)    = loch_f0(ny_neg + 1,is,iz,iagg)
							h_f0(iy,is,iz,iagg)       = h_f0(ny_neg + 1,is,iz,iagg)
							locxb_f0(iy,is,iz,iagg)   = locxb_f0(ny_neg + 1,is,iz,iagg)
							xb_f0(iy,is,iz,iagg)      = xb_f0(ny_neg + 1,is,iz,iagg)
							xbratio_f0(iy,is,iz,iagg) = xbratio_f0(ny_neg + 1,is,iz,iagg)
							pbuy_f0(iy,is,iz,iagg)    = pbuy_f0(ny_neg + 1,is,iz,iagg)
							Wrent_f1(iy,is,iz,iagg)	  = Wrent_f1(ny_neg + 1,is,iz,iagg)
							loch_f1(iy,is,iz,iagg)    = loch_f1(ny_neg + 1,is,iz,iagg)
							h_f1(iy,is,iz,iagg)       = h_f1(ny_neg + 1,is,iz,iagg)
							locxb_f1(iy,is,iz,iagg)   = locxb_f1(ny_neg + 1,is,iz,iagg)
							xb_f1(iy,is,iz,iagg)      = xb_f1(ny_neg + 1,is,iz,iagg)
							xbratio_f1(iy,is,iz,iagg) = xbratio_f1(ny_neg + 1,is,iz,iagg)
							pbuy_f1(iy,is,iz,iagg)    = pbuy_f1(ny_neg + 1,is,iz,iagg)
						enddo
					enddo
				enddo
			enddo
			!Mortgage foreclosure/payment subperiod
			do iagg = 1, nagg
				do iz = 1, nz
					do is = 1, ns
						do ig = 1, ng
							ih = hfun(ig)
							!Households without a mortgage
							do iy = 1, ny
								Wnosell_f0(iy,1,ig,is,iz,iagg) = Vown_f0(iy,1,ig,is,iz,iagg)
								d_f0(iy,1,ig,is,iz,iagg)       = 0.d0
							enddo
							!Households with a mortgage
							call omp_set_num_threads(num_threads)
							!$omp parallel private(iy,ydefault,iylow,iyupp,yweight,wdefault,W_maxsofar,m_maxsofar,imprime,ypay,wval)
							!$omp do
							do im = 2, nm
								do iy = ny_neg + 1, ny
									!Utility of defaulting
									ydefault = yy(iy) - max(0.d0,min(fdeficiency*yy(iy),mm(im)*hh(ih) - Jreo(ih,iz,iagg)))
									iylow    = ly(ydefault)
									iyupp    = uy(ydefault)
									yweight  = wy(ydefault)
									wdefault = (1 - yweight)*Wrent_f1(iylow,is,iz,iagg)	+ yweight*Wrent_f1(iyupp,is,iz,iagg)
									!Utility of making an optimal payment
									W_maxsofar = -1e9
									m_maxsofar = im
									do imprime = 1, im
										ypay = yy(iy) - (mm(im) - qm(iz,iagg)*mm(imprime))*hh(ih)
										if (ypay < 0.d0) then
											ypay = ypay/abs(ylb(ig,is,iz,iagg))
										endif
										iylow   = ly(ypay)
										iyupp   = uy(ypay)
										yweight = wy(ypay)
										if ((imprime == 1 .AND. ypay >= -1.d0) .OR. (imprime > 1 .AND. ypay >= 0.d0)) then
											wval = (1 - yweight)*Vown_f0(iylow,imprime,ig,is,iz,iagg) + yweight*Vown_f0(iyupp,imprime,ig,is,iz,iagg)
											if (wval > W_maxsofar) then
												W_maxsofar = wval
												m_maxsofar = imprime
											endif
										endif					        
									enddo
									!Optimal default/payment decision
									if (W_maxsofar >= wdefault) then
										Wnosell_f0(iy,im,ig,is,iz,iagg) = W_maxsofar
										d_f0(iy,im,ig,is,iz,iagg)       = 0.d0
										locm_f0(iy,im,ig,is,iz,iagg)    = m_maxsofar
										m_f0(iy,im,ig,is,iz,iagg)       = mm(m_maxsofar)
									else
										Wnosell_f0(iy,im,ig,is,iz,iagg) = wdefault
										d_f0(iy,im,ig,is,iz,iagg)       = 1.d0
										locm_f0(iy,im,ig,is,iz,iagg)    = 1
										m_f0(iy,im,ig,is,iz,iagg)       = 0.d0
									endif
								enddo
								do iy = 1, ny_neg
									Wnosell_f0(iy,im,ig,is,iz,iagg) = Wnosell_f0(ny_neg + 1,im,ig,is,iz,iagg)
									d_f0(iy,im,ig,is,iz,iagg)       = d_f0(ny_neg + 1,im,ig,is,iz,iagg)
									locm_f0(iy,im,ig,is,iz,iagg)    = locm_f0(ny_neg + 1,im,ig,is,iz,iagg)
									m_f0(iy,im,ig,is,iz,iagg)       = m_f0(ny_neg + 1,im,ig,is,iz,iagg)
								enddo
							enddo
							!$omp end do
							!$omp end parallel
						enddo
					enddo
				enddo
			enddo
			!House selling subperiod
			do iagg = 1, nagg
				do iz = 1, nz
					do is = 1, ns
						do ig = 1, ng
							ih = hfun(ig)
							!Households with bad credit
							call omp_set_num_threads(num_threads)
							!$omp parallel private(W_maxsofar,x_maxsofar,ix,ysell,iylow,iyupp,yweight,wval)
							!$omp do
							do iy = ny_neg + 1, ny
								W_maxsofar = -1e9
								x_maxsofar = nx
								do ix = 1, nx - 1
									ysell   = yy(iy) + xsell(ix,ih,iagg)*(1 - sellcost)
									iylow   = ly(ysell)
									iyupp   = uy(ysell)
									yweight = wy(ysell)
									if (Vown_f1(iy,ig,is,iz,iagg) - searchcost < 0.d0) then
										wval = -1e9
									else
										wval    = ((1 - prob_xs(ix))*(Vown_f1(iy,ig,is,iz,iagg) - searchcost)**(1 - gamma) + prob_xs(ix)&
										*((1 - yweight)*Wrent_f1(iylow,is,iz,iagg)**(1 - gamma) + yweight*Wrent_f1(iyupp,is,iz,iagg)**(1 - gamma)))**(1/(1 - gamma))
									endif
									if (wval > W_maxsofar) then
										W_maxsofar = wval
										x_maxsofar = ix
									endif
								enddo
								if (W_maxsofar > Vown_f1(iy,ig,is,iz,iagg)) then
									Wown_f1(iy,ig,is,iz,iagg)    = W_maxsofar
									locxs_f1(iy,ig,is,iz,iagg)   = x_maxsofar
									xs_f1(iy,ig,is,iz,iagg)      = xsell(x_maxsofar,ih,iagg)
									xsratio_f1(iy,ig,is,iz,iagg) = xsell(x_maxsofar,ih,iagg)/(ph(iagg)*hh(ih))
									psell_f1(iy,ig,is,iz,iagg)   = prob_xs(x_maxsofar)
								else
									Wown_f1(iy,ig,is,iz,iagg)    = Vown_f1(iy,ig,is,iz,iagg)
									locxs_f1(iy,ig,is,iz,iagg)   = nx
									xs_f1(iy,ig,is,iz,iagg)      = xsell(nx,ih,iagg)
									xsratio_f1(iy,ig,is,iz,iagg) = xsell(nx,ih,iagg)/(ph(iagg)*hh(ih))
									psell_f1(iy,ig,is,iz,iagg)   = 0.d0
								endif
							enddo
							!$omp end do
							!$omp end parallel
							do iy = 1, ny_neg
								Wown_f1(iy,ig,is,iz,iagg)    = Wown_f1(ny_neg + 1,ig,is,iz,iagg)
								locxs_f1(iy,ig,is,iz,iagg)   = locxs_f1(ny_neg + 1,ig,is,iz,iagg)
								xs_f1(iy,ig,is,iz,iagg)      = xs_f1(ny_neg + 1,ig,is,iz,iagg)
								xsratio_f1(iy,ig,is,iz,iagg) = xsratio_f1(ny_neg + 1,ig,is,iz,iagg)
								psell_f1(iy,ig,is,iz,iagg)   = psell_f1(ny_neg + 1,ig,is,iz,iagg)
							enddo
							!Households with good credit
							call omp_set_num_threads(num_threads)
							!$omp parallel private(iy,W_maxsofar,x_maxsofar,ix,ysell,iylow,iyupp,yweight,wval)
							!$omp do
							do im = 1, nm
								do iy = ny_neg + 1, ny
									W_maxsofar = -1e9
									x_maxsofar = nx
									do ix = 2, nx
										ysell   = yy(iy) - mm(im)*hh(ih) + xsell(nx - ix + 1,ih,iagg)*(1 - sellcost)
										iylow   = ly(ysell)
										iyupp   = uy(ysell)
										yweight = wy(ysell)
										if (ysell >= 0.d0) then
											if (Wnosell_f0(iy,im,ig,is,iz,iagg) - searchcost < 0.d0) then
												wval = -1e9
											else
												wval = ((1 - prob_xs(nx - ix + 1))*(Wnosell_f0(iy,im,ig,is,iz,iagg) - searchcost)**(1 - gamma) + prob_xs(nx - ix + 1)&
												*((1 - yweight)*Wrent_f0(iylow,is,iz,iagg)**(1 - gamma) + yweight*Wrent_f0(iyupp,is,iz,iagg)**(1 - gamma)))**(1/(1 - gamma))
											endif
											if (wval > W_maxsofar) then
												W_maxsofar = wval
												x_maxsofar = nx - ix + 1
											endif
										else
											exit
										endif
									enddo
									if (W_maxsofar > Wnosell_f0(iy,im,ig,is,iz,iagg)) then
										Wown_f0(iy,im,ig,is,iz,iagg)    = W_maxsofar
										locxs_f0(iy,im,ig,is,iz,iagg)   = x_maxsofar
										xs_f0(iy,im,ig,is,iz,iagg)      = xsell(x_maxsofar,ih,iagg)
										xsratio_f0(iy,im,ig,is,iz,iagg) = xsell(x_maxsofar,ih,iagg)/(ph(iagg)*hh(ih))
										psell_f0(iy,im,ig,is,iz,iagg)   = prob_xs(x_maxsofar)
									else
										Wown_f0(iy,im,ig,is,iz,iagg)    = Wnosell_f0(iy,im,ig,is,iz,iagg)
										locxs_f0(iy,im,ig,is,iz,iagg)   = nx
										xs_f0(iy,im,ig,is,iz,iagg)      = xsell(nx,ih,iagg)
										xsratio_f0(iy,im,ig,is,iz,iagg) = xsell(nx,ih,iagg)/(ph(iagg)*hh(ih))
										psell_f0(iy,im,ig,is,iz,iagg)   = 0.d0
									endif
								enddo
								do iy = 1, ny_neg
									Wown_f0(iy,im,ig,is,iz,iagg)    = Wown_f0(ny_neg + 1,im,ig,is,iz,iagg)
									locxs_f0(iy,im,ig,is,iz,iagg)   = locxs_f0(ny_neg + 1,im,ig,is,iz,iagg)
									xs_f0(iy,im,ig,is,iz,iagg)      = xs_f0(ny_neg + 1,im,ig,is,iz,iagg)
									xsratio_f0(iy,im,ig,is,iz,iagg) = xsratio_f0(ny_neg + 1,im,ig,is,iz,iagg)
									psell_f0(iy,im,ig,is,iz,iagg)   = psell_f0(ny_neg + 1,im,ig,is,iz,iagg)
								enddo
							enddo
							!$omp end do
							!$omp end parallel
						enddo
					enddo
				enddo
			enddo
			!Consumption/savings subperiod
			do iagg = 1, nagg
				do iz = 1, nz
					do is = 1, ns
						!Renters
						!Continuation utilities
						call omp_set_num_threads(num_threads)
						!$omp parallel private(izprime,isprime,ieprime,yprime_f0,iy0low,iy0upp,y0weight,yprime_f1,iy1low,iy1upp,y1weight,Vcont_e_f0,Vcont_e_f1,Vcont_s_f0,Vcont_s_f1,Vcont_z_f0,Vcont_z_f1)
						!$omp do
						do ibprime = 1, nb
							do izprime = 1, nz
								do isprime = 1, ns
									do ieprime = 1, ne
										yprime_f0 = (labor(isprime,ieprime)*wprime(iz,iagg,izprime) + bb(ibprime))*(1 - taxprime(iz,iagg,izprime))
										iy0low    = ly(yprime_f0)
										iy0upp    = uy(yprime_f0)
										y0weight  = wy(yprime_f0)
										yprime_f1 = ((1 - fincloss)*labor(isprime,ieprime)*wprime(iz,iagg,izprime) + bb(ibprime))*(1 - taxprime(iz,iagg,izprime))
										iy1low    = ly(yprime_f1)
										iy1upp    = uy(yprime_f1)
										y1weight  = wy(yprime_f1)
										!Households with good credit
										Vcont_e_f0(ieprime) = dot_product(waggprime(iz,iagg,izprime,:),(1 - y0weight)&
										*Wrent_f0(iy0low,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma) + y0weight&
										*Wrent_f0(iy0upp,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma))
										!Households with bad credit
										Vcont_e_f1(ieprime) = dot_product(waggprime(iz,iagg,izprime,:),pflag*((1 - y1weight)&
										*Wrent_f1(iy1low,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma) + y1weight&
										*Wrent_f1(iy1upp,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)) + (1 - pflag)&
										*((1 - y0weight)*Wrent_f0(iy0low,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma) + y0weight&
										*Wrent_f0(iy0upp,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)))
									enddo
									Vcont_s_f0(isprime) = dot_product(Vcont_e_f0,emeas)
									Vcont_s_f1(isprime) = dot_product(Vcont_e_f1,emeas)
								enddo
								Vcont_z_f0(izprime) = dot_product(Vcont_s_f0,prob_ss(is,:,iz,izprime))
								Vcont_z_f1(izprime) = dot_product(Vcont_s_f1,prob_ss(is,:,iz,izprime))
							enddo
							Vcont_rent_f0(ibprime) = dot_product(Vcont_z_f0,prob_zz(iz,:))
							Vcont_rent_f1(ibprime) = dot_product(Vcont_z_f1,prob_zz(iz,:))
						enddo
						!$omp end do
						!$omp end parallel
						!Optimization
						call omp_set_num_threads(num_threads)
						!$omp parallel private(V_maxsofar,b_maxsofar,c_maxsofar,hc_maxsofar,spend_maxsofar,ibprime,spending,hc,c,util,val)
						!$omp do
						do iy = ny_neg + 1, ny
							!Households with good credit
							V_maxsofar     = -1e8
							b_maxsofar     = 1
							c_maxsofar     = cshare(iz)*min_spend
							hc_maxsofar    = (1 - cshare(iz))*min_spend/pa
							spend_maxsofar = min_spend
							do ibprime = 1, nb
								spending = min_spend + yy(iy) - qb(iz,iagg)*bb(ibprime)
								if (spending > 0.d0) then
									if (spending >= max_a*(pa + (cshare(iz)/(1 - cshare(iz))*pa)**ies)) then
										hc = max_a
										c  = spending - pa*max_a
									else
										hc = spending/(pa + (cshare(iz)/(1 - cshare(iz))*pa)**ies)
										c  = spending*(cshare(iz)/(1 - cshare(iz))*pa)**ies/(pa + (cshare(iz)/(1 - cshare(iz))*pa)**ies)
									endif
									util = (cshare(iz)*c**((ies - 1)/ies) + (1 - cshare(iz))*hc**((ies - 1)/ies))**(ies/(ies - 1))
									!Lifetime utility
									val = ((1 - beta)*util**((1 - gamma)/theta)	+ beta*Vcont_rent_f0(ibprime)**(1/theta))**(theta/(1 - gamma))
									if (val > V_maxsofar) then
										V_maxsofar     = val
										b_maxsofar     = ibprime
										c_maxsofar     = c
										hc_maxsofar    = hc
										spend_maxsofar = spending
									endif
								else
									exit
								endif
							enddo
							Vnew_rent_f0(iy,is,iz,iagg) = V_maxsofar
							locbrent_f0(iy,is,iz,iagg)  = b_maxsofar
							brent_f0(iy,is,iz,iagg)     = bb(b_maxsofar)
							crent_f0(iy,is,iz,iagg)     = c_maxsofar
							hcrent_f0(iy,is,iz,iagg)    = hc_maxsofar
							spendrent_f0(iy,is,iz,iagg) = spend_maxsofar
							!Households with bad credit
							V_maxsofar     = -1e8
							b_maxsofar     = 1
							c_maxsofar     = cshare(iz)*min_spend
							hc_maxsofar    = (1 - cshare(iz))*min_spend/pa
							spend_maxsofar = min_spend
							do ibprime = 1, nb
								spending = min_spend + yy(iy) - qb(iz,iagg)*bb(ibprime)
								if (spending > 0.d0) then
									if (spending >= max_a*(pa + (cshare(iz)/(1 - cshare(iz))*pa)**ies)) then
										hc = max_a
										c  = spending - pa*max_a
									else
										hc = spending/(pa + (cshare(iz)/(1 - cshare(iz))*pa)**ies)
										c  = spending*(cshare(iz)/(1 - cshare(iz))*pa)**ies/(pa + (cshare(iz)/(1 - cshare(iz))*pa)**ies)
									endif
									util = (cshare(iz)*c**((ies - 1)/ies) + (1 - cshare(iz))*hc**((ies - 1)/ies))**(ies/(ies - 1))
									!Lifetime utility
									val = ((1 - beta)*util**((1 - gamma)/theta)	+ beta*Vcont_rent_f1(ibprime)**(1/theta))**(theta/(1 - gamma))
									if (val > V_maxsofar) then
										V_maxsofar     = val
										b_maxsofar     = ibprime
										c_maxsofar     = c
										hc_maxsofar    = hc
										spend_maxsofar = spending
									endif
								else
									exit
								endif
							enddo
							Vnew_rent_f1(iy,is,iz,iagg) = V_maxsofar
							locbrent_f1(iy,is,iz,iagg)  = b_maxsofar
							brent_f1(iy,is,iz,iagg)     = bb(b_maxsofar)
							crent_f1(iy,is,iz,iagg)     = c_maxsofar
							hcrent_f1(iy,is,iz,iagg)    = hc_maxsofar
							spendrent_f1(iy,is,iz,iagg) = spend_maxsofar
						enddo
						!$omp end do
						!$omp end parallel
						do iy = 1, ny_neg
							Vnew_rent_f0(iy,is,iz,iagg) = Vnew_rent_f0(ny_neg + 1,is,iz,iagg)
							locbrent_f0(iy,is,iz,iagg)  = locbrent_f0(ny_neg + 1,is,iz,iagg)
							brent_f0(iy,is,iz,iagg)     = brent_f0(ny_neg + 1,is,iz,iagg)
							crent_f0(iy,is,iz,iagg)     = crent_f0(ny_neg + 1,is,iz,iagg)
							hcrent_f0(iy,is,iz,iagg)    = hcrent_f0(ny_neg + 1,is,iz,iagg)
							spendrent_f0(iy,is,iz,iagg) = spendrent_f0(ny_neg + 1,is,iz,iagg)
							Vnew_rent_f1(iy,is,iz,iagg) = Vnew_rent_f1(ny_neg + 1,is,iz,iagg)
							locbrent_f1(iy,is,iz,iagg)  = locbrent_f1(ny_neg + 1,is,iz,iagg)
							brent_f1(iy,is,iz,iagg)     = brent_f1(ny_neg + 1,is,iz,iagg)
							crent_f1(iy,is,iz,iagg)     = crent_f1(ny_neg + 1,is,iz,iagg)
							hcrent_f1(iy,is,iz,iagg)    = hcrent_f1(ny_neg + 1,is,iz,iagg)
							spendrent_f1(iy,is,iz,iagg) = spendrent_f1(ny_neg + 1,is,iz,iagg)
						enddo
						do ig = 1, ng
							ih      = hfun(ig)
							iu      = ufun(ig)
							igshock = gfun(ih,2)
							!Homeowners with bad credit
							!Continuation utilities
							call omp_set_num_threads(num_threads)
							!$omp parallel private(izprime,isprime,ieprime,yprime_f0,iy0low,iy0upp,y0weight,yprime_f1,iy1low,iy1upp,y1weight,Vcont_e_f1,Vcont_s_f1,Vcont_z_f1)
							!$omp do
							do ibprime = 1, nb
								do izprime = 1, nz
									if (iu == 1 .AND. nu == 2) then
										do isprime = 1, ns
											do ieprime = 1, ne
												yprime_f0 = (labor(isprime,ieprime)*wprime(iz,iagg,izprime) + bb(ibprime))*(1 - taxprime(iz,iagg,izprime))
												iy0low    = ly(yprime_f0)
												iy0upp    = uy(yprime_f0)
												y0weight  = wy(yprime_f0)
												yprime_f1 = ((1 - fincloss)*labor(isprime,ieprime)*wprime(iz,iagg,izprime) + bb(ibprime))*(1 - taxprime(iz,iagg,izprime))
												iy1low    = ly(yprime_f1)
												iy1upp    = uy(yprime_f1)
												y1weight  = wy(yprime_f1)
												Vcont_e_f1(ieprime) = dot_product(waggprime(iz,iagg,izprime,:),pflag*((1 - y1weight)&
												*((1 - deprec_h)*((1 - pseparate)*Wown_f1(iy1low,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma) + pseparate&
												*Wown_f1(iy1low,igshock,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)) + deprec_h&
												*Wrent_f1(iy1low,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)) + y1weight&
												*((1 - deprec_h)*((1 - pseparate)*Wown_f1(iy1upp,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma) + pseparate&
												*Wown_f1(iy1upp,igshock,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)) + deprec_h&
												*Wrent_f1(iy1upp,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma))) + (1 - pflag)&
												*((1 - y0weight)*((1 - deprec_h)*((1 - pseparate)*Wown_f0(iy0low,1,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)&
												+ pseparate*Wown_f0(iy0low,1,igshock,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma))&
												+ deprec_h*Wrent_f0(iy0low,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)) + y0weight&
												*((1 - deprec_h)*((1 - pseparate)*Wown_f0(iy0upp,1,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma) + pseparate&
												*Wown_f0(iy0upp,1,igshock,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma))&
												+ deprec_h*Wrent_f0(iy0upp,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma))))
											enddo
											Vcont_s_f1(isprime) = dot_product(Vcont_e_f1,emeas)
										enddo
										Vcont_z_f1(izprime) = dot_product(Vcont_s_f1,prob_ss(is,:,iz,izprime))
									else
										do isprime = 1, ns
											do ieprime = 1, ne
												yprime_f0 = (labor(isprime,ieprime)*wprime(iz,iagg,izprime) + bb(ibprime))*(1 - taxprime(iz,iagg,izprime))
												iy0low    = ly(yprime_f0)
												iy0upp    = uy(yprime_f0)
												y0weight  = wy(yprime_f0)
												yprime_f1 = ((1 - fincloss)*labor(isprime,ieprime)*wprime(iz,iagg,izprime) + bb(ibprime))*(1 - taxprime(iz,iagg,izprime))
												iy1low    = ly(yprime_f1)
												iy1upp    = uy(yprime_f1)
												y1weight  = wy(yprime_f1)
												Vcont_e_f1(ieprime) = dot_product(waggprime(iz,iagg,izprime,:),pflag*((1 - y1weight)&
												*((1 - deprec_h)*Wown_f1(iy1low,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma) + deprec_h&
												*Wrent_f1(iy1low,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)) + y1weight&
												*((1 - deprec_h)*Wown_f1(iy1upp,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma) + deprec_h&
												*Wrent_f1(iy1upp,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma))) + (1 - pflag)&
												*((1 - y0weight)*((1 - deprec_h)*Wown_f0(iy0low,1,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)&
												+ deprec_h*Wrent_f0(iy0low,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)) + y0weight&
												*((1 - deprec_h)*Wown_f0(iy0upp,1,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)&
												+ deprec_h*Wrent_f0(iy0upp,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma))))
											enddo
											Vcont_s_f1(isprime) = dot_product(Vcont_e_f1,emeas)
										enddo
										Vcont_z_f1(izprime) = dot_product(Vcont_s_f1,prob_ss(is,:,iz,izprime))
									endif
								enddo
								Vcont_own_f1(ibprime) = dot_product(Vcont_z_f1,prob_zz(iz,:))
							enddo
							!$omp end do
							!$omp end parallel
							!Optimization
							call omp_set_num_threads(num_threads)
							!$omp parallel private(V_maxsofar,b_maxsofar,c_maxsofar,hc_maxsofar,spend_maxsofar,ibprime,spending,hcstay,cstay,ustay,hcmove,cmove,umove,hc,c,util,val)
							!$omp do
							do iy = ny_neg + 1, ny
								V_maxsofar     = -1e8
								b_maxsofar     = 1
								c_maxsofar     = cshare(iz)*min_spend
								hc_maxsofar    = (1 - cshare(iz))*min_spend/pa
								spend_maxsofar = min_spend
								do ibprime = 1, nb
									spending = min_spend + yy(iy) - qb(iz,iagg)*bb(ibprime)
									if (spending > 0.d0) then
										!Utility of occupying
										hcstay = utilown*uu(iu)*hh(ih)
										cstay  = spending
										ustay  = (cshare(iz)*cstay**((ies - 1)/ies) + (1 - cshare(iz))*hcstay**((ies - 1)/ies))**(ies/(ies - 1))
										!Utility of renting elsewhere
										if (spending >= max_a*(pa + (cshare(iz)/(1 - cshare(iz))*pa)**ies)) then
											hcmove = max_a
											cmove  = spending - pa*max_a
										else
											hcmove = spending/(pa + (cshare(iz)/(1 - cshare(iz))*pa)**ies)
											cmove  = spending*(cshare(iz)/(1 - cshare(iz))*pa)**ies/(pa + (cshare(iz)/(1 - cshare(iz))*pa)**ies)
										endif
										umove = (cshare(iz)*cmove**((ies - 1)/ies) + (1 - cshare(iz))*hcmove**((ies - 1)/ies))**(ies/(ies - 1))
										!Optimal occupation decision
										if (ustay >= umove) then
											util = ustay
											c    = cstay
											hc   = 0.d0
										else
											util = umove
											c    = cmove
											hc   = hcmove
										endif
										!Lifetime utility
										val = ((1 - beta)*util**((1 - gamma)/theta)	+ beta*Vcont_own_f1(ibprime)**(1/theta))**(theta/(1 - gamma))
										if (val > V_maxsofar) then
											V_maxsofar     = val
											b_maxsofar     = ibprime
											c_maxsofar     = c
											hc_maxsofar    = hc
											spend_maxsofar = spending
										endif
									else
										exit
									endif
								enddo
								Vnew_own_f1(iy,ig,is,iz,iagg) = V_maxsofar
								locbown_f1(iy,ig,is,iz,iagg)  = b_maxsofar
								bown_f1(iy,ig,is,iz,iagg)     = bb(b_maxsofar)
								cown_f1(iy,ig,is,iz,iagg)     = c_maxsofar
								hcown_f1(iy,ig,is,iz,iagg)    = hc_maxsofar
								spendown_f1(iy,ig,is,iz,iagg) = spend_maxsofar
							enddo
							!$omp end do
							!$omp end parallel
							do iy = 1, ny_neg
								Vnew_own_f1(iy,ig,is,iz,iagg) = Vnew_own_f1(ny_neg + 1,ig,is,iz,iagg)
								locbown_f1(iy,ig,is,iz,iagg)  = locbown_f1(ny_neg + 1,ig,is,iz,iagg)
								bown_f1(iy,ig,is,iz,iagg)     = bown_f1(ny_neg + 1,ig,is,iz,iagg)
								cown_f1(iy,ig,is,iz,iagg)     = cown_f1(ny_neg + 1,ig,is,iz,iagg)
								hcown_f1(iy,ig,is,iz,iagg)    = hcown_f1(ny_neg + 1,ig,is,iz,iagg)
								spendown_f1(iy,ig,is,iz,iagg) = spendown_f1(ny_neg + 1,ig,is,iz,iagg)
							enddo
							!Homeowners with good credit
							!Continuation utilities
							call omp_set_num_threads(num_threads)
							!$omp parallel private(ibprime,izprime,isprime,ieprime,yprime_f0,iy0low,iy0upp,y0weight,Vcont_e_f0,Vcont_s_f0,Vcont_z_f0)
							!$omp do
							do imprime = 1, nm
								do ibprime = 1, nb
									do izprime = 1, nz
										if (iu == 1 .AND. nu == 2) then
											do isprime = 1, ns
												do ieprime = 1, ne
													yprime_f0 = (labor(isprime,ieprime)*wprime(iz,iagg,izprime) + bb(ibprime))*(1 - taxprime(iz,iagg,izprime))
													iy0low    = ly(yprime_f0)
													iy0upp    = uy(yprime_f0)
													y0weight  = wy(yprime_f0)
													Vcont_e_f0(ieprime) = dot_product(waggprime(iz,iagg,izprime,:),(1 - y0weight)&
													*((1 - deprec_h)*((1 - pseparate)*Wown_f0(iy0low,imprime,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)&
													+ pseparate*Wown_f0(iy0low,imprime,igshock,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma))&
													+ deprec_h*Wrent_f0(iy0low,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)) + y0weight&
													*((1 - deprec_h)*((1 - pseparate)*Wown_f0(iy0upp,imprime,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)&
													+ pseparate*Wown_f0(iy0upp,imprime,igshock,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma))&
													+ deprec_h*Wrent_f0(iy0upp,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)))
												enddo
												Vcont_s_f0(isprime) = dot_product(Vcont_e_f0,emeas)
											enddo
											Vcont_z_f0(izprime) = dot_product(Vcont_s_f0,prob_ss(is,:,iz,izprime))
										else
											do isprime = 1, ns
												do ieprime = 1, ne
													yprime_f0 = (labor(isprime,ieprime)*wprime(iz,iagg,izprime) + bb(ibprime))*(1 - taxprime(iz,iagg,izprime))
													iy0low    = ly(yprime_f0)
													iy0upp    = uy(yprime_f0)
													y0weight  = wy(yprime_f0)
													Vcont_e_f0(ieprime) = dot_product(waggprime(iz,iagg,izprime,:),(1 - y0weight)&
													*((1 - deprec_h)*Wown_f0(iy0low,imprime,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)&
													+ deprec_h*Wrent_f0(iy0low,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)) + y0weight&
													*((1 - deprec_h)*Wown_f0(iy0upp,imprime,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)&
													+ deprec_h*Wrent_f0(iy0upp,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)))
												enddo
												Vcont_s_f0(isprime) = dot_product(Vcont_e_f0,emeas)
											enddo
											Vcont_z_f0(izprime) = dot_product(Vcont_s_f0,prob_ss(is,:,iz,izprime))
										endif
									enddo
									Vcont_own_f0(ibprime,imprime) = dot_product(Vcont_z_f0,prob_zz(iz,:))
								enddo
							enddo
							!$omp end do
							!$omp end parallel
							!Optimization
							!Households without a mortgage
							call omp_set_num_threads(num_threads)
							!$omp parallel private(cash_at_hand,V_maxsofar,m_maxsofar,b_maxsofar,c_maxsofar,hc_maxsofar,spend_maxsofar,imprime,ibprime,spending,hcstay,cstay,ustay,hcmove,cmove,umove,hc,c,util,val)
							!$omp do
							do iy = 1, ny
								if (iy <= ny_neg) then
									cash_at_hand = yy(iy)*abs(ylb(ig,is,iz,iagg))
								else
									cash_at_hand = yy(iy)
								endif
								V_maxsofar     = -1e8
								m_maxsofar     = 1
								b_maxsofar     = 1
								c_maxsofar     = cshare(iz)*min_spend
								hc_maxsofar    = (1 - cshare(iz))*min_spend/pa
								spend_maxsofar = min_spend
								do imprime = 1, nm
									do ibprime = 1, nb
										spending = min_spend + cash_at_hand + qm0(imprime,ibprime,ig,is,iz,iagg)*mm(imprime)*hh(ih) - qb(iz,iagg)*bb(ibprime)
										if (spending > 0.d0 .AND. qm0(imprime,ibprime,ig,is,iz,iagg)*mm(imprime) <= max_ltv*ph(iagg)) then
											!Utility of occupying
											hcstay = utilown*uu(iu)*hh(ih)
											cstay  = spending
											ustay  = (cshare(iz)*cstay**((ies - 1)/ies) + (1 - cshare(iz))*hcstay**((ies - 1)/ies))**(ies/(ies - 1))
											!Utility of renting elsewhere
											if (spending >= max_a*(pa + (cshare(iz)/(1 - cshare(iz))*pa)**ies)) then
												hcmove = max_a
												cmove  = spending - pa*max_a
											else
												hcmove = spending/(pa + (cshare(iz)/(1 - cshare(iz))*pa)**ies)
												cmove  = spending*(cshare(iz)/(1 - cshare(iz))*pa)**ies/(pa + (cshare(iz)/(1 - cshare(iz))*pa)**ies)
											endif
											umove = (cshare(iz)*cmove**((ies - 1)/ies) + (1 - cshare(iz))*hcmove**((ies - 1)/ies))**(ies/(ies - 1))
											!Optimal occupation decision
											if (ustay >= umove) then
												util = ustay
												c    = cstay
												hc   = 0.d0
											else
												util = umove
												c    = cmove
												hc   = hcmove
											endif
											!Lifetime utility
											val = ((1 - beta)*util**((1 - gamma)/theta)	+ beta*Vcont_own_f0(ibprime,imprime)**(1/theta))**(theta/(1 - gamma))
											if (val > V_maxsofar) then
												V_maxsofar     = val
												m_maxsofar     = imprime
												b_maxsofar     = ibprime
												c_maxsofar     = c
												hc_maxsofar    = hc
												spend_maxsofar = spending
											endif
										endif
									enddo
								enddo
								Vnew_own_f0(iy,1,ig,is,iz,iagg) = V_maxsofar
								locm_f0(iy,1,ig,is,iz,iagg)     = m_maxsofar
								m_f0(iy,1,ig,is,iz,iagg)        = mm(m_maxsofar)
								locbown_f0(iy,1,ig,is,iz,iagg)  = b_maxsofar
								bown_f0(iy,1,ig,is,iz,iagg)     = bb(b_maxsofar)
								cown_f0(iy,1,ig,is,iz,iagg)     = c_maxsofar
								hcown_f0(iy,1,ig,is,iz,iagg)    = hc_maxsofar
								spendown_f0(iy,1,ig,is,iz,iagg) = spend_maxsofar
							enddo
							!$omp end do
							!$omp end parallel
							call omp_set_num_threads(num_threads)
							!$omp parallel private(iy,V_maxsofar,b_maxsofar,c_maxsofar,hc_maxsofar,spend_maxsofar,ibprime_start,ibprime,spending,hcstay,cstay,ustay,hcmove,cmove,umove,hc,c,util,val)
							!$omp do
							do im = 2, nm
								!Households with a mortgage
								do iy = ny_neg + 1, ny
									V_maxsofar     = -1e8
									b_maxsofar     = 1
									c_maxsofar     = cshare(iz)*min_spend
									hc_maxsofar    = (1 - cshare(iz))*min_spend/pa
									spend_maxsofar = min_spend
									if (iy == ny_neg + 1) then
										ibprime_start = 1
									else
										ibprime_start = locbown_f0(iy - 1,im,ig,is,iz,iagg)
									endif
									do ibprime = ibprime_start, nb
										spending = min_spend + yy(iy) - qb(iz,iagg)*bb(ibprime)
										if (spending > 0.d0) then
											!Utility of occupying
											hcstay = utilown*uu(iu)*hh(ih)
											cstay  = spending
											ustay  = (cshare(iz)*cstay**((ies - 1)/ies) + (1 - cshare(iz))*hcstay**((ies - 1)/ies))**(ies/(ies - 1))
											!Utility of renting elsewhere
											if (spending >= max_a*(pa + (cshare(iz)/(1 - cshare(iz))*pa)**ies)) then
												hcmove = max_a
												cmove  = spending - pa*max_a
											else
												hcmove = spending/(pa + (cshare(iz)/(1 - cshare(iz))*pa)**ies)
												cmove  = spending*(cshare(iz)/(1 - cshare(iz))*pa)**ies/(pa + (cshare(iz)/(1 - cshare(iz))*pa)**ies)
											endif
											umove = (cshare(iz)*cmove**((ies - 1)/ies) + (1 - cshare(iz))*hcmove**((ies - 1)/ies))**(ies/(ies - 1))
											!Optimal occupation decision
											if (ustay >= umove) then
												util = ustay
												c    = cstay
												hc   = 0.d0
											else
												util = umove
												c    = cmove
												hc   = hcmove
											endif
											!Lifetime utility
											val = ((1 - beta)*util**((1 - gamma)/theta)	+ beta*Vcont_own_f0(ibprime,im)**(1/theta))**(theta/(1 - gamma))
											if (val > V_maxsofar) then
												V_maxsofar     = val
												b_maxsofar     = ibprime
												c_maxsofar     = c
												hc_maxsofar    = hc
												spend_maxsofar = spending
											endif
										else
											exit
										endif
									enddo
									Vnew_own_f0(iy,im,ig,is,iz,iagg) = V_maxsofar
									locbown_f0(iy,im,ig,is,iz,iagg)  = b_maxsofar
									bown_f0(iy,im,ig,is,iz,iagg)     = bb(b_maxsofar)
									cown_f0(iy,im,ig,is,iz,iagg)     = c_maxsofar
									hcown_f0(iy,im,ig,is,iz,iagg)    = hc_maxsofar
									spendown_f0(iy,im,ig,is,iz,iagg) = spend_maxsofar
								enddo
								do iy = 1, ny_neg
									Vnew_own_f0(iy,im,ig,is,iz,iagg) = Vnew_own_f0(ny_neg + 1,im,ig,is,iz,iagg)
									locbown_f0(iy,im,ig,is,iz,iagg)  = locbown_f0(ny_neg + 1,im,ig,is,iz,iagg)
									bown_f0(iy,im,ig,is,iz,iagg)     = bown_f0(ny_neg + 1,im,ig,is,iz,iagg)
									cown_f0(iy,im,ig,is,iz,iagg)     = cown_f0(ny_neg + 1,im,ig,is,iz,iagg)
									hcown_f0(iy,im,ig,is,iz,iagg)    = hcown_f0(ny_neg + 1,im,ig,is,iz,iagg)
									spendown_f0(iy,im,ig,is,iz,iagg) = spendown_f0(ny_neg + 1,im,ig,is,iz,iagg)
								enddo
							enddo
							!$omp end do
							!$omp end parallel
						enddo
					enddo
				enddo
			enddo
			err_valfn = max(maxval(abs(Vnew_own_f0 - Vown_f0)),maxval(abs(Vnew_rent_f0 - Vrent_f0)),maxval(abs(Vnew_own_f1 - Vown_f1)),maxval(abs(Vnew_rent_f1 - Vrent_f1)))
			if (err_valfn < minerr_noaccel) then
				minerr_noaccel = err_valfn
			endif
			Vown_f0  = Vnew_own_f0
			Vrent_f0 = Vnew_rent_f0
			Vown_f1  = Vnew_own_f1
			Vrent_f1 = Vnew_rent_f1
			write(*, '(a22, i4, a12, f20.6)'), '          Iteration = ' ,iter_valfn, ', err_val = ' ,err_valfn
			if (err_valfn < toler) then
				exit
			endif
		else
			!Accelerator
			!House buying subperiod
			do iagg = 1, nagg
				do iz = 1, nz
					do is = 1, ns
						call omp_set_num_threads(num_threads)
						!$omp parallel private(ih,ig,ix,ybuy,iylow,iyupp,yweight)
						!$omp do
						do iy = ny_neg + 1, ny
							!Households with good credit
							ih   = loch_f0(iy,is,iz,iagg)
							ig   = gfun(ih,1)
							ix   = locxb_f0(iy,is,iz,iagg)
							ybuy = yy(iy) - xbuy(ix,ih,iagg)
							if (ybuy < 0.d0) then
								ybuy = ybuy/abs(ylb(ig,is,iz,iagg))
							endif
							iylow                   = ly(ybuy)
							iyupp                   = uy(ybuy)
							yweight                 = wy(ybuy)
							Wrent_f0(iy,is,iz,iagg) = max(Vrent_f0(iy,is,iz,iagg),((1 - prob_xb(ix))*Vrent_f0(iy,is,iz,iagg)**(1 - gamma) + prob_xb(ix)*((1 - yweight)&
							*Vown_f0(iylow,1,ig,is,iz,iagg)**(1 - gamma) + yweight*Vown_f0(iyupp,1,ig,is,iz,iagg)**(1 - gamma)))**(1/(1 - gamma)))
							!Households with bad credit
							ih					    = loch_f1(iy,is,iz,iagg)
							ig					    = gfun(ih,1)
							ix						= locxb_f1(iy,is,iz,iagg)
							ybuy					= yy(iy) - xbuy(ix,ih,iagg)
							iylow					= ly(ybuy)
							iyupp					= uy(ybuy)
							yweight					= wy(ybuy)
							Wrent_f1(iy,is,iz,iagg) = max(Vrent_f1(iy,is,iz,iagg),((1 - prob_xb(ix))*Vrent_f1(iy,is,iz,iagg)**(1 - gamma) + prob_xb(ix)*((1 - yweight)&
							*Vown_f1(iylow,ig,is,iz,iagg)**(1 - gamma) + yweight*Vown_f1(iyupp,ig,is,iz,iagg)**(1 - gamma)))**(1/(1 - gamma)))
						enddo
						!$omp end do
						!$omp end parallel
						do iy = 1, ny_neg
							Wrent_f0(iy,is,iz,iagg) = Wrent_f0(ny_neg + 1,is,iz,iagg)
							Wrent_f1(iy,is,iz,iagg) = Wrent_f1(ny_neg + 1,is,iz,iagg)
						enddo
					enddo
				enddo
			enddo
			!Mortgage foreclosure/payment subperiod
			do iagg = 1, nagg
				do iz = 1, nz
					do is = 1, ns
						do ig = 1, ng
							ih = hfun(ig)
							!Households without a mortgage
							do iy = 1, ny
								Wnosell_f0(iy,1,ig,is,iz,iagg) = Vown_f0(iy,1,ig,is,iz,iagg)
							enddo
							!Households with a mortgage
							call omp_set_num_threads(num_threads)
							!$omp parallel private(iy,ydefault,iylow,iyupp,yweight,wdefault,imprime,ypay,wval)
							!$omp do
							do im = 2, nm
								do iy = ny_neg + 1, ny
									!Utility of defaulting
									ydefault = yy(iy) - max(0.d0,min(fdeficiency*yy(iy),mm(im)*hh(ih) - Jreo(ih,iz,iagg)))
									iylow    = ly(ydefault)
									iyupp    = uy(ydefault)
									yweight  = wy(ydefault)
									wdefault = (1 - yweight)*Wrent_f1(iylow,is,iz,iagg)	+ yweight*Wrent_f1(iyupp,is,iz,iagg)
									!Utility of making an optimal payment
									imprime = locm_f0(iy,im,ig,is,iz,iagg)
									ypay    = yy(iy) - (mm(im) - qm(iz,iagg)*mm(imprime))*hh(ih)
									if (ypay < 0.d0) then
										ypay = ypay/abs(ylb(ig,is,iz,iagg))
									endif
									iylow   = ly(ypay)
									iyupp   = uy(ypay)
									yweight = wy(ypay)
									if ((imprime == 1 .AND. ypay >= -1.d0) .OR. (imprime > 1 .AND. ypay >= 0.d0)) then
										wval = (1 - yweight)*Vown_f0(iylow,imprime,ig,is,iz,iagg) + yweight*Vown_f0(iyupp,imprime,ig,is,iz,iagg)
										!Optimal default/payment decision
										if (wval >= wdefault) then
											Wnosell_f0(iy,im,ig,is,iz,iagg) = wval
										else
											Wnosell_f0(iy,im,ig,is,iz,iagg) = wdefault
										endif
									else
										Wnosell_f0(iy,im,ig,is,iz,iagg) = wdefault
									endif
								enddo
								do iy = 1, ny_neg
									Wnosell_f0(iy,im,ig,is,iz,iagg) = Wnosell_f0(ny_neg + 1,im,ig,is,iz,iagg)
								enddo
							enddo
							!$omp end do
							!$omp end parallel
						enddo
					enddo
				enddo
			enddo
			!House selling subperiod
			do iagg = 1, nagg
				do iz = 1, nz
					do is = 1, ns
						do ig = 1, ng
							ih = hfun(ig)
							!Households with bad credit
							call omp_set_num_threads(num_threads)
							!$omp parallel private(ix,ysell,iylow,iyupp,yweight)
							!$omp do
							do iy = ny_neg + 1, ny
								ix						  = locxs_f1(iy,ig,is,iz,iagg)
								ysell					  = yy(iy) + xsell(ix,ih,iagg)*(1 - sellcost)
								iylow					  = ly(ysell)
								iyupp			  		  = uy(ysell)
								yweight					  = wy(ysell)
								Wown_f1(iy,ig,is,iz,iagg) = max(Vown_f1(iy,ig,is,iz,iagg),((1 - prob_xs(ix))*(max(0.d0,Vown_f1(iy,ig,is,iz,iagg) - searchcost))**(1 - gamma)&
								+ prob_xs(ix)*((1 - yweight)*Wrent_f1(iylow,is,iz,iagg)**(1 - gamma) + yweight*Wrent_f1(iyupp,is,iz,iagg)**(1 - gamma)))**(1/(1 - gamma)))
							enddo
							!$omp end do
							!$omp end parallel
							do iy = 1, ny_neg
								Wown_f1(iy,ig,is,iz,iagg) = Wown_f1(ny_neg + 1,ig,is,iz,iagg)
							enddo
							!Households with good credit
							call omp_set_num_threads(num_threads)
							!$omp parallel private(iy,ix,ysell,iylow,iyupp,yweight)
							!$omp do
							do im = 1, nm
								do iy = ny_neg + 1, ny
									ix      = locxs_f0(iy,im,ig,is,iz,iagg)
									ysell   = yy(iy) - mm(im)*hh(ih) + xsell(ix,ih,iagg)*(1 - sellcost)
									iylow   = ly(ysell)
									iyupp   = uy(ysell)
									yweight = wy(ysell)
									if (ysell >= 0.d0) then
										Wown_f0(iy,im,ig,is,iz,iagg) = max(Wnosell_f0(iy,im,ig,is,iz,iagg),((1 - prob_xs(ix))&
										*(max(0.d0,Wnosell_f0(iy,im,ig,is,iz,iagg) - searchcost))**(1 - gamma) + prob_xs(ix)*((1 - yweight)*Wrent_f0(iylow,is,iz,iagg)**(1 - gamma)&
										+ yweight*Wrent_f0(iyupp,is,iz,iagg)**(1 - gamma)))**(1/(1 - gamma)))
									endif
								enddo
								do iy = 1, ny_neg
									Wown_f0(iy,im,ig,is,iz,iagg) = Wown_f0(ny_neg + 1,im,ig,is,iz,iagg)
								enddo
							enddo
							!$omp end do
							!$omp end parallel
						enddo
					enddo
				enddo
			enddo
			!Consumption/savings subperiod
			do iagg = 1, nagg
				do iz = 1, nz
					do is = 1, ns
						!Renters
						call omp_set_num_threads(num_threads)
						!$omp parallel private(V_maxsofar,ibprime,spending,hc,c,util,izprime,isprime,ieprime,yprime_f0,iy0low,iy0upp,y0weight,Vcont_e_f0,Vcont_s_f0,Vcont_z_f0,yprime_f1,iy1low,iy1upp,y1weight,Vcont_e_f1,Vcont_s_f1,Vcont_z_f1,val)
						!$omp do
						do iy = ny_neg + 1, ny
							!Households with good credit
							V_maxsofar = -1e8
							ibprime    = locbrent_f0(iy,is,iz,iagg)
							spending   = min_spend + yy(iy) - qb(iz,iagg)*bb(ibprime)
							if (spending > 0.d0) then
								if (spending >= max_a*(pa + (cshare(iz)/(1 - cshare(iz))*pa)**ies)) then
									hc = max_a
									c  = spending - pa*max_a
								else
									hc = spending/(pa + (cshare(iz)/(1 - cshare(iz))*pa)**ies)
									c  = spending*(cshare(iz)/(1 - cshare(iz))*pa)**ies/(pa + (cshare(iz)/(1 - cshare(iz))*pa)**ies)
								endif
								util = (cshare(iz)*c**((ies - 1)/ies) + (1 - cshare(iz))*hc**((ies - 1)/ies))**(ies/(ies - 1))
								!Continuation utilities
								do izprime = 1, nz
									do isprime = 1, ns
										do ieprime = 1, ne
											yprime_f0 = (labor(isprime,ieprime)*wprime(iz,iagg,izprime) + bb(ibprime))*(1 - taxprime(iz,iagg,izprime))
											iy0low    = ly(yprime_f0)
											iy0upp    = uy(yprime_f0)
											y0weight  = wy(yprime_f0)
											!Households with good credit
											Vcont_e_f0(ieprime) = dot_product(waggprime(iz,iagg,izprime,:),(1 - y0weight)&
											*Wrent_f0(iy0low,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma) + y0weight&
											*Wrent_f0(iy0upp,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma))
										enddo
										Vcont_s_f0(isprime) = dot_product(Vcont_e_f0,emeas)
									enddo
									Vcont_z_f0(izprime) = dot_product(Vcont_s_f0,prob_ss(is,:,iz,izprime))
								enddo
								!Lifetime utility
								val = ((1 - beta)*util**((1 - gamma)/theta)	+ beta*dot_product(Vcont_z_f0,prob_zz(iz,:))**(1/theta))**(theta/(1 - gamma))
								if (val > V_maxsofar) then
									Vnew_rent_f0(iy,is,iz,iagg) = val
								endif
							endif
							!Households with bad credit
							V_maxsofar = -1e8
							ibprime    = locbrent_f1(iy,is,iz,iagg)
							spending   = min_spend + yy(iy) - qb(iz,iagg)*bb(ibprime)
							if (spending > 0.d0) then
								if (spending >= max_a*(pa + (cshare(iz)/(1 - cshare(iz))*pa)**ies)) then
									hc = max_a
									c  = spending - pa*max_a
								else
									hc = spending/(pa + (cshare(iz)/(1 - cshare(iz))*pa)**ies)
									c  = spending*(cshare(iz)/(1 - cshare(iz))*pa)**ies/(pa + (cshare(iz)/(1 - cshare(iz))*pa)**ies)
								endif
								util = (cshare(iz)*c**((ies - 1)/ies) + (1 - cshare(iz))*hc**((ies - 1)/ies))**(ies/(ies - 1))
								!Continuation utilities
								do izprime = 1, nz
									do isprime = 1, ns
										do ieprime = 1, ne
											yprime_f0 = (labor(isprime,ieprime)*wprime(iz,iagg,izprime) + bb(ibprime))*(1 - taxprime(iz,iagg,izprime))
											iy0low    = ly(yprime_f0)
											iy0upp    = uy(yprime_f0)
											y0weight  = wy(yprime_f0)
											yprime_f1 = ((1 - fincloss)*labor(isprime,ieprime)*wprime(iz,iagg,izprime) + bb(ibprime))*(1 - taxprime(iz,iagg,izprime))
											iy1low    = ly(yprime_f1)
											iy1upp    = uy(yprime_f1)
											y1weight  = wy(yprime_f1)
											!Households with bad credit
											Vcont_e_f1(ieprime) = dot_product(waggprime(iz,iagg,izprime,:),pflag*((1 - y1weight)&
											*Wrent_f1(iy1low,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma) + y1weight&
											*Wrent_f1(iy1upp,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)) + (1 - pflag)&
											*((1 - y0weight)*Wrent_f0(iy0low,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma) + y0weight&
											*Wrent_f0(iy0upp,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)))
										enddo
										Vcont_s_f1(isprime) = dot_product(Vcont_e_f1,emeas)
									enddo
									Vcont_z_f1(izprime) = dot_product(Vcont_s_f1,prob_ss(is,:,iz,izprime))
								enddo
								!Lifetime utility
								val = ((1 - beta)*util**((1 - gamma)/theta)	+ beta*dot_product(Vcont_z_f1,prob_zz(iz,:))**(1/theta))**(theta/(1 - gamma))
								if (val > V_maxsofar) then
									Vnew_rent_f1(iy,is,iz,iagg) = val
								endif
							endif
						enddo
						!$omp end do
						!$omp end parallel
						do iy = 1, ny_neg
							Vnew_rent_f0(iy,is,iz,iagg) = Vnew_rent_f0(ny_neg + 1,is,iz,iagg)
							Vnew_rent_f1(iy,is,iz,iagg) = Vnew_rent_f1(ny_neg + 1,is,iz,iagg)
						enddo
						do ig = 1, ng
							ih      = hfun(ig)
							iu      = ufun(ig)
							igshock = gfun(ih,2)
							!Homeowners with bad credit
							call omp_set_num_threads(num_threads)
							!$omp parallel private(V_maxsofar,ibprime,spending,hcstay,cstay,ustay,hcmove,cmove,umove,hc,c,util,izprime,isprime,ieprime,yprime_f0,iy0low,iy0upp,y0weight,yprime_f1,iy1low,iy1upp,y1weight,Vcont_e_f1,Vcont_s_f1,Vcont_z_f1,val)
							!$omp do
							do iy = ny_neg + 1, ny
								V_maxsofar = -1e8
								ibprime    = locbown_f1(iy,ig,is,iz,iagg)
								spending   = min_spend + yy(iy) - qb(iz,iagg)*bb(ibprime)
								if (spending > 0.d0) then
									!Utility of occupying
									hcstay = utilown*uu(iu)*hh(ih)
									cstay  = spending
									ustay  = (cshare(iz)*cstay**((ies - 1)/ies) + (1 - cshare(iz))*hcstay**((ies - 1)/ies))**(ies/(ies - 1))
									!Utility of renting elsewhere
									if (spending >= max_a*(pa + (cshare(iz)/(1 - cshare(iz))*pa)**ies)) then
										hcmove = max_a
										cmove  = spending - pa*max_a
									else
										hcmove = spending/(pa + (cshare(iz)/(1 - cshare(iz))*pa)**ies)
										cmove  = spending*(cshare(iz)/(1 - cshare(iz))*pa)**ies/(pa + (cshare(iz)/(1 - cshare(iz))*pa)**ies)
									endif
									umove = (cshare(iz)*cmove**((ies - 1)/ies) + (1 - cshare(iz))*hcmove**((ies - 1)/ies))**(ies/(ies - 1))
									!Optimal occupation decision
									if (ustay >= umove) then
										util = ustay
										c    = cstay
										hc   = 0.d0
									else
										util = umove
										c    = cmove
										hc   = hcmove
									endif
									!Continuation utilities
									do izprime = 1, nz
										if (iu == 1 .AND. nu == 2) then
											do isprime = 1, ns
												do ieprime = 1, ne
													yprime_f0 = (labor(isprime,ieprime)*wprime(iz,iagg,izprime) + bb(ibprime))*(1 - taxprime(iz,iagg,izprime))
													iy0low    = ly(yprime_f0)
													iy0upp    = uy(yprime_f0)
													y0weight  = wy(yprime_f0)
													yprime_f1 = ((1 - fincloss)*labor(isprime,ieprime)*wprime(iz,iagg,izprime) + bb(ibprime))*(1 - taxprime(iz,iagg,izprime))
													iy1low    = ly(yprime_f1)
													iy1upp    = uy(yprime_f1)
													y1weight  = wy(yprime_f1)
													Vcont_e_f1(ieprime) = dot_product(waggprime(iz,iagg,izprime,:),pflag*((1 - y1weight)&
													*((1 - deprec_h)*((1 - pseparate)*Wown_f1(iy1low,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma) + pseparate&
													*Wown_f1(iy1low,igshock,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)) + deprec_h&
													*Wrent_f1(iy1low,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)) + y1weight&
													*((1 - deprec_h)*((1 - pseparate)*Wown_f1(iy1upp,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma) + pseparate&
													*Wown_f1(iy1upp,igshock,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)) + deprec_h&
													*Wrent_f1(iy1upp,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma))) + (1 - pflag)&
													*((1 - y0weight)*((1 - deprec_h)*((1 - pseparate)*Wown_f0(iy0low,1,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)&
													+ pseparate*Wown_f0(iy0low,1,igshock,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma))&
													+ deprec_h*Wrent_f0(iy0low,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)) + y0weight&
													*((1 - deprec_h)*((1 - pseparate)*Wown_f0(iy0upp,1,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma) + pseparate&
													*Wown_f0(iy0upp,1,igshock,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma))&
													+ deprec_h*Wrent_f0(iy0upp,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma))))
												enddo
												Vcont_s_f1(isprime) = dot_product(Vcont_e_f1,emeas)
											enddo
											Vcont_z_f1(izprime) = dot_product(Vcont_s_f1,prob_ss(is,:,iz,izprime))
										else
											do isprime = 1, ns
												do ieprime = 1, ne
													yprime_f0 = (labor(isprime,ieprime)*wprime(iz,iagg,izprime) + bb(ibprime))*(1 - taxprime(iz,iagg,izprime))
													iy0low    = ly(yprime_f0)
													iy0upp    = uy(yprime_f0)
													y0weight  = wy(yprime_f0)
													yprime_f1 = ((1 - fincloss)*labor(isprime,ieprime)*wprime(iz,iagg,izprime) + bb(ibprime))*(1 - taxprime(iz,iagg,izprime))
													iy1low    = ly(yprime_f1)
													iy1upp    = uy(yprime_f1)
													y1weight  = wy(yprime_f1)
													Vcont_e_f1(ieprime) = dot_product(waggprime(iz,iagg,izprime,:),pflag*((1 - y1weight)&
													*((1 - deprec_h)*Wown_f1(iy1low,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma) + deprec_h&
													*Wrent_f1(iy1low,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)) + y1weight&
													*((1 - deprec_h)*Wown_f1(iy1upp,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma) + deprec_h&
													*Wrent_f1(iy1upp,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma))) + (1 - pflag)&
													*((1 - y0weight)*((1 - deprec_h)*Wown_f0(iy0low,1,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)&
													+ deprec_h*Wrent_f0(iy0low,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)) + y0weight&
													*((1 - deprec_h)*Wown_f0(iy0upp,1,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)&
													+ deprec_h*Wrent_f0(iy0upp,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma))))
												enddo
												Vcont_s_f1(isprime) = dot_product(Vcont_e_f1,emeas)
											enddo
											Vcont_z_f1(izprime) = dot_product(Vcont_s_f1,prob_ss(is,:,iz,izprime))
										endif
									enddo
									!Lifetime utility
									val = ((1 - beta)*util**((1 - gamma)/theta)	+ beta*dot_product(Vcont_z_f1,prob_zz(iz,:))**(1/theta))**(theta/(1 - gamma))
									if (val > V_maxsofar) then
										Vnew_own_f1(iy,ig,is,iz,iagg) = val
									endif
								endif
							enddo
							!$omp end do
							!$omp end parallel
							do iy = 1, ny_neg
								Vnew_own_f1(iy,ig,is,iz,iagg) = Vnew_own_f1(ny_neg + 1,ig,is,iz,iagg)
							enddo
							!Homeowners with good credit
							!Households without a mortgage
							call omp_set_num_threads(num_threads)
							!$omp parallel private(cash_at_hand,V_maxsofar,imprime,ibprime,spending,hcstay,cstay,ustay,hcmove,cmove,umove,hc,c,util,izprime,isprime,ieprime,yprime_f0,iy0low,iy0upp,y0weight,Vcont_e_f0,Vcont_s_f0,Vcont_z_f0,val)
							!$omp do
							do iy = 1, ny
								if (iy <= ny_neg) then
									cash_at_hand = yy(iy)*abs(ylb(ig,is,iz,iagg))
								else
									cash_at_hand = yy(iy)
								endif
								V_maxsofar = -1e8
								imprime    = locm_f0(iy,1,ig,is,iz,iagg)
								ibprime    = locbown_f0(iy,1,ig,is,iz,iagg)
								spending   = min_spend + cash_at_hand + qm0(imprime,ibprime,ig,is,iz,iagg)*mm(imprime)&
								*hh(ih) - qb(iz,iagg)*bb(ibprime)
								if (spending > 0.d0) then
									!Utility of occupying
									hcstay = utilown*uu(iu)*hh(ih)
									cstay  = spending
									ustay  = (cshare(iz)*cstay**((ies - 1)/ies) + (1 - cshare(iz))*hcstay**((ies - 1)/ies))**(ies/(ies - 1))
									!Utility of renting elsewhere
									if (spending >= max_a*(pa + (cshare(iz)/(1 - cshare(iz))*pa)**ies)) then
										hcmove = max_a
										cmove  = spending - pa*max_a
									else
										hcmove = spending/(pa + (cshare(iz)/(1 - cshare(iz))*pa)**ies)
										cmove  = spending*(cshare(iz)/(1 - cshare(iz))*pa)**ies/(pa + (cshare(iz)/(1 - cshare(iz))*pa)**ies)
									endif
									umove = (cshare(iz)*cmove**((ies - 1)/ies) + (1 - cshare(iz))*hcmove**((ies - 1)/ies))**(ies/(ies - 1))
									!Optimal occupation decision
									if (ustay >= umove) then
										util = ustay
										c    = cstay
										hc   = 0.d0
									else
										util = umove
										c    = cmove
										hc   = hcmove
									endif
									!Continuation utilities
									do izprime = 1, nz
										if (iu == 1 .AND. nu == 2) then
											do isprime = 1, ns
												do ieprime = 1, ne
													yprime_f0 = (labor(isprime,ieprime)*wprime(iz,iagg,izprime) + bb(ibprime))*(1 - taxprime(iz,iagg,izprime))
													iy0low    = ly(yprime_f0)
													iy0upp    = uy(yprime_f0)
													y0weight  = wy(yprime_f0)
													Vcont_e_f0(ieprime) = dot_product(waggprime(iz,iagg,izprime,:),(1 - y0weight)&
													*((1 - deprec_h)*((1 - pseparate)*Wown_f0(iy0low,imprime,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)&
													+ pseparate*Wown_f0(iy0low,imprime,igshock,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma))&
													+ deprec_h*Wrent_f0(iy0low,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)) + y0weight&
													*((1 - deprec_h)*((1 - pseparate)*Wown_f0(iy0upp,imprime,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)&
													+ pseparate*Wown_f0(iy0upp,imprime,igshock,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma))&
													+ deprec_h*Wrent_f0(iy0upp,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)))
												enddo
												Vcont_s_f0(isprime) = dot_product(Vcont_e_f0,emeas)
											enddo
											Vcont_z_f0(izprime) = dot_product(Vcont_s_f0,prob_ss(is,:,iz,izprime))
										else
											do isprime = 1, ns
												do ieprime = 1, ne
													yprime_f0 = (labor(isprime,ieprime)*wprime(iz,iagg,izprime) + bb(ibprime))*(1 - taxprime(iz,iagg,izprime))
													iy0low    = ly(yprime_f0)
													iy0upp    = uy(yprime_f0)
													y0weight  = wy(yprime_f0)
													Vcont_e_f0(ieprime) = dot_product(waggprime(iz,iagg,izprime,:),(1 - y0weight)&
													*((1 - deprec_h)*Wown_f0(iy0low,imprime,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)&
													+ deprec_h*Wrent_f0(iy0low,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)) + y0weight&
													*((1 - deprec_h)*Wown_f0(iy0upp,imprime,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)&
													+ deprec_h*Wrent_f0(iy0upp,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)))
												enddo
												Vcont_s_f0(isprime) = dot_product(Vcont_e_f0,emeas)
											enddo
											Vcont_z_f0(izprime) = dot_product(Vcont_s_f0,prob_ss(is,:,iz,izprime))
										endif
									enddo
									!Lifetime utility
									val = ((1 - beta)*util**((1 - gamma)/theta)	+ beta*dot_product(Vcont_z_f0,prob_zz(iz,:))**(1/theta))**(theta/(1 - gamma))
									if (val > V_maxsofar) then
										Vnew_own_f0(iy,1,ig,is,iz,iagg) = val
									endif
								endif
							enddo
							!$omp end do
							!$omp end parallel
							call omp_set_num_threads(num_threads)
							!$omp parallel private(iy,V_maxsofar,ibprime,spending,hcstay,cstay,ustay,hcmove,cmove,umove,hc,c,util,izprime,isprime,ieprime,yprime_f0,iy0low,iy0upp,y0weight,Vcont_e_f0,Vcont_s_f0,Vcont_z_f0,val)
							!$omp do
							do im = 2, nm
								!Households with a mortgage
								do iy = ny_neg + 1, ny
									V_maxsofar = -1e8
									ibprime    = locbown_f0(iy,im,ig,is,iz,iagg)
									spending   = min_spend + yy(iy) - qb(iz,iagg)*bb(ibprime)
									if (spending > 0.d0) then
										!Utility of occupying
										hcstay = utilown*uu(iu)*hh(ih)
										cstay  = spending
										ustay  = (cshare(iz)*cstay**((ies - 1)/ies) + (1 - cshare(iz))*hcstay**((ies - 1)/ies))**(ies/(ies - 1))
										!Utility of renting elsewhere
										if (spending >= max_a*(pa + (cshare(iz)/(1 - cshare(iz))*pa)**ies)) then
											hcmove = max_a
											cmove  = spending - pa*max_a
										else
											hcmove = spending/(pa + (cshare(iz)/(1 - cshare(iz))*pa)**ies)
											cmove  = spending*(cshare(iz)/(1 - cshare(iz))*pa)**ies/(pa + (cshare(iz)/(1 - cshare(iz))*pa)**ies)
										endif
										umove = (cshare(iz)*cmove**((ies - 1)/ies) + (1 - cshare(iz))*hcmove**((ies - 1)/ies))**(ies/(ies - 1))
										!Optimal occupation decision
										if (ustay >= umove) then
											util = ustay
											c    = cstay
											hc   = 0.d0
										else
											util = umove
											c    = cmove
											hc   = hcmove
										endif
										!Continuation utilities
										do izprime = 1, nz
											if (iu == 1 .AND. nu == 2) then
												do isprime = 1, ns
													do ieprime = 1, ne
														yprime_f0 = (labor(isprime,ieprime)*wprime(iz,iagg,izprime) + bb(ibprime))*(1 - taxprime(iz,iagg,izprime))
														iy0low    = ly(yprime_f0)
														iy0upp    = uy(yprime_f0)
														y0weight  = wy(yprime_f0)
														Vcont_e_f0(ieprime) = dot_product(waggprime(iz,iagg,izprime,:),(1 - y0weight)&
														*((1 - deprec_h)*((1 - pseparate)*Wown_f0(iy0low,im,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)&
														+ pseparate*Wown_f0(iy0low,im,igshock,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma))&
														+ deprec_h*Wrent_f0(iy0low,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)) + y0weight&
														*((1 - deprec_h)*((1 - pseparate)*Wown_f0(iy0upp,im,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)&
														+ pseparate*Wown_f0(iy0upp,im,igshock,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma))&
														+ deprec_h*Wrent_f0(iy0upp,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)))
													enddo
													Vcont_s_f0(isprime) = dot_product(Vcont_e_f0,emeas)
												enddo
												Vcont_z_f0(izprime) = dot_product(Vcont_s_f0,prob_ss(is,:,iz,izprime))
											else
												do isprime = 1, ns
													do ieprime = 1, ne
														yprime_f0 = (labor(isprime,ieprime)*wprime(iz,iagg,izprime) + bb(ibprime))*(1 - taxprime(iz,iagg,izprime))
														iy0low    = ly(yprime_f0)
														iy0upp    = uy(yprime_f0)
														y0weight  = wy(yprime_f0)
														Vcont_e_f0(ieprime) = dot_product(waggprime(iz,iagg,izprime,:),(1 - y0weight)&
														*((1 - deprec_h)*Wown_f0(iy0low,im,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)&
														+ deprec_h*Wrent_f0(iy0low,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)) + y0weight&
														*((1 - deprec_h)*Wown_f0(iy0upp,im,ig,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)&
														+ deprec_h*Wrent_f0(iy0upp,isprime,izprime,iaggprime(iz,iagg,izprime,:))**(1 - gamma)))
													enddo
													Vcont_s_f0(isprime) = dot_product(Vcont_e_f0,emeas)
												enddo
												Vcont_z_f0(izprime) = dot_product(Vcont_s_f0,prob_ss(is,:,iz,izprime))
											endif
										enddo
										!Lifetime utility
										val = ((1 - beta)*util**((1 - gamma)/theta)	+ beta*dot_product(Vcont_z_f0,prob_zz(iz,:))**(1/theta))**(theta/(1 - gamma))
										if (val > V_maxsofar) then
											Vnew_own_f0(iy,im,ig,is,iz,iagg) = val
										endif
									endif
								enddo
								do iy = 1, ny_neg
									Vnew_own_f0(iy,im,ig,is,iz,iagg) = Vnew_own_f0(ny_neg + 1,im,ig,is,iz,iagg)
								enddo
							enddo
							!$omp end do
							!$omp end parallel
						enddo
					enddo
				enddo
			enddo
			err_valfn = max(maxval(abs(Vnew_own_f0 - Vown_f0)),maxval(abs(Vnew_rent_f0 - Vrent_f0)),maxval(abs(Vnew_own_f1 - Vown_f1)),maxval(abs(Vnew_rent_f1 - Vrent_f1)))
			if (err_valfn < minerr_accel) then
				minerr_accel = err_valfn
			endif
			Vown_f0  = Vnew_own_f0
			Vrent_f0 = Vnew_rent_f0
			Vown_f1  = Vnew_own_f1
			Vrent_f1 = Vnew_rent_f1
			write(*, '(a22, i4, a12, f20.6)'), '          Iteration = ' ,iter_valfn, ', err_val = ' ,err_valfn
		endif
	enddo
	
	!Compute optimal bond holdings as of the subperiod 1 state for households with good credit
	do iz = 1, nz
		do iagg = 1, nagg
			do is = 1, ns
				do ig = 1, ng
					ih = hfun(ig)
					!Households without a mortgage
					do iy = ny_neg + 1, ny
						bown_f0_sp1(iy,1,ig,is,iz,iagg)    = bown_f0(iy,1,ig,is,iz,iagg)
						locbown_f0_sp1(iy,1,ig,is,iz,iagg) = locbown_f0(iy,1,ig,is,iz,iagg)
					enddo
					do iy = 1, ny_neg
						bown_f0_sp1(iy,1,ig,is,iz,iagg)	   = bown_f0_sp1(ny_neg + 1,1,ig,is,iz,iagg)
						locbown_f0_sp1(iy,1,ig,is,iz,iagg) = locbown_f0_sp1(ny_neg + 1,1,ig,is,iz,iagg)
					enddo
					!Households with a mortgage
					do im = 2, nm
						do iy = ny_neg + 1, ny
							imprime = locm_f0(iy,im,ig,is,iz,iagg)
							ypay    = yy(iy) - (mm(im) - qm(iz,iagg)*mm(imprime))*hh(ih)
							if (ypay < 0.d0) then
								ypay = ypay/abs(ylb(ig,is,iz,iagg))
							endif
							iylow   = ly(ypay)
							iyupp   = uy(ypay)
							yweight = wy(ypay)
							!Bond holdings as of the subperiod 1 state
							bown_f0_sp1(iy,im,ig,is,iz,iagg) = (1 - yweight)*bown_f0(iylow,imprime,ig,is,iz,iagg) + yweight*bown_f0(iyupp,imprime,ig,is,iz,iagg)
							if (yweight >= 0.5d0) then
								locbown_f0_sp1(iy,im,ig,is,iz,iagg) = locbown_f0(iyupp,imprime,ig,is,iz,iagg)
							else
								locbown_f0_sp1(iy,im,ig,is,iz,iagg) = locbown_f0(iylow,imprime,ig,is,iz,iagg)
							endif
						enddo
						do iy = 1, ny_neg
							bown_f0_sp1(iy,im,ig,is,iz,iagg)	= bown_f0_sp1(ny_neg + 1,im,ig,is,iz,iagg)
							locbown_f0_sp1(iy,im,ig,is,iz,iagg) = locbown_f0_sp1(ny_neg + 1,im,ig,is,iz,iagg)
						enddo
					enddo
				enddo
			enddo
		enddo
	enddo

end subroutine sub_valfn_ez_ces