%------------------------------------------------------------------------
%Example: Single plot, two curves, two y-axes
%------------------------------------------------------------------------

x0 = 51;
x1 = 225;
is = 1;

x  = yy(:,1,is,1,1);
y1 = xb_f0(:,is,1,1);
y2 = pbuy_f0(:,is,1,1);
legend1 = 'Submarket';
legend2 = 'p(buy)';

[haxes,hline1,hline2] = plotyy(x(x0:x1),y1(x0:x1),x(x0:x1),y2(x0:x1));
%Position
set(gcf,'position',[200 200 625 275]);
%Title
%title('Buying Market','FontName','Garamond');
%Curves
set(hline1,'MarkerSize',5,'Marker','o','LineWidth',2,'Color',[1 0 0]);
set(hline2,'MarkerSize',5,'Marker','x','LineWidth',2,'Linestyle','--','Color',[0 0 1]);
%Legend
legend(legend1,legend2)
%Left y-axis
axes(haxes(1))
ylabel('Purchase price','FontName','Garamond');
ylim([0 ceil(y1(x1))]);
set(haxes(1),'YTick',0:0.2*ceil(y1(x1)):ceil(y1(x1)),'YGrid','on','FontName','Garamond','YColor',[0 0 0]);
%x-axis
xlabel('Cash at hand','FontName','Garamond');
set(gca,'XLim',[0.5*floor(x(x0)*2) 0.5*ceil(x(x1)*2)],'XTick',ceil(0.5*floor(x(x0)*2)):1:floor(0.5*ceil(x(x1)*2)),'FontName','Garamond');
%Right y-axis
axes(haxes(2))
ylabel('Trade probability','FontName','Garamond');
ylim([0 ceil(y2(x1))]);
set(haxes(2),'YTick',0:0.2*ceil(y2(x1)):ceil(y2(x1)),'FontName','Garamond','YColor',[0 0 0]);
%x-axis
set(gca,'XLim',[0.5*floor(x(x0)*2) 0.5*ceil(x(x1)*2)],'XTick',ceil(0.5*floor(x(x0)*2)):1:floor(0.5*ceil(x(x1)*2)),'FontName','Garamond');

%------------------------------------------------------------------------
%Example: Single plot, four curves, two y-axes
%------------------------------------------------------------------------
x0 = 51;
x1 = 225;
is = 1;

x  = yy(:,1,is,1,1);
y1 = [xb_f0(:,is,1,1) xb_f0(:,is,2,5)];
y2 = [pbuy_f0(:,is,1,1) pbuy_f0(:,is,2,5)];
legend1 = 'Submarket, low';
legend2 = 'Submarket, high';
legend3 = 'p(buy), low';
legend4 = 'p(buy), high';

[haxes,hline1,hline2] = plotyy(x(x0:x1),y1(x0:x1,:),x(x0:x1),y2(x0:x1,:));
%Position
set(gcf,'position',[200 200 625 275]);
%Title
%title('Buying Market','FontName','Garamond');
%Curves
set(hline1(1),'MarkerSize',5,'Marker','o','Color','r');
set(hline1(2),'MarkerSize',5,'Marker','+','Color','m');
set(hline2(1),'MarkerSize',5,'Marker','x','Color','b');
set(hline2(2),'MarkerSize',5,'Marker','s','Color','g');
%Legend
legend(legend1,legend2,legend3,legend4);
%Left y-axis
axes(haxes(1))
ylabel('Purchase price','FontName','Garamond');
ylim([0 max(ceil(y1(x1,:)))]);
set(haxes(1),'YTick',0:0.2*max(ceil(y1(x1,:))):max(ceil(y1(x1,:))),'YGrid','on','FontName','Garamond','YColor',[0 0 0]);
%x-axis
xlabel('Cash at hand','FontName','Garamond');
set(gca,'XLim',[0.5*floor(x(x0)*2) 0.5*ceil(x(x1)*2)],'XTick',ceil(0.5*floor(x(x0)*2)):1:floor(0.5*ceil(x(x1)*2)),'FontName','Garamond');
%Right y-axis
axes(haxes(2))
ylabel('Trade probability','FontName','Garamond');
ylim([0 max(ceil(y2(x1,:)))]);
set(haxes(2),'YTick',0:0.2*max(ceil(y2(x1,:))):max(ceil(y2(x1,:))),'FontName','Garamond','YColor',[0 0 0]);
%x-axis
set(gca,'XLim',[0.5*floor(x(x0)*2) 0.5*ceil(x(x1)*2)],'XTick',ceil(0.5*floor(x(x0)*2)):1:floor(0.5*ceil(x(x1)*2)),'FontName','Garamond');

%------------------------------------------------------------------------
%Example: Two subplots, two curves each, one axis each
%------------------------------------------------------------------------

x0 = 51;
x1 = 225;
is = 1;

% Create figure
figure1 = figure;
%Position
set(gcf,'position',[200 200 625 275]);

%%%Subplot 1%%%
x  = yy(:,1,is,1,1);
y1 = pbuy_f0(:,is,1,6);
y2 = pbuy_f0(:,is,1,10);

% Create subplot
subplot1 = subplot(1,2,1,'Parent',figure1,'FontName','Garamond','YTick',0:0.2:1,'XTick',0:3:15);
box('on');
hold('all');
% Create multiple lines using matrix input to plot
plot1 = plot(x,y1,x,y2,'Parent',subplot1,'LineWidth',1);
set(plot1(1),'MarkerSize',5,'Marker','o','Color','b','DisplayName','low p_h');
set(plot1(2),'MarkerSize',5,'Marker','*','Color','r','DisplayName','high p_h');
% Create xlabel
xlabel('Cash at hand','FontName','Garamond');
% Create ylabel
ylabel('Buying probability','FontName','Garamond');
%Axis limits
xlim([0 ceil(x(x1))]);
ylim([0 ceil(max(max(y1(x0:x1),y2(x0:x1))))]);
% Create title
title({'Probability of Buying'},'FontName','Garamond');

%%%Subplot 2%%%
x  = yy(:,1,is,1,1);
y1 = xb_f0(:,is,1,6);
y2 = xb_f0(:,is,1,10);

% Create subplot
subplot2 = subplot(1,2,2,'Parent',figure1,'FontName','Garamond','YTick',0:1:ceil(max(max(y1(x0:x1),y2(x0:x1)))),'XTick',0:3:15);
box('on');
hold('all');
% Create multiple lines using matrix input to plot
plot2 = plot(x,y1,x,y2,'Parent',subplot2,'LineWidth',2);
set(plot2(1),'LineStyle',':','Color','b','DisplayName','low p_h');
set(plot2(2),'LineStyle','--','Color','r','DisplayName','high p_h');
% Create xlabel
xlabel('Cash at hand','FontName','Garamond');
% Create ylabel
ylabel('Purchase price','FontName','Garamond');
%Axis limits
xlim([0 ceil(x(x1))]);
ylim([0 ceil(max(max(y1(x0:x1),y2(x0:x1))))]);
% Create title
title({'Buying Submarket Choice'},'FontName','Garamond');
% Create legend
legend(subplot1,'show');
legend(subplot2,'show');