%%%Load first series

%Output, consumption, and investment
load IRF\Baseline\Impulse_z23\yc_t.txt
load IRF\Baseline\Impulse_z23\yh_t.txt
load IRF\Baseline\Impulse_z23\gdp_t.txt
load IRF\Baseline\Impulse_z23\cons_t.txt
load IRF\Baseline\Impulse_z23\hcons_t.txt
load IRF\Baseline\Impulse_z23\invest_k_t.txt
%House prices, sales, months supply, and foreclosures
load IRF\Baseline\Impulse_z23\ph_t.txt
load IRF\Baseline\Impulse_z23\existsales_t.txt
load IRF\Baseline\Impulse_z23\months_supply_t.txt
load IRF\Baseline\Impulse_z23\frate_t.txt
load IRF\Baseline\Impulse_z23\reosales_t.txt
%Net worth, assets, housing wealth, and mortgage debt
load IRF\Baseline\Impulse_z23\networth_t.txt
load IRF\Baseline\Impulse_z23\assets_t.txt
load IRF\Baseline\Impulse_z23\hwealth_t.txt
load IRF\Baseline\Impulse_z23\mortdebt_t.txt

yc            = yc_t;
gdp           = gdp_t;
cons          = cons_t;
hcons         = hcons_t;
invest_k      = invest_k_t;
invest_h      = ph_t.*yh_t;
ph            = ph_t;
existsales    = existsales_t;
months_supply = months_supply_t;
frate         = frate_t;
reosales      = reosales_t;
networth      = networth_t;
assets        = assets_t;
hwealth       = hwealth_t;
mortdebt      = mortdebt_t;

%output = cons + hcons + invest_h + invest_k;
output = yc;

clear *_t;

x0 = 260;
x1 = 420;
tq = (x0:x1)'/4-(x0 + 40)/4;

load IRF\Baseline\Impulse_z23\z_t.txt
if (z_t(x0 + 41) > z_t(x0))
    boom = 1;
else
    boom = 0;
end

%------------------------------------------------------------------------
%Figure 1: Y vs. C, Ch, I, Ih, Assets, and Mortgage Debt
%------------------------------------------------------------------------

figure1 = figure;
%Position
set(gcf,'position',[50 50 800 950]);

%%%Subplot 1%%%
x  = tq;
y1 = output(x0:x1)/output(x0);
y2 = cons(x0:x1)/cons(x0);

if (boom == 1)
    ymin = 0.99*min(min(abs(y1)),min(abs(y2)));
    ymax = 1.01*max(max(abs(y1)),max(abs(y2)));
else
    ymin = 0.99*min(min(abs(y1)),min(abs(y2)));
    ymax = 1.01*max(max(abs(y1)),max(abs(y2)));
end

%Create subplot
subplot1 = subplot(4,3,1,'Parent',figure1,'XTick',0:10:1/0.1*floor(0.1*tq(length(tq))),'Xgrid','on');
box('on');
hold('all');
%Create multiple lines using matrix input to plot
plot1 = plot(x,y1,x,y2,'Parent',subplot1);
set(plot1(1),'Color',[0.701960784313725 0.780392156862745 1],'LineWidth',3);
set(plot1(2),'Color',[0.0431372549019608 0.517647058823529 0.780392156862745],'LineWidth',3);
%Create ylabel
%ylabel('Consumption (normalized)');
xlabel('Time (years)');
%Axis limits
xlim([tq(1) tq(length(tq))]);
ylim([ymin ymax]);
%Create title
title('Composite Consumption');

%%%Subplot 2%%%
x  = tq;
y1 = output(x0:x1)/output(x0);
y2 = hcons(x0:x1)/hcons(x0);

if (boom == 1)
    ymin = 0.99*min(min(abs(y1)),min(abs(y2)));
    ymax = 1.01*max(max(abs(y1)),max(abs(y2)));
else
    ymin = 0.99*min(min(abs(y1)),min(abs(y2)));
    ymax = 1.01*max(max(abs(y1)),max(abs(y2)));
end

%Create subplot
subplot2 = subplot(4,3,2,'Parent',figure1,'XTick',0:10:1/0.1*floor(0.1*tq(length(tq))),'Xgrid','on');
box('on');
hold('all');
%Create multiple lines using matrix input to plot
plot2 = plot(x,y1,x,y2,'Parent',subplot2);
set(plot2(1),'Color',[0.701960784313725 0.780392156862745 1],'LineWidth',3);
set(plot2(2),'Color',[0.0431372549019608 0.517647058823529 0.780392156862745],'LineWidth',3);
%Create ylabel
%ylabel('Housing services (normalized)');
xlabel('Time (years)');
%Axis limits
xlim([tq(1) tq(length(tq))]);
ylim([ymin ymax]);
%Create title
title('Housing Services');

%%%Subplot 3%%%
x  = tq;
y1 = output(x0:x1)/output(x0);
y2 = invest_k(x0:x1)/invest_k(x0);

if (boom == 1)
    ymin = 0.99*min(min(abs(y1)),min(abs(y2)));
    ymax = 1.01*max(max(abs(y1)),max(abs(y2)));
else
    ymin = 0.99*min(min(abs(y1)),min(abs(y2)));
    ymax = 1.01*max(max(abs(y1)),max(abs(y2)));
end

%Create subplot
subplot3 = subplot(4,3,3,'Parent',figure1,'XTick',0:10:1/0.1*floor(0.1*tq(length(tq))),'Xgrid','on');
box('on');
hold('all');
%Create multiple lines using matrix input to plot
plot3 = plot(x,y1,x,y2,'Parent',subplot3);
set(plot3(1),'Color',[0.701960784313725 0.780392156862745 1],'LineWidth',3);
set(plot3(2),'Color',[0.0431372549019608 0.517647058823529 0.780392156862745],'LineWidth',3);
%Create ylabel
%ylabel('Non-residential investment (normalized)');
xlabel('Time (years)');
%Axis limits
xlim([tq(1) tq(length(tq))]);
ylim([ymin ymax]);
%Create title
title('Non-Residential Investment');

%%%Subplot 4%%%
x  = tq;
y1 = output(x0:x1)/output(x0);
y2 = invest_h(x0:x1)/invest_h(x0);

if (boom == 1)
    ymin = 0.99*min(min(abs(y1)),min(abs(y2)));
    ymax = 1.01*max(max(abs(y1)),max(abs(y2)));
else
    ymin = 0.99*min(min(abs(y1)),min(abs(y2)));
    ymax = 1.01*max(max(abs(y1)),max(abs(y2)));
end

% Create subplot
subplot4 = subplot(4,3,4,'Parent',figure1,'XTick',0:10:1/0.1*floor(0.1*tq(length(tq))),'Xgrid','on');
box('on');
hold('all');
% Create multiple lines using matrix input to plot
plot4 = plot(x,y1,x,y2,'Parent',subplot4);
set(plot4(1),'Color',[0.701960784313725 0.780392156862745 1],'LineWidth',3);
set(plot4(2),'Color',[0.0431372549019608 0.517647058823529 0.780392156862745],'LineWidth',3);
% Create ylabel
%ylabel('Residential investment (normalized)');
xlabel('Time (years)');
%Axis limits
xlim([tq(1) tq(length(tq))]);
ylim([ymin ymax]);
%Create title
title('Residential Investment');

%%%Subplot 5%%%
x  = tq;
y1 = output(x0:x1)/output(x0);
y2 = assets(x0:x1)/assets(x0);

if (boom == 1)
    ymin = 0.99*min(min(abs(y1)),min(abs(y2)));
    ymax = 1.01*max(max(abs(y1)),max(abs(y2)));
else
    ymin = 0.99*min(min(abs(y1)),min(abs(y2)));
    ymax = 1.01*max(max(abs(y1)),max(abs(y2)));
end

%Create subplot
subplot5 = subplot(4,3,5,'Parent',figure1,'XTick',0:10:1/0.1*floor(0.1*tq(length(tq))),'Xgrid','on');
box('on');
hold('all');
%Create multiple lines using matrix input to plot
plot5 = plot(x,y1,x,y2,'Parent',subplot5);
set(plot5(1),'Color',[0.701960784313725 0.780392156862745 1],'LineWidth',3);
set(plot5(2),'Color',[0.0431372549019608 0.517647058823529 0.780392156862745],'LineWidth',3);
%Create ylabel
%ylabel('Financial assets (normalized)');
xlabel('Time (years)');
%Axis limits
xlim([tq(1) tq(length(tq))]);
ylim([ymin ymax]);
%Create title
title('Financial Assets');

%%%Subplot 6%%%
x  = tq;
y1 = output(x0:x1)/output(x0);
y2 = mortdebt(x0:x1)/mortdebt(x0);

if (boom == 1)
    ymin = 0.99*min(min(abs(y1)),min(abs(y2)));
    ymax = 1.01*max(max(abs(y1)),max(abs(y2)));
else
    ymin = 0.99*min(min(abs(y1)),min(abs(y2)));
    ymax = 1.01*max(max(abs(y1)),max(abs(y2)));
end

% Create subplot
subplot6 = subplot(4,3,6,'Parent',figure1,'XTick',0:10:1/0.1*floor(0.1*tq(length(tq))),'Xgrid','on');
box('on');
hold('all');
% Create multiple lines using matrix input to plot
plot6 = plot(x,y1,x,y2,'Parent',subplot6);
set(plot6(1),'Color',[0.701960784313725 0.780392156862745 1],'LineWidth',3);
set(plot6(2),'Color',[0.0431372549019608 0.517647058823529 0.780392156862745],'LineWidth',3);
% Create ylabel
%ylabel('Mortgage debt (normalized)');
xlabel('Time (years)');
%Axis limits
xlim([tq(1) tq(length(tq))]);
ylim([ymin ymax]);
%Create title
title('Mortgage Debt');

%%%Subplot 7%%%
x  = tq;
y1 = output(x0:x1)/output(x0);
y2 = ph(x0:x1)/ph(x0);

if (boom == 1)
    ymin = 0.99*min(min(abs(y1)),min(abs(y2)));
    ymax = 1.01*max(max(abs(y1)),max(abs(y2)));
else
    ymin = 0.99*min(min(abs(y1)),min(abs(y2)));
    ymax = 1.01*max(max(abs(y1)),max(abs(y2)));
end

%Create subplot
subplot7 = subplot(4,3,7,'Parent',figure1,'XTick',0:10:1/0.1*floor(0.1*tq(length(tq))),'Xgrid','on');
box('on');
hold('all');
%Create multiple lines using matrix input to plot
plot7 = plot(x,y1,x,y2,'Parent',subplot7);
set(plot7(1),'Color',[0.701960784313725 0.780392156862745 1],'LineWidth',3);
set(plot7(2),'Color',[0.0431372549019608 0.517647058823529 0.780392156862745],'LineWidth',3);
%Create ylabel
%ylabel('House prices (normalized)');
xlabel('Time (years)');
%Axis limits
xlim([tq(1) tq(length(tq))]);
ylim([ymin ymax]);
%Create title
title('House Prices');

%%%Subplot 8%%%
x  = tq;
y1 = output(x0:x1)/output(x0);
y2 = existsales(x0:x1)/existsales(x0);

if (boom == 1)
    ymin = 0.99*min(min(abs(y1)),min(abs(y2)));
    ymax = 1.01*max(max(abs(y1)),max(abs(y2)));
else
    ymin = 0.99*min(min(abs(y1)),min(abs(y2)));
    ymax = 1.01*max(max(abs(y1)),max(abs(y2)));
end

%Create subplot
subplot8 = subplot(4,3,8,'Parent',figure1,'XTick',0:10:1/0.1*floor(0.1*tq(length(tq))),'Xgrid','on');
box('on');
hold('all');
%Create multiple lines using matrix input to plot
plot8 = plot(x,y1,x,y2,'Parent',subplot8);
set(plot8(1),'Color',[0.701960784313725 0.780392156862745 1],'LineWidth',3);
set(plot8(2),'Color',[0.0431372549019608 0.517647058823529 0.780392156862745],'LineWidth',3);
%Create ylabel
%ylabel('Existing sales (normalized)');
xlabel('Time (years)');
%Axis limits
xlim([tq(1) tq(length(tq))]);
%ylim([ymin ymax]);
%Create title
title('Homeowner + REO Sales');

%%%Subplot 9%%%
x  = tq;
y1 = output(x0:x1)/output(x0);
y2 = (existsales(x0:x1) - reosales(x0:x1))/(existsales(x0) - reosales(x0));

if (boom == 1)
    ymin = 0.99*min(min(abs(y1)),min(abs(y2)));
    ymax = 1.01*max(max(abs(y1)),max(abs(y2)));
else
    ymin = 0.99*min(min(abs(y1)),min(abs(y2)));
    ymax = 1.01*max(max(abs(y1)),max(abs(y2)));
end

%Create subplot
subplot9 = subplot(4,3,9,'Parent',figure1,'XTick',0:10:1/0.1*floor(0.1*tq(length(tq))),'Xgrid','on');
box('on');
hold('all');
%Create multiple lines using matrix input to plot
plot9 = plot(x,y1,x,y2,'Parent',subplot9);
set(plot9(1),'Color',[0.701960784313725 0.780392156862745 1],'LineWidth',3);
set(plot9(2),'Color',[0.0431372549019608 0.517647058823529 0.780392156862745],'LineWidth',3);
%Create ylabel
%ylabel('Homeowner sales (normalized)');
xlabel('Time (years)');
%Axis limits
xlim([tq(1) tq(length(tq))]);
ylim([ymin ymax]);
%Create title
title('Homeowner Sales');

%%%Subplot 10%%%
x = tq;
y = months_supply(x0:x1);

if (boom == 1)
    ymin = 0.99*min(min(abs(y1)),min(abs(y2)));
    ymax = 1.01*max(max(abs(y1)),max(abs(y2)));
else
    ymin = 0.99*min(min(abs(y1)),min(abs(y2)));
    ymax = 1.01*max(max(abs(y1)),max(abs(y2)));
end

%Create subplot
subplot10 = subplot(4,3,10,'Parent',figure1,'XTick',0:10:1/0.1*floor(0.1*tq(length(tq))),'Xgrid','on');
box('on');
hold('all');
%Create multiple lines using matrix input to plot
plot10 = plot(x,y,'Parent',subplot10);
set(plot10(1),'Color',[0.0431372549019608 0.517647058823529 0.780392156862745],'LineWidth',3);
%Create ylabel
%ylabel('Months of supply');
xlabel('Time (years)');
%Axis limits
xlim([tq(1) tq(length(tq))]);
%ylim([ymin ymax]);
%Create title
title('Months of Supply');

%%%Subplot 11%%%
x = tq;
y = 400*frate(x0:x1);

if (boom == 1)
    ymin = 0.99*min(min(abs(y1)),min(abs(y2)));
    ymax = 1.01*max(max(abs(y1)),max(abs(y2)));
else
    ymin = 0.99*min(min(abs(y1)),min(abs(y2)));
    ymax = 1.01*max(max(abs(y1)),max(abs(y2)));
end

% Create subplot
subplot11 = subplot(4,3,11,'Parent',figure1,'XTick',0:10:1/0.1*floor(0.1*tq(length(tq))),'Xgrid','on');
box('on');
hold('all');
% Create multiple lines using matrix input to plot
plot11 = plot(x,y,'Parent',subplot11);
set(plot11(1),'Color',[0.0431372549019608 0.517647058823529 0.780392156862745],'LineWidth',3);
% Create ylabel
%ylabel('Foreclosure rate (%)');
xlabel('Time (years)');
%Axis limits
xlim([tq(1) tq(length(tq))]);
%ylim([ymin ymax]);
%Create title
title('Foreclosure Rate');

%%%Subplot 12%%%
x = tq;
y = reosales(x0:x1)/existsales(x0)*100;

if (boom == 1)
    ymin = 0;
    ymax = 1.01*max(y);
else
    ymin = 0;
    ymax = 1.01*max(y);
end

%Create subplot
subplot12 = subplot(4,3,12,'Parent',figure1,'XTick',0:10:1/0.1*floor(0.1*tq(length(tq))),'Xgrid','on');
box('on');
hold('all');
%Create multiple lines using matrix input to plot
plot12 = plot(x,y,'Parent',subplot12);
set(plot12(1),'Color',[0.0431372549019608 0.517647058823529 0.780392156862745],'LineWidth',3);
%Create ylabel
%ylabel('REO sale share (%)');
xlabel('Time (years)');
%Axis limits
xlim([tq(1) tq(length(tq))]);
ylim([ymin ymax]);
%Create title
title('Foreclosure Sale Share');

MakeTightFigure;