function [table1,table2,table3,table4] = aggresults_data(deflator,houseprice,yearstart,yearend,lambda,numlags)
%example: [table1,table2,table3,table4] = aggresults_data(2,2,1975,2010,100000000,4);
load data_combined.mat
%deflator   = 2;
%houseprice = 2;
%yearstart  = 1975;
%yearend    = 2010;
%lambda     = 100000000;
%numlags    = 4;

%-----------------------------------------------------------------------
%Index of variables
%-----------------------------------------------------------------------
%C
%CT
%Ch
%G
%I
%IT
%Ih
%NX
%Y
%cpi
%exist_inventories1982
%exist_mossupply1982
%exist_sales
%foreclosures_allpastdue_nsa
%foreclosures_allpastdue_sa
%foreclosures_pastdue90_nsa
%foreclosures_pastdue90_sa
%foreclosures_seriousdelinquent_nsa
%foreclosures_starts_nsa
%foreclosures_starts_sa
%gdpdef
%new_forsale_nsa
%new_forsale_sa
%new_mosforsale
%new_mossupply_nsa
%new_mossupply_sa
%new_sales_nsa
%new_sales_sa
%pce
%ph_CS1987_nsa
%ph_CS1987_sa
%ph_CShybrid
%ph_FHFA_all
%ph_FMAC
%ph_NAR_median
%ph_newmean
%ph_newmedian
%tq

%Note: ph_all = [ph_FHFA_all ph_FMAC ph_NAR_median ph_CShybrid ph_CS1987_sa
%ph_CS1987_nsa ph_newconstquality ph_newmean ph_newmedian]

%-----------------------------------------------------------------------
%Generate data matrix and HP filter data
%-----------------------------------------------------------------------
%Locate time indices that correspond to yearstart and yearend
for t=1:length(tq)
    if (tq(t) >= yearstart)
        T0 = t;
        break
    end
end
for t=1:length(tq)
    if (tq(t) >= yearend + 1)
        T1 = t - 1;
        break
    end
    if (t == length(tq))
        T1 = t;
    end
end

data1  = [Y-G-NX CT C Ch IT I Ih];
data1  = data1(T0:T1,:);
n      = size(data1,2);
cpi    = cpi(T0:T1);
pce    = pce(T0:T1);
gdpdef = gdpdef(T0:T1);

if (deflator == 1)
    %CPI
    data1 = data1./(cpi*ones(1,n))*cpi(1);
elseif (deflator == 2)
    %Personal consumption expenditures index
    data1 = data1./(pce*ones(1,n))*pce(1);
else
    %GDP deflator
    data1 = data1./(gdpdef*ones(1,n))*gdpdef(1);
end

%-----------------------------------------------------------------------
%Compute business cycle statistics and cross correlations with GDP
%-----------------------------------------------------------------------
table1 = buscycstats(data1,lambda);
table2 = crosscorrGDP(data1,lambda,numlags);

%Table 3
ph          = ph_all(T0:T1,houseprice);
tradevol    = exist_sales(T0:T1,:) + new_sales_sa(T0:T1,:);
Yh          = new_sales_sa(T0:T1,:);
existsales  = exist_sales(T0:T1,:);
inventories = exist_inventories1982(T0:T1,:);
mos_supply  = exist_mossupply1982(T0:T1,:);
%frate       = foreclosures_pastdue90_sa(T0:T1,:);
frate       = foreclosures_starts_sa(T0:T1,:);

%Convert from nominal to real
if (deflator == 1)
    %CPI
    ph = ph./cpi*cpi(1);
elseif (deflator == 2)
    %Personal consumption expenditures index
    ph = ph./pce*pce(1);
else
    %GDP deflator
    ph = ph./gdpdef*gdpdef(1);
end

data3  = [data1(:,1) ph tradevol Yh existsales inventories mos_supply frate];
table3 = -999*ones(size(data3,2),1 + 2*numlags + 1);
for i = 1:size(data3,2)
    for t = 1:T1 - T0 + 1
        if (data3(t,i) > 0)
            T0temp = t;
            break
        end
    end
    table3temp  = crosscorrGDP([data1(T0temp:T1 - T0 + 1,1) data3(T0temp:T1 - T0 + 1,i)],lambda,numlags);
    table3(i,:) = table3temp(2,:);
end

%Table 4
%Convert from nominal to real
if (deflator == 1)
    %CPI
    networth = networth(T0:T1)./cpi*cpi(1);
    assets   = assets(T0:T1)./cpi*cpi(1);
    hwealth  = hwealth(T0:T1)./cpi*cpi(1);
    mortdebt = mortdebt(T0:T1)./cpi*cpi(1);
elseif (deflator == 2)
    %Personal consumption expenditures index
    networth = networth(T0:T1)./pce*pce(1);
    assets   = assets(T0:T1)./pce*pce(1);
    hwealth  = hwealth(T0:T1)./pce*pce(1);
    mortdebt = mortdebt(T0:T1)./pce*pce(1);
else
    %GDP deflator
    networth = networth(T0:T1)./gdpdef*gdpdef(1);
    assets   = assets(T0:T1)./gdpdef*gdpdef(1);
    hwealth  = hwealth(T0:T1)./gdpdef*gdpdef(1);
    mortdebt = mortdebt(T0:T1)./gdpdef*gdpdef(1);
end
data4  = [data1(:,1) networth assets hwealth mortdebt];
table4 = crosscorrGDP(data4,lambda,numlags);

table1 = round(table1*100)/100;
table2 = round(table2*100)/100;
table3 = round(table3*100)/100;
table4 = round(table4*100)/100;

%Shorter tables
disp(quickcorr(existsales,ph,lambda));
%disp(quickcorr(existsales,inventories,lambda));
disp(quickcorr(existsales,mos_supply,lambda));
disp(quickcorr(existsales,frate,lambda));