function [table1,table2,table3,table4] = aggresults_model_noK(T0,T1,lambda,numlags)

%-----------------------------------------------------------------------
%Load model simulated data
%-----------------------------------------------------------------------
%House prices
load Model\ph_t.txt
load Model\pa.txt

ph_t = ph_t(T0:T1);

%Output, consumption, and investment
load Model\gdp_t.txt
load Model\gnp_t.txt
load Model\yc_t.txt
load Model\yh_t.txt
load Model\invest_h_t.txt
load Model\cons_t.txt
load Model\hcons_t.txt

gdp_t      = gdp_t(T0:T1);
gnp_t      = gnp_t(T0:T1);
yc_t       = yc_t(T0:T1);
yh_t       = yh_t(T0:T1);
invest_h_t = invest_h_t(T0:T1);
cons_t     = cons_t(T0:T1);
hcons_t    = hcons_t(T0:T1);

%Wealth
load Model\networth_t.txt
load Model\assets_t.txt
load Model\hwealth_t.txt
load Model\mortdebt_t.txt

networth_t = networth_t(T0:T1);
assets_t   = assets_t(T0:T1);
hwealth_t  = hwealth_t(T0:T1);
mortdebt_t = mortdebt_t(T0:T1);

%Sales, trade volume, and foreclosures
load Model\inventories_t.txt
load Model\existsales_t.txt
load Model\months_supply_t.txt
load Model\tradevol_t.txt
load Model\frate_t.txt
load Model\reosales_t.txt

inventories_t   = inventories_t(T0:T1);
existsales_t    = existsales_t(T0:T1);
months_supply_t = months_supply_t(T0:T1);
tradevol_t      = tradevol_t(T0:T1);
frate_t         = frate_t(T0:T1);
reosales_t      = reosales_t(T0:T1);

%Miscellaneous
load Model\ownrate_t.txt
load Model\htot_t.txt
load Model\hownstock_t.txt
load Model\xsdiscount_t.txt
load Model\TOMsell_t.txt
load Model\TOMbuy_t.txt
load Model\REOdiscount_t.txt

%-----------------------------------------------------------------------
%Prepare data for analysis
%-----------------------------------------------------------------------
%Output, consumption, and investment
Cc   = 4*cons_t;            %Non-housing consumption
Ch   = 4*hcons_t;           %Housing consumption
C    = Cc + Ch;             %Total consumption
Ih   = 4*invest_h_t;        %Housing investment (incl. real estate agent costs)
phyh = 4*ph_t.*yh_t;        %Housing investment (excl. real estate agent costs)
YN   = 4*gnp_t;             %GDP - government spending - net exports - real estate agent costs
YN   = YN - (Ih - phyh);

%Housing prices, sales, and inventories
ph = ph_t;                      %House prices
tradevol    = 4*tradevol_t;     %Total house sales
Yh          = 4*yh_t;           %New house sales
existsales  = 4*existsales_t;   %Existing house sales
%reosales    = 4*reosales_t;    %REO sales
inventories = inventories_t;    %Inventories
mos_supply  = months_supply_t;  %Months' supply of inventories
frate       = 4*frate_t;        %Foreclosure rate

%-----------------------------------------------------------------------
%Compute business cycle statistics and cross correlations with GDP
%-----------------------------------------------------------------------
%Table 1: std dev, std dev rel to GDP, corr with GDP, annual
%autocorrelation, share of GDP
%Table 2: std dev rel to GDP, lagged correlations with GDP

data1  = [YN C Cc Ch phyh];
table1 = buscycstats(data1,lambda);
table2 = crosscorrGDP(data1,lambda,numlags);
data3  = [YN ph tradevol Yh existsales inventories mos_supply frate];
table3 = crosscorrGDP(data3,lambda,numlags);
data4  = [YN networth_t assets_t hwealth_t mortdebt_t];
table4 = crosscorrGDP(data4,lambda,numlags);