function [table1,table2,table3,table4] = aggresults_model(T0,T1,lambda,numlags)

%-----------------------------------------------------------------------
%Load model simulated data
%-----------------------------------------------------------------------
%House prices
load Model\ph_t.txt
load Model\pa.txt

ph_t = ph_t(T0:T1);

%Output, consumption, and investment
load Model\gdp_t.txt
load Model\yc_t.txt
load Model\yh_t.txt
load Model\invest_k_t.txt
load Model\invest_h_t.txt
load Model\cons_t.txt
load Model\hcons_t.txt

gdp_t        = gdp_t(T0:T1);
yc_t         = yc_t(T0:T1);
yh_t         = yh_t(T0:T1);
invest_k_t   = invest_k_t(T0:T1);
invest_h_t   = invest_h_t(T0:T1);
cons_t       = cons_t(T0:T1);
hcons_t      = hcons_t(T0:T1);

%Wealth
load Model\networth_t.txt
load Model\assets_t.txt
load Model\hwealth_t.txt
load Model\mortdebt_t.txt

networth_t = networth_t(T0:T1);
assets_t   = assets_t(T0:T1);
hwealth_t  = hwealth_t(T0:T1);
mortdebt_t = mortdebt_t(T0:T1);

%Sales, trade volume, and foreclosures
load Model\inventories_t.txt
load Model\existsales_t.txt
load Model\months_supply_t.txt
load Model\tradevol_t.txt
load Model\frate_t.txt
load Model\reosales_t.txt

existsales_t    = existsales_t(T0:T1);
tradevol_t      = tradevol_t(T0:T1);
frate_t         = frate_t(T0:T1);
reosales_t      = reosales_t(T0:T1);
inventories_t   = inventories_t(T0:T1);
months_supply_t = months_supply_t(T0:T1);

%Recourse welfare
load Model\welfare_change_all_t.txt
load Model\welfare_change_renters_t.txt
load Model\welfare_change_ltv80_t.txt
load Model\welfare_change_ltv90_t.txt
load Model\welfare_change_ltv95_t.txt

welfare_change_all_t     = welfare_change_all_t(T0:T1);
welfare_change_renters_t = welfare_change_renters_t(T0:T1);
welfare_change_ltv80_t   = welfare_change_ltv80_t(T0:T1);
welfare_change_ltv90_t   = welfare_change_ltv90_t(T0:T1);
welfare_change_ltv95_t   = welfare_change_ltv95_t(T0:T1);

%-----------------------------------------------------------------------
%Prepare data for analysis
%-----------------------------------------------------------------------
%Output, consumption, and investment
Cc   = 4*cons_t;            %Non-housing consumption
Ch   = 4*hcons_t;           %Housing consumption
C    = Cc + Ch;             %Total consumption
Ic   = 4*invest_k_t;        %Non-housing investment
Ih   = 4*invest_h_t;        %Housing investment (incl. real estate agent costs)
phyh = 4*ph_t.*yh_t;        %Housing investment (excl. real estate agent costs)
%I    = Ic + Ih;            %Total investment (incl. real estate agent costs)
I    = Ic + phyh;           %Total investment (excl. real estate agent costs)
%Y    = 4*gdp_t;            %GDP
Y    = C + I;

%Housing prices, sales, and inventories
ph          = ph_t;             %House prices
tradevol    = 4*tradevol_t;     %Total house sales
Yh          = 4*yh_t;           %New house sales
existsales  = 4*existsales_t;   %Existing house sales
reosales    = 4*reosales_t;    %REO sales
inventories = inventories_t;    %Inventories
mos_supply  = months_supply_t;  %Months' supply of inventories
frate       = 4*frate_t;        %Foreclosure rate

%Welfare change
welfare_change_all     = welfare_change_all_t;
welfare_change_renters = welfare_change_renters_t;
welfare_change_ltv80   = welfare_change_ltv80_t;
welfare_change_ltv90   = welfare_change_ltv90_t;
welfare_change_ltv95   = welfare_change_ltv95_t;

%-----------------------------------------------------------------------
%Compute business cycle statistics and cross correlations with GDP
%-----------------------------------------------------------------------
%Table 1: std dev, std dev rel to GDP, corr with GDP, annual
%autocorrelation, share of GDP
%Table 2: std dev rel to GDP, lagged correlations with GDP

data1  = [Y C Cc Ch I Ic phyh];
table1 = buscycstats(data1,lambda);
table2 = crosscorrGDP(data1,lambda,numlags);
data3  = [Y ph tradevol Yh existsales inventories mos_supply frate];
table3 = crosscorrGDP(data3,lambda,numlags);
data4  = [Y networth_t assets_t hwealth_t mortdebt_t];
table4 = crosscorrGDP(data4,lambda,numlags);

table1 = round(table1*100)/100;
table2 = round(table2*100)/100;
table3 = round(table3*100)/100;
table4 = round(table4*100)/100;

[et,ed] = hpfilter(log(existsales - reosales),lambda);
[pt,pd] = hpfilter(log(ph),lambda);
[it,id] = hpfilter(log(inventories),lambda);
[mt,md] = hpfilter(log(mos_supply),lambda);
[ft,fd] = hpfilter(log(frate),lambda);

disp('Correlations of sales with prices, months supply, and foreclosures');
disp(corr(ed,pd));
%disp(corr(ed,id));
disp(corr(ed,md));
disp(corr(ed,fd));

disp('Mean and standard deviation of recourse welfare change');
disp([mean(welfare_change_all) std(welfare_change_all) max(welfare_change_all) min(welfare_change_all)]);
disp([mean(welfare_change_renters) std(welfare_change_renters) max(welfare_change_renters) min(welfare_change_renters)]);
disp([mean(welfare_change_ltv80) std(welfare_change_ltv80) max(welfare_change_ltv80) min(welfare_change_ltv80)]);
disp([mean(welfare_change_ltv90) std(welfare_change_ltv90) max(welfare_change_ltv90) min(welfare_change_ltv90)]);
disp([mean(welfare_change_ltv95) std(welfare_change_ltv95) max(welfare_change_ltv95) min(welfare_change_ltv95)]);

disp('Correlation of house prices and recourse welfare change');
disp(corr(ph,welfare_change_all));
disp(corr(ph,welfare_change_renters));
disp(corr(ph,welfare_change_ltv80));
disp(corr(ph,welfare_change_ltv90));
disp(corr(ph,welfare_change_ltv95));

Tstart = T0;
Tstop  = T0 + 3000;

figure;
plot(Tstart:Tstop,ph(Tstart:Tstop),'Linewidth',2);

figure;
xvar = (ph(Tstart:Tstop) - mean(ph(Tstart:Tstop)))/(max(ph(Tstart:Tstop)) - min(ph(Tstart:Tstop)));
yvar = (welfare_change_all(Tstart:Tstop) - mean(welfare_change_all(Tstart:Tstop)))/(max(welfare_change_all(Tstart:Tstop)) - min(welfare_change_all(Tstart:Tstop)));
plot(Tstart:Tstop,xvar,Tstart:Tstop,yvar,'Linewidth',2);

figure;
xvar = (ph(Tstart:Tstop) - mean(ph(Tstart:Tstop)))/(max(ph(Tstart:Tstop)) - min(ph(Tstart:Tstop)));
yvar = (welfare_change_renters(Tstart:Tstop) - mean(welfare_change_renters(Tstart:Tstop)))/(max(welfare_change_renters(Tstart:Tstop)) - min(welfare_change_renters(Tstart:Tstop)));
plot(Tstart:Tstop,xvar,Tstart:Tstop,yvar,'Linewidth',2);

figure;
xvar = (ph(Tstart:Tstop) - mean(ph(Tstart:Tstop)))/(max(ph(Tstart:Tstop)) - min(ph(Tstart:Tstop)));
yvar = (welfare_change_ltv80(Tstart:Tstop) - mean(welfare_change_ltv80(Tstart:Tstop)))/(max(welfare_change_ltv80(Tstart:Tstop)) - min(welfare_change_ltv80(Tstart:Tstop)));
plot(Tstart:Tstop,xvar,Tstart:Tstop,yvar,'Linewidth',2);

figure;
xvar = (ph(Tstart:Tstop) - mean(ph(Tstart:Tstop)))/(max(ph(Tstart:Tstop)) - min(ph(Tstart:Tstop)));
yvar = (welfare_change_ltv90(Tstart:Tstop) - mean(welfare_change_ltv90(Tstart:Tstop)))/(max(welfare_change_ltv90(Tstart:Tstop)) - min(welfare_change_ltv90(Tstart:Tstop)));
plot(Tstart:Tstop,xvar,Tstart:Tstop,yvar,'Linewidth',2);

figure;
xvar = (ph(Tstart:Tstop) - mean(ph(Tstart:Tstop)))/(max(ph(Tstart:Tstop)) - min(ph(Tstart:Tstop)));
yvar = (welfare_change_ltv95(Tstart:Tstop) - mean(welfare_change_ltv95(Tstart:Tstop)))/(max(welfare_change_ltv95(Tstart:Tstop)) - min(welfare_change_ltv95(Tstart:Tstop)));
plot(Tstart:Tstop,xvar,Tstart:Tstop,yvar,'Linewidth',2);

disp('Correlation of GDP and recourse welfare change');
disp(corr(Y,welfare_change_all));
disp(corr(Y,welfare_change_renters));
disp(corr(Y,welfare_change_ltv80));
disp(corr(Y,welfare_change_ltv90));
disp(corr(Y,welfare_change_ltv95));